@@ -67,15 +67,29 @@ namespace H5 {
 //--------------------------------------------------------------------------
 Group CommonFG::createGroup( const char* name, size_t size_hint ) const
 {
-   // Call C routine H5Gcreate to create the named group, giving the
+   // Create group creation property list for size_hint
+   hid_t gcpl_id = H5Pcreate(H5P_GROUP_CREATE);
+
+   // If the creation of the property list failed, throw an exception
+   if( gcpl_id < 0 )
+      throwException("createGroup", "H5Pcreate failed");
+
+   // Set the local heap size hint
+   if( H5Pset_local_heap_size_hint(gcpl_id, size_hint) < 0) {
+      H5Pclose(gcpl_id);
+      throwException("createGroup", "H5Pset_local_heap_size failed");
+   }
+
+   // Call C routine H5Gcreate2 to create the named group, giving the
    // location id which can be a file id or a group id
-   hid_t group_id = H5Gcreate( getLocId(), name, size_hint );
+   hid_t group_id = H5Gcreate2( getLocId(), name, H5P_DEFAULT, gcpl_id, H5P_DEFAULT );
+
+   // Close the group creation property list
+   H5Pclose(gcpl_id);
 
    // If the creation of the group failed, throw an exception
    if( group_id < 0 )
-   {
-      throwException("createGroup", "H5Gcreate failed");
-   }
+      throwException("createGroup", "H5Gcreate2 failed");
 
    // No failure, create and return the Group object
    Group group( group_id );
@@ -86,7 +100,7 @@ Group CommonFG::createGroup( const char* name, size_t size_hint ) const
 // Function:	CommonFG::createGroup
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 Group CommonFG::createGroup( const H5std_string& name, size_t size_hint ) const
@@ -105,14 +119,14 @@ Group CommonFG::createGroup( const H5std_string& name, size_t size_hint ) const
 //--------------------------------------------------------------------------
 Group CommonFG::openGroup( const char* name ) const
 {
-   // Call C routine H5Gopen to open the named group, giving the
+   // Call C routine H5Gopen2 to open the named group, giving the
    // location id which can be a file id or a group id
-   hid_t group_id = H5Gopen( getLocId(), name );
+   hid_t group_id = H5Gopen2( getLocId(), name, H5P_DEFAULT );
 
    // If the opening of the group failed, throw an exception
    if( group_id < 0 )
    {
-      throwException("openGroup", "H5Gopen failed");
+      throwException("openGroup", "H5Gopen2 failed");
    }
 
    // No failure, create and return the Group object
@@ -124,7 +138,7 @@ Group CommonFG::openGroup( const char* name ) const
 // Function:	CommonFG::openGroup
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 Group CommonFG::openGroup( const H5std_string& name ) const
@@ -150,13 +164,13 @@ DataSet CommonFG::createDataSet( const char* name, const DataType& data_type, co
    hid_t space_id = data_space.getId();
    hid_t create_plist_id = create_plist.getId();
 
-   // Call C routine H5Dcreate to create the named dataset
-   hid_t dataset_id = H5Dcreate( getLocId(), name, type_id, space_id, create_plist_id );
+   // Call C routine H5Dcreate2 to create the named dataset
+   hid_t dataset_id = H5Dcreate2( getLocId(), name, type_id, space_id, H5P_DEFAULT, create_plist_id, H5P_DEFAULT );
 
    // If the creation of the dataset failed, throw an exception
    if( dataset_id < 0 )
    {
-      throwException("createDataSet", "H5Dcreate failed");
+      throwException("createDataSet", "H5Dcreate2 failed");
    }
 
    // No failure, create and return the DataSet object
@@ -168,7 +182,7 @@ DataSet CommonFG::createDataSet( const char* name, const DataType& data_type, co
 // Function:	CommonFG::createDataSet
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 DataSet CommonFG::createDataSet( const H5std_string& name, const DataType& data_type, const DataSpace& data_space, const DSetCreatPropList& create_plist ) const
@@ -186,15 +200,13 @@ DataSet CommonFG::createDataSet( const H5std_string& name, const DataType& data_
 //--------------------------------------------------------------------------
 DataSet CommonFG::openDataSet( const char* name ) const
 {
-   // Call C function H5Dopen to open the specified dataset, giving
+   // Call C function H5Dopen2 to open the specified dataset, giving
    // the location id and the dataset's name
-   hid_t dataset_id = H5Dopen( getLocId(), name );
+   hid_t dataset_id = H5Dopen2( getLocId(), name, H5P_DEFAULT );
 
    // If the dataset's opening failed, throw an exception
-   if( dataset_id < 0 )
-   {
-      throwException("openDataSet", "H5Dopen failed");
-   }
+   if(dataset_id < 0)
+      throwException("openDataSet", "H5Dopen2 failed");
 
    // No failure, create and return the DataSet object
    DataSet dataset( dataset_id );
@@ -205,7 +217,7 @@ DataSet CommonFG::openDataSet( const char* name ) const
 // Function:	CommonFG::openDataSet
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 DataSet CommonFG::openDataSet( const H5std_string& name ) const
@@ -229,12 +241,27 @@ DataSet CommonFG::openDataSet( const H5std_string& name ) const
 ///		specified location.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
-void CommonFG::link( H5G_link_t link_type, const char* curr_name, const char* new_name ) const
+void CommonFG::link( H5L_type_t link_type, const char* curr_name, const char* new_name ) const
 {
-   herr_t ret_value = H5Glink( getLocId(), link_type, curr_name, new_name );
+    herr_t ret_value;
+
+    switch(link_type) {
+        case H5L_TYPE_HARD:
+            ret_value = H5Lcreate_hard( getLocId(), curr_name, H5L_SAME_LOC, new_name, H5P_DEFAULT, H5P_DEFAULT );
+            break;
+
+        case H5L_TYPE_SOFT:
+            ret_value = H5Lcreate_soft( curr_name, getLocId(), new_name, H5P_DEFAULT, H5P_DEFAULT );
+            break;
+
+        default:
+            throwException("link", "unknown link type");
+            break;
+    } /* end switch */
+
    if( ret_value < 0 )
    {
-      throwException("link", "H5Glink failed");
+      throwException("link", "creating link failed");
    }
 }
 
@@ -242,10 +269,10 @@ void CommonFG::link( H5G_link_t link_type, const char* curr_name, const char* ne
 // Function:	CommonFG::link
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a curr_name and \a new_name.
+///		\c std::string for \a curr_name and \a new_name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
-void CommonFG::link( H5G_link_t link_type, const H5std_string& curr_name, const H5std_string& new_name ) const
+void CommonFG::link( H5L_type_t link_type, const H5std_string& curr_name, const H5std_string& new_name ) const
 {
    link( link_type, curr_name.c_str(), new_name.c_str() );
 }
@@ -259,10 +286,10 @@ void CommonFG::link( H5G_link_t link_type, const H5std_string& curr_name, const
 //--------------------------------------------------------------------------
 void CommonFG::unlink( const char* name ) const
 {
-   herr_t ret_value = H5Gunlink( getLocId(), name );
+   herr_t ret_value = H5Ldelete( getLocId(), name, H5P_DEFAULT );
    if( ret_value < 0 )
    {
-      throwException("unlink", "H5Gunlink failed");
+      throwException("unlink", "H5Ldelete failed");
    }
 }
 
@@ -270,7 +297,7 @@ void CommonFG::unlink( const char* name ) const
 // Function:	CommonFG::unlink
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void CommonFG::unlink( const H5std_string& name ) const
@@ -288,15 +315,15 @@ void CommonFG::unlink( const H5std_string& name ) const
 ///		Exercise care in moving groups as it is possible to render
 ///		data in a file inaccessible with Group::move. Please refer
 ///		to the Group Interface in the HDF5 User's Guide at:
-/// <A HREF="../Groups.html">../Groups.html</A>
+/// http://hdf.ncsa.uiuc.edu/HDF5/doc/Groups.html
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void CommonFG::move( const char* src, const char* dst ) const
 {
-   herr_t ret_value = H5Gmove( getLocId(), src, dst );
+   herr_t ret_value = H5Lmove( getLocId(), src, H5L_SAME_LOC, dst, H5P_DEFAULT, H5P_DEFAULT );
    if( ret_value < 0 )
    {
-      throwException("move", "H5Gmove failed");
+      throwException("move", "H5Lmove failed");
    }
 }
 
@@ -304,7 +331,7 @@ void CommonFG::move( const char* src, const char* dst ) const
 // Function:	CommonFG::move
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a src and \a dst.
+///		\c std::string for \a src and \a dst.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void CommonFG::move( const H5std_string& src, const H5std_string& dst ) const
@@ -312,6 +339,7 @@ void CommonFG::move( const H5std_string& src, const H5std_string& dst ) const
    move( src.c_str(), dst.c_str() );
 }
 
+#ifndef H5_NO_DEPRECATED_SYMBOLS
 //--------------------------------------------------------------------------
 // Function:	CommonFG::getObjinfo
 ///\brief	Returns information about an object.
@@ -322,7 +350,7 @@ void CommonFG::move( const H5std_string& src, const H5std_string& dst ) const
 ///\par Description
 ///		For more information, please refer to the C layer Reference
 ///		Manual at:
-/// <A HREF="../RM_H5G.html#Group-GetObjinfo">../RM_H5G.html#Group-GetObjinfo</A>
+/// http://hdf.ncsa.uiuc.edu/HDF5/doc/RM_H5G.html#Group-GetObjinfo
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void CommonFG::getObjinfo( const char* name, hbool_t follow_link, H5G_stat_t& statbuf ) const
@@ -338,7 +366,7 @@ void CommonFG::getObjinfo( const char* name, hbool_t follow_link, H5G_stat_t& st
 // Function:	CommonFG::getObjinfo
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void CommonFG::getObjinfo( const H5std_string& name, hbool_t follow_link, H5G_stat_t& statbuf ) const
@@ -366,13 +394,14 @@ void CommonFG::getObjinfo( const char* name, H5G_stat_t& statbuf ) const
 // Function:	CommonFG::getObjinfo
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - Nov, 2005
 //--------------------------------------------------------------------------
 void CommonFG::getObjinfo( const H5std_string& name, H5G_stat_t& statbuf ) const
 {
    getObjinfo( name.c_str(), statbuf );
 }
+#endif /* H5_NO_DEPRECATED_SYMBOLS */
 
 //--------------------------------------------------------------------------
 // Function:	CommonFG::getLinkval
@@ -387,10 +416,10 @@ H5std_string CommonFG::getLinkval( const char* name, size_t size ) const
 {
    char* value_C = new char[size+1];  // temporary C-string for C API
 
-   herr_t ret_value = H5Gget_linkval( getLocId(), name, size, value_C );
+   herr_t ret_value = H5Lget_val( getLocId(), name, value_C, size, H5P_DEFAULT );
    if( ret_value < 0 )
    {
-      throwException("getLinkval", "H5Gget_linkval failed");
+      throwException("getLinkval", "H5Lget_val failed");
    }
    H5std_string value = H5std_string( value_C );
    delete []value_C;
@@ -401,7 +430,7 @@ H5std_string CommonFG::getLinkval( const char* name, size_t size ) const
 // Function:	CommonFG::getLinkval
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 H5std_string CommonFG::getLinkval( const H5std_string& name, size_t size ) const
@@ -426,10 +455,10 @@ H5std_string CommonFG::getLinkval( const H5std_string& name, size_t size ) const
 //--------------------------------------------------------------------------
 void CommonFG::setComment( const char* name, const char* comment ) const
 {
-   herr_t ret_value = H5Gset_comment( getLocId(), name, comment );
+   herr_t ret_value = H5Oset_comment_by_name( getLocId(), name, comment, H5P_DEFAULT );
    if( ret_value < 0 )
    {
-      throwException("setComment", "H5Gset_comment failed");
+      throwException("setComment", "H5Oset_comment_by_name failed");
    }
 }
 
@@ -437,7 +466,7 @@ void CommonFG::setComment( const char* name, const char* comment ) const
 // Function:	CommonFG::setComment
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name and \a comment.
+///		\c std::string for \a name and \a comment.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void CommonFG::setComment( const H5std_string& name, const H5std_string& comment ) const
@@ -454,10 +483,10 @@ void CommonFG::setComment( const H5std_string& name, const H5std_string& comment
 //--------------------------------------------------------------------------
 void CommonFG::removeComment(const char* name) const
 {
-   herr_t ret_value = H5Gset_comment(getLocId(), name, NULL);
+   herr_t ret_value = H5Oset_comment_by_name(getLocId(), name, NULL, H5P_DEFAULT);
    if( ret_value < 0 )
    {
-      throwException("removeComment", "H5Gset_comment failed");
+      throwException("removeComment", "H5Oset_comment_by_name failed");
    }
 }
 
@@ -465,7 +494,7 @@ void CommonFG::removeComment(const char* name) const
 // Function:	CommonFG::removeComment
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - May 2005
 //--------------------------------------------------------------------------
 void CommonFG::removeComment(const H5std_string& name) const
@@ -488,23 +517,23 @@ H5std_string CommonFG::getComment (const H5std_string& name) const
 
    // temporary C-string for the object's comment
    char* comment_C = new char[bufsize+1];
-   herr_t ret_value = H5Gget_comment (loc_id, name.c_str(), bufsize, comment_C);
+   herr_t ret_value = H5Oget_comment_by_name(loc_id, name.c_str(), comment_C, bufsize, H5P_DEFAULT);
 
    // if the actual length of the comment is longer than the anticipated
-   // value, then call H5Gget_comment again with the correct value
+   // value, then call H5Oget_comment_by_name again with the correct value
    if (ret_value > bufsize)
    {
 	bufsize = ret_value;
 	delete []comment_C;
 	comment_C = new char[bufsize+1];
-	ret_value = H5Gget_comment (loc_id, name.c_str(), bufsize, comment_C);
+	ret_value = H5Oget_comment_by_name(loc_id, name.c_str(), comment_C, bufsize, H5P_DEFAULT);
    }
 
-   // if H5Gget_comment returns SUCCEED, return the string comment,
+   // if H5Oget_comment_by_name returns SUCCEED, return the string comment,
    // otherwise, throw an exception
    if( ret_value < 0 )
    {
-      throwException("getComment", "H5Gget_comment failed");
+      throwException("getComment", "H5Oget_comment_by_name failed");
    }
    H5std_string comment = H5std_string(comment_C);
    delete []comment_C;
@@ -526,12 +555,12 @@ H5std_string CommonFG::getComment( const char* name, size_t bufsize ) const
    // temporary C-string for the object's comment
    char* comment_C = new char[bufsize+1];
 
-   herr_t ret_value = H5Gget_comment( getLocId(), name, bufsize, comment_C );
+   herr_t ret_value = H5Oget_comment_by_name( getLocId(), name, comment_C, bufsize, H5P_DEFAULT );
 
-   // if H5Gget_comment returns SUCCEED, return the string comment
+   // if H5Oget_comment_by_name returns SUCCEED, return the string comment
    if( ret_value < 0 )
    {
-      throwException("getComment", "H5Gget_comment failed");
+      throwException("getComment", "H5Oget_comment_by_name failed");
    }
    H5std_string comment = H5std_string(comment_C);
    delete []comment_C;
@@ -542,7 +571,7 @@ H5std_string CommonFG::getComment( const char* name, size_t bufsize ) const
 // Function:	CommonFG::getComment
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 H5std_string CommonFG::getComment( const H5std_string& name, size_t bufsize ) const
@@ -579,7 +608,7 @@ void CommonFG::mount( const char* name, H5File& child, PropList& plist ) const
 // Function:	CommonFG::mount
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void CommonFG::mount( const H5std_string& name, H5File& child, PropList& plist ) const
@@ -610,7 +639,7 @@ void CommonFG::unmount( const char* name ) const
 // Function:	CommonFG::unmount
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void CommonFG::unmount( const H5std_string& name ) const
@@ -628,15 +657,14 @@ void CommonFG::unmount( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 DataType CommonFG::openDataType( const char* name ) const
 {
-   // Call C function H5Topen to open the named datatype in this group,
+   // Call C function H5Topen2 to open the named datatype in this group,
    // given either the file or group id
-   hid_t type_id = H5Topen(getLocId(), name);
+   hid_t type_id = H5Topen2(getLocId(), name, H5P_DEFAULT);
 
    // If the datatype's opening failed, throw an exception
    if( type_id < 0 )
-   {
-      throwException("openDataType", "H5Topen failed");
-   }
+      throwException("openDataType", "H5Topen2 failed");
+
    // No failure, create and return the DataType object
    DataType data_type(type_id);
    return(data_type);
@@ -646,7 +674,7 @@ DataType CommonFG::openDataType( const char* name ) const
 // Function:	CommonFG::openDataType
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 DataType CommonFG::openDataType( const H5std_string& name ) const
@@ -664,15 +692,14 @@ DataType CommonFG::openDataType( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 ArrayType CommonFG::openArrayType( const char* name ) const
 {
-   // Call C function H5Topen to open the named datatype in this group,
+   // Call C function H5Topen2 to open the named datatype in this group,
    // given either the file or group id
-   hid_t type_id = H5Topen(getLocId(), name);
+   hid_t type_id = H5Topen2(getLocId(), name, H5P_DEFAULT);
 
    // If the datatype's opening failed, throw an exception
    if( type_id < 0 )
-   {
-      throwException("openArrayType", "H5Topen failed");
-   }
+      throwException("openArrayType", "H5Topen2 failed");
+
    // No failure, create and return the ArrayType object
    ArrayType array_type (type_id);
    return(array_type);
@@ -682,7 +709,7 @@ ArrayType CommonFG::openArrayType( const char* name ) const
 // Function:	CommonFG::openArrayType
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - Jul, 2005
 //--------------------------------------------------------------------------
 ArrayType CommonFG::openArrayType( const H5std_string& name ) const
@@ -700,15 +727,14 @@ ArrayType CommonFG::openArrayType( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 CompType CommonFG::openCompType( const char* name ) const
 {
-   // Call C function H5Topen to open the named datatype in this group,
+   // Call C function H5Topen2 to open the named datatype in this group,
    // given either the file or group id
-   hid_t type_id = H5Topen(getLocId(), name);
+   hid_t type_id = H5Topen2(getLocId(), name, H5P_DEFAULT);
 
    // If the datatype's opening failed, throw an exception
    if( type_id < 0 )
-   {
-      throwException("openCompType", "H5Topen failed");
-   }
+      throwException("openCompType", "H5Topen2 failed");
+
    // No failure, create and return the CompType object
    CompType comp_type(type_id);
    return(comp_type);
@@ -718,7 +744,7 @@ CompType CommonFG::openCompType( const char* name ) const
 // Function:	CommonFG::openCompType
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 CompType CommonFG::openCompType( const H5std_string& name ) const
@@ -736,15 +762,14 @@ CompType CommonFG::openCompType( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 EnumType CommonFG::openEnumType( const char* name ) const
 {
-   // Call C function H5Topen to open the named datatype in this group,
+   // Call C function H5Topen2 to open the named datatype in this group,
    // given either the file or group id
-   hid_t type_id = H5Topen(getLocId(), name);
+   hid_t type_id = H5Topen2(getLocId(), name, H5P_DEFAULT);
 
    // If the datatype's opening failed, throw an exception
    if( type_id < 0 )
-   {
-      throwException("openEnumType", "H5Topen failed");
-   }
+      throwException("openEnumType", "H5Topen2 failed");
+
    // No failure, create and return the EnumType object
    EnumType enum_type(type_id);
    return(enum_type);
@@ -754,7 +779,7 @@ EnumType CommonFG::openEnumType( const char* name ) const
 // Function:	CommonFG::openEnumType
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 EnumType CommonFG::openEnumType( const H5std_string& name ) const
@@ -772,15 +797,14 @@ EnumType CommonFG::openEnumType( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 IntType CommonFG::openIntType( const char* name ) const
 {
-   // Call C function H5Topen to open the named datatype in this group,
+   // Call C function H5Topen2 to open the named datatype in this group,
    // given either the file or group id
-   hid_t type_id = H5Topen(getLocId(), name);
+   hid_t type_id = H5Topen2(getLocId(), name, H5P_DEFAULT);
 
    // If the datatype's opening failed, throw an exception
    if( type_id < 0 )
-   {
-      throwException("openIntType", "H5Topen failed");
-   }
+      throwException("openIntType", "H5Topen2 failed");
+
    // No failure, create and return the IntType object
    IntType int_type(type_id);
    return(int_type);
@@ -790,7 +814,7 @@ IntType CommonFG::openIntType( const char* name ) const
 // Function:	CommonFG::openIntType
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 IntType CommonFG::openIntType( const H5std_string& name ) const
@@ -808,15 +832,14 @@ IntType CommonFG::openIntType( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 FloatType CommonFG::openFloatType( const char* name ) const
 {
-   // Call C function H5Topen to open the named datatype in this group,
+   // Call C function H5Topen2 to open the named datatype in this group,
    // given either the file or group id
-   hid_t type_id = H5Topen(getLocId(), name);
+   hid_t type_id = H5Topen2(getLocId(), name, H5P_DEFAULT);
 
    // If the datatype's opening failed, throw an exception
    if( type_id < 0 )
-   {
-      throwException("openFloatType", "H5Topen failed");
-   }
+      throwException("openFloatType", "H5Topen2 failed");
+
    // No failure, create and return the FloatType object
    FloatType float_type(type_id);
    return(float_type);
@@ -826,7 +849,7 @@ FloatType CommonFG::openFloatType( const char* name ) const
 // Function:	CommonFG::openFloatType
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 FloatType CommonFG::openFloatType( const H5std_string& name ) const
@@ -844,15 +867,14 @@ FloatType CommonFG::openFloatType( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 StrType CommonFG::openStrType( const char* name ) const
 {
-   // Call C function H5Topen to open the named datatype in this group,
+   // Call C function H5Topen2 to open the named datatype in this group,
    // given either the file or group id
-   hid_t type_id = H5Topen(getLocId(), name);
+   hid_t type_id = H5Topen2(getLocId(), name, H5P_DEFAULT);
 
    // If the datatype's opening failed, throw an exception
    if( type_id < 0 )
-   {
-      throwException("openStrType", "H5Topen failed");
-   }
+      throwException("openStrType", "H5Topen2 failed");
+
    // No failure, create and return the StrType object
    StrType str_type(type_id);
    return(str_type);
@@ -862,7 +884,7 @@ StrType CommonFG::openStrType( const char* name ) const
 // Function:	CommonFG::openStrType
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 StrType CommonFG::openStrType( const H5std_string& name ) const
@@ -880,15 +902,14 @@ StrType CommonFG::openStrType( const H5std_string& name ) const
 //--------------------------------------------------------------------------
 VarLenType CommonFG::openVarLenType( const char* name ) const
 {
-   // Call C function H5Topen to open the named datatype in this group,
+   // Call C function H5Topen2 to open the named datatype in this group,
    // given either the file or group id
-   hid_t type_id = H5Topen(getLocId(), name);
+   hid_t type_id = H5Topen2(getLocId(), name, H5P_DEFAULT);
 
    // If the datatype's opening failed, throw an exception
    if( type_id < 0 )
-   {
-      throwException("openVarLenType", "H5Topen failed");
-   }
+      throwException("openVarLenType", "H5Topen2 failed");
+
    // No failure, create and return the VarLenType object
    VarLenType varlen_type(type_id);
    return(varlen_type);
@@ -898,7 +919,7 @@ VarLenType CommonFG::openVarLenType( const char* name ) const
 // Function:	CommonFG::openVarLenType
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - Jul, 2005
 //--------------------------------------------------------------------------
 VarLenType CommonFG::openVarLenType( const H5std_string& name ) const
@@ -906,6 +927,7 @@ VarLenType CommonFG::openVarLenType( const H5std_string& name ) const
    return( openVarLenType( name.c_str()) );
 }
 
+#ifndef H5_NO_DEPRECATED_SYMBOLS
 //--------------------------------------------------------------------------
 // Function:	CommonFG::iterateElems
 ///\brief	Iterates a user's function over the entries of a group.
@@ -933,13 +955,14 @@ int CommonFG::iterateElems( const char* name, int *idx, H5G_iterate_t op , void*
 // Function:	CommonFG::iterateElems
 ///\brief	This is an overloaded member function, provided for convenience.
 ///		It differs from the above function in that it takes an
-///		\c H5std_string for \a name.
+///		\c std::string for \a name.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 int CommonFG::iterateElems( const H5std_string& name, int *idx, H5G_iterate_t op , void* op_data )
 {
    return( iterateElems( name.c_str(), idx, op, op_data ));
 }
+#endif /* H5_NO_DEPRECATED_SYMBOLS */
 
 //--------------------------------------------------------------------------
 // Function:	CommonFG::getNumObjs
@@ -950,13 +973,14 @@ int CommonFG::iterateElems( const H5std_string& name, int *idx, H5G_iterate_t op
 //--------------------------------------------------------------------------
 hsize_t CommonFG::getNumObjs() const
 {
-   hsize_t num_objs;
-   herr_t ret_value = H5Gget_num_objs(getLocId(), &num_objs);
+   H5G_info_t 		ginfo;                  /* Group information */
+
+   herr_t ret_value = H5Gget_info(getLocId(), &ginfo);
    if(ret_value < 0)
    {
-      throwException("getNumObjs", "H5Gget_num_objs failed");
+      throwException("getNumObjs", "H5Gget_info failed");
    }
-   return (num_objs);
+   return (ginfo.nlinks);
 }
 
 //--------------------------------------------------------------------------
@@ -976,16 +1000,16 @@ hsize_t CommonFG::getNumObjs() const
 //--------------------------------------------------------------------------
 H5std_string CommonFG::getObjnameByIdx(hsize_t idx) const
 {
-    // call H5Gget_objname_by_idx with name as NULL to get its length
-    ssize_t name_len = H5Gget_objname_by_idx(getLocId(), idx, NULL, 0);
+    // call H5Lget_name_by_idx with name as NULL to get its length
+    ssize_t name_len = H5Lget_name_by_idx(getLocId(), ".", H5_INDEX_NAME, H5_ITER_INC, idx, NULL, 0, H5P_DEFAULT);
     if(name_len < 0)
     {
-      throwException("getObjnameByIdx", "H5Gget_objname_by_idx failed");
+      throwException("getObjnameByIdx", "H5Lget_name_by_idx failed");
     }
 
     // now, allocate C buffer to get the name
     char* name_C = new char[name_len+1];
-    name_len = H5Gget_objname_by_idx(getLocId(), idx, name_C, name_len+1);
+    name_len = H5Lget_name_by_idx(getLocId(), ".", H5_INDEX_NAME, H5_ITER_INC, idx, name_C, name_len+1, H5P_DEFAULT);
 
     // clean up and return the string
     H5std_string name = H5std_string(name_C);
@@ -1012,17 +1036,18 @@ H5std_string CommonFG::getObjnameByIdx(hsize_t idx) const
 //--------------------------------------------------------------------------
 ssize_t CommonFG::getObjnameByIdx(hsize_t idx, H5std_string& name, size_t size) const
 {
-   char* name_C = new char[size+1];
-   ssize_t name_len = H5Gget_objname_by_idx(getLocId(), idx, name_C, size+1);
+   char* name_C = new char[size];
+   ssize_t name_len = H5Lget_name_by_idx(getLocId(), ".", H5_INDEX_NAME, H5_ITER_INC, idx, name_C, size, H5P_DEFAULT);
    if(name_len < 0)
    {
-      throwException("getObjnameByIdx", "H5Gget_objname_by_idx failed");
+      throwException("getObjnameByIdx", "H5Lget_name_by_idx failed");
    }
    name = H5std_string(name_C);
    delete []name_C;
    return (name_len);
 }
 
+#ifndef H5_NO_DEPRECATED_SYMBOLS
 //--------------------------------------------------------------------------
 // Function:	CommonFG::getObjTypeByIdx
 ///\brief	Returns the type of an object in this group, given the
@@ -1070,6 +1095,8 @@ H5G_obj_t CommonFG::getObjTypeByIdx(hsize_t idx, H5std_string& type_name) const
    }
    return (obj_type);
 }
+#endif /* H5_NO_DEPRECATED_SYMBOLS */
+
 //--------------------------------------------------------------------------
 // Function:	CommonFG default constructor
 ///\brief	Default constructor.
@@ -1087,3 +1114,4 @@ CommonFG::~CommonFG() {}
 #ifndef H5_NO_NAMESPACE
 }
 #endif
+
