@@ -68,11 +68,11 @@ DataSet::DataSet( const DataSet& original ) : AbstractDs( original ) {}
 //--------------------------------------------------------------------------
 // Function:	DataSet overload constructor - dereference
 ///\brief	Given a reference to some object, returns that dataset
-///\param	obj - IN: Dataset reference object is in or location of
+///             obj - IN: Dataset reference object is in or location of
 ///                   object that the dataset is located within.
 ///\param	ref - IN: Reference pointer
 ///\exception	H5::DataSetIException
-///\par Description
+///\parDescription
 ///		\c obj can be DataSet, Group, H5File, or named DataType, that 
 ///		is a datatype that has been named by DataType::commit.
 // Programmer	Binh-Minh Ribler - Oct, 2006
@@ -296,7 +296,7 @@ void DataSet::read( void* buf, const DataType& mem_type, const DataSpace& mem_sp
 //--------------------------------------------------------------------------
 // Function:	DataSet::read
 ///\brief	This is an overloaded member function, provided for convenience.
-///		It takes a reference to a \c H5std_string for the buffer.
+///		It takes a reference to a \c std::string for the buffer.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataSet::read( H5std_string& strg, const DataType& mem_type, const DataSpace& mem_space, const DataSpace& file_space, const DSetMemXferPropList& xfer_plist ) const
@@ -347,7 +347,7 @@ void DataSet::write( const void* buf, const DataType& mem_type, const DataSpace&
 //--------------------------------------------------------------------------
 // Function:	DataSet::write
 ///\brief	This is an overloaded member function, provided for convenience.
-///		It takes a reference to a \c H5std_string for the buffer.
+///		It takes a reference to a \c std::string for the buffer.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataSet::write( const H5std_string& strg, const DataType& mem_type, const DataSpace& mem_space, const DataSpace& file_space, const DSetMemXferPropList& xfer_plist ) const
@@ -400,16 +400,14 @@ int DataSet::iterateElems( void* buf, const DataType& type, const DataSpace& spa
 ///		For more information, please see the Description section in
 ///		C layer Reference Manual at:
 ///\par
-/// <A HREF="../RM_H5D.html#Dataset-Extend">../RM_H5D.html#Dataset-Extend</A>
+/// http://hdf.ncsa.uiuc.edu/HDF5/doc/RM_H5D.html#Dataset-Extend
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataSet::extend( const hsize_t* size ) const
 {
-   herr_t ret_value = H5Dextend( id, size );
-   if( ret_value < 0 )  // raise exception when H5Dextend returns a neg value
-   {
-      throw DataSetIException("DataSet::extend", "H5Dextend failed");
-   }
+   herr_t ret_value = H5Dset_extent( id, size );
+   if( ret_value < 0 )  // raise exception when H5Dset_extent returns a neg value
+      throw DataSetIException("DataSet::extend", "H5Dset_extent failed");
 }
 
 //--------------------------------------------------------------------------
@@ -503,18 +501,19 @@ void* DataSet::Reference(const H5std_string& name) const
    return(Reference(name.c_str()));
 }
 
+#ifndef H5_NO_DEPRECATED_SYMBOLS
 //--------------------------------------------------------------------------
 // Function:	DataSet::getObjType
 ///\brief	Retrieves the type of object that an object reference points to.
-///\param	ref      - IN: Reference to query
 ///\param	ref_type - IN: Type of reference to query, valid values are:
-///		\li \c H5R_OBJECT - Reference is an object reference.
-///		\li \c H5R_DATASET_REGION - Reference is a dataset region reference.
+///		\li \c H5R_OBJECT \tReference is an object reference.
+///		\li \c H5R_DATASET_REGION \tReference is a dataset region reference.
+///\param	ref      - IN: Reference to query
 ///\return	An object type, which can be one of the following:
-///		\li \c H5G_LINK (0) - Object is a symbolic link.
-///		\li \c H5G_GROUP (1) - Object is a group.
-///		\li \c H5G_DATASET (2) - Object is a dataset.
-///		\li \c H5G_TYPE Object (3) - is a named datatype
+///		\li \c H5G_LINK (0) \tObject is a symbolic link.
+///		\li \c H5G_GROUP (1) \tObject is a group.
+///		\li \c H5G_DATASET (2) \tObject is a dataset.
+///		\li \c H5G_TYPE Object (3) \tis a named datatype
 ///\exception	H5::DataSetIException
 // Programmer	Binh-Minh Ribler - May, 2004
 //--------------------------------------------------------------------------
@@ -527,6 +526,7 @@ H5G_obj_t DataSet::getObjType(void *ref, H5R_type_t ref_type) const
       throw DataSetIException("DataSet::getObjType", E.getDetailMsg());
    }
 }
+#endif /* H5_NO_DEPRECATED_SYMBOLS */
 
 //--------------------------------------------------------------------------
 // Function:	DataSet::getRegion
@@ -576,7 +576,7 @@ void DataSet::close()
 // Programmer	Binh-Minh Ribler - 2000
 // Modification
 //		- Replaced resetIdComponent() with decRefCount() to use C
-//		library ID reference counting mechanism - BMR, Feb 20, 2005
+//		library ID reference counting mechanism - BMR, Jun 1, 2004
 //		- Replaced decRefCount with close() to let the C library
 //		handle the reference counting - BMR, Jun 1, 2006
 //--------------------------------------------------------------------------
