@@ -100,7 +100,7 @@ DataSpace::DataSpace( const DataSpace& original ) : IdComponent( original ) {}
 // Programmer	Binh-Minh Ribler - 2000
 // Modification
 //		- Replaced resetIdComponent() with decRefCount() to use C
-//		library ID reference counting mechanism - BMR, Feb 20, 2005
+//		library ID reference counting mechanism - BMR, Jun 1, 2004
 //		- Replaced decRefCount with close() to let the C library
 //		handle the reference counting - BMR, Jun 1, 2006
 //--------------------------------------------------------------------------
@@ -137,7 +137,7 @@ void DataSpace::copy( const DataSpace& like_space )
 DataSpace& DataSpace::operator=( const DataSpace& rhs )
 {
     if (this != &rhs)
-	copy(rhs);
+        copy(rhs);
     return(*this);
 }
 
@@ -401,7 +401,7 @@ hssize_t DataSpace::getSelectElemNpoints () const
 ///\par Description
 ///		For more information, please refer to the C layer Reference
 ///		Manual at:
-/// <A HREF="../RM_H5S.html#Dataspace-SelectElemPointList">../RM_H5S.html#Dataspace-SelectElemPointList</A>
+/// http://hdf.ncsa.uiuc.edu/HDF5/doc/RM_H5S.html#Dataspace-SelectElemPointList
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataSpace::getSelectElemPointlist ( hsize_t startpoint, hsize_t numpoints, hsize_t *buf ) const
@@ -425,7 +425,7 @@ void DataSpace::getSelectElemPointlist ( hsize_t startpoint, hsize_t numpoints,
 ///\par Description
 ///		For more information, please refer to the C layer Reference
 ///		Manual at:
-/// <A HREF="../RM_H5S.html#Dataspace-SelectBounds">../RM_H5S.html#Dataspace-SelectBounds</A>
+/// http://hdf.ncsa.uiuc.edu/HDF5/doc/RM_H5S.html#Dataspace-SelectBounds
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataSpace::getSelectBounds ( hsize_t* start, hsize_t* end ) const
@@ -451,10 +451,10 @@ void DataSpace::getSelectBounds ( hsize_t* start, hsize_t* end ) const
 ///\par Description
 ///		For more information, please refer to the C layer Reference
 ///		Manual at:
-/// <A HREF="../RM_H5S.html#Dataspace-SelectElements">../RM_H5S.html#Dataspace-SelectElements</A>
+/// http://hdf.ncsa.uiuc.edu/HDF5/doc/RM_H5S.html#Dataspace-SelectElements
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
-void DataSpace::selectElements ( H5S_seloper_t op, const size_t num_elements, const hsize_t *coord[ ] ) const
+void DataSpace::selectElements ( H5S_seloper_t op, const size_t num_elements, const hsize_t *coord) const
 {
    herr_t ret_value;
    ret_value = H5Sselect_elements( id, op, num_elements, coord );
@@ -533,7 +533,7 @@ bool DataSpace::selectValid () const
 ///\par Description
 ///		For more information, please refer to the C layer Reference
 ///		Manual at:
-/// <A HREF="../RM_H5S.html#Dataspace-SelectHyperslab">../RM_H5S.html#Dataspace-SelectHyperslab</A>
+/// http://hdf.ncsa.uiuc.edu/HDF5/doc/RM_H5S.html#Dataspace-SelectHyperslab
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataSpace::selectHyperslab( H5S_seloper_t op, const hsize_t *count, const hsize_t *start, const hsize_t *stride, const hsize_t *block ) const
@@ -575,7 +575,7 @@ void DataSpace::close()
 // Programmer	Binh-Minh Ribler - 2000
 // Modification
 //		- Replaced resetIdComponent() with decRefCount() to use C
-//		library ID reference counting mechanism - BMR, Feb 20, 2005
+//		library ID reference counting mechanism - BMR, Jun 1, 2004
 //		- Replaced decRefCount with close() to let the C library
 //		handle the reference counting - BMR, Jun 1, 2006
 //--------------------------------------------------------------------------
