@@ -13,6 +13,10 @@
  * access to either file, you may request a copy from help@hdfgroup.org.     *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+#ifdef H5_VMS
+#include <iostream>
+#endif /*H5_VMS*/
+
 #include <string>
 
 #include "H5Include.h"
@@ -144,7 +148,7 @@ int IdComponent::getCounter() const
 H5I_type_t IdComponent::getHDFObjType(const hid_t obj_id)
 {
     H5I_type_t id_type = H5Iget_type(obj_id);
-    if (id_type <= H5I_BADID || id_type >= H5I_NGROUPS)
+    if (id_type <= H5I_BADID || id_type >= H5I_NTYPES)
 	return H5I_BADID; // invalid
     else
 	return id_type; // valid type
@@ -248,13 +252,19 @@ IdComponent::~IdComponent() {}
 ///		where the failure occurs.  The class-name is provided by
 ///		fromClass().  This string will be used by a base class when
 ///		an exception is thrown.
-// Programmer	Binh-Minh Ribler - Oct 10, 2005
+// Programmer	Binh-Minh Ribler - Aug 6, 2005
 //--------------------------------------------------------------------------
 H5std_string IdComponent::inMemFunc(const char* func_name) const
 {
+#ifdef H5_VMS
+   H5std_string full_name = fromClass();
+   full_name.append("::");
+   full_name.append(func_name);
+#else
    H5std_string full_name = func_name;
    full_name.insert(0, "::");
    full_name.insert(0, fromClass());
+#endif /*H5_VMS*/
    return (full_name);
 }
 
@@ -372,9 +382,9 @@ void IdComponent::reference(void* ref, const char* name) const
 // Function:    IdComponent::reference
 ///\brief       This is an overloaded function, provided for your convenience.
 ///             It differs from the above function in that it takes an
-///             \c H5std_string for the object's name.
+///             \c std::string for the object's name.
 ///\param       ref - IN: Reference pointer
-///\param       name - IN: Name of the object to be referenced - \c H5std_string
+///\param       name - IN: Name of the object to be referenced - \c std::string
 // Programmer   Binh-Minh Ribler - May, 2004
 //--------------------------------------------------------------------------
 void IdComponent::reference(void* ref, const H5std_string& name) const
@@ -426,6 +436,7 @@ void IdComponent::dereference(IdComponent& obj, void* ref)
    }
 }
 
+#ifndef H5_NO_DEPRECATED_SYMBOLS
 //--------------------------------------------------------------------------
 // Function:	IdComponent::p_get_obj_type (protected)
 // Purpose	Retrieves the type of object that an object reference points to.
@@ -442,22 +453,15 @@ void IdComponent::dereference(IdComponent& obj, void* ref)
 //--------------------------------------------------------------------------
 H5G_obj_t IdComponent::p_get_obj_type(void *ref, H5R_type_t ref_type) const
 {
-#ifdef H5_WANT_H5_V1_4_COMPAT
-   H5G_obj_t obj_type = H5Rget_object_type(id, ref);
-#else
-   H5G_obj_t obj_type = H5Rget_obj_type(id, ref_type, ref);
-#endif
+   H5G_obj_t obj_type = H5Rget_obj_type1(id, ref_type, ref);
 
    if (obj_type == H5G_UNKNOWN)
    {
-#ifdef H5_WANT_H5_V1_4_COMPAT
-      throw IdComponentException("", "H5Rget_object_type failed");
-#else
       throw IdComponentException("", "H5Rget_obj_type failed");
-#endif
    }
    return(obj_type);
 }
+#endif /* H5_NO_DEPRECATED_SYMBOLS */
 
 //--------------------------------------------------------------------------
 // Function:	IdComponent::p_get_region (protected)
@@ -494,7 +498,7 @@ hid_t IdComponent::p_get_region(void *ref, H5R_type_t ref_type) const
 bool IdComponent::p_valid_id(const hid_t obj_id) const
 {
     H5I_type_t id_type = H5Iget_type(obj_id);
-    if (id_type <= H5I_BADID || id_type >= H5I_NGROUPS)
+    if (id_type <= H5I_BADID || id_type >= H5I_NTYPES)
 	return false;
     else
 	return true;
