@@ -36,13 +36,14 @@
 #endif
 
 #include "h5test.h"
-#include "H5Cpp.h"	// C++ API header file
-#include "h5cpputil.h"
+#include "H5Cpp.h"
 
 #ifndef H5_NO_NAMESPACE
-using namespace H5;
+    using namespace H5;
 #endif
 
+#include "h5cpputil.h"
+
 
 /*-------------------------------------------------------------------------
  * Function:	test_report
@@ -99,7 +100,7 @@ void issue_fail_msg(const char* where, int line, const char* file_name,
 {
     //if (GetTestVerbosity()>=VERBO_HI)
     {
-        cerr << "ERROR>>> From " << where << " at line " << line
+        cerr << ">>> FAILED in " << where << " at line " << line
              << " in " << file_name << " - " << message << endl << endl;
     }
 }
@@ -111,12 +112,12 @@ void issue_fail_msg(const char* where, int line, const char* file_name,
  *		different, the function will print out a message and the
  *		different values.  This function is made to reuse the code
  *		segment that is used in various places throughout
- *		the test code.  Where the C version of this code segment
+ *		the test code.  Where the C version of this code segment 
  *		"goto error," this function will return -1.
  *
- * Return:	Success:        0
+ * Return:	Success:	0
  *
- *		Failure:        -1
+ *		Failure:	-1
  *
  * Programmer:	Binh-Minh Ribler (using C code segment for checking values)
  *		Friday, February 6, 2001
@@ -131,7 +132,7 @@ int check_values (hsize_t i, hsize_t j, int apoint, int acheck)
     {
 	cerr << "    Read different values than written.\n" << endl;
 	cerr << "    At index " << (unsigned long)i << "," <<
-	(unsigned long)j << endl;
+   	(unsigned long)j << endl;
 	return -1;
     }
     return 0;
@@ -159,3 +160,25 @@ InvalidActionException::InvalidActionException(const H5std_string func_name, con
 //--------------------------------------------------------------------------
 InvalidActionException::~InvalidActionException() {}
 
+//--------------------------------------------------------------------------
+// Function:    TestFailedException default constructor
+//--------------------------------------------------------------------------
+TestFailedException::TestFailedException():Exception(){}
+
+//--------------------------------------------------------------------------
+// Function:    TestFailedException overloaded constructor
+//
+// Purpose:	Creates an TestFailedException with the name of the function,
+//              which the failure should have occurred but didn't, and a
+//		message explaining why it should fail.
+// Parameters
+//		func_name - IN: Name of the function where failure should occur
+//		message   - IN: Message
+//--------------------------------------------------------------------------
+TestFailedException::TestFailedException(const H5std_string func_name, const H5std_string message) : Exception(func_name, message) {}
+
+//--------------------------------------------------------------------------
+// Function:    TestFailedException destructor
+//--------------------------------------------------------------------------
+TestFailedException::~TestFailedException() {}
+
