@@ -40,7 +40,7 @@
 #include "H5Cpp.h"	// C++ API header file
 
 #ifndef H5_NO_NAMESPACE
-using namespace H5;
+    using namespace H5;
 #endif
 
 #include "h5cpputil.h"	// C++ utilility header file
@@ -89,6 +89,9 @@ struct space4_struct {
     char c2;
  } space4_data={'v',987123,(float)-3.14,'g'}; /* Test data for 4th dataspace */
 
+/* Null dataspace */
+int space5_data = 7;
+
 /*-------------------------------------------------------------------------
  *
  * Function:	test_h5s_basic
@@ -109,8 +112,7 @@ struct space4_struct {
  *		       with a special routine.
  *-------------------------------------------------------------------------
  */
-static void
-test_h5s_basic(void)
+static void test_h5s_basic()
 {
     hsize_t		dims1[] = {SPACE1_DIM1, SPACE1_DIM2, SPACE1_DIM3};
     hsize_t		dims2[] = {SPACE2_DIM1, SPACE2_DIM2, SPACE2_DIM3,
@@ -119,11 +121,9 @@ test_h5s_basic(void)
     hsize_t		tmax[4];
 
     // Output message about test being performed
-    MESSAGE(5, ("Testing Dataspace Manipulation\n"));
-
-    try
-    { // beginning of first try block
+    SUBTEST("Testing Dataspace Manipulation");
 
+    try {
 	// Create simple dataspace sid1
 	DataSpace sid1 (SPACE1_RANK, dims1 );
 
@@ -139,9 +139,10 @@ test_h5s_basic(void)
 	verify_val(rank, SPACE1_RANK, "DataSpace::getSimpleExtentNdims", __LINE__, __FILE__);
 
 	// Retrieves dimension size of dataspace sid1 and verify it
-	int ndims;              // Number of dimensions
+	int ndims;		// Number of dimensions
 	hsize_t	tdims[4];	// Dimension array to test with
 	ndims = sid1.getSimpleExtentDims( tdims );
+	verify_val(ndims, SPACE1_RANK, "DataSpace::getSimpleExtentDims", __LINE__, __FILE__);
 	verify_val(HDmemcmp(tdims, dims1, SPACE1_RANK * sizeof(unsigned)), 0,
 	   "DataSpace::getSimpleExtentDims", __LINE__, __FILE__);
 
@@ -209,7 +210,7 @@ test_h5s_basic(void)
     // CHECK_I(ret, "H5Fclose");  // leave this here, later, fake a failure
 		// in the p_close see how this will handle it. - BMR
 
-    // Verify that incorrect dimensions don't work
+	// Verify that incorrect dimensions don't work
 	dims1[0] = 0;
 	try {
 	    DataSpace wrongdim_ds (SPACE1_RANK, dims1);
@@ -230,7 +231,9 @@ test_h5s_basic(void)
 	}
 	catch (DataSpaceIException E) // catching use of incorrect dimensions
 	{} // do nothing, exception expected
-    }	// end of outer try block
+
+	PASSED();
+    }	// end of try block
 
     catch (InvalidActionException E)
     {
@@ -240,7 +243,7 @@ test_h5s_basic(void)
     // catch all other exceptions
     catch (Exception E)
     {
-        issue_fail_msg(E.getCFuncName(), __LINE__, __FILE__, E.getCDetailMsg());
+        issue_fail_msg("test_h5s_basic()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }   // test_h5s_basic()
 
@@ -264,14 +267,12 @@ test_h5s_basic(void)
  *		       with a special routine.
  *-------------------------------------------------------------------------
  */
-static void
-test_h5s_scalar_write(void)
+static void test_h5s_scalar_write()
 {
     // Output message about test being performed
-    MESSAGE(5, ("Testing Scalar Dataspace Writing\n"));
+    SUBTEST("Testing Scalar Dataspace Writing");
 
-    try
-    {
+    try {
 	// Create file
 	H5File fid1(DATAFILE, H5F_ACC_TRUNC);
 
@@ -298,14 +299,15 @@ test_h5s_scalar_write(void)
 	ext_type = sid1.getSimpleExtentType();
 	verify_val(ext_type, H5S_SCALAR, "DataSpace::getSimpleExtentType", __LINE__, __FILE__);
 
-	// Create a dataset
-	DataSet dataset = fid1.createDataSet("Dataset1", PredType::NATIVE_UINT,sid1);
-
+	// Create and write a dataset
+	DataSet dataset = fid1.createDataSet("Dataset1", PredType::NATIVE_UINT,sid1); 
 	dataset.write(&space3_data, PredType::NATIVE_UINT);
+
+	PASSED();
     } // end of try block
     catch (Exception E)
     {
-	issue_fail_msg(E.getCFuncName(), __LINE__, __FILE__, E.getCDetailMsg());
+	issue_fail_msg("test_h5s_scalar_write()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }   // test_h5s_scalar_write()
 
@@ -329,16 +331,14 @@ test_h5s_scalar_write(void)
  *		       with a special routine.
  *-------------------------------------------------------------------------
  */
-static void
-test_h5s_scalar_read(void)
+static void test_h5s_scalar_read()
 {
     hsize_t		tdims[4];	// Dimension array to test with
 
     // Output message about test being performed
-    MESSAGE(5, ("Testing Scalar Dataspace Reading\n"));
+    SUBTEST("Testing Scalar Dataspace Reading");
 
-    try
-    {
+    try {
 	// Create file
 	H5File fid1(DATAFILE, H5F_ACC_RDWR);
 
@@ -358,18 +358,75 @@ test_h5s_scalar_read(void)
 	ndims = sid1.getSimpleExtentDims(tdims);
 	verify_val(ndims, 0, "DataSpace::getSimpleExtentDims", __LINE__, __FILE__);
 
+	// Read data back and verify it
 	unsigned      	rdata;      	// Scalar data read in
 	dataset.read(&rdata, PredType::NATIVE_UINT);
 	verify_val(rdata, space3_data, "DataSet::read", __LINE__, __FILE__);
+
+	PASSED();
     }   // end of try block
     catch (Exception E)
     {
 	// all the exceptions caused by negative returned values by C APIs
-	issue_fail_msg(E.getCFuncName(), __LINE__, __FILE__, E.getCDetailMsg());
+	issue_fail_msg("test_h5s_scalar_read()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 
 }   // test_h5s_scalar_read()
 
+/*-------------------------------------------------------------------------
+ *
+ * Function:	test_h5s_null
+ *
+ * Purpose:	Test null H5S (dataspace) code
+ *
+ * Return:	none
+ *
+ * Programmer:	Raymond Lu (using C version)
+ *              May 18, 2004
+ *
+ * Modifications:
+ *      January, 2005: C tests' macro VERIFY casts values to 'long' for all
+ *		       cases.  Since there are no operator<< for 'long long'
+ *		       or int64 in VS C++ ostream, I casted the hssize_t values
+ *		       passed to verify_val to 'long' as well.  If problems
+ *		       arises later, this will have to be specificly handled
+ *		       with a special routine.
+ *-------------------------------------------------------------------------
+ */
+static void test_h5s_null()
+{
+    // Output message about test being performed
+    SUBTEST("Testing Null Dataspace Writing");
+
+    try {
+	// Create file
+	H5File fid1(DATAFILE, H5F_ACC_TRUNC);
+
+	// Create scalar dataspace
+	DataSpace sid1(H5S_NULL);
+
+	hssize_t	n;	 	// Number of dataspace elements
+	n = sid1.getSimpleExtentNpoints();
+	verify_val((long)n, 0, "DataSpace::getSimpleExtentNpoints", __LINE__, __FILE__);
+
+	// Create a dataset
+	DataSet dataset = fid1.createDataSet("Dataset1", PredType::NATIVE_UINT,sid1);
+
+        // Try to write nothing to the dataset
+	dataset.write(&space5_data, PredType::NATIVE_INT);
+
+        // Read the data.  Make sure no change to the buffer
+	dataset.read(&space5_data, PredType::NATIVE_INT);
+	verify_val(space5_data, 7, "DataSet::read", __LINE__, __FILE__);
+
+	PASSED();
+    } // end of try block
+    catch (Exception E)
+    {
+	issue_fail_msg("test_h5s_null()", __LINE__, __FILE__, E.getCDetailMsg());
+    }
+}   // test_h5s_null()
+
 /*-------------------------------------------------------------------------
  *
  * Function:	test_h5s_compound_scalar_write
@@ -391,14 +448,12 @@ test_h5s_scalar_read(void)
  *		       with a special routine.
  *-------------------------------------------------------------------------
  */
-static void
-test_h5s_compound_scalar_write(void)
+static void test_h5s_compound_scalar_write()
 {
     // Output message about test being performed
-    MESSAGE(5, ("Testing Compound Dataspace Writing\n"));
+    SUBTEST("Testing Compound Dataspace Writing");
 
-    try
-    {
+    try {
 	// Create file
 	H5File fid1(DATAFILE, H5F_ACC_TRUNC);
 
@@ -432,15 +487,16 @@ test_h5s_compound_scalar_write(void)
 	ndims = sid1.getSimpleExtentDims(tdims);
 	verify_val(ndims, 0, "DataSpace::getSimpleExtentDims", __LINE__, __FILE__);
 
-	// Create a dataset
+	// Create and write a dataset
 	DataSet dataset = fid1.createDataSet("Dataset1", tid1, sid1);
-
 	dataset.write(&space4_data, tid1);
+
+	PASSED();
     }	// end of try block
     catch (Exception E)
     {
 	// all the exceptions caused by negative returned values by C APIs
-	issue_fail_msg(E.getCFuncName(), __LINE__, __FILE__, E.getCDetailMsg());
+	issue_fail_msg("test_h5s_compound_scalar_write()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }   // test_h5s_compound_scalar_write()
 
@@ -465,15 +521,13 @@ test_h5s_compound_scalar_write(void)
  *		       with a special routine.
  *-------------------------------------------------------------------------
  */
-static void
-test_h5s_compound_scalar_read(void)
+static void test_h5s_compound_scalar_read()
 {
     hsize_t		tdims[4];	// Dimension array to test with
 
     // Output message about test being performed
-    MESSAGE(5, ("Testing Compound Dataspace Reading\n"));
-    try
-    {
+    SUBTEST("Testing Compound Dataspace Reading");
+    try {
 	// Create file
 	H5File fid1(DATAFILE, H5F_ACC_RDWR);
 
@@ -511,11 +565,12 @@ test_h5s_compound_scalar_read(void)
             TestErrPrintf("scalar data different: space4_data.c1=%c, read_data4.c1=%c\n",
 		space4_data.c1, rdata.c2);
 	} // end if
+	PASSED();
     }   // end of try block
     catch (Exception E)
     {
 	// all the exceptions caused by negative returned values by C APIs
-	issue_fail_msg(E.getCFuncName(), __LINE__, __FILE__, E.getCDetailMsg());
+	issue_fail_msg("test_h5s_compound_scalar_read()", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }   // test_h5s_compound_scalar_read()
 
@@ -533,8 +588,10 @@ test_h5s_compound_scalar_read(void)
  * Modifications:
  *-------------------------------------------------------------------------
  */
-void
-test_h5s(void)
+#ifdef __cplusplus
+extern "C"
+#endif
+void test_h5s()
 {
     // Output message about test being performed
     MESSAGE(5, ("Testing Dataspaces\n"));
@@ -542,6 +599,7 @@ test_h5s(void)
     test_h5s_basic();		// Test basic H5S code
     test_h5s_scalar_write();	// Test scalar H5S writing code
     test_h5s_scalar_read();	// Test scalar H5S reading code
+    test_h5s_null();		// Test null H5S code
     test_h5s_compound_scalar_write();	// Test compound datatype scalar H5S writing code
     test_h5s_compound_scalar_read();	// Test compound datatype scalar H5S reading code
 }   // test_h5s()
@@ -561,9 +619,11 @@ test_h5s(void)
  *
  *-------------------------------------------------------------------------
  */
-void
-cleanup_h5s(void)
+#ifdef __cplusplus
+extern "C"
+#endif
+void cleanup_h5s()
 {
-    remove(DATAFILE.c_str());
+    HDremove(DATAFILE.c_str());
 }   // cleanup_h5s
 
