@@ -87,7 +87,8 @@ main (void)
     * Buffers' initialization.
     */
    vector[0] = vector[MSPACE1_DIM - 1] = -1;
-   for (i = 1; i < MSPACE1_DIM - 1; i++) vector[i] = i;
+   for(i = 1; i < MSPACE1_DIM - 1; i++)
+       vector[i] = i;
 
    /*
     * Create a file.
@@ -109,7 +110,7 @@ main (void)
      * Create dataset in the file. Notice that creation
      * property list plist is used.
      */
-    dataset = H5Dcreate(file, "Matrix in file", H5T_NATIVE_INT, fid, plist);
+    dataset = H5Dcreate2(file, "Matrix in file", H5T_NATIVE_INT, fid, H5P_DEFAULT, plist, H5P_DEFAULT);
 
     /*
      * Select hyperslab for the dataset in the file, using 3x2 blocks,
@@ -170,8 +171,7 @@ main (void)
     coord[2][0] = 3; coord[2][1] = 5;
     coord[3][0] = 5; coord[3][1] = 6;
 
-    ret = H5Sselect_elements(fid, H5S_SELECT_SET, NPOINTS,
-                             (const hsize_t **)coord);
+    ret = H5Sselect_elements(fid, H5S_SELECT_SET, NPOINTS, coord);
 
     /*
      * Write new selection of points to the dataset.
@@ -216,7 +216,7 @@ main (void)
     /*
      * Open the dataset.
      */
-    dataset = H5Dopen(file,"Matrix in file");
+    dataset = H5Dopen2(file, "Matrix in file", H5P_DEFAULT);
 
     /*
      * Get dataspace of the open dataset.
@@ -302,8 +302,9 @@ main (void)
      *                     0  0  0  0  0  0  0  0  0
      *                     0  0  0  0  0  0  0  0  0
      */
-    for (i=0; i < MSPACE_DIM1; i++) {
-        for(j=0; j < MSPACE_DIM2; j++) printf("%3d  ", matrix_out[i][j]);
+    for(i = 0; i < MSPACE_DIM1; i++) {
+        for(j = 0; j < MSPACE_DIM2; j++)
+            printf("%3d  ", matrix_out[i][j]);
         printf("\n");
     }
 
@@ -330,3 +331,4 @@ main (void)
 
     return 0;
 }
+
