@@ -64,7 +64,7 @@
               INTEGER FUNCTION h5eclear_c()
               USE H5GLOBAL
               !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5ECLEAR_C'::h5eclear_c
+              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5ECLEAR_C'::h5eclear_c
               !DEC$ ENDIF
               END FUNCTION h5eclear_c
             END INTERFACE
@@ -113,7 +113,7 @@
               INTEGER FUNCTION h5eprint_c1(name, namelen)
               USE H5GLOBAL
               !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5EPRINT_C1'::h5eprint_c1
+              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5EPRINT_C1'::h5eprint_c1
               !DEC$ ENDIF
               !DEC$ATTRIBUTES reference :: name
               INTEGER :: namelen
@@ -126,7 +126,7 @@
               INTEGER FUNCTION h5eprint_c2()
               USE H5GLOBAL
               !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5EPRINT_C2'::h5eprint_c2
+              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5EPRINT_C2'::h5eprint_c2
               !DEC$ ENDIF
               END FUNCTION h5eprint_c2
             END INTERFACE
@@ -148,6 +148,7 @@
 !		error_no	- mojor error number 
 ! Outputs:  
 !		name		- character string describing the error
+!		namelen		- number of characters in the name buffer
 !		hdferr:		- error code		
 !				 	Success:  0
 !				 	Failure: -1   
@@ -163,7 +164,7 @@
 ! Comment:		
 !----------------------------------------------------------------------
 
-          SUBROUTINE h5eget_major_f(error_no, name, hdferr)
+          SUBROUTINE h5eget_major_f(error_no, name, namelen, hdferr)
 !
 !This definition is needed for Windows DLLs
 !DEC$if defined(BUILD_HDF5_DLL)
@@ -173,24 +174,26 @@
             INTEGER, INTENT(IN) :: error_no !Major error number
             CHARACTER(LEN=*), INTENT(OUT) :: name ! Character string describing
                                                   ! the error.
+            INTEGER(SIZE_T), INTENT(IN) :: namelen !Anticipated number of characters in name.
             INTEGER, INTENT(OUT) :: hdferr          ! Error code
 
 !            INTEGER, EXTERNAL :: h5eget_major_c
 !  MS FORTRAN needs explicit interface for C functions called here.
 !
             INTERFACE
-              INTEGER FUNCTION h5eget_major_c(error_no, name)
+              INTEGER FUNCTION h5eget_major_c(error_no, name, namelen)
               USE H5GLOBAL
               !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5EGET_MAJOR_C'::h5eget_major_c
+              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5EGET_MAJOR_C'::h5eget_major_c
               !DEC$ ENDIF
               !DEC$ATTRIBUTES reference :: name
               INTEGER :: error_no
-               CHARACTER(LEN=*) :: name 
+              CHARACTER(LEN=*) :: name 
+              INTEGER(SIZE_T), INTENT(IN) :: namelen
               END FUNCTION h5eget_major_c
             END INTERFACE
 
-            hdferr = h5eget_major_c(error_no, name) 
+            hdferr = h5eget_major_c(error_no, name, namelen) 
           END SUBROUTINE h5eget_major_f
 
 !----------------------------------------------------------------------
@@ -240,7 +243,7 @@
               INTEGER FUNCTION h5eget_minor_c(error_no, name)
               USE H5GLOBAL
               !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5EGET_MINOR_C'::h5eget_minor_c
+              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5EGET_MINOR_C'::h5eget_minor_c
               !DEC$ ENDIF
               !DEC$ATTRIBUTES reference :: name
               INTEGER :: error_no
@@ -300,7 +303,7 @@
               INTEGER FUNCTION h5eset_auto_c(printflag)
               USE H5GLOBAL
               !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-              !MS$ATTRIBUTES C,reference,alias:'_H5ESET_AUTO_C'::h5eset_auto_c
+              !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5ESET_AUTO_C'::h5eset_auto_c
               !DEC$ ENDIF
               INTEGER :: printflag
               END FUNCTION h5eset_auto_c
