@@ -20,7 +20,7 @@
 
 /*----------------------------------------------------------------------------
  * Name:        h5topen_c
- * Purpose:     Call H5Topen to open a datatype
+ * Purpose:     Call H5Topen2 to open a datatype
  * Inputs:      loc_id - file or group identifier
  *              name - name of the datatype within file or  group
  *              namelen - name length
@@ -33,36 +33,36 @@
 int_f
 nh5topen_c (hid_t_f *loc_id, _fcd name, int_f *namelen, hid_t_f *type_id)
 {
-     int ret_value = -1;
-     char *c_name;
-     int c_namelen;
-     hid_t c_type_id;
-     hid_t c_loc_id;
+    char *c_name = NULL;
+    hid_t c_type_id;
+    int ret_value = -1;
 
-     /*
-      * Convert FORTRAN name to C name
-      */
-     c_namelen = *namelen;
-     c_name = (char *)HD5f2cstring(name, c_namelen);
-     if (c_name == NULL) return ret_value;
+    /*
+     * Convert FORTRAN name to C name
+     */
+    if(NULL == (c_name = (char *)HD5f2cstring(name, (size_t)*namelen)))
+        goto done;
 
-     /*
-      * Call H5Topen function.
-      */
-     c_loc_id = *loc_id;
-     c_type_id = H5Topen(c_loc_id, c_name);
+    /*
+     * Call H5Topen2 function.
+     */
+    if((c_type_id = H5Topen2((hid_t)*loc_id, c_name, H5P_DEFAULT)) < 0)
+        goto done;
+    *type_id = (hid_t_f)c_type_id;
 
-     if (c_type_id < 0) return ret_value;
-     *type_id = (hid_t_f)c_type_id;
-     HDfree(c_name);
-     ret_value = 0;
-     return ret_value;
+    ret_value = 0;
+
+done:
+    if(c_name)
+        HDfree(c_name);
+
+    return ret_value;
 }
 
 
 /*----------------------------------------------------------------------------
  * Name:        h5tcommit_c
- * Purpose:     Call H5Tcommit to commit a datatype
+ * Purpose:     Call H5Tcommit2 to commit a datatype
  * Inputs:      loc_id - file or group identifier
  *              name - name of the datatype within file or  group
  *              namelen - name length
@@ -73,32 +73,25 @@ nh5topen_c (hid_t_f *loc_id, _fcd name, int_f *namelen, hid_t_f *type_id)
  * Modifications:
  *---------------------------------------------------------------------------*/
 int_f
-nh5tcommit_c (hid_t_f *loc_id, _fcd name, int_f *namelen, hid_t_f *type_id)
+nh5tcommit_c(hid_t_f *loc_id, _fcd name, int_f *namelen, hid_t_f *type_id)
 {
-     int ret_value = -1;
-     char *c_name;
-     int c_namelen;
-     hid_t c_type_id;
-     hid_t c_loc_id;
-     herr_t status;
+    char *c_name = NULL;
+    int ret_value = -1;
 
-     /*
-      * Convert FORTRAN name to C name
-      */
-     c_namelen = *namelen;
-     c_name = (char *)HD5f2cstring(name, c_namelen);
-     if (c_name == NULL) return ret_value;
+    /* Convert FORTRAN name to C name */
+    if(NULL == (c_name = (char *)HD5f2cstring(name, (size_t)*namelen)))
+        goto done;
 
-     /*
-      * Call H5Tcommit function.
-      */
-     c_loc_id = *loc_id;
-     c_type_id = *type_id;
-     status = H5Tcommit(c_loc_id, c_name, c_type_id);
-     HDfree(c_name);
-     if (status < 0) return ret_value;
-     ret_value = 0;
-     return ret_value;
+    /* Call H5Tcommit2 function */
+    if(H5Tcommit2((hid_t)*loc_id, c_name, (hid_t)*type_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT) < 0)
+        goto done;
+
+    ret_value = 0;
+
+done:
+    if(c_name)
+        HDfree(c_name);
+    return ret_value;
 }
 
 /*----------------------------------------------------------------------------
@@ -994,7 +987,7 @@ nh5tget_member_name_c ( hid_t_f *type_id ,int_f* idx, _fcd member_name, int_f *n
   c_name = H5Tget_member_name(c_type_id, c_index);
   if (c_name == NULL ) return ret_value;
 
-  HD5packFstring(c_name, _fcdtocp(member_name), (int)strlen(c_name));
+  HD5packFstring(c_name, _fcdtocp(member_name), strlen(c_name));
   *namelen = (int_f)strlen(c_name);
   HDfree(c_name);
   ret_value = 0;
@@ -1018,7 +1011,7 @@ nh5tget_member_index_c (hid_t_f *type_id, _fcd name, int_f *namelen, int_f *idx)
 {
      int ret_value = -1;
      char *c_name;
-     int c_namelen;
+     size_t c_namelen;
      hid_t c_type_id;
      int c_index;
 
@@ -1077,7 +1070,7 @@ nh5tget_member_offset_c ( hid_t_f *type_id ,int_f* member_no, size_t_f * offset)
 
 /*----------------------------------------------------------------------------
  * Name:        h5tget_array_dims_c
- * Purpose:     Call H5Tget_array_dims to get
+ * Purpose:     Call H5Tget_array_dims2 to get
  *              dimensions of array datatype
  * Inputs:      type_id - identifier of the array datatype
  * Outputs:     dims -  dimensions(sizes of dimensions) of the array
@@ -1090,33 +1083,23 @@ nh5tget_member_offset_c ( hid_t_f *type_id ,int_f* member_no, size_t_f * offset)
 int_f
 nh5tget_array_dims_c ( hid_t_f *type_id , hsize_t_f * dims)
 {
-  int ret_value = -1;
-  hid_t c_type_id;
-  hsize_t * c_dims;
-  int rank, i;
-  herr_t status;
+    hsize_t c_dims[H5S_MAX_RANK];
+    int rank, i;
+    int ret_value = -1;
 
-  rank = H5Tget_array_ndims((hid_t)*type_id);
-  if (rank < 0) return ret_value;
-  c_dims = (hsize_t*)malloc(sizeof(hsize_t)*rank);
-  if(!c_dims) return ret_value;
+    if((rank = H5Tget_array_ndims((hid_t)*type_id)) < 0)
+        goto DONE;
 
-  c_type_id = (hid_t)*type_id;
-  status = H5Tget_array_dims(c_type_id, c_dims, NULL);
-  if (status < 0) {
-             HDfree(c_dims);
-             return ret_value;
-  }
+    if(H5Tget_array_dims2((hid_t)*type_id, c_dims) < 0)
+        goto DONE;
 
-  for(i =0; i < rank; i++)
-  {
-      dims[rank-i-1] = (hsize_t_f)c_dims[i];
-  }
+    for(i = 0; i < rank; i++)
+        dims[(rank - i) - 1] = (hsize_t_f)c_dims[i];
 
-  ret_value = 0;
-  HDfree(c_dims);
+    ret_value = 0;
 
-  return ret_value;
+DONE:
+    return ret_value;
 }
 
 /*----------------------------------------------------------------------------
@@ -1255,7 +1238,7 @@ nh5tinsert_c(hid_t_f *type_id, _fcd name, int_f* namelen, size_t_f *offset, hid_
   hid_t c_type_id;
   hid_t c_field_id;
   char* c_name;
-  int c_namelen;
+  size_t c_namelen;
   size_t c_offset;
   herr_t error;
 
@@ -1303,7 +1286,7 @@ nh5tpack_c(hid_t_f * type_id)
 
 /*----------------------------------------------------------------------------
  * Name:        h5tarray_create_c
- * Purpose:     Call H5Tarray_create to create array datatype
+ * Purpose:     Call H5Tarray_create2 to create array datatype
  * Inputs:      base_id - identifier of array base datatype
  *              rank - array's rank
  *              dims - Size of new member array
@@ -1316,38 +1299,26 @@ nh5tpack_c(hid_t_f * type_id)
 int_f
 nh5tarray_create_c(hid_t_f * base_id, int_f *rank, hsize_t_f* dims, hid_t_f* type_id)
 {
-  int ret_value = -1;
-  hid_t c_base_id;
-  hid_t c_type_id;
-  int c_rank;
-  hsize_t *c_dims;
-  int i;
-
-  c_dims = (hsize_t*)malloc(sizeof(hsize_t)*(*rank));
-  if(!c_dims) return ret_value;
+    hsize_t c_dims[H5S_MAX_RANK];
+    hid_t c_type_id;
+    unsigned u;                 /* Local index variable */
+    int ret_value = -1;
 
 
-  /*
-   * Transpose dimension arrays because of C-FORTRAN storage order
-   */
-  for (i = 0; i < *rank ; i++) {
-     c_dims[i] =  (hsize_t)dims[*rank - i - 1];
-  }
+    /*
+     * Transpose dimension arrays because of C-FORTRAN storage order
+     */
+    for(u = 0; u < (unsigned)*rank ; u++)
+        c_dims[u] =  (hsize_t)dims[(*rank - u) - 1];
 
-  c_base_id = (hid_t)*base_id;
-  c_rank = (int)*rank;
-  c_type_id = H5Tarray_create(c_base_id, c_rank, c_dims, NULL);
+    if((c_type_id = H5Tarray_create2((hid_t)*base_id, (unsigned)*rank, c_dims)) < 0)
+        goto DONE;
 
-  if(c_type_id < 0) {
-          HDfree(c_dims);
-          return ret_value;
-  }
-
-  *type_id = (hid_t_f)c_type_id;
-  ret_value = 0;
-  HDfree(c_dims);
-  return ret_value;
+    *type_id = (hid_t_f)c_type_id;
+    ret_value = 0;
 
+DONE:
+    return ret_value;
 }
 
 
@@ -1397,8 +1368,8 @@ nh5tenum_insert_c(hid_t_f *type_id, _fcd name, int_f* namelen, int_f* value)
   int ret_value = -1;
   hid_t c_type_id;
   char* c_name;
-  int c_namelen;
-  int c_value;
+  size_t c_namelen;
+  int_f c_value;
   herr_t error;
 
   c_namelen = *namelen;
@@ -1438,13 +1409,13 @@ nh5tenum_nameof_c(hid_t_f *type_id, int_f* value, _fcd name, size_t_f* namelen)
   char* c_name;
   size_t c_namelen;
   herr_t error;
-  int c_value;
+  int_f c_value;
   c_value = *value;
   c_namelen = ((size_t)*namelen) +1;
   c_name = (char *)malloc(sizeof(char)*c_namelen);
-  c_type_id = *type_id;
+  c_type_id = (hid_t)*type_id;
   error = H5Tenum_nameof(c_type_id, &c_value, c_name, c_namelen);
-  HD5packFstring(c_name, _fcdtocp(name), (int)strlen(c_name));
+  HD5packFstring(c_name, _fcdtocp(name), strlen(c_name));
   HDfree(c_name);
 
   if(error < 0) return ret_value;
@@ -1472,18 +1443,17 @@ nh5tenum_valueof_c(hid_t_f *type_id, _fcd name, int_f* namelen, int_f* value)
   int ret_value = -1;
   hid_t c_type_id;
   char* c_name;
-  int c_namelen;
-  int c_value;
+  size_t c_namelen;
   herr_t error;
   c_namelen = *namelen;
   c_name = (char *)HD5f2cstring(name, c_namelen);
   if (c_name == NULL) return ret_value;
 
   c_type_id = *type_id;
-  error = H5Tenum_valueof(c_type_id, c_name, &c_value);
+  error = H5Tenum_valueof(c_type_id, c_name, value);
   HDfree(c_name);
+
   if(error < 0) return ret_value;
-  *value = (int_f)c_value;
   ret_value = 0;
   return ret_value;
 }
@@ -1541,7 +1511,7 @@ nh5tset_tag_c(hid_t_f* type_id, _fcd tag, int_f* namelen)
   hid_t c_type_id;
   herr_t status;
   char* c_tag;
-  int c_namelen;
+  size_t c_namelen;
 
   c_namelen = *namelen;
   c_tag = (char *)HD5f2cstring(tag, c_namelen);
@@ -1578,7 +1548,7 @@ nh5tget_tag_c(hid_t_f* type_id, _fcd tag, int_f* taglen)
   c_tag = H5Tget_tag(c_type_id);
   if (c_tag == NULL ) return ret_value;
 
-  HD5packFstring(c_tag, _fcdtocp(tag), (int)strlen(c_tag));
+  HD5packFstring(c_tag, _fcdtocp(tag), strlen(c_tag));
   *taglen = (int_f)HDstrlen(c_tag);
   HDfree(c_tag);
   ret_value = 0;
@@ -1634,7 +1604,6 @@ nh5tis_variable_str_c ( hid_t_f *type_id , int_f *flag )
   if ( status < 0  ) ret_value = -1;
   return ret_value;
 }
-
 /*----------------------------------------------------------------------------
  * Name:        h5tget_member_class_c
  * Purpose:     Call H5Tget_member_class to detrmine ithe class of the compound
