@@ -60,7 +60,7 @@
         INTERFACE
           INTEGER FUNCTION h5open_c()
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5OPEN_C'::h5open_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5OPEN_C'::h5open_c
           !DEC$ ENDIF
           END FUNCTION h5open_c
         END INTERFACE
@@ -71,13 +71,12 @@
           INTEGER(HID_T), DIMENSION(FLOATING_TYPES_LEN) :: f_types
           INTEGER(HID_T), DIMENSION(INTEGER_TYPES_LEN) :: i_types   
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5INIT_TYPES_C'::h5init_types_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5INIT_TYPES_C'::h5init_types_c
           !DEC$ ENDIF
           END FUNCTION h5init_types_c
         END INTERFACE
         INTERFACE
           INTEGER FUNCTION h5init_flags_c(i_H5D_flags, &
-                                i_H5E_flags, &
                                 i_H5F_flags, &
                                 i_H5FD_flags, &
                                 i_H5FD_hid_flags, &
@@ -93,16 +92,15 @@
           INTEGER i_H5G_flags(H5G_FLAGS_LEN)
           INTEGER i_H5D_flags(H5D_FLAGS_LEN)
           INTEGER i_H5FD_flags(H5FD_FLAGS_LEN)
-          INTEGER i_H5FD_hid_flags(H5FD_HID_FLAGS_LEN)
-          INTEGER i_H5E_flags(H5E_FLAGS_LEN)
+          INTEGER(HID_T) i_H5FD_hid_flags(H5FD_HID_FLAGS_LEN)
           INTEGER i_H5I_flags(H5I_FLAGS_LEN)
-          INTEGER i_H5P_flags(H5P_FLAGS_LEN)
+          INTEGER(HID_T) i_H5P_flags(H5P_FLAGS_LEN)
           INTEGER i_H5R_flags(H5R_FLAGS_LEN)
           INTEGER i_H5S_flags(H5S_FLAGS_LEN)
           INTEGER i_H5T_flags(H5T_FLAGS_LEN)
           INTEGER i_H5Z_flags(H5Z_FLAGS_LEN)
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5INIT_FLAGS_C'::h5init_flags_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5INIT_FLAGS_C'::h5init_flags_c
           !DEC$ ENDIF
           END FUNCTION h5init_flags_c
         END INTERFACE
@@ -111,14 +109,13 @@
           USE H5GLOBAL
           INTEGER i_H5LIB_flags(H5LIB_FLAGS_LEN)
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5INIT1_FLAGS_C'::h5init1_flags_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5INIT1_FLAGS_C'::h5init1_flags_c
           !DEC$ ENDIF
           END FUNCTION h5init1_flags_c
         END INTERFACE
         error_0 = h5open_c()
         error_1 = h5init_types_c(predef_types, floating_types, integer_types)
         error_2 = h5init_flags_c(H5D_flags, &
-                                H5E_flags, &
                                 H5F_flags, &
                                 H5FD_flags, &
                                 H5FD_hid_flags, &
@@ -172,7 +169,7 @@
         INTERFACE
           INTEGER FUNCTION h5close_c()
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5CLOSE_C'::h5close_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5CLOSE_C'::h5close_c
           !DEC$ ENDIF
           END FUNCTION h5close_c
         END INTERFACE
@@ -188,7 +185,7 @@
           INTEGER(HID_T), DIMENSION(F_TYPES_LEN) :: f_types
           INTEGER(HID_T), DIMENSION(I_TYPES_LEN) :: i_types   
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5CLOSE_TYPES_C'::h5close_types_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5CLOSE_TYPES_C'::h5close_types_c
           !DEC$ ENDIF
           END FUNCTION h5close_types_c
         END INTERFACE
@@ -236,7 +233,7 @@
         INTERFACE
           INTEGER FUNCTION h5get_libversion_c(majnum, minnum, relnum)
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5GET_LIBVERSION_C'::h5get_libversion_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5GET_LIBVERSION_C'::h5get_libversion_c
           !DEC$ ENDIF
           INTEGER, INTENT(OUT) :: majnum, minnum, relnum
           END FUNCTION h5get_libversion_c
@@ -283,7 +280,7 @@
         INTERFACE
           INTEGER FUNCTION h5check_version_c(majnum, minnum, relnum)
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5CHECK_VERSION_C'::h5check_version_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5CHECK_VERSION_C'::h5check_version_c
           !DEC$ ENDIF
           INTEGER, INTENT(IN) :: majnum, minnum, relnum
           END FUNCTION h5check_version_c
@@ -327,7 +324,7 @@
         INTERFACE
           INTEGER FUNCTION h5garbage_collect_c()
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5GARBAGE_COLLECT_C'::h5garbage_collect_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5GARBAGE_COLLECT_C'::h5garbage_collect_c
           !DEC$ ENDIF
           END FUNCTION h5garbage_collect_c
         END INTERFACE
@@ -370,7 +367,7 @@
         INTERFACE
           INTEGER FUNCTION h5dont_atexit_c()
           !DEC$ IF DEFINED(HDF5F90_WINDOWS)
-          !MS$ATTRIBUTES C,reference,alias:'_H5DONT_ATEXIT_C'::h5dont_atexit_c
+          !DEC$ ATTRIBUTES C,reference,decorate,alias:'H5DONT_ATEXIT_C'::h5dont_atexit_c
           !DEC$ ENDIF
           END FUNCTION h5dont_atexit_c
         END INTERFACE
