@@ -202,19 +202,21 @@
       COMMON /H5G_FLAGS/ H5G_flags
 
       INTEGER :: H5G_UNKNOWN_F
-      INTEGER :: H5G_LINK_F
       INTEGER :: H5G_GROUP_F
       INTEGER :: H5G_DATASET_F
       INTEGER :: H5G_TYPE_F
+! XXX: Fix problems with H5G_LINK_F! - QAK
+      INTEGER :: H5G_LINK_F
       INTEGER :: H5G_LINK_ERROR_F
       INTEGER :: H5G_LINK_HARD_F
       INTEGER :: H5G_LINK_SOFT_F
 
       EQUIVALENCE(H5G_flags(1), H5G_UNKNOWN_F)
-      EQUIVALENCE(H5G_flags(2), H5G_LINK_F)
-      EQUIVALENCE(H5G_flags(3), H5G_GROUP_F)
-      EQUIVALENCE(H5G_flags(4), H5G_DATASET_F)
-      EQUIVALENCE(H5G_flags(5), H5G_TYPE_F) 
+      EQUIVALENCE(H5G_flags(2), H5G_GROUP_F)
+      EQUIVALENCE(H5G_flags(3), H5G_DATASET_F)
+      EQUIVALENCE(H5G_flags(4), H5G_TYPE_F) 
+! XXX: Fix problems with H5G_LINK_F! - QAK
+      EQUIVALENCE(H5G_flags(5), H5G_LINK_F)
       EQUIVALENCE(H5G_flags(6), H5G_LINK_ERROR_F) 
       EQUIVALENCE(H5G_flags(7), H5G_LINK_HARD_F) 
       EQUIVALENCE(H5G_flags(8), H5G_LINK_SOFT_F) 
@@ -310,13 +312,11 @@
       EQUIVALENCE(H5FD_flags(9), H5FD_MEM_LHEAP_F)
       EQUIVALENCE(H5FD_flags(10), H5FD_MEM_OHDR_F)
       EQUIVALENCE(H5FD_flags(11), H5FD_MEM_NTYPES_F)
-
-
 !
 ! H5FD file drivers flags declaration
 !
-      INTEGER, PARAMETER :: H5FD_HID_FLAGS_LEN = 8
-      INTEGER H5FD_hid_flags(H5FD_HID_FLAGS_LEN)
+      INTEGER, PARAMETER :: H5FD_HID_FLAGS_LEN = 7
+      INTEGER(HID_T) H5FD_hid_flags(H5FD_HID_FLAGS_LEN)
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$ ATTRIBUTES DLLEXPORT :: /H5FD_HID_FLAGS/
 !DEC$endif
@@ -329,7 +329,6 @@
       INTEGER(HID_T) :: H5FD_MULTI_F
       INTEGER(HID_T) :: H5FD_SEC2_F
       INTEGER(HID_T) :: H5FD_STDIO_F
-      INTEGER(HID_T) :: H5FD_STREAM_F
 
       EQUIVALENCE(H5FD_hid_flags(1), H5FD_CORE_F)
       EQUIVALENCE(H5FD_hid_flags(2), H5FD_FAMILY_F)
@@ -338,68 +337,8 @@
       EQUIVALENCE(H5FD_hid_flags(5), H5FD_MULTI_F)
       EQUIVALENCE(H5FD_hid_flags(6), H5FD_SEC2_F)
       EQUIVALENCE(H5FD_hid_flags(7), H5FD_STDIO_F)
-      EQUIVALENCE(H5FD_hid_flags(8), H5FD_STREAM_F)
 
       
-!
-! H5E flags declaration
-!
-      INTEGER, PARAMETER :: H5E_FLAGS_LEN = 24
-      INTEGER H5E_flags(H5E_FLAGS_LEN)
-!DEC$if defined(BUILD_HDF5_DLL)
-!DEC$ ATTRIBUTES DLLEXPORT :: /H5E_FLAGS/
-!DEC$endif
-      COMMON /H5E_FLAGS/ H5E_flags
-
-      INTEGER ::  H5E_NONE_MAJOR_F 
-      INTEGER ::  H5E_ARGS_F 
-      INTEGER ::  H5E_RESOURCE_F 
-      INTEGER ::  H5E_INTERNAL_F 
-      INTEGER ::  H5E_FILE_F 
-      INTEGER ::  H5E_IO_F 
-      INTEGER ::  H5E_FUNC_F 
-      INTEGER ::  H5E_ATOM_F 
-      INTEGER ::  H5E_CACHE_F 
-      INTEGER ::  H5E_BTREE_F 
-      INTEGER ::  H5E_SYM_F 
-      INTEGER ::  H5E_HEAP_F 
-      INTEGER ::  H5E_OHDR_F 
-      INTEGER ::  H5E_DATATYPE_F 
-      INTEGER ::  H5E_DATASPACE_F 
-      INTEGER ::  H5E_DATASET_F 
-      INTEGER ::  H5E_STORAGE_F 
-      INTEGER ::  H5E_PLIST_F 
-      INTEGER ::  H5E_ATTR_F 
-      INTEGER ::  H5E_PLINE_F 
-      INTEGER ::  H5E_EFL_F 
-      INTEGER ::  H5E_REFERENCE_F
-      INTEGER ::  H5E_VFL_F 
-      INTEGER ::  H5E_TBBT_F 
-
-      EQUIVALENCE(H5E_flags(1), H5E_NONE_MAJOR_F)
-      EQUIVALENCE(H5E_flags(2), H5E_ARGS_F)
-      EQUIVALENCE(H5E_flags(3), H5E_RESOURCE_F)
-      EQUIVALENCE(H5E_flags(4), H5E_INTERNAL_F)
-      EQUIVALENCE(H5E_flags(5), H5E_FILE_F)
-      EQUIVALENCE(H5E_flags(6), H5E_IO_F)
-      EQUIVALENCE(H5E_flags(7), H5E_FUNC_F)
-      EQUIVALENCE(H5E_flags(8), H5E_ATOM_F)
-      EQUIVALENCE(H5E_flags(9), H5E_CACHE_F)
-      EQUIVALENCE(H5E_flags(10), H5E_BTREE_F)
-      EQUIVALENCE(H5E_flags(11), H5E_SYM_F)
-      EQUIVALENCE(H5E_flags(12), H5E_HEAP_F)
-      EQUIVALENCE(H5E_flags(13), H5E_OHDR_F)
-      EQUIVALENCE(H5E_flags(14), H5E_DATATYPE_F)
-      EQUIVALENCE(H5E_flags(15), H5E_DATASPACE_F)
-      EQUIVALENCE(H5E_flags(16), H5E_DATASET_F)
-      EQUIVALENCE(H5E_flags(17), H5E_STORAGE_F)
-      EQUIVALENCE(H5E_flags(18), H5E_PLIST_F)
-      EQUIVALENCE(H5E_flags(19), H5E_ATTR_F)
-      EQUIVALENCE(H5E_flags(20), H5E_PLINE_F)
-      EQUIVALENCE(H5E_flags(21), H5E_EFL_F)
-      EQUIVALENCE(H5E_flags(22), H5E_REFERENCE_F)
-      EQUIVALENCE(H5E_flags(23), H5E_VFL_F)
-      EQUIVALENCE(H5E_flags(24), H5E_TBBT_F)
 
 !
 ! H5I flags declaration
@@ -431,27 +370,27 @@
 ! H5P flags declaration
 !
       INTEGER, PARAMETER :: H5P_FLAGS_LEN = 7 
-      INTEGER H5P_flags(H5P_FLAGS_LEN)
+      INTEGER(HID_T) H5P_flags(H5P_FLAGS_LEN)
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$ ATTRIBUTES DLLEXPORT :: /H5P_FLAGS/
 !DEC$endif
       COMMON /H5P_FLAGS/ H5P_flags
 
-      INTEGER ::  H5P_FILE_CREATE_F 
-      INTEGER ::  H5P_FILE_ACCESS_F 
-      INTEGER ::  H5P_DATASET_CREATE_F
-      INTEGER ::  H5P_DATASET_XFER_F 
-      INTEGER ::  H5P_MOUNT_F 
-      INTEGER ::  H5P_DEFAULT_F 
-      INTEGER ::  H5P_NO_CLASS_F 
+      INTEGER(HID_T) ::  H5P_FILE_CREATE_F 
+      INTEGER(HID_T) ::  H5P_FILE_ACCESS_F 
+      INTEGER(HID_T) ::  H5P_DATASET_CREATE_F
+      INTEGER(HID_T) ::  H5P_DATASET_XFER_F 
+      INTEGER(HID_T) ::  H5P_FILE_MOUNT_F 
+      INTEGER(HID_T) ::  H5P_DEFAULT_F 
+      INTEGER(HID_T) ::  H5P_ROOT_F 
 
       EQUIVALENCE(H5P_flags(1), H5P_FILE_CREATE_F)
       EQUIVALENCE(H5P_flags(2), H5P_FILE_ACCESS_F)
       EQUIVALENCE(H5P_flags(3), H5P_DATASET_CREATE_F)
       EQUIVALENCE(H5P_flags(4), H5P_DATASET_XFER_F)
-      EQUIVALENCE(H5P_flags(5), H5P_MOUNT_F)
+      EQUIVALENCE(H5P_flags(5), H5P_FILE_MOUNT_F)
       EQUIVALENCE(H5P_flags(6), H5P_DEFAULT_F)
-      EQUIVALENCE(H5P_flags(7), H5P_NO_CLASS_F)
+      EQUIVALENCE(H5P_flags(7), H5P_ROOT_F)
 
 !
 ! H5P flags declaration
@@ -472,7 +411,7 @@
 !
 ! H5S flags declaration
 !
-      INTEGER, PARAMETER :: H5S_FLAGS_LEN = 19
+      INTEGER, PARAMETER :: H5S_FLAGS_LEN = 20 
       INTEGER H5S_flags(H5S_FLAGS_LEN)
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$ ATTRIBUTES DLLEXPORT :: /H5S_FLAGS/
@@ -481,6 +420,7 @@
 
       INTEGER :: H5S_SCALAR_F 
       INTEGER :: H5S_SIMPLE_F 
+      INTEGER :: H5S_NULL_F 
 
       INTEGER :: H5S_UNLIMITED_F
       INTEGER :: H5S_ALL_F
@@ -505,32 +445,33 @@
 
       EQUIVALENCE(H5S_flags(1), H5S_SCALAR_F)
       EQUIVALENCE(H5S_flags(2), H5S_SIMPLE_F)
-      EQUIVALENCE(H5S_flags(3), H5S_SELECT_SET_F)
-      EQUIVALENCE(H5S_flags(4), H5S_SELECT_OR_F)
-      EQUIVALENCE(H5S_flags(5), H5S_UNLIMITED_F)
-      EQUIVALENCE(H5S_flags(6), H5S_ALL_F)
+      EQUIVALENCE(H5S_flags(3), H5S_NULL_F)
+      EQUIVALENCE(H5S_flags(4), H5S_SELECT_SET_F)
+      EQUIVALENCE(H5S_flags(5), H5S_SELECT_OR_F)
+      EQUIVALENCE(H5S_flags(6), H5S_UNLIMITED_F)
+      EQUIVALENCE(H5S_flags(7), H5S_ALL_F)
 
-      EQUIVALENCE(H5S_flags(7), H5S_SELECT_NOOP_F)
-      EQUIVALENCE(H5S_flags(8), H5S_SELECT_AND_F) 
-      EQUIVALENCE(H5S_flags(9), H5S_SELECT_XOR_F)
-      EQUIVALENCE(H5S_flags(10), H5S_SELECT_NOTB_F)
-      EQUIVALENCE(H5S_flags(11), H5S_SELECT_NOTA_F)
-      EQUIVALENCE(H5S_flags(12), H5S_SELECT_APPEND_F) 
-      EQUIVALENCE(H5S_flags(13), H5S_SELECT_PREPEND_F) 
-      EQUIVALENCE(H5S_flags(14), H5S_SELECT_INVALID_F)
+      EQUIVALENCE(H5S_flags(8), H5S_SELECT_NOOP_F)
+      EQUIVALENCE(H5S_flags(9), H5S_SELECT_AND_F) 
+      EQUIVALENCE(H5S_flags(10), H5S_SELECT_XOR_F)
+      EQUIVALENCE(H5S_flags(11), H5S_SELECT_NOTB_F)
+      EQUIVALENCE(H5S_flags(12), H5S_SELECT_NOTA_F)
+      EQUIVALENCE(H5S_flags(13), H5S_SELECT_APPEND_F) 
+      EQUIVALENCE(H5S_flags(14), H5S_SELECT_PREPEND_F) 
+      EQUIVALENCE(H5S_flags(15), H5S_SELECT_INVALID_F)
 
 
-      EQUIVALENCE(H5S_flags(15), H5S_SEL_ERROR_F)
-      EQUIVALENCE(H5S_flags(16), H5S_SEL_NONE_F)
-      EQUIVALENCE(H5S_flags(17), H5S_SEL_POINTS_F)
-      EQUIVALENCE(H5S_flags(18), H5S_SEL_HYPERSLABS_F)
-      EQUIVALENCE(H5S_flags(19), H5S_SEL_ALL_F)
+      EQUIVALENCE(H5S_flags(16), H5S_SEL_ERROR_F)
+      EQUIVALENCE(H5S_flags(17), H5S_SEL_NONE_F)
+      EQUIVALENCE(H5S_flags(18), H5S_SEL_POINTS_F)
+      EQUIVALENCE(H5S_flags(19), H5S_SEL_HYPERSLABS_F)
+      EQUIVALENCE(H5S_flags(20), H5S_SEL_ALL_F)
 
 
 !
 ! H5T flags declaration
 !
-      INTEGER, PARAMETER :: H5T_FLAGS_LEN = 30
+      INTEGER, PARAMETER :: H5T_FLAGS_LEN = 31
       INTEGER H5T_flags(H5T_FLAGS_LEN)
 !DEC$if defined(BUILD_HDF5_DLL)
 !DEC$ ATTRIBUTES DLLEXPORT :: /H5T_FLAGS/
@@ -563,6 +504,7 @@
       INTEGER ::  H5T_NORM_MSBSET_F
       INTEGER ::  H5T_NORM_NONE_F 
       INTEGER ::  H5T_CSET_ASCII_F
+      INTEGER ::  H5T_CSET_UTF8_F
       INTEGER ::  H5T_STR_NULLTERM_F 
       INTEGER ::  H5T_STR_NULLPAD_F 
       INTEGER ::  H5T_STR_SPACEPAD_F
@@ -592,12 +534,13 @@
       EQUIVALENCE(H5T_flags(22), H5T_NORM_MSBSET_F)
       EQUIVALENCE(H5T_flags(23), H5T_NORM_NONE_F)
       EQUIVALENCE(H5T_flags(24), H5T_CSET_ASCII_F)
-      EQUIVALENCE(H5T_flags(25), H5T_STR_NULLTERM_F)
-      EQUIVALENCE(H5T_flags(26), H5T_STR_NULLPAD_F)
-      EQUIVALENCE(H5T_flags(27), H5T_STR_SPACEPAD_F)
-      EQUIVALENCE(H5T_flags(28), H5T_STR_ERROR_F)
-      EQUIVALENCE(H5T_flags(29), H5T_VLEN_F)
-      EQUIVALENCE(H5T_flags(30), H5T_ARRAY_F)
+      EQUIVALENCE(H5T_flags(25), H5T_CSET_UTF8_F)
+      EQUIVALENCE(H5T_flags(26), H5T_STR_NULLTERM_F)
+      EQUIVALENCE(H5T_flags(27), H5T_STR_NULLPAD_F)
+      EQUIVALENCE(H5T_flags(28), H5T_STR_SPACEPAD_F)
+      EQUIVALENCE(H5T_flags(29), H5T_STR_ERROR_F)
+      EQUIVALENCE(H5T_flags(30), H5T_VLEN_F)
+      EQUIVALENCE(H5T_flags(31), H5T_ARRAY_F)
 
 !
 ! H5Z flags declaration
