@@ -629,7 +629,6 @@
                write(*,*) " File access lists should be equal, error "
                total_error=total_error + 1
           endif
-
           CALL h5fopen_f(fix_filename, H5F_ACC_RDWR_F, fid2, error, access_prp=fapl2)
                if( error .ne. -1) then
                    total_error = total_error + 1
@@ -654,7 +653,7 @@
                  write(*,*) "Wrong number of open objects reported, error"
                endif
           allocate(obj_ids(obj_countf), stat = error) 
-          CALL h5fget_obj_ids_f(fid, H5F_OBJ_FILE_F, -1, obj_ids, error)
+          CALL h5fget_obj_ids_f(fid, H5F_OBJ_FILE_F, obj_countf, obj_ids, error)
                CALL check("h5fget_obj_ids_f",error,total_error)
           if(error .eq. 0) then
              do i = 1, obj_countf
@@ -679,8 +678,10 @@
                  write(*,*) "File should be closed at this point, error"
               endif 
 
-          if(cleanup) CALL h5_cleanup_f(filename, H5P_DEFAULT_F, error)
+          if(cleanup) then
+              CALL h5_cleanup_f(filename, H5P_DEFAULT_F, error)
               CALL check("h5_cleanup_f", error, total_error)
+          endif
           deallocate(obj_ids)
           RETURN
 
@@ -696,6 +697,8 @@
           LOGICAL, INTENT(IN) :: cleanup
           INTEGER, INTENT(OUT) :: total_error 
           INTEGER              :: error
+          INTEGER flag 
+          INTEGER :: free_space_out
      
           !
           CHARACTER(LEN=10), PARAMETER :: filename = "file_space"
@@ -720,7 +723,7 @@
                CALL check("h5fget_freespace_f",error,total_error)
                if(error .eq.0 .and. free_space .ne. 0) then
                  total_error = total_error + 1
-                 write(*,*) "Wrong amount of free space reported, ", free_space
+                 write(*,*) "1: Wrong amount of free space reported, ", free_space
                endif
 
           ! Create group in the file.
@@ -736,7 +739,7 @@
                CALL check("h5fget_freespace_f",error,total_error)
                if(error .eq.0 .and. free_space .ne. 0) then
                  total_error = total_error + 1
-                 write(*,*) "Wrong amount of free space reported, ", free_space
+                 write(*,*) "2: Wrong amount of free space reported, ", free_space
                endif
 
           !Unlink the group
@@ -746,9 +749,9 @@
           ! Check the free space now
           CALL h5fget_freespace_f(fid, free_space, error)
                CALL check("h5fget_freespace_f",error,total_error)
-               if(error .eq.0 .and. free_space .ne. 976) then
+               if(error .eq.0 .and. free_space .ne. 0) then
                  total_error = total_error + 1
-                 write(*,*) "Wrong amount of free space reported, ", free_space
+                 write(*,*) "3: Wrong amount of free space reported, ", free_space
                endif
 
           CALL h5fclose_f(fid, error)
