@@ -45,6 +45,7 @@
                       !specified dataset
      INTEGER(SIZE_T) :: namesize
      INTEGER(HSIZE_T) :: size, buf_size
+     INTEGER :: idx
 
      buf_size = 4*1024*1024
 
@@ -96,10 +97,13 @@
      CALL check("h5pclose_f", error, total_error)
      CALL h5sclose_f(space_id, error)
      CALL check("h5sclose_f", error, total_error)
-     ! Read dataset creation information 
+     CALL h5fclose_f(file_id, error)
+
+     CALL h5fopen_f(fix_filename, H5F_ACC_RDWR_F, file_id, error)
      CALL h5dopen_f(file_id, "dset1", dataset_id, error)
      CALL check("h5dopen_f",error,total_error)
     
+     ! Read dataset creation information 
      CALL h5dget_create_plist_f(dataset_id, plist_id, error)
      CALL check("h5dget_create_plist_f",error,total_error)
      CALL h5pget_external_count_f(plist_id, count, error)
@@ -109,7 +113,8 @@
          total_error = total_error + 1
      end if
      namesize = 10
-     CALL h5pget_external_f(plist_id, 0, namesize, name, file_offset, &
+     idx = 0
+     CALL h5pget_external_f(plist_id, idx, namesize, name, file_offset, &
                             file_bytes, error)
      CALL check("h5pget_external_f",error,total_error)
      if(file_offset .ne. 0 ) then
@@ -150,7 +155,6 @@
           INTEGER(HID_T) :: dspace_id     ! Dataspace identifier
           INTEGER(HID_T) :: dtype_id      ! Datatype identifier
           INTEGER(HID_T) :: fapl, fapl_1  ! File access property list identifier
-          INTEGER(HID_T) :: driver
           INTEGER, DIMENSION(0:H5FD_MEM_NTYPES_F-1) :: memb_map, memb_map_out
           INTEGER(HID_T), DIMENSION(0:H5FD_MEM_NTYPES_F-1) :: memb_fapl, memb_fapl_out
           CHARACTER(LEN=20), DIMENSION(0:H5FD_MEM_NTYPES_F-1) :: memb_name, memb_name_out
@@ -164,8 +168,7 @@
 
           INTEGER, DIMENSION(4,6) :: dset_data, data_out ! Data buffers
           INTEGER     ::   error ! Error flag
- 
-
+          INTEGER(HID_T) :: driver
           INTEGER     :: i, j    !general purpose integers
           INTEGER(HSIZE_T), DIMENSION(2) :: data_dims
           INTEGER :: mdc_nelmts
@@ -241,6 +244,12 @@
  
           CALL h5fcreate_f(fix_filename, H5F_ACC_TRUNC_F, file_id, error, access_prp = fapl)
               CALL check("h5fcreate_f", error, total_error)
+          if(error .ne. 0) then
+             write(*,*) "Cannot create file using multi-file driver... Exiting...."
+             total_error = 1
+             call h5pclose_f(fapl, error)
+             return
+          endif  
 
 
           ! 
@@ -285,7 +294,6 @@
               CALL check("h5fclose_f", error, total_error)
           CALL h5pclose_f(fapl, error)
               CALL check("h5pclose_f", error, total_error)
-
          ! 
           ! Open the existing file.
           !
