@@ -13,12 +13,11 @@
  * access to either file, you may request a copy from help@hdfgroup.org.     *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#include "H5IM.h"
-
+#include "H5IMprivate.h"
+#include "H5LTprivate.h"
 #include <string.h>
 #include <stdlib.h>
 
-
 /*-------------------------------------------------------------------------
  * Function: H5IMmake_image_8bit
  *
@@ -26,7 +25,7 @@
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: June 13, 2001
  *
@@ -45,35 +44,31 @@ herr_t H5IMmake_image_8bit( hid_t loc_id,
                             hsize_t height,
                             const unsigned char *buffer )
 {
- int      rank = 3;
- hsize_t  dims[3];
+ hsize_t  dims[IMAGE8_RANK];
 
   /* Initialize the image dimensions */
  dims[0] = height;
  dims[1] = width;
- dims[2] = 1;
 
  /* Make the dataset */
- if ( H5LTmake_dataset( loc_id, dset_name, rank, dims, H5T_NATIVE_UCHAR, buffer ) < 0 )
+ if ( H5LTmake_dataset( loc_id, dset_name, IMAGE8_RANK, dims, H5T_NATIVE_UCHAR, buffer ) < 0)
   return -1;
 
  /* Attach the CLASS attribute */
- if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", "IMAGE" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", IMAGE_CLASS ) < 0)
   return -1;
 
  /* Attach the VERSION attribute */
- if ( H5LTset_attribute_string( loc_id, dset_name, "IMAGE_VERSION", "1.2" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "IMAGE_VERSION", IMAGE_VERSION ) < 0)
   return -1;
 
  /* Attach the IMAGE_SUBCLASS attribute */
- if ( H5LTset_attribute_string( loc_id, dset_name, "IMAGE_SUBCLASS", "IMAGE_INDEXED" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "IMAGE_SUBCLASS", "IMAGE_INDEXED" ) < 0)
   return -1;
 
  return 0;
 }
 
-
-
 /*-------------------------------------------------------------------------
  * Function: H5IMmake_image_24bit
  *
@@ -81,7 +76,7 @@ herr_t H5IMmake_image_8bit( hid_t loc_id,
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: June 13, 2001
  *
@@ -106,8 +101,7 @@ herr_t H5IMmake_image_24bit( hid_t loc_id,
                              const char *interlace,
                              const unsigned char *buffer )
 {
- int      rank = 3;
- hsize_t  dims[3];
+ hsize_t  dims[IMAGE24_RANK];
 
  /* Initialize the image dimensions */
 
@@ -116,36 +110,36 @@ herr_t H5IMmake_image_24bit( hid_t loc_id,
   /* Number of color planes is defined as the third dimension */
   dims[0] = height;
   dims[1] = width;
-  dims[2] = 3;
+  dims[2] = IMAGE24_RANK;
  }
  else
  if ( strcmp( interlace, "INTERLACE_PLANE" ) == 0 )
  {
   /* Number of color planes is defined as the first dimension */
-  dims[0] = 3;
+  dims[0] = IMAGE24_RANK;
   dims[1] = height;
   dims[2] = width;
  }
  else return -1;
 
  /* Make the dataset */
- if ( H5LTmake_dataset( loc_id, dset_name, rank, dims, H5T_NATIVE_UCHAR, buffer ) < 0 )
+ if ( H5LTmake_dataset( loc_id, dset_name, IMAGE24_RANK, dims, H5T_NATIVE_UCHAR, buffer ) < 0)
   return -1;
 
  /* Attach the CLASS attribute */
- if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", "IMAGE" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "CLASS", IMAGE_CLASS ) < 0)
   return -1;
 
  /* Attach the VERSION attribute */
- if ( H5LTset_attribute_string( loc_id, dset_name, "IMAGE_VERSION", "1.2" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "IMAGE_VERSION", IMAGE_VERSION ) < 0)
   return -1;
 
  /* Attach the IMAGE_SUBCLASS attribute */
- if ( H5LTset_attribute_string( loc_id, dset_name, "IMAGE_SUBCLASS", "IMAGE_TRUECOLOR" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "IMAGE_SUBCLASS", "IMAGE_TRUECOLOR" ) < 0)
   return -1;
 
  /* Attach the INTERLACE_MODE attribute. This attributes is only for true color images */
- if ( H5LTset_attribute_string( loc_id, dset_name, "INTERLACE_MODE", interlace ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, dset_name, "INTERLACE_MODE", interlace ) < 0)
   return -1;
 
  return 0;
@@ -161,7 +155,7 @@ herr_t H5IMmake_image_24bit( hid_t loc_id,
  *
  * Return:
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: May 28, 2001
  *
@@ -171,30 +165,23 @@ herr_t H5IMmake_image_24bit( hid_t loc_id,
  *
  *-------------------------------------------------------------------------
  */
-
-static herr_t find_palette( hid_t loc_id, const char *name, void  *op_data )
+static herr_t
+find_palette(hid_t loc_id, const char *name, const H5A_info_t *ainfo,
+    void *op_data)
 {
+    int ret = H5_ITER_CONT;
 
- /* Define a default zero value for return. This will cause the iterator to continue if
-  * the palette attribute is not found yet.
-  */
+    /* Shut compiler up */
+    loc_id = loc_id; ainfo = ainfo; op_data = op_data;
 
- int ret = 0;
+    /* Define a positive value for return value if the attribute was found. This will
+     * cause the iterator to immediately return that positive value,
+     * indicating short-circuit success
+     */
+    if(strcmp(name, "PALETTE") == 0)
+        ret = H5_ITER_STOP;
 
- /* Shut compiler */
- loc_id=loc_id;
- op_data=op_data;
-
- /* Define a positive value for return value if the attribute was found. This will
-  * cause the iterator to immediately return that positive value,
-  * indicating short-circuit success
-  */
-
- if( strcmp( name, "PALETTE" ) == 0 )
-  ret = 1;
-
-
- return ret;
+    return ret;
 }
 
 
@@ -205,28 +192,21 @@ static herr_t find_palette( hid_t loc_id, const char *name, void  *op_data )
  *
  * Return: Success: 1, Failure: 0
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: May 11, 2001
  *
  * Comments:
- *  The function uses H5Aiterate with the operator function find_palette
+ *  The function uses H5Aiterate2 with the operator function find_palette
  *
  * Modifications:
  *
  *-------------------------------------------------------------------------
  */
 
-static herr_t H5IM_find_palette( hid_t loc_id )
+herr_t H5IM_find_palette( hid_t loc_id )
 {
-
- unsigned int attr_num;     /* Starting attribute to look up */
- herr_t       ret;
-
- attr_num = 0;
- ret = H5Aiterate( loc_id, &attr_num, find_palette, 0 );
-
- return ret;
+    return H5Aiterate2(loc_id, H5_INDEX_NAME, H5_ITER_INC, NULL, find_palette, NULL);
 }
 
 
@@ -238,7 +218,7 @@ static herr_t H5IM_find_palette( hid_t loc_id )
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: July 25, 2001
  *
@@ -252,15 +232,15 @@ static herr_t H5IM_find_palette( hid_t loc_id )
  */
 
 herr_t H5IMget_image_info( hid_t loc_id,
-                     const char *dset_name,
-                     hsize_t *width,
-                     hsize_t *height,
-                     hsize_t *planes,
-                     char *interlace,
-                     hssize_t *npals )
+                           const char *dset_name,
+                           hsize_t *width,
+                           hsize_t *height,
+                           hsize_t *planes,
+                           char *interlace,
+                           hssize_t *npals )
 {
  hid_t   did, sid;
- hsize_t dims[3];
+ hsize_t dims[IMAGE24_RANK];
  hid_t   attr_id;
  hid_t   attr_type;
  int     has_attr;
@@ -272,38 +252,38 @@ herr_t H5IMget_image_info( hid_t loc_id,
  *npals = 0;
 
  /* Open the dataset. */
- if ( (did = H5Dopen( loc_id, dset_name )) < 0 )
+ if((did = H5Dopen2(loc_id, dset_name, H5P_DEFAULT)) < 0)
   return -1;
 
  /* Try to find the attribute "INTERLACE_MODE" on the >>image<< dataset */
- has_attr = H5LT_find_attribute( did, "INTERLACE_MODE" );
+ has_attr = H5LT_find_attribute(did, "INTERLACE_MODE");
 
  /* It exists, get it */
- if ( has_attr == 1 )
+ if(has_attr == 1)
  {
 
-  if ( (attr_id = H5Aopen_name( did, "INTERLACE_MODE" )) < 0 )
+  if((attr_id = H5Aopen(did, "INTERLACE_MODE", H5P_DEFAULT)) < 0)
    goto out;
 
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
 
-  if ( H5Aread( attr_id, attr_type, interlace ) < 0 )
+  if(H5Aread(attr_id, attr_type, interlace) < 0)
    goto out;
 
-  if ( H5Tclose( attr_type )  < 0 )
+  if(H5Tclose(attr_type) < 0)
    goto out;
 
-  if ( H5Aclose( attr_id )  < 0 )
+  if(H5Aclose(attr_id) < 0)
    goto out;
  }
 
  /* Get the dataspace handle */
- if ( (sid = H5Dget_space( did )) < 0 )
+ if ( (sid = H5Dget_space( did )) < 0)
   goto out;
 
  /* Get dimensions */
- if ( H5Sget_simple_extent_dims( sid, dims, NULL) < 0 )
+ if ( H5Sget_simple_extent_dims( sid, dims, NULL) < 0)
   goto out;
 
  /* Initialize the image dimensions */
@@ -334,11 +314,11 @@ herr_t H5IMget_image_info( hid_t loc_id,
  {
   *height = dims[0];
   *width  = dims[1];
-  *planes = dims[2];
+  *planes = 1;
  }
 
  /* Close */
- if ( H5Sclose( sid ) < 0 )
+ if ( H5Sclose( sid ) < 0)
   goto out;
 
 
@@ -346,48 +326,48 @@ herr_t H5IMget_image_info( hid_t loc_id,
 
 
  /* Try to find the attribute "PALETTE" on the >>image<< dataset */
- has_pal = H5IM_find_palette( did );
+ has_pal = H5IM_find_palette(did);
 
- if ( has_pal ==  1 )
+ if(has_pal ==  1)
  {
 
-  if ( (attr_id = H5Aopen_name( did, "PALETTE" )) < 0 )
+  if((attr_id = H5Aopen(did, "PALETTE", H5P_DEFAULT)) < 0)
    goto out;
 
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
 
-  if ( (attr_class = H5Tget_class( attr_type )) < 0 )
+  if((attr_class = H5Tget_class(attr_type)) < 0)
    goto out;
 
   /* Check if it is really a reference */
 
-  if ( attr_class == H5T_REFERENCE )
+  if(attr_class == H5T_REFERENCE)
   {
 
    /* Get the reference(s) */
 
-   if ( (attr_space_id = H5Aget_space( attr_id )) < 0 )
+   if ( (attr_space_id = H5Aget_space( attr_id )) < 0)
     goto out;
 
    *npals = H5Sget_simple_extent_npoints( attr_space_id );
 
-   if ( H5Sclose( attr_space_id ) < 0 )
+   if ( H5Sclose( attr_space_id ) < 0)
     goto out;
 
   } /* H5T_REFERENCE */
 
-  if ( H5Tclose( attr_type ) < 0 )
+  if ( H5Tclose( attr_type ) < 0)
    goto out;
 
   /* Close the attribute. */
-  if ( H5Aclose( attr_id ) < 0 )
+  if ( H5Aclose( attr_id ) < 0)
    goto out;
 
  }
 
  /* End access to the dataset and release resources used by it. */
- if ( H5Dclose( did ) < 0 )
+ if ( H5Dclose( did ) < 0)
   goto out;
 
  return 0;
@@ -406,7 +386,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: June 13, 2001
  *
@@ -426,11 +406,11 @@ herr_t H5IMread_image( hid_t loc_id,
  hid_t   did;
 
  /* Open the dataset. */
- if ( (did = H5Dopen( loc_id, dset_name )) < 0 )
+ if((did = H5Dopen2(loc_id, dset_name, H5P_DEFAULT)) < 0)
   return -1;
 
  /* Read */
- if ( H5Dread( did, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, buffer ) < 0 )
+ if ( H5Dread( did, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, buffer ) < 0)
   goto out;
 
  /* End access to the dataset and release resources used by it. */
@@ -453,7 +433,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: May 01, 2001
  *
@@ -487,11 +467,11 @@ herr_t H5IMmake_palette( hid_t loc_id,
   return -1;
 
  /* Attach the attribute "CLASS" to the >>palette<< dataset*/
- if ( H5LTset_attribute_string( loc_id, pal_name, "CLASS", "PALETTE" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, pal_name, "CLASS", PALETTE_CLASS ) < 0)
   return -1;
 
  /* Attach the attribute "PAL_VERSION" to the >>palette<< dataset*/
- if ( H5LTset_attribute_string( loc_id, pal_name, "PAL_VERSION", "1.2" ) < 0 )
+ if ( H5LTset_attribute_string( loc_id, pal_name, "PAL_VERSION", "1.2" ) < 0)
   return -1;
 
  return 0;
@@ -506,7 +486,7 @@ herr_t H5IMmake_palette( hid_t loc_id,
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: May 01, 2001
  *
@@ -529,10 +509,10 @@ herr_t H5IMlink_palette( hid_t loc_id,
 
 {
  hid_t       image_id;
- hid_t       attr_type;
- hid_t       attr_id;
- hid_t       attr_space_id;
- hid_t       attr_class;
+ hid_t       attr_type=-1;
+ hid_t       attr_id=-1;
+ hid_t       attr_space_id=-1;
+ hid_t       attr_class=-1;
  hobj_ref_t  ref;         /* write a new reference */
  hobj_ref_t  *refbuf;     /* buffer to read references */
  hssize_t    n_refs;
@@ -546,7 +526,7 @@ herr_t H5IMlink_palette( hid_t loc_id,
   */
 
  /* First we get the image id */
- if ( (image_id = H5Dopen( loc_id, image_name )) < 0 )
+ if((image_id = H5Dopen2(loc_id, image_name, H5P_DEFAULT)) < 0)
   return -1;
 
  /* Try to find the attribute "PALETTE" on the >>image<< dataset */
@@ -556,33 +536,33 @@ herr_t H5IMlink_palette( hid_t loc_id,
  * It does not exist. We create the attribute and one reference
  *-------------------------------------------------------------------------
  */
- if ( ok_pal == 0 )
+ if(ok_pal == 0 )
  {
-  if ( (attr_space_id = H5Screate( H5S_SCALAR )) < 0 )
+  if((attr_space_id = H5Screate(H5S_SCALAR)) < 0)
    goto out;
 
   /* Create the attribute type for the reference */
-  if ( (attr_type = H5Tcopy( H5T_STD_REF_OBJ )) < 0 )
+  if((attr_type = H5Tcopy(H5T_STD_REF_OBJ)) < 0)
    goto out;
 
   /* Create the attribute "PALETTE" to be attached to the image*/
-  if ( (attr_id = H5Acreate( image_id, "PALETTE", attr_type, attr_space_id, H5P_DEFAULT )) < 0 )
+  if((attr_id = H5Acreate2(image_id, "PALETTE", attr_type, attr_space_id, H5P_DEFAULT, H5P_DEFAULT)) < 0)
    goto out;
 
   /* Create a reference. The reference is created on the local id.  */
-  if ( H5Rcreate( &ref, loc_id, pal_name, H5R_OBJECT, -1 ) < 0 )
+  if(H5Rcreate(&ref, loc_id, pal_name, H5R_OBJECT, -1) < 0)
    goto out;
 
   /* Write the attribute with the reference */
-  if ( H5Awrite( attr_id, attr_type, &ref ) < 0 )
+  if(H5Awrite(attr_id, attr_type, &ref) < 0)
    goto out;
 
   /* close */
-  if ( H5Sclose( attr_space_id ) < 0 )
+  if(H5Sclose(attr_space_id) < 0)
    goto out;
-  if ( H5Tclose( attr_type ) < 0 )
+  if ( H5Tclose( attr_type ) < 0)
    goto out;
-  if ( H5Aclose( attr_id ) < 0 )
+  if(H5Aclose(attr_id) < 0)
    goto out;
 
  }
@@ -591,63 +571,62 @@ herr_t H5IMlink_palette( hid_t loc_id,
  * The attribute already exists, open it
  *-------------------------------------------------------------------------
  */
- else if ( ok_pal ==  1 )
-  
+ else if(ok_pal ==  1)
  {
-  if ( (attr_id = H5Aopen_name( image_id, "PALETTE" )) < 0 )
+  if((attr_id = H5Aopen(image_id, "PALETTE", H5P_DEFAULT)) < 0)
    goto out;
   
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
   
-  if ( (attr_class = H5Tget_class( attr_type )) < 0 )
+  if((attr_class = H5Tget_class(attr_type)) < 0)
    goto out;
   
   /* Get and save the old reference(s) */
-  if ( (attr_space_id = H5Aget_space( attr_id )) < 0 )
+  if((attr_space_id = H5Aget_space(attr_id)) < 0)
    goto out;
   
-  n_refs = H5Sget_simple_extent_npoints( attr_space_id );
+  n_refs = H5Sget_simple_extent_npoints(attr_space_id);
   
   dim_ref = n_refs + 1;
   
   refbuf = malloc( sizeof(hobj_ref_t) * (int)dim_ref );
   
-  if ( H5Aread( attr_id, attr_type, refbuf ) < 0 )
+  if ( H5Aread( attr_id, attr_type, refbuf ) < 0)
    goto out;
  
   /* The attribute must be deleted, in order to the new one can reflect the changes*/
-  if ( H5Adelete( image_id, "PALETTE" ) < 0 )
+  if(H5Adelete(image_id, "PALETTE") < 0)
    goto out;
   
   /* Create a new reference for this palette. */
-  if ( H5Rcreate( &ref, loc_id, pal_name, H5R_OBJECT, -1 ) < 0 )
+  if ( H5Rcreate( &ref, loc_id, pal_name, H5R_OBJECT, -1 ) < 0)
    goto out;
   
   refbuf[n_refs] = ref;
   
   /* Create the data space for the new references */
-  if ( H5Sclose( attr_space_id ) < 0 )
+  if(H5Sclose(attr_space_id) < 0)
    goto out;
   
-  if ( (attr_space_id = H5Screate_simple( 1, &dim_ref, NULL )) < 0 )
+  if((attr_space_id = H5Screate_simple(1, &dim_ref, NULL)) < 0)
    goto out;
   
   /* Create the attribute again with the changes of space */
-  if ( H5Aclose( attr_id ) < 0 )
+  if(H5Aclose(attr_id) < 0)
    goto out;
 
-  if ( (attr_id = H5Acreate( image_id, "PALETTE", attr_type, attr_space_id, H5P_DEFAULT )) < 0 )
+  if((attr_id = H5Acreate2(image_id, "PALETTE", attr_type, attr_space_id, H5P_DEFAULT, H5P_DEFAULT)) < 0)
    goto out;
   
   /* Write the attribute with the new references */
-  if ( H5Awrite( attr_id, attr_type, refbuf ) < 0 )
+  if(H5Awrite(attr_id, attr_type, refbuf) < 0)
    goto out;
   
   /* close */
-  if ( H5Sclose( attr_space_id ) < 0 )
+  if(H5Sclose(attr_space_id) < 0)
    goto out;
-  if ( H5Aclose( attr_id ) < 0 )
+  if(H5Aclose(attr_id) < 0)
    goto out;
   
   free( refbuf );
@@ -655,7 +634,7 @@ herr_t H5IMlink_palette( hid_t loc_id,
  } /* ok_pal ==  1 */
 
   /* Close the image dataset. */
- if ( H5Dclose( image_id ) < 0 )
+ if ( H5Dclose( image_id ) < 0)
   return -1;
 
  return 0;
@@ -669,6 +648,7 @@ out:
 }
 
 
+
 /*-------------------------------------------------------------------------
  * Function: H5IMunlink_palette
  *
@@ -676,7 +656,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: September 10, 2001
  *
@@ -713,49 +693,48 @@ herr_t H5IMunlink_palette( hid_t loc_id,
   */
 
  /* First we get the image id */
- if ( (image_id = H5Dopen( loc_id, image_name )) < 0 )
+ if((image_id = H5Dopen2(loc_id, image_name, H5P_DEFAULT)) < 0)
   return -1;
 
  /* Try to find the attribute "PALETTE" on the >>image<< dataset */
- ok_pal = H5LT_find_attribute( image_id, "PALETTE" );
+ ok_pal = H5LT_find_attribute(image_id, "PALETTE");
 
  /* It does not exist. Nothing to do */
- if ( ok_pal == 0 )
+ if(ok_pal == 0)
   return -1;
 
  /* The attribute exists, open it */
- else if ( ok_pal ==  1 )
+ else if(ok_pal ==  1)
  {
-  if ( (attr_id = H5Aopen_name( image_id, "PALETTE" )) < 0 )
+  if((attr_id = H5Aopen(image_id, "PALETTE", H5P_DEFAULT)) < 0)
    goto out;
 
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
 
-  if ( (attr_class = H5Tget_class( attr_type )) < 0 )
+  if((attr_class = H5Tget_class(attr_type)) < 0)
    goto out;
 
   /* Check if it is really a reference */
-  if ( attr_class == H5T_REFERENCE )
+  if(attr_class == H5T_REFERENCE)
   {
-
-   /* Deelete the attribute */
-   if ( H5Adelete( image_id, "PALETTE" ) < 0 )
+   /* Delete the attribute */
+   if(H5Adelete(image_id, "PALETTE") < 0)
     goto out;
 
   }  /* H5T_REFERENCE */
 
-  if ( H5Tclose( attr_type ) < 0 )
+  if(H5Tclose(attr_type) < 0)
    goto out;
 
   /* Close the attribute. */
-  if ( H5Aclose( attr_id ) < 0 )
+  if(H5Aclose(attr_id) < 0)
    goto out;
 
  } /* ok_pal */
 
   /* Close the image dataset. */
- if ( H5Dclose( image_id ) < 0 )
+ if(H5Dclose(image_id) < 0)
   return -1;
 
  return 0;
@@ -773,7 +752,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: July 22, 2001
  *
@@ -799,49 +778,49 @@ herr_t H5IMget_npalettes( hid_t loc_id,
  *npals = 0;
 
  /* Open the dataset. */
- if ( (image_id = H5Dopen( loc_id, image_name )) < 0 )
+ if((image_id = H5Dopen2(loc_id, image_name, H5P_DEFAULT)) < 0)
   return -1;
 
  /* Try to find the attribute "PALETTE" on the >>image<< dataset */
- has_pal = H5IM_find_palette( image_id );
+ has_pal = H5IM_find_palette(image_id);
 
- if ( has_pal ==  1 )
+ if(has_pal ==  1 )
  {
 
-  if ( (attr_id = H5Aopen_name( image_id, "PALETTE" )) < 0 )
+  if((attr_id = H5Aopen(image_id, "PALETTE", H5P_DEFAULT)) < 0)
    goto out;
 
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
 
-  if ( (attr_class = H5Tget_class( attr_type )) < 0 )
+  if((attr_class = H5Tget_class(attr_type)) < 0)
    goto out;
 
   /* Check if it is really a reference */
 
-  if ( attr_class == H5T_REFERENCE )
+  if(attr_class == H5T_REFERENCE)
   {
-   if ( (attr_space_id = H5Aget_space( attr_id )) < 0 )
+   if((attr_space_id = H5Aget_space(attr_id)) < 0)
     goto out;
 
    *npals = H5Sget_simple_extent_npoints( attr_space_id );
 
-   if ( H5Sclose( attr_space_id ) < 0 )
+   if ( H5Sclose( attr_space_id ) < 0)
     goto out;
 
   } /* H5T_REFERENCE */
 
-  if ( H5Tclose( attr_type ) < 0 )
+  if ( H5Tclose( attr_type ) < 0)
    goto out;
 
   /* Close the attribute. */
-  if ( H5Aclose( attr_id ) < 0 )
+  if ( H5Aclose( attr_id ) < 0)
    goto out;
 
  }
 
      /* Close the image dataset. */
- if ( H5Dclose( image_id ) < 0 )
+ if ( H5Dclose( image_id ) < 0)
   return -1;
 
  return 0;
@@ -860,7 +839,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: July 22, 2001
  *
@@ -880,9 +859,9 @@ herr_t H5IMget_palette_info( hid_t loc_id,
 {
  hid_t      image_id;
  int        has_pal;
- hid_t      attr_type;
+ hid_t      attr_type=-1;
  hid_t      attr_id;
- hid_t      attr_space_id;
+ hid_t      attr_space_id=-1;
  hid_t      attr_class;
  hssize_t   n_refs;
  hsize_t    dim_ref;
@@ -892,59 +871,59 @@ herr_t H5IMget_palette_info( hid_t loc_id,
  hsize_t    pal_maxdims[2];
  
  /* Open the dataset. */
- if ( (image_id = H5Dopen( loc_id, image_name )) < 0 )
+ if((image_id = H5Dopen2(loc_id, image_name, H5P_DEFAULT)) < 0)
   return -1;
  
  /* Try to find the attribute "PALETTE" on the >>image<< dataset */
- has_pal = H5IM_find_palette( image_id );
+ has_pal = H5IM_find_palette(image_id);
  
- if ( has_pal ==  1 )
+ if(has_pal ==  1)
  {
-  if ( (attr_id = H5Aopen_name( image_id, "PALETTE" )) < 0 )
+  if((attr_id = H5Aopen(image_id, "PALETTE", H5P_DEFAULT)) < 0)
    goto out;
   
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
   
-  if ( (attr_class = H5Tget_class( attr_type )) < 0 )
+  if((attr_class = H5Tget_class(attr_type)) < 0)
    goto out;
   
   /* Get the reference(s) */
-  if ( (attr_space_id = H5Aget_space( attr_id )) < 0 )
+  if((attr_space_id = H5Aget_space(attr_id)) < 0)
    goto out;
   
-  n_refs = H5Sget_simple_extent_npoints( attr_space_id );
+  n_refs = H5Sget_simple_extent_npoints(attr_space_id);
   
   dim_ref = n_refs;
   
   refbuf = malloc( sizeof(hobj_ref_t) * (int)dim_ref );
   
-  if ( H5Aread( attr_id, attr_type, refbuf ) < 0 )
+  if ( H5Aread( attr_id, attr_type, refbuf ) < 0)
    goto out;
   
   /* Get the actual palette */
-  if ( (pal_id = H5Rdereference( image_id, H5R_OBJECT, &refbuf[pal_number] )) < 0 )
+  if ( (pal_id = H5Rdereference( image_id, H5R_OBJECT, &refbuf[pal_number] )) < 0)
    goto out;
   
-  if ( (pal_space_id = H5Dget_space( pal_id )) < 0 )
+  if ( (pal_space_id = H5Dget_space( pal_id )) < 0)
    goto out;
   
-  if ( H5Sget_simple_extent_ndims( pal_space_id ) < 0 )
+  if ( H5Sget_simple_extent_ndims( pal_space_id ) < 0)
    goto out;
   
-  if ( H5Sget_simple_extent_dims( pal_space_id, pal_dims, pal_maxdims ) < 0 )
+  if ( H5Sget_simple_extent_dims( pal_space_id, pal_dims, pal_maxdims ) < 0)
    goto out;
   
   /* close */
   if (H5Dclose(pal_id)<0)
    goto out;
-  if ( H5Sclose( pal_space_id ) < 0 )
+  if ( H5Sclose( pal_space_id ) < 0)
    goto out;
-  if ( H5Sclose( attr_space_id ) < 0 )
+  if ( H5Sclose( attr_space_id ) < 0)
    goto out;
-  if ( H5Tclose( attr_type ) < 0 )
+  if ( H5Tclose( attr_type ) < 0)
    goto out;
-  if ( H5Aclose( attr_id ) < 0 )
+  if ( H5Aclose( attr_id ) < 0)
    goto out;
   free( refbuf );
   
@@ -952,7 +931,7 @@ herr_t H5IMget_palette_info( hid_t loc_id,
  }
  
  /* Close the image dataset. */
- if ( H5Dclose( image_id ) < 0 )
+ if ( H5Dclose( image_id ) < 0)
   return -1;
  
  return 0;
@@ -966,6 +945,7 @@ out:
  
 }
 
+
 /*-------------------------------------------------------------------------
  * Function: H5IMget_palette
  *
@@ -973,7 +953,7 @@ out:
  *
  * Return: Success: 0, Failure: -1
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: August 30, 2001
  *
@@ -993,9 +973,9 @@ herr_t H5IMget_palette( hid_t loc_id,
 {
  hid_t      image_id;
  int        has_pal;
- hid_t      attr_type;
+ hid_t      attr_type=-1;
  hid_t      attr_id;
- hid_t      attr_space_id;
+ hid_t      attr_space_id=-1;
  hid_t      attr_class;
  hssize_t   n_refs;
  hsize_t    dim_ref;
@@ -1003,58 +983,58 @@ herr_t H5IMget_palette( hid_t loc_id,
  hid_t      pal_id;
 
  /* Open the dataset. */
- if ( (image_id = H5Dopen( loc_id, image_name )) < 0 )
+ if((image_id = H5Dopen2(loc_id, image_name, H5P_DEFAULT)) < 0)
   return -1;
 
  /* Try to find the attribute "PALETTE" on the >>image<< dataset */
- has_pal = H5IM_find_palette( image_id );
+ has_pal = H5IM_find_palette(image_id);
 
- if ( has_pal ==  1 )
+ if(has_pal ==  1 )
  {
-  if ( (attr_id = H5Aopen_name( image_id, "PALETTE" )) < 0 )
+  if((attr_id = H5Aopen(image_id, "PALETTE", H5P_DEFAULT)) < 0)
    goto out;
   
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
   
-  if ( (attr_class = H5Tget_class( attr_type )) < 0 )
+  if((attr_class = H5Tget_class(attr_type)) < 0)
    goto out;
   
   /* Get the reference(s) */
-  if ( (attr_space_id = H5Aget_space( attr_id )) < 0 )
+  if((attr_space_id = H5Aget_space(attr_id)) < 0)
    goto out;
   
-  n_refs = H5Sget_simple_extent_npoints( attr_space_id );
+  n_refs = H5Sget_simple_extent_npoints(attr_space_id);
   
   dim_ref = n_refs;
   
   refbuf = malloc( sizeof(hobj_ref_t) * (int)dim_ref );
   
-  if ( H5Aread( attr_id, attr_type, refbuf ) < 0 )
+  if ( H5Aread( attr_id, attr_type, refbuf ) < 0)
    goto out;
   
   /* Get the palette id */
-  if ( (pal_id = H5Rdereference( image_id, H5R_OBJECT, &refbuf[pal_number] )) < 0 )
+  if ( (pal_id = H5Rdereference( image_id, H5R_OBJECT, &refbuf[pal_number] )) < 0)
    goto out;
   
   /* Read the palette dataset */
-  if ( H5Dread( pal_id, H5Dget_type(pal_id), H5S_ALL, H5S_ALL, H5P_DEFAULT, pal_data ) < 0 )
+  if ( H5Dread( pal_id, H5Dget_type(pal_id), H5S_ALL, H5S_ALL, H5P_DEFAULT, pal_data ) < 0)
    goto out;
 
   /* close */
   if (H5Dclose(pal_id)<0)
    goto out;
-  if ( H5Sclose( attr_space_id ) < 0 )
+  if ( H5Sclose( attr_space_id ) < 0)
    goto out;
-  if ( H5Tclose( attr_type ) < 0 )
+  if ( H5Tclose( attr_type ) < 0)
    goto out;
-  if ( H5Aclose( attr_id ) < 0 )
+  if ( H5Aclose( attr_id ) < 0)
    goto out;
   free( refbuf );
  }
 
  /* Close the image dataset. */
- if ( H5Dclose( image_id ) < 0 )
+ if ( H5Dclose( image_id ) < 0)
   return -1;
 
  return 0;
@@ -1075,7 +1055,7 @@ out:
  *
  * Return: true, false, fail
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: August 30, 2001
  *
@@ -1102,48 +1082,47 @@ herr_t H5IMis_image( hid_t loc_id,
  ret = -1;
 
  /* Open the dataset. */
- if ( (did = H5Dopen( loc_id, dset_name )) < 0 )
+ if((did = H5Dopen2(loc_id, dset_name, H5P_DEFAULT)) < 0)
   return -1;
 
  /* Try to find the attribute "CLASS" on the dataset */
- has_class = H5LT_find_attribute( did, "CLASS" );
+ has_class = H5LT_find_attribute(did, "CLASS");
 
- if ( has_class ==  0 )
+ if(has_class == 0)
  {
-  H5Dclose( did );
+  H5Dclose(did);
   return 0;
  }
-
- else if ( has_class ==  1 )
+ else if(has_class ==  1)
  {
 
-  if ( (attr_id = H5Aopen_name( did, "CLASS" )) < 0 )
+  if((attr_id = H5Aopen(did, "CLASS", H5P_DEFAULT)) < 0)
    goto out;
 
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
 
-  if ( H5Tget_class( attr_type ) < 0 )
+  if(H5Tget_class(attr_type) < 0)
    goto out;
 
-  if ( H5Aread( attr_id, attr_type, attr_data ) < 0 )
+  if(H5Aread(attr_id, attr_type, attr_data) < 0)
     goto out;
 
-  if( strcmp( attr_data, "IMAGE" ) == 0 )
+  if(strcmp(attr_data, IMAGE_CLASS) == 0)
    ret = 1;
   else
    ret = 0;
 
-  if ( H5Tclose( attr_type ) < 0 )
+  if ( H5Tclose( attr_type ) < 0)
    goto out;
 
-  if ( H5Aclose( attr_id ) < 0 )
+  if ( H5Aclose( attr_id ) < 0)
    goto out;
 
  }
 
  /* Close the dataset. */
- if ( H5Dclose( did ) < 0 )
+ if ( H5Dclose( did ) < 0)
   return -1;
 
  return ret;
@@ -1154,8 +1133,6 @@ out:
 
 }
 
-
-
 /*-------------------------------------------------------------------------
  * Function: H5IMis_palette
  *
@@ -1163,7 +1140,7 @@ out:
  *
  * Return: true, false, fail
  *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
+ * Programmer: Pedro Vicente Nunes, pvn@ncsa.uiuc.edu
  *
  * Date: August 30, 2001
  *
@@ -1190,48 +1167,47 @@ herr_t H5IMis_palette( hid_t loc_id,
  ret = -1;
 
  /* Open the dataset. */
- if ( (did = H5Dopen( loc_id, dset_name )) < 0 )
+ if((did = H5Dopen2(loc_id, dset_name, H5P_DEFAULT)) < 0)
   return -1;
 
-    /* Try to find the attribute "CLASS" on the dataset */
- has_class = H5LT_find_attribute( did, "CLASS" );
+ /* Try to find the attribute "CLASS" on the dataset */
+ has_class = H5LT_find_attribute(did, "CLASS");
 
- if ( has_class ==  0 )
+ if(has_class ==  0)
  {
   H5Dclose( did );
   return 0;
  }
-
- else if ( has_class ==  1 )
+ else if(has_class ==  1)
  {
 
-  if ( (attr_id = H5Aopen_name( did, "CLASS" )) < 0 )
+  if((attr_id = H5Aopen(did, "CLASS", H5P_DEFAULT)) < 0)
    goto out;
 
-  if ( (attr_type = H5Aget_type( attr_id )) < 0 )
+  if((attr_type = H5Aget_type(attr_id)) < 0)
    goto out;
 
-  if ( H5Tget_class( attr_type ) < 0 )
+  if(H5Tget_class(attr_type) < 0)
    goto out;
 
-  if ( H5Aread( attr_id, attr_type, attr_data ) < 0 )
+  if(H5Aread(attr_id, attr_type, attr_data) < 0)
     goto out;
 
-  if( strcmp( attr_data, "PALETTE" ) == 0 )
+  if(strcmp(attr_data, PALETTE_CLASS) == 0)
    ret = 1;
   else
    ret = 0;
 
-  if ( H5Tclose( attr_type ) < 0 )
+  if ( H5Tclose( attr_type ) < 0)
    goto out;
 
-  if ( H5Aclose( attr_id ) < 0 )
+  if ( H5Aclose( attr_id ) < 0)
    goto out;
 
  }
 
-     /* Close the dataset. */
- if ( H5Dclose( did ) < 0 )
+ /* Close the dataset. */
+ if ( H5Dclose( did ) < 0)
   return -1;
 
  return ret;
@@ -1241,3 +1217,4 @@ out:
  return -1;
 
 }
+
