@@ -14,13 +14,13 @@
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #include <stdlib.h>
-
-
-#include "H5LT.h"
-
+#include <string.h>
+#include "h5hltest.h"
+#include "H5LTpublic.h"
 
 #define FILE_NAME "test_lite1.h5"
 #define FILE_NAME2 "test_lite2.h5"
+#define INPUT_FILE "dtype_file.txt"
 
 #define DSET0_NAME "2D int array"
 #define DSET1_NAME "dataset char"
@@ -45,6 +45,9 @@
 #define ATTR10_NAME "attr float"
 #define ATTR11_NAME "attr double"
 
+/*Initial input buffer size for testing H5LTtext_to_dtype()*/
+#define BUF_SIZE 1024
+
 static herr_t make_attributes( hid_t loc_id, const char* obj_name );
 
 
@@ -72,8 +75,8 @@ static int test_dsets( void )
  float   data_float_out[DIM];
  double  data_double_in[DIM]  = {1,2,3,4,5,6};
  double  data_double_out[DIM];
-	char    *data_string_in = "This is a string";
-	char    data_string_out[20];
+ const char    *data_string_in = "This is a string";
+ char    data_string_out[20];
  int     i;
 
 
@@ -93,7 +96,7 @@ static int test_dsets( void )
 
  /* Read dataset using the basic HDF5 API */
 
- if ( ( dataset_id = H5Dopen ( file_id, DSET0_NAME) ) < 0 )
+ if ( ( dataset_id = H5Dopen2(file_id, DSET0_NAME, H5P_DEFAULT) ) < 0 )
   goto out;
 
  if ( H5Dread ( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data_int_out ) < 0 )
@@ -395,87 +398,82 @@ out:
  *-------------------------------------------------------------------------
  */
 
-static int test_attr( void )
+static int test_attr(void)
 {
+    hid_t   file_id;
+    hid_t   dataset_id;
+    hid_t   group_id;
+    hid_t   space_id;
+    hsize_t dims[1] = { 5 };
 
- hid_t   file_id;
- hid_t   dataset_id;
- hid_t   group_id;
- hid_t   space_id;
- hsize_t dims[1] = { 5 };
+    /* Create a new file using default properties. */
+    file_id = H5Fcreate(FILE_NAME2, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
 
- /* Create a new file using default properties. */
- file_id = H5Fcreate( FILE_NAME2, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT );
+    /*-------------------------------------------------------------------------
+    * Create a dataset named "dset" on the root group
+    *-------------------------------------------------------------------------
+    */
 
-/*-------------------------------------------------------------------------
- * Create a dataset named "dset" on the root group
- *-------------------------------------------------------------------------
- */
+    /* Create the data space  */
+    if((space_id = H5Screate_simple(1, dims, NULL)) < 0) goto out;
 
- /* Create the data space  */
- if ((space_id = H5Screate_simple( 1, dims, NULL ))<0) goto out;
+    /* Create the dataset */
+    if((dataset_id = H5Dcreate2(file_id , "dset", H5T_NATIVE_INT, space_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) goto out;
 
- /* Create the dataset */
- if ((dataset_id = H5Dcreate( file_id , "dset", H5T_NATIVE_INT, space_id,
-  H5P_DEFAULT ))<0) goto out;
+    /* Close */
+    H5Dclose(dataset_id);
 
- /* Close */
- H5Dclose( dataset_id );
+    /*-------------------------------------------------------------------------
+    * Create a group named "grp" on the root group
+    *-------------------------------------------------------------------------
+    */
 
-/*-------------------------------------------------------------------------
- * Create a group named "grp" on the root group
- *-------------------------------------------------------------------------
- */
+    /* Create a group. */
+    if((group_id = H5Gcreate2(file_id, "grp", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) goto out;
 
- /* Create a group. */
- if ((group_id = H5Gcreate( file_id, "grp", 0 ))<0)
-  goto out;
+    /* Close */
+    H5Gclose(group_id);
 
- /* Close */
- H5Gclose( group_id );
+    /*-------------------------------------------------------------------------
+    *
+    * Create attributes in the root group
+    * Note that we are calling the H5LTset_attribute functions with the name "."
+    *
+    *-------------------------------------------------------------------------
+    */
+    if(make_attributes(file_id, ".") < 0) goto out;
 
-/*-------------------------------------------------------------------------
- *
- * Create attributes in the root group
-	* Note that we are calling the H5LTset_attribute functions with the name "."
- *
- *-------------------------------------------------------------------------
- */
- if (make_attributes( file_id, "." )<0)
-  goto out;
+    /*-------------------------------------------------------------------------
+    *
+    * Create attributes in the dataset "dset"
+    *
+    *-------------------------------------------------------------------------
+    */
+    if(make_attributes(file_id, "dset") < 0) goto out;
 
-/*-------------------------------------------------------------------------
- *
- * Create attributes in the dataset "dset"
- *
- *-------------------------------------------------------------------------
- */
- if (make_attributes( file_id, "dset" )<0)
-  goto out;
+    /*-------------------------------------------------------------------------
+    *
+    * Create attributes in the group "grp"
+    *
+    *-------------------------------------------------------------------------
+    */
+    if(make_attributes(file_id, "grp") < 0) goto out;
 
-/*-------------------------------------------------------------------------
- *
- * Create attributes in the group "grp"
-	*
- *-------------------------------------------------------------------------
- */
- if (make_attributes( file_id, "grp" )<0)
-  goto out;
+    /*-------------------------------------------------------------------------
+    * end
+    *-------------------------------------------------------------------------
+    */
+    /* Close the file. */
+    H5Fclose(file_id);
 
-/*-------------------------------------------------------------------------
- * end
- *-------------------------------------------------------------------------
- */
- /* Close the file. */
- H5Fclose( file_id );
-
- return 0;
+    return 0;
 
 out:
- /* Close the file. */
-	H5Fclose( file_id );
- H5_FAILED();
- return -1;
+    /* Close the file. */
+    H5Fclose(file_id);
+
+    H5_FAILED();
+    return -1;
 }
 
 /*-------------------------------------------------------------------------
@@ -556,7 +554,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_char");
 
  /* Set the attribute */
- if ( H5LTset_attribute_char( loc_id, obj_name, ATTR2_NAME, attr_char_in, 5 ) < 0 )
+ if ( H5LTset_attribute_char( loc_id, obj_name, ATTR2_NAME, attr_char_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -600,7 +598,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_short");
 
  /* Set the attribute */
- if ( H5LTset_attribute_short( loc_id, obj_name, ATTR3_NAME, attr_short_in, 5 ) < 0 )
+ if ( H5LTset_attribute_short( loc_id, obj_name, ATTR3_NAME, attr_short_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -646,7 +644,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_int");
 
  /* Set the attribute */
- if ( H5LTset_attribute_int( loc_id, obj_name, ATTR4_NAME, attr_int_in, 5 ) < 0 )
+ if ( H5LTset_attribute_int( loc_id, obj_name, ATTR4_NAME, attr_int_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -690,7 +688,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_long");
 
  /* Set the attribute */
- if ( H5LTset_attribute_long( loc_id, obj_name, ATTR5_NAME, attr_long_in, 5 ) < 0 )
+ if ( H5LTset_attribute_long( loc_id, obj_name, ATTR5_NAME, attr_long_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -734,7 +732,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_uchar");
 
  /* Set the attribute */
- if ( H5LTset_attribute_uchar( loc_id, obj_name, ATTR6_NAME, attr_uchar_in, 5 ) < 0 )
+ if ( H5LTset_attribute_uchar( loc_id, obj_name, ATTR6_NAME, attr_uchar_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -778,7 +776,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_ushort");
 
  /* Set the attribute */
- if ( H5LTset_attribute_ushort( loc_id, obj_name, ATTR7_NAME, attr_ushort_in, 5 ) < 0 )
+ if ( H5LTset_attribute_ushort( loc_id, obj_name, ATTR7_NAME, attr_ushort_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -824,7 +822,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_uint");
 
  /* Set the attribute */
- if ( H5LTset_attribute_uint( loc_id, obj_name, ATTR8_NAME, attr_uint_in, 5 ) < 0 )
+ if ( H5LTset_attribute_uint( loc_id, obj_name, ATTR8_NAME, attr_uint_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -868,7 +866,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_ulong");
 
  /* Set the attribute */
- if ( H5LTset_attribute_ulong( loc_id, obj_name, ATTR9_NAME, attr_ulong_in, 5 ) < 0 )
+ if ( H5LTset_attribute_ulong( loc_id, obj_name, ATTR9_NAME, attr_ulong_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -913,7 +911,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_float");
 
  /* Set the attribute */
- if ( H5LTset_attribute_float( loc_id, obj_name, ATTR10_NAME, attr_float_in, 5 ) < 0 )
+ if ( H5LTset_attribute_float( loc_id, obj_name, ATTR10_NAME, attr_float_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -958,7 +956,7 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  TESTING("H5LTset_attribute_double");
 
  /* Set the attribute */
- if ( H5LTset_attribute_double( loc_id, obj_name, ATTR11_NAME, attr_double_in, 5 ) < 0 )
+ if ( H5LTset_attribute_double( loc_id, obj_name, ATTR11_NAME, attr_double_in, (size_t)5 ) < 0 )
   return -1;
 
 	PASSED();
@@ -1044,11 +1042,601 @@ static herr_t make_attributes( hid_t loc_id, const char* obj_name )
  return 0;
 }
 
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_integers().
+ *-------------------------------------------------------------------------
+ */
+static int test_integers(void)
+{
+    hid_t   dtype;
+    char*   dt_str;
+    size_t  str_len;
+
+    TESTING3("\n        text for integer types");
+
+    if((dtype = H5LTtext_to_dtype("H5T_NATIVE_INT\n", H5LT_DDL))<0)
+        goto out;
+    if(!H5Tequal(dtype, H5T_NATIVE_INT))
+        goto out;
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    if((dtype = H5LTtext_to_dtype("H5T_STD_I8BE\n", H5LT_DDL))<0)
+        goto out;
+    if(!H5Tequal(dtype, H5T_STD_I8BE))
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    if(strcmp(dt_str, "H5T_STD_I8BE"))
+        goto out;
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    if((dtype = H5LTtext_to_dtype("H5T_STD_U16LE\n", H5LT_DDL))<0)
+        goto out;
+    if(!H5Tequal(dtype, H5T_STD_U16LE))
+        goto out;
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_fps().
+ *-------------------------------------------------------------------------
+ */
+static int test_fps(void)
+{
+    hid_t   dtype;
+    char*   dt_str;
+    size_t  str_len;
+
+    TESTING3("        text for floating-point types");
+
+    if((dtype = H5LTtext_to_dtype("H5T_NATIVE_LDOUBLE\n", H5LT_DDL))<0)
+        goto out;
+    if(!H5Tequal(dtype, H5T_NATIVE_LDOUBLE))
+        goto out;
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    if((dtype = H5LTtext_to_dtype("H5T_IEEE_F32BE\n", H5LT_DDL))<0)
+        goto out;
+    if(!H5Tequal(dtype, H5T_IEEE_F32BE))
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    if(strcmp(dt_str, "H5T_IEEE_F32BE"))
+        goto out;
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    if((dtype = H5LTtext_to_dtype("H5T_IEEE_F64LE\n", H5LT_DDL))<0)
+        goto out;
+    if(!H5Tequal(dtype, H5T_IEEE_F64LE))
+        goto out;
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_strings().
+ *-------------------------------------------------------------------------
+ */
+static int test_strings(void)
+{
+    hid_t   dtype;
+    size_t  str_size;
+    H5T_str_t   str_pad;
+    H5T_cset_t  str_cset;
+    H5T_class_t type_class;
+    char*   dt_str;
+    size_t  str_len;
+
+    TESTING3("        text for string types");
+
+    if((dtype = H5LTtext_to_dtype("H5T_STRING { STRSIZE 13; STRPAD H5T_STR_NULLTERM; CSET H5T_CSET_ASCII; CTYPE H5T_C_S1; }", H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_STRING)
+        goto out;
+
+    str_size = H5Tget_size(dtype);
+    if(str_size != 13)
+        goto out;
+
+    str_pad = H5Tget_strpad(dtype);
+    if(str_pad != H5T_STR_NULLTERM)
+        goto out;
+
+    str_cset = H5Tget_cset(dtype);
+    if(str_cset != H5T_CSET_ASCII)
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    if(strcmp(dt_str, "H5T_STRING {\n      STRSIZE 13;\n      STRPAD H5T_STR_NULLTERM;\n      CSET H5T_CSET_ASCII;\n      CTYPE H5T_C_S1;\n   }")) {
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    if((dtype = H5LTtext_to_dtype("H5T_STRING { STRSIZE H5T_VARIABLE; STRPAD H5T_STR_NULLPAD; CSET H5T_CSET_ASCII; CTYPE H5T_C_S1; }", H5LT_DDL))<0)
+        goto out;
+
+    if(!H5Tis_variable_str(dtype))
+        goto out;
+
+    str_pad = H5Tget_strpad(dtype);
+    if(str_pad != H5T_STR_NULLPAD)
+        goto out;
+
+    str_cset = H5Tget_cset(dtype);
+    if(str_cset != H5T_CSET_ASCII)
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    if(strcmp(dt_str, "H5T_STRING {\n      STRSIZE H5T_VARIABLE;\n      STRPAD H5T_STR_NULLPAD;\n      CSET H5T_CSET_ASCII;\n      CTYPE H5T_C_S1;\n   }")) {
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_opaques().
+ *-------------------------------------------------------------------------
+ */
+static int test_opaques(void)
+{
+    hid_t   dtype;
+    size_t  opq_size;
+    H5T_class_t type_class;
+    char*   dt_str;
+    size_t  str_len;
+
+    TESTING3("        text for opaque types");
+
+    if((dtype = H5LTtext_to_dtype("H5T_OPAQUE { OPQ_SIZE 19; OPQ_TAG \"This is a tag for opaque type\"; }", H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_OPAQUE)
+        goto out;
+
+    if((opq_size = H5Tget_size(dtype)) == 0)
+        goto out;
+    if(opq_size != 19)
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    if(strcmp(dt_str, "H5T_OPAQUE {\n      OPQ_SIZE 19;\n      OPQ_TAG \"This is a tag for opaque type\";\n   }")) {
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_enums().
+ *-------------------------------------------------------------------------
+ */
+static int test_enums(void)
+{
+    hid_t   dtype;
+    size_t  size = 16;
+    char    name1[16];
+    int     value1 = 7;
+    const char    *name2 = "WHITE";
+    int     value2;
+    H5T_class_t type_class;
+    char*   dt_str;
+    size_t  str_len;
+
+    TESTING3("        text for enum types");
+
+    if((dtype = H5LTtext_to_dtype("H5T_ENUM { H5T_STD_I32LE; \"RED\" 5; \"GREEN\" 6; \"BLUE\" 7; \"WHITE\" 8; }", H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_ENUM)
+        goto out;
+
+    if(H5Tenum_nameof(dtype, &value1, name1, size)<0)
+        goto out;
+    if(strcmp(name1, "BLUE"))
+        goto out;
+
+    if(H5Tenum_valueof(dtype, name2, &value2)<0)
+        goto out;
+    if(value2 != 8)
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    /*if(strcmp(dt_str, "H5T_ENUM {\n      H5T_STD_I32LE;\n      \"RED\"            5;\n      \"GREEN\"          6;\n      \"BLUE\"           7;\n      \"WHITE\"          8;\n   }")) {
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }*/
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_variables().
+ *-------------------------------------------------------------------------
+ */
+static int test_variables(void)
+{
+    hid_t   dtype;
+    H5T_class_t type_class;
+    char*   dt_str;
+    size_t  str_len;
+
+    TESTING3("        text for variable types");
+
+    if((dtype = H5LTtext_to_dtype("H5T_VLEN { H5T_NATIVE_CHAR }\n", H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_VLEN)
+        goto out;
+
+    if(H5Tis_variable_str(dtype))
+        goto out;
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    if((dtype = H5LTtext_to_dtype("H5T_VLEN { H5T_VLEN { H5T_STD_I32BE } }", H5LT_DDL))<0)
+        goto out;
+
+    if(H5Tis_variable_str(dtype))
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    if(strcmp(dt_str, "H5T_VLEN {\n      H5T_VLEN {\n         H5T_STD_I32BE\n      }\n   }")) {
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_arrays().
+ *-------------------------------------------------------------------------
+ */
+static int test_arrays(void)
+{
+    hid_t   dtype;
+    int     ndims;
+    hsize_t dims[3];
+    H5T_class_t type_class;
+    char*   dt_str;
+    size_t  str_len;
+
+    TESTING3("        text for array types");
+
+    if((dtype = H5LTtext_to_dtype("H5T_ARRAY { [5][7][13] H5T_ARRAY { [17][19] H5T_COMPOUND { H5T_STD_I8BE \"arr_compound_1\"; H5T_STD_I32BE \"arr_compound_2\"; } } }", H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_ARRAY)
+        goto out;
+
+    if((ndims = H5Tget_array_ndims(dtype))<0)
+        goto out;
+    if(ndims != 3)
+        goto out;
+
+    if(H5Tget_array_dims2(dtype, dims) < 0)
+        goto out;
+    if(dims[0] != 5 || dims[1] != 7 || dims[2] != 13)
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    /*if(strcmp(dt_str, "H5T_ARRAY { [5][7][13] H5T_ARRAY { [17][19] H5T_COMPOUND { H5T_STD_I8BE \"arr_compound_1\"; H5T_STD_I32BE \"arr_compound_2\"; } } }")) {
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }*/
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_compounds().
+ *-------------------------------------------------------------------------
+ */
+static int test_compounds(void)
+{
+    hid_t   dtype;
+    int     nmembs;
+    char    *memb_name = NULL;
+    H5T_class_t memb_class;
+    H5T_class_t type_class;
+    char*   dt_str;
+    size_t  str_len;
+
+    TESTING3("        text for compound types");
+
+    if((dtype = H5LTtext_to_dtype("H5T_COMPOUND { H5T_STD_I16BE \"one_field\" : 2; H5T_STD_U8LE \"two_field\" : 6; }", H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_COMPOUND)
+        goto out;
+
+    if((nmembs = H5Tget_nmembers(dtype))<0)
+        goto out;
+    if(nmembs != 2)
+        goto out;
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    if(strcmp(dt_str, "H5T_COMPOUND {\n      H5T_STD_I16BE \"one_field\" : 2;\n      H5T_STD_U8LE \"two_field\" : 6;\n   }")) {
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    if((dtype = H5LTtext_to_dtype("H5T_COMPOUND { H5T_STD_I32BE \"i32_field\"; H5T_STD_I16BE \"i16_field\"; H5T_COMPOUND  { H5T_STD_I16BE \"sec_field\"; H5T_COMPOUND { H5T_STD_I32BE \"thd_field\"; } \"grandchild\"; } \"child_compound\"; H5T_STD_I8BE  \"i8_field\"; }", H5LT_DDL))<0)
+        goto out;
+
+    if((memb_name = H5Tget_member_name(dtype, 1)) == NULL)
+        goto out;
+    if(strcmp(memb_name, "i16_field"))
+        goto out;
+    free(memb_name);
+
+    if((memb_class = H5Tget_member_class(dtype, 2))<0)
+        goto out;
+    if(memb_class != H5T_COMPOUND)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * subroutine for test_text_dtype(): test_complicated_compound().
+ *-------------------------------------------------------------------------
+ */
+static int test_complicated_compound(void)
+{
+    hid_t   dtype;
+    int     nmembs;
+    H5T_class_t type_class;
+    char*   line=NULL;
+    FILE    *fp;
+    size_t  size = 1024;
+    char    *srcdir = getenv("srcdir"); /* the source directory */
+    char    filename[1024]="";
+
+    TESTING3("        text for complicated compound types");
+
+    /* compose the name of the file to open, using the srcdir, if appropriate */
+    if(srcdir)
+    {
+      strcpy(filename, srcdir);
+      strcat(filename, "/");
+    }
+    strcat(filename, INPUT_FILE);
+
+    /* Open input file */
+    fp = fopen(filename, "r");
+    if(fp == NULL)
+    {
+       printf( "Could not find file %s. Try set $srcdir \n", filename);
+       goto out;
+    }
+
+    /* This part reads in the input as a string in a slow manner.  GNU C
+     * Library has convenient function getline() but isn't available on
+     * all machines.
+     */
+    if((line = (char*)calloc(size, sizeof(char)))==NULL)
+        goto out;
+    if(fgets(line, (int)size, fp)==NULL)
+        goto out;
+    while(strlen(line)==size-1) {
+        size *= 2;
+        if(line)
+            free(line);
+        if((line = (char*)calloc(size, sizeof(char)))==NULL)
+            goto out;
+        if(fseek(fp, 0L, SEEK_SET)!=0)
+            goto out;
+        if(fgets(line, (int)size, fp)==NULL)
+            goto out;
+    }
+
+    fclose(fp);
+
+    if((dtype = H5LTtext_to_dtype(line, H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_COMPOUND)
+        goto out;
+
+    /* There should be 101 compound members */
+    if((nmembs = H5Tget_nmembers(dtype))<0)
+        goto out;
+    if(nmembs != 101)
+        goto out;
+
+    if(line)
+        free(line);
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
+/*-------------------------------------------------------------------------
+ * test H5LTtext_to_dtype function
+ *-------------------------------------------------------------------------
+ */
+static int test_text_dtype(void)
+{
+    TESTING("H5LTtext_to_dtype");
+
+    if(test_integers()<0)
+        goto out;
+
+    if(test_fps()<0)
+        goto out;
+
+    if(test_strings()<0)
+        goto out;
+
+    if(test_opaques()<0)
+        goto out;
+
+    if(test_enums()<0)
+        goto out;
+
+    if(test_variables()<0)
+        goto out;
+
+    if(test_arrays()<0)
+        goto out;
+
+    if(test_compounds()<0)
+        goto out;
+
+    if(test_complicated_compound()<0)
+        goto out;
+
+    return 0;
+
+out:
+    return -1;
+}
+
 /*-------------------------------------------------------------------------
  * the main program
  *-------------------------------------------------------------------------
  */
-
 int main( void )
 {
  int  nerrors=0;
@@ -1059,6 +1647,9 @@ int main( void )
  /* test attribute functions */
  nerrors += test_attr();
 
+ /* test text-dtype functions */
+ nerrors += test_text_dtype();
+
  /* check for errors */
  if (nerrors)
   goto error;
