@@ -13,21 +13,15 @@
  * access to either file, you may request a copy from help@hdfgroup.org.     *
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#include "H5TA.h"
-
 #include <stdlib.h>
-#include <assert.h>
+#include <string.h>
+#include "h5hltest.h"
+#include "H5TBpublic.h"
 
 #define TEST_FILE_BE "test_table_be.hdf5"
 #define TEST_FILE_LE "test_table_le.hdf5"
 #define TEST_FILE_CRAY "test_table_cray.hdf5"
 
-/* Compiler optimization on Cray X1s can cause this file not to compile.
- * Turn it off.
- */
-#ifdef __crayx1
-#pragma OPTIMIZE OFF
-#endif
 
 /*-------------------------------------------------------------------------
  * Table API test
@@ -360,7 +354,6 @@ int test_table(hid_t fid, int write)
  PASSED();
  }
 
-
  TESTING2("reading table");
 
 /*-------------------------------------------------------------------------
@@ -1361,7 +1354,6 @@ int test_table(hid_t fid, int write)
  PASSED();
  }
 
-
 /*-------------------------------------------------------------------------
  *
  * Functions tested:
@@ -1380,7 +1372,6 @@ int test_table(hid_t fid, int write)
                     chunk_size,fill,compress,wbuf)<0)
                     goto out;
 
-
  /* delete the field */
  if ( H5TBdelete_field(fid, "table14", "Pressure" ) < 0 )
   goto out;
