@@ -29,7 +29,6 @@
 #include <fcntl.h>
 #include <math.h>
 #include <sys/time.h>
-#include <sys/uio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -43,13 +42,13 @@
 
 #include <zlib.h>
 
-#if defined(MSDOS) || defined(OS2) || defined(WIN32)
+#if defined(MSDOS) || defined(OS2) || defined(_WIN32)
 #  include <fcntl.h>
 #  include <io.h>
 #  define SET_BINARY_MODE(file)     setmode(fileno(file), O_BINARY)
 #else
 #  define SET_BINARY_MODE(file)     /* nothing */
-#endif  /* MSDOS || OS2 || WIN32 */
+#endif  /* MSDOS || OS2 || _WIN32 */
 
 #ifdef VMS
 #  define unlink        delete
