@@ -19,14 +19,53 @@
  * Purpose:	Dataspace I/O functions.
  */
 
+/****************/
+/* Module Setup */
+/****************/
+
 #define H5D_PACKAGE		/*suppress error about including H5Dpkg	  */
 
 
+/***********/
+/* Headers */
+/***********/
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Dpkg.h"		/* Datasets				*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5FLprivate.h"	/* Free Lists                           */
 
+/****************/
+/* Local Macros */
+/****************/
+
+/******************/
+/* Local Typedefs */
+/******************/
+
+/********************/
+/* Local Prototypes */
+/********************/
+
+/* Struct for holding the vectorized I/O operation buffers "loosely" */
+typedef struct {
+    enum {
+        H5S_SELECT_READ,                /* Read selection */
+        H5S_SELECT_WRITE                /* Write selection */
+    } op_type;
+    union {
+        void *rbuf;                          /* Buffer for read */
+        const void *wbuf;                    /* Buffer to write */
+    } u;
+} H5D_select_buf_t;
+
+/*********************/
+/* Package Variables */
+/*********************/
+
+/*******************/
+/* Local Variables */
+/*******************/
+
 /* Declare a free list to manage sequences of size_t */
 H5FL_SEQ_DEFINE_STATIC(size_t);
 
@@ -48,22 +87,20 @@ H5FL_SEQ_DEFINE_STATIC(hsize_t);
  * Programmer:	Quincey Koziol
  *              Thursday, June 20, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 herr_t
 H5D_select_fscat (H5D_io_info_t *io_info,
     const H5S_t *space, H5S_sel_iter_t *iter, size_t nelmts,
-    const void *_buf)
+    haddr_t chunk_addr, void *chunk/*in*/, const void *_buf)
 {
     const uint8_t *buf=_buf;       /* Alias for pointer arithmetic */
-    hsize_t _off[H5D_XFER_HYPER_VECTOR_SIZE_DEF];             /* Array to store sequence offsets */
+    hsize_t _off[H5D_IO_VECTOR_SIZE];             /* Array to store sequence offsets */
     hsize_t *off=NULL;             /* Pointer to sequence offsets */
     hsize_t mem_off;               /* Offset in memory */
     size_t mem_curr_seq;           /* "Current sequence" in memory */
     size_t dset_curr_seq;          /* "Current sequence" in dataset */
-    size_t _len[H5D_XFER_HYPER_VECTOR_SIZE_DEF];              /* Array to store sequence lengths */
+    size_t _len[H5D_IO_VECTOR_SIZE];              /* Array to store sequence lengths */
     size_t *len=NULL;              /* Array to store sequence lengths */
     size_t orig_mem_len, mem_len;  /* Length of sequence in memory */
     size_t  nseq;                  /* Number of sequences generated */
@@ -81,7 +118,7 @@ H5D_select_fscat (H5D_io_info_t *io_info,
     assert(TRUE==H5P_isa_class(io_info->dxpl_id,H5P_DATASET_XFER));
 
     /* Allocate the vector I/O arrays */
-    if(io_info->dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
+    if(io_info->dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
         if((len = H5FL_SEQ_MALLOC(size_t,io_info->dxpl_cache->vec_size))==NULL)
             HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O length vector array");
         if((off = H5FL_SEQ_MALLOC(hsize_t,io_info->dxpl_cache->vec_size))==NULL)
@@ -104,7 +141,7 @@ H5D_select_fscat (H5D_io_info_t *io_info,
         mem_off=0;
 
         /* Write sequence list out */
-        if ((*io_info->ops.writevv)(io_info, nseq, &dset_curr_seq, len, off, 1, &mem_curr_seq, &mem_len, &mem_off, buf)<0)
+        if((*io_info->ops.writevv)(io_info, nseq, &dset_curr_seq, len, off, (size_t)1, &mem_curr_seq, &mem_len, &mem_off, chunk_addr, chunk, buf) < 0)
             HGOTO_ERROR(H5E_DATASPACE, H5E_WRITEERROR, FAIL, "write error");
 
         /* Update buffer */
@@ -115,7 +152,7 @@ H5D_select_fscat (H5D_io_info_t *io_info,
     } /* end while */
 
 done:
-    if(io_info->dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
+    if(io_info->dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
         if(len!=NULL)
             H5FL_SEQ_FREE(size_t,len);
         if(off!=NULL)
@@ -144,22 +181,20 @@ done:
  * Programmer:	Quincey Koziol
  *              Monday, June 24, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 size_t
 H5D_select_fgath (H5D_io_info_t *io_info,
     const H5S_t *space, H5S_sel_iter_t *iter, size_t nelmts,
-    void *_buf/*out*/)
+    haddr_t chunk_addr, void *chunk/*in*/, void *_buf/*out*/)
 {
-    uint8_t *buf=_buf;          /* Alias for pointer arithmetic */
-    hsize_t _off[H5D_XFER_HYPER_VECTOR_SIZE_DEF];          /* Array to store sequence offsets */
+    uint8_t *buf=(uint8_t*)_buf;          /* Alias for pointer arithmetic */
+    hsize_t _off[H5D_IO_VECTOR_SIZE];          /* Array to store sequence offsets */
     hsize_t *off=NULL;          /* Pointer to sequence offsets */
     hsize_t mem_off;            /* Offset in memory */
     size_t mem_curr_seq;        /* "Current sequence" in memory */
     size_t dset_curr_seq;       /* "Current sequence" in dataset */
-    size_t _len[H5D_XFER_HYPER_VECTOR_SIZE_DEF];           /* Array to store sequence lengths */
+    size_t _len[H5D_IO_VECTOR_SIZE];           /* Array to store sequence lengths */
     size_t *len=NULL;           /* Pointer to sequence lengths */
     size_t orig_mem_len, mem_len;       /* Length of sequence in memory */
     size_t nseq;                /* Number of sequences generated */
@@ -178,7 +213,7 @@ H5D_select_fgath (H5D_io_info_t *io_info,
     assert (_buf);
 
     /* Allocate the vector I/O arrays */
-    if(io_info->dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
+    if(io_info->dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
         if((len = H5FL_SEQ_MALLOC(size_t,io_info->dxpl_cache->vec_size))==NULL)
             HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, 0, "can't allocate I/O length vector array");
         if((off = H5FL_SEQ_MALLOC(hsize_t,io_info->dxpl_cache->vec_size))==NULL)
@@ -201,7 +236,8 @@ H5D_select_fgath (H5D_io_info_t *io_info,
         mem_off=0;
 
         /* Read sequence list in */
-        if ((*io_info->ops.readvv)(io_info, nseq, &dset_curr_seq, len, off, 1, &mem_curr_seq, &mem_len, &mem_off, buf)<0)
+        if((*io_info->ops.readvv)(io_info, nseq, &dset_curr_seq, len, off, (size_t)1, 
+                                  &mem_curr_seq, &mem_len, &mem_off, chunk_addr, chunk, buf) < 0)
             HGOTO_ERROR(H5E_DATASPACE, H5E_READERROR, 0, "read error");
 
         /* Update buffer */
@@ -212,7 +248,7 @@ H5D_select_fgath (H5D_io_info_t *io_info,
     } /* end while */
 
 done:
-    if(io_info->dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
+    if(io_info->dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
         if(len!=NULL)
             H5FL_SEQ_FREE(size_t,len);
         if(off!=NULL)
@@ -235,8 +271,6 @@ done:
  * Programmer:	Quincey Koziol
  *              Monday, July 8, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 herr_t
@@ -246,9 +280,9 @@ H5D_select_mscat (const void *_tscat_buf, const H5S_t *space,
 {
     uint8_t *buf=(uint8_t *)_buf;   /* Get local copies for address arithmetic */
     const uint8_t *tscat_buf=(const uint8_t *)_tscat_buf;
-    hsize_t _off[H5D_XFER_HYPER_VECTOR_SIZE_DEF];          /* Array to store sequence offsets */
+    hsize_t _off[H5D_IO_VECTOR_SIZE];          /* Array to store sequence offsets */
     hsize_t *off=NULL;          /* Pointer to sequence offsets */
-    size_t _len[H5D_XFER_HYPER_VECTOR_SIZE_DEF];           /* Array to store sequence lengths */
+    size_t _len[H5D_IO_VECTOR_SIZE];           /* Array to store sequence lengths */
     size_t *len=NULL;           /* Pointer to sequence lengths */
     size_t curr_len;            /* Length of bytes left to process in sequence */
     size_t nseq;                /* Number of sequences generated */
@@ -266,7 +300,7 @@ H5D_select_mscat (const void *_tscat_buf, const H5S_t *space,
     assert (buf);
 
     /* Allocate the vector I/O arrays */
-    if(dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
+    if(dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
         if((len = H5FL_SEQ_MALLOC(size_t,dxpl_cache->vec_size))==NULL)
             HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O length vector array");
         if((off = H5FL_SEQ_MALLOC(hsize_t,dxpl_cache->vec_size))==NULL)
@@ -299,7 +333,7 @@ H5D_select_mscat (const void *_tscat_buf, const H5S_t *space,
     } /* end while */
 
 done:
-    if(dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
+    if(dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
         if(len!=NULL)
             H5FL_SEQ_FREE(size_t,len);
         if(off!=NULL)
@@ -324,8 +358,6 @@ done:
  * Programmer:	Quincey Koziol
  *              Monday, June 24, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 size_t
@@ -335,9 +367,9 @@ H5D_select_mgath (const void *_buf, const H5S_t *space,
 {
     const uint8_t *buf=(const uint8_t *)_buf;   /* Get local copies for address arithmetic */
     uint8_t *tgath_buf=(uint8_t *)_tgath_buf;
-    hsize_t _off[H5D_XFER_HYPER_VECTOR_SIZE_DEF];          /* Array to store sequence offsets */
+    hsize_t _off[H5D_IO_VECTOR_SIZE];          /* Array to store sequence offsets */
     hsize_t *off=NULL;          /* Pointer to sequence offsets */
-    size_t _len[H5D_XFER_HYPER_VECTOR_SIZE_DEF];           /* Array to store sequence lengths */
+    size_t _len[H5D_IO_VECTOR_SIZE];           /* Array to store sequence lengths */
     size_t *len=NULL;           /* Pointer to sequence lengths */
     size_t curr_len;            /* Length of bytes left to process in sequence */
     size_t nseq;                /* Number of sequences generated */
@@ -355,7 +387,7 @@ H5D_select_mgath (const void *_buf, const H5S_t *space,
     assert (tgath_buf);
 
     /* Allocate the vector I/O arrays */
-    if(dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
+    if(dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
         if((len = H5FL_SEQ_MALLOC(size_t,dxpl_cache->vec_size))==NULL)
             HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, 0, "can't allocate I/O length vector array");
         if((off = H5FL_SEQ_MALLOC(hsize_t,dxpl_cache->vec_size))==NULL)
@@ -388,7 +420,7 @@ H5D_select_mgath (const void *_buf, const H5S_t *space,
     } /* end while */
 
 done:
-    if(dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
+    if(dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
         if(len!=NULL)
             H5FL_SEQ_FREE(size_t,len);
         if(off!=NULL)
@@ -397,6 +429,196 @@ done:
     FUNC_LEAVE_NOAPI(ret_value);
 }   /* H5D_select_mgath() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5D_select_io
+ *
+ * Purpose:	Perform I/O directly from application memory and a file
+ *
+ * Return:	Non-negative on success/Negative on failure
+ *
+ * Programmer:	Quincey Koziol
+ *              Tuesday, November 27, 2007
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5D_select_io(H5D_io_info_t *io_info,
+    size_t nelmts, size_t elmt_size,
+    const H5S_t *file_space, const H5S_t *mem_space,
+    haddr_t addr, void *chunk/*in*/,
+    const H5D_select_buf_t *io_buf)
+{
+    H5S_sel_iter_t mem_iter;    /* Memory selection iteration info */
+    hbool_t mem_iter_init = 0;  /* Memory selection iteration info has been initialized */
+    H5S_sel_iter_t file_iter;   /* File selection iteration info */
+    hbool_t file_iter_init = 0;	/* File selection iteration info has been initialized */
+    hsize_t _mem_off[H5D_IO_VECTOR_SIZE];      /* Array to store sequence offsets in memory */
+    hsize_t *mem_off = NULL;    /* Pointer to sequence offsets in memory */
+    hsize_t _file_off[H5D_IO_VECTOR_SIZE];     /* Array to store sequence offsets in the file */
+    hsize_t *file_off = NULL;   /* Pointer to sequence offsets in the file */
+    size_t _mem_len[H5D_IO_VECTOR_SIZE];       /* Array to store sequence lengths in memory */
+    size_t *mem_len = NULL;     /* Pointer to sequence lengths in memory */
+    size_t _file_len[H5D_IO_VECTOR_SIZE];      /* Array to store sequence lengths in the file */
+    size_t *file_len = NULL;    /* Pointer to sequence lengths in the file */
+    size_t curr_mem_seq;        /* Current memory sequence to operate on */
+    size_t curr_file_seq;       /* Current file sequence to operate on */
+    size_t mem_nseq;            /* Number of sequences generated in the file */
+    size_t file_nseq;           /* Number of sequences generated in memory */
+    ssize_t tmp_file_len;       /* Temporary number of bytes in file sequence */
+    herr_t ret_value = SUCCEED; /* Return value */
+
+    FUNC_ENTER_NOAPI(H5D_select_io, FAIL)
+
+    /* Check args */
+    HDassert(io_info);
+    HDassert(io_info->dset);
+    HDassert(io_info->store);
+    HDassert(TRUE == H5P_isa_class(io_info->dxpl_id, H5P_DATASET_XFER));
+    HDassert(io_buf->u.rbuf);
+
+    /* Allocate the vector I/O arrays */
+    if(io_info->dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
+        if(NULL == (mem_len = H5FL_SEQ_MALLOC(size_t,io_info->dxpl_cache->vec_size)))
+            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O length vector array")
+        if(NULL == (mem_off = H5FL_SEQ_MALLOC(hsize_t,io_info->dxpl_cache->vec_size)))
+            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O offset vector array")
+        if(NULL == (file_len = H5FL_SEQ_MALLOC(size_t,io_info->dxpl_cache->vec_size)))
+            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O length vector array")
+        if(NULL == (file_off = H5FL_SEQ_MALLOC(hsize_t,io_info->dxpl_cache->vec_size)))
+            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O offset vector array")
+    } /* end if */
+    else {
+        mem_len = _mem_len;
+        mem_off = _mem_off;
+        file_len = _file_len;
+        file_off = _file_off;
+    } /* end else */
+
+    /* Check for only one element in selection */
+    if(nelmts == 1) {
+        /* Get offset of first element in selections */
+        if(H5S_SELECT_OFFSET(file_space, file_off) < 0)
+            HGOTO_ERROR(H5E_INTERNAL, H5E_UNSUPPORTED, FAIL, "can't retrieve file selection offset")
+        if(H5S_SELECT_OFFSET(mem_space, mem_off) < 0)
+            HGOTO_ERROR(H5E_INTERNAL, H5E_UNSUPPORTED, FAIL, "can't retrieve memory selection offset")
+
+        /* Set up necessary information for I/O operation */
+        file_nseq = mem_nseq = 1;
+        curr_mem_seq = curr_file_seq = 0;
+        *file_off *= elmt_size;
+        *mem_off *= elmt_size;
+        *file_len = *mem_len = elmt_size;
+
+        /* Perform I/O on memory and file sequences */
+        if(io_buf->op_type == H5S_SELECT_READ) {
+            if((tmp_file_len = (*io_info->ops.readvv)(io_info,
+                    file_nseq, &curr_file_seq, file_len, file_off,
+                    mem_nseq, &curr_mem_seq, mem_len, mem_off,
+                    addr, chunk, io_buf->u.rbuf)) < 0)
+                HGOTO_ERROR(H5E_DATASPACE, H5E_READERROR, FAIL, "read error")
+        } /* end if */
+        else {
+            HDassert(io_buf->op_type == H5S_SELECT_WRITE);
+            if((tmp_file_len = (*io_info->ops.writevv)(io_info,
+                    file_nseq, &curr_file_seq, file_len, file_off,
+                    mem_nseq, &curr_mem_seq, mem_len, mem_off,
+                    addr, chunk, io_buf->u.wbuf)) < 0)
+                HGOTO_ERROR(H5E_DATASPACE, H5E_WRITEERROR, FAIL, "write error")
+        } /* end else */
+
+        /* Decrement number of elements left to process */
+        HDassert((tmp_file_len % elmt_size) == 0);
+    } /* end if */
+    else {
+        size_t mem_nelem;           /* Number of elements used in memory sequences */
+        size_t file_nelem;          /* Number of elements used in file sequences */
+
+        /* Initialize file iterator */
+        if(H5S_select_iter_init(&file_iter, file_space, elmt_size) < 0)
+            HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to initialize selection iterator")
+        file_iter_init = 1;	/* File selection iteration info has been initialized */
+
+        /* Initialize memory iterator */
+        if(H5S_select_iter_init(&mem_iter, mem_space, elmt_size) < 0)
+            HGOTO_ERROR(H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to initialize selection iterator")
+        mem_iter_init = 1;	/* Memory selection iteration info has been initialized */
+
+        /* Initialize sequence counts */
+        curr_mem_seq = curr_file_seq = 0;
+        mem_nseq = file_nseq = 0;
+
+        /* Loop, until all bytes are processed */
+        while(nelmts > 0) {
+            /* Check if more file sequences are needed */
+            if(curr_file_seq >= file_nseq) {
+                /* Get sequences for file selection */
+                if(H5S_SELECT_GET_SEQ_LIST(file_space, H5S_GET_SEQ_LIST_SORTED, &file_iter, io_info->dxpl_cache->vec_size, nelmts, &file_nseq, &file_nelem, file_off, file_len) < 0)
+                    HGOTO_ERROR(H5E_INTERNAL, H5E_UNSUPPORTED, FAIL, "sequence length generation failed")
+
+                /* Start at the beginning of the sequences again */
+                curr_file_seq = 0;
+            } /* end if */
+
+            /* Check if more memory sequences are needed */
+            if(curr_mem_seq >= mem_nseq) {
+                /* Get sequences for memory selection */
+                if(H5S_SELECT_GET_SEQ_LIST(mem_space, 0, &mem_iter, io_info->dxpl_cache->vec_size, nelmts, &mem_nseq, &mem_nelem, mem_off, mem_len) < 0)
+                    HGOTO_ERROR(H5E_INTERNAL, H5E_UNSUPPORTED, FAIL, "sequence length generation failed")
+
+                /* Start at the beginning of the sequences again */
+                curr_mem_seq = 0;
+            } /* end if */
+
+            /* Perform I/O on memory and file sequences */
+            if(io_buf->op_type == H5S_SELECT_READ) {
+                if((tmp_file_len = (*io_info->ops.readvv)(io_info,
+                        file_nseq, &curr_file_seq, file_len, file_off,
+                        mem_nseq, &curr_mem_seq, mem_len, mem_off,
+                        addr, chunk, io_buf->u.rbuf)) < 0)
+                    HGOTO_ERROR(H5E_DATASPACE, H5E_READERROR, FAIL, "read error")
+            } /* end if */
+            else {
+                HDassert(io_buf->op_type == H5S_SELECT_WRITE);
+                if((tmp_file_len = (*io_info->ops.writevv)(io_info,
+                        file_nseq, &curr_file_seq, file_len, file_off,
+                        mem_nseq, &curr_mem_seq, mem_len, mem_off,
+                        addr, chunk, io_buf->u.wbuf)) < 0)
+                    HGOTO_ERROR(H5E_DATASPACE, H5E_WRITEERROR, FAIL, "write error")
+            } /* end else */
+
+            /* Decrement number of elements left to process */
+            HDassert((tmp_file_len % elmt_size) == 0);
+            nelmts -= (tmp_file_len / elmt_size);
+        } /* end while */
+    } /* end else */
+
+done:
+    /* Release file selection iterator */
+    if(file_iter_init)
+        if(H5S_SELECT_ITER_RELEASE(&file_iter) < 0)
+            HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release selection iterator")
+
+    /* Release memory selection iterator */
+    if(mem_iter_init)
+        if(H5S_SELECT_ITER_RELEASE(&mem_iter) < 0)
+            HDONE_ERROR(H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release selection iterator")
+
+    /* Free vector arrays */
+    if(io_info->dxpl_cache->vec_size != H5D_IO_VECTOR_SIZE) {
+        if(file_len != NULL)
+            H5FL_SEQ_FREE(size_t, file_len);
+        if(file_off != NULL)
+            H5FL_SEQ_FREE(hsize_t, file_off);
+        if(mem_len != NULL)
+            H5FL_SEQ_FREE(size_t, mem_len);
+        if(mem_off != NULL)
+            H5FL_SEQ_FREE(hsize_t, mem_off);
+    } /* end if */
+
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5D_select_io() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5D_select_read
@@ -408,138 +630,30 @@ done:
  * Programmer:	Quincey Koziol
  *              Tuesday, July 23, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 herr_t
 H5D_select_read(H5D_io_info_t *io_info,
     size_t nelmts, size_t elmt_size,
     const H5S_t *file_space, const H5S_t *mem_space,
+    haddr_t addr, void *chunk/*in*/,
     void *buf/*out*/)
 {
-    H5S_sel_iter_t mem_iter;    /* Memory selection iteration info */
-    hbool_t mem_iter_init=0;    /* Memory selection iteration info has been initialized */
-    H5S_sel_iter_t file_iter;   /* File selection iteration info */
-    hbool_t file_iter_init=0;	/* File selection iteration info has been initialized */
-    hsize_t _mem_off[H5D_XFER_HYPER_VECTOR_SIZE_DEF];      /* Array to store sequence offsets in memory */
-    hsize_t *mem_off=NULL;      /* Pointer to sequence offsets in memory */
-    hsize_t _file_off[H5D_XFER_HYPER_VECTOR_SIZE_DEF];     /* Array to store sequence offsets in the file */
-    hsize_t *file_off=NULL;     /* Pointer to sequence offsets in the file */
-    size_t _mem_len[H5D_XFER_HYPER_VECTOR_SIZE_DEF];       /* Array to store sequence lengths in memory */
-    size_t *mem_len=NULL;       /* Pointer to sequence lengths in memory */
-    size_t _file_len[H5D_XFER_HYPER_VECTOR_SIZE_DEF];      /* Array to store sequence lengths in the file */
-    size_t *file_len=NULL;      /* Pointer to sequence lengths in the file */
-    size_t mem_nseq;            /* Number of sequences generated in the file */
-    size_t file_nseq;           /* Number of sequences generated in memory */
-    size_t mem_nelem;           /* Number of elements used in memory sequences */
-    size_t file_nelem;          /* Number of elements used in file sequences */
-    size_t curr_mem_seq;        /* Current memory sequence to operate on */
-    size_t curr_file_seq;       /* Current file sequence to operate on */
-    ssize_t tmp_file_len;       /* Temporary number of bytes in file sequence */
-    herr_t ret_value=SUCCEED;   /* Return value */
+    H5D_select_buf_t io_buf;      /* Selection I/O operation to perform */
+    herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_NOAPI(H5D_select_read, FAIL);
+    FUNC_ENTER_NOAPI(H5D_select_read, FAIL)
 
-    /* Check args */
-    assert(io_info);
-    assert(io_info->dset);
-    assert(io_info->dxpl_cache);
-    assert(io_info->store);
-    assert(buf);
-    assert(TRUE==H5P_isa_class(io_info->dxpl_id,H5P_DATASET_XFER));
+    /* Construct proper I/O operation */
+    io_buf.op_type = H5S_SELECT_READ;
+    io_buf.u.rbuf = buf;
 
-    /* Initialize file iterator */
-    if (H5S_select_iter_init(&file_iter, file_space, elmt_size)<0)
-        HGOTO_ERROR (H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to initialize selection iterator");
-    file_iter_init=1;	/* File selection iteration info has been initialized */
-
-    /* Initialize memory iterator */
-    if (H5S_select_iter_init(&mem_iter, mem_space, elmt_size)<0)
-        HGOTO_ERROR (H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to initialize selection iterator");
-    mem_iter_init=1;	/* Memory selection iteration info has been initialized */
-
-    /* Allocate the vector I/O arrays */
-    if(io_info->dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
-        if((mem_len = H5FL_SEQ_MALLOC(size_t,io_info->dxpl_cache->vec_size))==NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O length vector array");
-        if((mem_off = H5FL_SEQ_MALLOC(hsize_t,io_info->dxpl_cache->vec_size))==NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O offset vector array");
-        if((file_len = H5FL_SEQ_MALLOC(size_t,io_info->dxpl_cache->vec_size))==NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O length vector array");
-        if((file_off = H5FL_SEQ_MALLOC(hsize_t,io_info->dxpl_cache->vec_size))==NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O offset vector array");
-    } /* end if */
-    else {
-        mem_len=_mem_len;
-        mem_off=_mem_off;
-        file_len=_file_len;
-        file_off=_file_off;
-    } /* end else */
-
-    /* Initialize sequence counts */
-    curr_mem_seq=curr_file_seq=0;
-    mem_nseq=file_nseq=0;
-
-    /* Loop, until all bytes are processed */
-    while(nelmts>0) {
-        /* Check if more file sequences are needed */
-        if(curr_file_seq>=file_nseq) {
-            /* Get sequences for file selection */
-            if(H5S_SELECT_GET_SEQ_LIST(file_space,H5S_GET_SEQ_LIST_SORTED,&file_iter,io_info->dxpl_cache->vec_size,nelmts,&file_nseq,&file_nelem,file_off,file_len)<0)
-                HGOTO_ERROR (H5E_INTERNAL, H5E_UNSUPPORTED, FAIL, "sequence length generation failed");
-
-            /* Start at the beginning of the sequences again */
-            curr_file_seq=0;
-        } /* end if */
-
-        /* Check if more memory sequences are needed */
-        if(curr_mem_seq>=mem_nseq) {
-            /* Get sequences for memory selection */
-            if(H5S_SELECT_GET_SEQ_LIST(mem_space,0,&mem_iter,io_info->dxpl_cache->vec_size,nelmts,&mem_nseq,&mem_nelem,mem_off,mem_len)<0)
-                HGOTO_ERROR (H5E_INTERNAL, H5E_UNSUPPORTED, FAIL, "sequence length generation failed");
-
-            /* Start at the beginning of the sequences again */
-            curr_mem_seq=0;
-        } /* end if */
-
-        /* Read file sequences into current memory sequence */
-        if ((tmp_file_len=(*io_info->ops.readvv)(io_info,
-                file_nseq, &curr_file_seq, file_len, file_off,
-                mem_nseq, &curr_mem_seq, mem_len, mem_off,
-                buf))<0)
-            HGOTO_ERROR(H5E_DATASPACE, H5E_READERROR, FAIL, "read error");
-
-        /* Decrement number of elements left to process */
-        assert((tmp_file_len%elmt_size)==0);
-        nelmts-=(tmp_file_len/elmt_size);
-    } /* end while */
+    /* Call generic selection operation */
+    if(H5D_select_io(io_info, nelmts, elmt_size, file_space, mem_space, addr, chunk, &io_buf) < 0)
+        HGOTO_ERROR(H5E_DATASPACE, H5E_READERROR, FAIL, "read error")
 
 done:
-    /* Release file selection iterator */
-    if(file_iter_init) {
-        if (H5S_SELECT_ITER_RELEASE(&file_iter)<0)
-            HDONE_ERROR (H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release selection iterator");
-    } /* end if */
-
-    /* Release memory selection iterator */
-    if(mem_iter_init) {
-        if (H5S_SELECT_ITER_RELEASE(&mem_iter)<0)
-            HDONE_ERROR (H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release selection iterator");
-    } /* end if */
-
-    /* Free vector arrays */
-    if(io_info->dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
-        if(file_len!=NULL)
-            H5FL_SEQ_FREE(size_t,file_len);
-        if(file_off!=NULL)
-            H5FL_SEQ_FREE(hsize_t,file_off);
-        if(mem_len!=NULL)
-            H5FL_SEQ_FREE(size_t,mem_len);
-        if(mem_off!=NULL)
-            H5FL_SEQ_FREE(hsize_t,mem_off);
-    } /* end if */
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5D_select_read() */
 
 
@@ -553,137 +667,29 @@ done:
  * Programmer:	Quincey Koziol
  *              Tuesday, July 23, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 herr_t
 H5D_select_write(H5D_io_info_t *io_info,
     size_t nelmts, size_t elmt_size,
     const H5S_t *file_space, const H5S_t *mem_space,
+    haddr_t addr, void *chunk/*in*/,
     const void *buf/*out*/)
 {
-    H5S_sel_iter_t mem_iter;    /* Memory selection iteration info */
-    hbool_t mem_iter_init=0;    /* Memory selection iteration info has been initialized */
-    H5S_sel_iter_t file_iter;   /* File selection iteration info */
-    hbool_t file_iter_init=0;	/* File selection iteration info has been initialized */
-    hsize_t _mem_off[H5D_XFER_HYPER_VECTOR_SIZE_DEF];      /* Array to store sequence offsets in memory */
-    hsize_t *mem_off=NULL;      /* Pointer to sequence offsets in memory */
-    hsize_t _file_off[H5D_XFER_HYPER_VECTOR_SIZE_DEF];     /* Array to store sequence offsets in the file */
-    hsize_t *file_off=NULL;     /* Pointer to sequence offsets in the file */
-    size_t _mem_len[H5D_XFER_HYPER_VECTOR_SIZE_DEF];       /* Array to store sequence lengths in memory */
-    size_t *mem_len=NULL;       /* Pointer to sequence lengths in memory */
-    size_t _file_len[H5D_XFER_HYPER_VECTOR_SIZE_DEF];      /* Array to store sequence lengths in the file */
-    size_t *file_len=NULL;      /* Pointer to sequence lengths in the file */
-    size_t mem_nseq;            /* Number of sequences generated in the file */
-    size_t file_nseq;           /* Number of sequences generated in memory */
-    size_t mem_nelem;           /* Number of elements used in memory sequences */
-    size_t file_nelem;          /* Number of elements used in file sequences */
-    size_t curr_mem_seq;        /* Current memory sequence to operate on */
-    size_t curr_file_seq;       /* Current file sequence to operate on */
-    ssize_t tmp_file_len;       /* Temporary number of bytes in file sequence */
-    herr_t ret_value=SUCCEED;   /* Return value */
+    H5D_select_buf_t io_buf;      /* Selection I/O operation to perform */
+    herr_t ret_value = SUCCEED; /* Return value */
 
-    FUNC_ENTER_NOAPI(H5D_select_write, FAIL);
+    FUNC_ENTER_NOAPI(H5D_select_write, FAIL)
 
-    /* Check args */
-    assert(io_info);
-    assert(io_info->dset);
-    assert(io_info->store);
-    assert(TRUE==H5P_isa_class(io_info->dxpl_id,H5P_DATASET_XFER));
-    assert(buf);
+    /* Construct proper I/O operation */
+    io_buf.op_type = H5S_SELECT_WRITE;
+    io_buf.u.wbuf = buf;
 
-    /* Allocate the vector I/O arrays */
-    if(io_info->dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
-        if((mem_len = H5FL_SEQ_MALLOC(size_t,io_info->dxpl_cache->vec_size))==NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O length vector array");
-        if((mem_off = H5FL_SEQ_MALLOC(hsize_t,io_info->dxpl_cache->vec_size))==NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O offset vector array");
-        if((file_len = H5FL_SEQ_MALLOC(size_t,io_info->dxpl_cache->vec_size))==NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O length vector array");
-        if((file_off = H5FL_SEQ_MALLOC(hsize_t,io_info->dxpl_cache->vec_size))==NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "can't allocate I/O offset vector array");
-    } /* end if */
-    else {
-        mem_len=_mem_len;
-        mem_off=_mem_off;
-        file_len=_file_len;
-        file_off=_file_off;
-    } /* end else */
-
-    /* Initialize file iterator */
-    if (H5S_select_iter_init(&file_iter, file_space, elmt_size)<0)
-        HGOTO_ERROR (H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to initialize selection iterator");
-    file_iter_init=1;	/* File selection iteration info has been initialized */
-
-    /* Initialize memory iterator */
-    if (H5S_select_iter_init(&mem_iter, mem_space, elmt_size)<0)
-        HGOTO_ERROR (H5E_DATASPACE, H5E_CANTINIT, FAIL, "unable to initialize selection iterator");
-    mem_iter_init=1;	/* Memory selection iteration info has been initialized */
-
-    /* Initialize sequence counts */
-    curr_mem_seq=curr_file_seq=0;
-    mem_nseq=file_nseq=0;
-
-    /* Loop, until all bytes are processed */
-    while(nelmts>0) {
-        /* Check if more file sequences are needed */
-        if(curr_file_seq>=file_nseq) {
-            /* Get sequences for file selection */
-            if(H5S_SELECT_GET_SEQ_LIST(file_space,H5S_GET_SEQ_LIST_SORTED,&file_iter,io_info->dxpl_cache->vec_size,nelmts,&file_nseq,&file_nelem,file_off,file_len)<0)
-                HGOTO_ERROR (H5E_INTERNAL, H5E_UNSUPPORTED, FAIL, "sequence length generation failed");
-
-            /* Start at the beginning of the sequences again */
-            curr_file_seq=0;
-        } /* end if */
-
-        /* Check if more memory sequences are needed */
-        if(curr_mem_seq>=mem_nseq) {
-            /* Get sequences for memory selection */
-            if(H5S_SELECT_GET_SEQ_LIST(mem_space,0,&mem_iter,io_info->dxpl_cache->vec_size,nelmts,&mem_nseq,&mem_nelem,mem_off,mem_len)<0)
-                HGOTO_ERROR (H5E_INTERNAL, H5E_UNSUPPORTED, FAIL, "sequence length generation failed");
-
-            /* Start at the beginning of the sequences again */
-            curr_mem_seq=0;
-        } /* end if */
-
-        /* Write memory sequences into file sequences */
-        if ((tmp_file_len=(*io_info->ops.writevv)(io_info,
-                file_nseq, &curr_file_seq, file_len, file_off,
-                mem_nseq, &curr_mem_seq, mem_len, mem_off,
-                buf))<0)
-            HGOTO_ERROR(H5E_DATASPACE, H5E_WRITEERROR, FAIL, "write error");
-
-        /* Decrement number of elements left to process */
-        assert((tmp_file_len%elmt_size)==0);
-        nelmts-=(tmp_file_len/elmt_size);
-    } /* end while */
+    /* Call generic selection operation */
+    if(H5D_select_io(io_info, nelmts, elmt_size, file_space, mem_space, addr, chunk, &io_buf) < 0)
+        HGOTO_ERROR(H5E_DATASPACE, H5E_WRITEERROR, FAIL, "write error")
 
 done:
-    /* Release file selection iterator */
-    if(file_iter_init) {
-        if (H5S_SELECT_ITER_RELEASE(&file_iter)<0)
-            HDONE_ERROR (H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release selection iterator");
-    } /* end if */
-
-    /* Release memory selection iterator */
-    if(mem_iter_init) {
-        if (H5S_SELECT_ITER_RELEASE(&mem_iter)<0)
-            HDONE_ERROR (H5E_DATASPACE, H5E_CANTRELEASE, FAIL, "unable to release selection iterator");
-    } /* end if */
-
-    /* Free vector arrays */
-    if(io_info->dxpl_cache->vec_size!=H5D_XFER_HYPER_VECTOR_SIZE_DEF) {
-        if(file_len!=NULL)
-            H5FL_SEQ_FREE(size_t,file_len);
-        if(file_off!=NULL)
-            H5FL_SEQ_FREE(hsize_t,file_off);
-        if(mem_len!=NULL)
-            H5FL_SEQ_FREE(size_t,mem_len);
-        if(mem_off!=NULL)
-            H5FL_SEQ_FREE(hsize_t,mem_off);
-    } /* end if */
-
-    FUNC_LEAVE_NOAPI(ret_value);
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5D_select_write() */
 
