@@ -59,7 +59,10 @@ static hid_t H5FD_FAMILY_g = 0;
 typedef struct H5FD_family_t {
     H5FD_t	pub;		/*public stuff, must be first		*/
     hid_t	memb_fapl_id;	/*file access property list for members	*/
-    hsize_t	memb_size;	/*maximum size of each member file	*/
+    hsize_t	memb_size;	/*actual size of each member file	*/
+    hsize_t	pmem_size;	/*member size passed in from property	*/
+    hsize_t	mem_newsize;	/*new member size passed in as private
+                                 *property. It's used only by h5repart  */
     unsigned	nmembs;		/*number of family members		*/
     unsigned	amembs;		/*number of member slots allocated	*/
     H5FD_t	**memb;		/*dynamic array of member pointers	*/
@@ -85,14 +88,19 @@ static void *H5FD_family_fapl_copy(const void *_old_fa);
 static herr_t H5FD_family_fapl_free(void *_fa);
 static void *H5FD_family_dxpl_copy(const void *_old_dx);
 static herr_t H5FD_family_dxpl_free(void *_dx);
+static hsize_t H5FD_family_sb_size(H5FD_t *_file);
+static herr_t H5FD_family_sb_encode(H5FD_t *_file, char *name/*out*/,
+		     unsigned char *buf/*out*/);
+static herr_t H5FD_family_sb_decode(H5FD_t *_file, const char *name,
+                    const unsigned char *buf);
 static H5FD_t *H5FD_family_open(const char *name, unsigned flags,
 				hid_t fapl_id, haddr_t maxaddr);
 static herr_t H5FD_family_close(H5FD_t *_file);
 static int H5FD_family_cmp(const H5FD_t *_f1, const H5FD_t *_f2);
 static herr_t H5FD_family_query(const H5FD_t *_f1, unsigned long *flags);
-static haddr_t H5FD_family_get_eoa(H5FD_t *_file);
-static herr_t H5FD_family_set_eoa(H5FD_t *_file, haddr_t eoa);
-static haddr_t H5FD_family_get_eof(H5FD_t *_file);
+static haddr_t H5FD_family_get_eoa(const H5FD_t *_file, H5FD_mem_t type);
+static herr_t H5FD_family_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t eoa);
+static haddr_t H5FD_family_get_eof(const H5FD_t *_file);
 static herr_t  H5FD_family_get_handle(H5FD_t *_file, hid_t fapl, void** file_handle);
 static herr_t H5FD_family_read(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr,
 			       size_t size, void *_buf/*out*/);
@@ -104,10 +112,10 @@ static herr_t H5FD_family_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing);
 static const H5FD_class_t H5FD_family_g = {
     "family",					/*name			*/
     HADDR_MAX,					/*maxaddr		*/
-    H5F_CLOSE_WEAK,				/* fc_degree		*/
-    NULL,					/*sb_size		*/
-    NULL,					/*sb_encode		*/
-    NULL,					/*sb_decode		*/
+    H5F_CLOSE_WEAK,				/*fc_degree		*/
+    H5FD_family_sb_size,			/*sb_size		*/
+    H5FD_family_sb_encode,			/*sb_encode		*/
+    H5FD_family_sb_decode,			/*sb_decode		*/
     sizeof(H5FD_family_fapl_t),			/*fapl_size		*/
     H5FD_family_fapl_get,			/*fapl_get		*/
     H5FD_family_fapl_copy,			/*fapl_copy		*/
@@ -243,14 +251,15 @@ H5FD_family_term(void)
  *-------------------------------------------------------------------------
  */
 herr_t
-H5Pset_fapl_family(hid_t fapl_id, hsize_t memb_size, hid_t memb_fapl_id)
+H5Pset_fapl_family(hid_t fapl_id, hsize_t msize, hid_t memb_fapl_id)
 {
     herr_t ret_value;
-    H5FD_family_fapl_t	fa;
+    H5FD_family_fapl_t	fa={0, -1};
     H5P_genplist_t *plist;      /* Property list pointer */
 
     FUNC_ENTER_API(H5Pset_fapl_family, FAIL)
-    H5TRACE3("e","ihi",fapl_id,memb_size,memb_fapl_id);
+    H5TRACE3("e", "ihi", fapl_id, msize, memb_fapl_id);
+
 
     /* Check arguments */
     if(TRUE != H5P_isa_class(fapl_id, H5P_FILE_ACCESS))
@@ -265,7 +274,7 @@ H5Pset_fapl_family(hid_t fapl_id, hsize_t memb_size, hid_t memb_fapl_id)
      * Initialize driver specific information. No need to copy it into the FA
      * struct since all members will be copied by H5P_set_driver().
      */
-    fa.memb_size = memb_size;
+    fa.memb_size = msize;
     fa.memb_fapl_id = memb_fapl_id;
 
     if(NULL == (plist = H5I_object(fapl_id)))
@@ -300,7 +309,7 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5Pget_fapl_family(hid_t fapl_id, hsize_t *memb_size/*out*/,
+H5Pget_fapl_family(hid_t fapl_id, hsize_t *msize/*out*/,
 		   hid_t *memb_fapl_id/*out*/)
 {
     H5FD_family_fapl_t	*fa;
@@ -308,7 +317,7 @@ H5Pget_fapl_family(hid_t fapl_id, hsize_t *memb_size/*out*/,
     herr_t      ret_value=SUCCEED;       /* Return value */
 
     FUNC_ENTER_API(H5Pget_fapl_family, FAIL)
-    H5TRACE3("e","ixx",fapl_id,memb_size,memb_fapl_id);
+    H5TRACE3("e", "ixx", fapl_id, msize, memb_fapl_id);
 
     if(NULL == (plist = H5P_object_verify(fapl_id,H5P_FILE_ACCESS)))
         HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a file access list")
@@ -316,8 +325,8 @@ H5Pget_fapl_family(hid_t fapl_id, hsize_t *memb_size/*out*/,
         HGOTO_ERROR(H5E_PLIST, H5E_BADVALUE, FAIL, "incorrect VFL driver")
     if (NULL==(fa=H5P_get_driver_info(plist)))
         HGOTO_ERROR(H5E_PLIST, H5E_BADVALUE, FAIL, "bad VFL driver info")
-    if (memb_size)
-        *memb_size = fa->memb_size;
+    if (msize)
+        *msize = fa->memb_size;
     if (memb_fapl_id) {
         if(NULL == (plist = H5I_object(fa->memb_fapl_id)))
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a file access list")
@@ -549,6 +558,139 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 }
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5FD_family_sb_size
+ *
+ * Purpose:	Returns the size of the private information to be stored in
+ *		the superblock.
+ *
+ * Return:	Success:	The super block driver data size.
+ *
+ *		Failure:	never fails
+ *
+ * Programmer:	Raymond Lu
+ *              Tuesday, May 10, 2005
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static hsize_t
+H5FD_family_sb_size(H5FD_t UNUSED *_file)
+{
+    hsize_t		ret_value = 0; /*size of header*/
+
+    FUNC_ENTER_NOAPI(H5FD_family_sb_size, UFAIL)
+
+    /* 8 bytes field for the size of member file size field should be
+     * enough for now. */
+    ret_value += 8;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+}
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5FD_family_sb_encode
+ *
+ * Purpose:	Encode driver information for the superblock. The NAME
+ *		argument is a nine-byte buffer which will be initialized with
+ *		an eight-character name/version number and null termination.
+ *
+ *		The encoding is the member file size and name template.
+ *
+ * Return:	Success:	0
+ *
+ *		Failure:	-1
+ *
+ * Programmer:	Raymond Lu
+ *              Tuesday, May 10, 2005
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5FD_family_sb_encode(H5FD_t *_file, char *name/*out*/, unsigned char *buf/*out*/)
+{
+    H5FD_family_t	*file = (H5FD_family_t*)_file;
+
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5FD_family_sb_encode)
+
+    /* Name and version number */
+    HDstrncpy(name, "NCSAfami", (size_t)8);
+    name[8] = '\0';
+
+    /* Store member file size */
+    UINT64ENCODE(buf, (uint64_t)file->memb_size);
+
+    FUNC_LEAVE_NOAPI(SUCCEED)
+} /* end H5FD_family_sb_encode() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5FD_family_sb_decode
+ *
+ * Purpose:	This function has 2 seperate purpose.  One is to decodes the
+ *              superblock information for this driver. The NAME argument is
+ *              the eight-character (plus null termination) name stored in i
+ *              the file.  The FILE argument is updated according to the
+ *              information in the superblock.
+ *
+ * Return:	Success:	0
+ *
+ *		Failure:	-1
+ *
+ * Programmer:	Raymond Lu
+ *              Tuesday, May 10, 2005
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5FD_family_sb_decode(H5FD_t *_file, const char UNUSED *name, const unsigned char *buf)
+{
+    H5FD_family_t	*file = (H5FD_family_t*)_file;
+    uint64_t            msize;
+    herr_t ret_value = SUCCEED;   /* Return value */
+
+    FUNC_ENTER_NOAPI(H5FD_family_sb_decode, FAIL)
+
+    /* Read member file size. Skip name template for now although it's saved. */
+    UINT64DECODE(buf, msize);
+
+    /* For h5repart only. Private property of new member size is used to signal
+     * h5repart is being used to change member file size.  h5repart will open
+     * files for read and write.  When the files are closed, metadata will be
+     * flushed to the files and updated this new size */
+    if(file->mem_newsize) {
+        file->memb_size = file->mem_newsize;
+        HGOTO_DONE(ret_value)
+    } /* end if */
+
+    /* Default - use the saved member size */
+    if(file->pmem_size == H5F_FAMILY_DEFAULT)
+       file->pmem_size = msize;
+
+    /* Check if member size from file access property is correct */
+    if(msize != file->pmem_size) {
+        char                err_msg[128];
+
+        sprintf(err_msg, "family member size should be %lu, is %lu", (unsigned long)msize, (unsigned long)file->pmem_size);
+        HGOTO_ERROR(H5E_FILE, H5E_BADVALUE, FAIL, err_msg)
+    } /* end if */
+
+    /* Update member file size to the size saved in the superblock.
+     * That's the size intended to be. */
+    file->memb_size = msize;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5FD_family_sb_decode() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5FD_family_open
@@ -573,6 +715,14 @@ done:
  *              If there are at least 2 member files, member size can only be equal
  *              the 1st member size.
  *
+ *              Raymond Lu
+ *              Tuesday, May 24, 2005
+ *              The modification described above has been changed.  The major checking
+ *              is done in H5F_read_superblock.  Member file size is saved in the
+ *              superblock now.  H5F_read_superblock() reads this saved size and compare
+ *              to the size passed in from file access property.  Wrong size will
+ *              result in a failure.
+ *
  *-------------------------------------------------------------------------
  */
 static H5FD_t *
@@ -582,7 +732,8 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
     H5FD_family_t	*file=NULL;
     H5FD_t     		*ret_value=NULL;
     char		memb_name[4096], temp[4096];
-    hsize_t		eof1=HADDR_UNDEF, eof2=HADDR_UNDEF;
+    hsize_t		eof=HADDR_UNDEF;
+    hsize_t             fam_newsize = 0;
     unsigned		t_flags = flags & ~H5F_ACC_CREAT;
     H5P_genplist_t      *plist;      /* Property list pointer */
 
@@ -601,13 +752,21 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
         file->memb_fapl_id = H5P_FILE_ACCESS_DEFAULT;
         if(H5I_inc_ref(file->memb_fapl_id)<0)
             HGOTO_ERROR(H5E_VFL, H5E_CANTINC, NULL, "unable to increment ref count on VFL driver")
-        file->memb_size = 1024*1024*1024; /*1GB*/
+        file->memb_size = 1024*1024*1024; /*1GB. Actual member size to be updated later */
+        file->pmem_size = 1024*1024*1024; /*1GB. Member size passed in through property */
+        file->mem_newsize = 0;            /*New member size used by h5repart only       */
     } else {
         H5FD_family_fapl_t *fa;
 
         if(NULL == (plist = H5I_object(fapl_id)))
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, NULL, "not a file access property list")
         fa = H5P_get_driver_info(plist);
+
+        /* New family file size. It's used by h5repart only. */
+        if(H5P_exist_plist(plist, H5F_ACS_FAMILY_NEWSIZE_NAME) > 0)
+            if(H5P_get(plist, H5F_ACS_FAMILY_NEWSIZE_NAME, &fam_newsize) < 0)
+                HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, NULL, "can't get new family member size")
+
         if(fa->memb_fapl_id==H5P_FILE_ACCESS_DEFAULT) {
             if(H5I_inc_ref(fa->memb_fapl_id)<0)
                 HGOTO_ERROR(H5E_VFL, H5E_CANTINC, NULL, "unable to increment ref count on VFL driver")
@@ -618,7 +777,9 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
                 HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, NULL, "not a file access property list")
             file->memb_fapl_id = H5P_copy_plist(plist);
         } /* end else */
-        file->memb_size = fa->memb_size;
+        file->memb_size = fa->memb_size; /* Actual member size to be updated later */
+        file->pmem_size = fa->memb_size; /* Member size passed in through property */
+        file->mem_newsize = fam_newsize; /* New member size passed in through property */
     }
     file->name = H5MM_strdup(name);
     file->flags = flags;
@@ -626,6 +787,7 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
     /* Check that names are unique */
     sprintf(memb_name, name, 0);
     sprintf(temp, name, 1);
+
     if (!strcmp(memb_name, temp))
         HGOTO_ERROR(H5E_FILE, H5E_FILEEXISTS, NULL, "file names not unique")
 
@@ -656,33 +818,17 @@ H5FD_family_open(const char *name, unsigned flags, hid_t fapl_id,
         if (!file->memb[file->nmembs]) {
             if (0==file->nmembs)
                 HGOTO_ERROR(H5E_FILE, H5E_CANTOPENFILE, NULL, "unable to open member file")
-            H5Eclear();
+            H5E_clear_stack(NULL);
             break;
         }
         file->nmembs++;
     }
 
-    /*
-     * Get file size of the first 2 member files if exist.  Check if user sets
-     * reasonable member size.
+    /* If the file is reopened and there's only one member file existing, this file maybe
+     * smaller than the size specified through H5Pset_fapl_family().  Update the actual
+     * member size.
      */
-    if(HADDR_UNDEF==(eof1 = H5FD_get_eof(file->memb[0])))
-	HGOTO_ERROR(H5E_VFL, H5E_CANTINIT, NULL, "file get eof1 request failed")
-    if(file->memb[1] && (HADDR_UNDEF==(eof2 = H5FD_get_eof(file->memb[1]))))
-	HGOTO_ERROR(H5E_VFL, H5E_CANTINIT, NULL, "file get eof2 request failed")
-
-    if(eof1 && (eof2==HADDR_UNDEF || !eof2)) {
-        /* If there is only 1 member file, new member size can't be smaller than
-         * current member size.
-         */
-        if(file->memb_size<eof1)
-            file->memb_size = eof1;
-    } else if(eof1 && eof2) {
-        /* If there are at least 2 member files, new member size can only be equal
-         * to the 1st member size
-         */
-        file->memb_size = eof1;
-    }
+    if ((eof=H5FDget_eof(file->memb[0]))) file->memb_size = eof;
 
     ret_value=(H5FD_t *)file;
 
@@ -692,6 +838,8 @@ done:
         unsigned nerrors=0;     /* Number of errors closing member files */
         unsigned u;             /* Local index variable */
 
+        /* Close as many members as possible. Use private function here to avoid clearing
+         * the error stack. We need the error message to indicate wrong member file size. */
         for (u=0; u<file->nmembs; u++)
             if (file->memb[u])
                 if (H5FD_close(file->memb[u])<0)
@@ -739,10 +887,11 @@ H5FD_family_close(H5FD_t *_file)
 
     FUNC_ENTER_NOAPI(H5FD_family_close, FAIL)
 
-    /* Close as many members as possible */
+    /* Close as many members as possible. Use private function here to avoid clearing
+     * the error stack. We need the error message to indicate wrong member file size. */
     for (u=0; u<file->nmembs; u++) {
         if (file->memb[u]) {
-            if (H5FDclose(file->memb[u])<0)
+            if (H5FD_close(file->memb[u])<0)
                 nerrors++;
             else
                 file->memb[u] = NULL;
@@ -858,13 +1007,16 @@ done:
  *              Wednesday, August  4, 1999
  *
  * Modifications:
+ *              Raymond Lu
+ *              21 Dec. 2006
+ *              Added the parameter TYPE.  It's only used for MULTI driver.
  *
  *-------------------------------------------------------------------------
  */
 static haddr_t
-H5FD_family_get_eoa(H5FD_t *_file)
+H5FD_family_get_eoa(const H5FD_t *_file, H5FD_mem_t UNUSED type)
 {
-    H5FD_family_t	*file = (H5FD_family_t*)_file;
+    const H5FD_family_t	*file = (const H5FD_family_t*)_file;
     haddr_t ret_value;   /* Return value */
 
     FUNC_ENTER_NOAPI(H5FD_family_get_eoa, HADDR_UNDEF)
@@ -890,11 +1042,14 @@ done:
  *              Wednesday, August  4, 1999
  *
  * Modifications:
+ *              Raymond Lu
+ *              21 Dec. 2006
+ *              Added the parameter TYPE.  It's only used for MULTI driver.
  *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5FD_family_set_eoa(H5FD_t *_file, haddr_t eoa)
+H5FD_family_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t eoa)
 {
     H5FD_family_t	*file = (H5FD_family_t*)_file;
     haddr_t		addr=eoa;
@@ -933,11 +1088,11 @@ H5FD_family_set_eoa(H5FD_t *_file, haddr_t eoa)
         /* Set the EOA marker for the member */
         H5_CHECK_OVERFLOW(file->memb_size,hsize_t,haddr_t);
         if (addr>(haddr_t)file->memb_size) {
-            if(H5FD_set_eoa(file->memb[u], (haddr_t)file->memb_size)<0)
+            if(H5FD_set_eoa(file->memb[u], type, (haddr_t)file->memb_size)<0)
                 HGOTO_ERROR(H5E_FILE, H5E_CANTINIT, FAIL, "unable to set file eoa")
             addr -= file->memb_size;
         } else {
-            if(H5FD_set_eoa(file->memb[u], addr)<0)
+            if(H5FD_set_eoa(file->memb[u], type, addr)<0)
                 HGOTO_ERROR(H5E_FILE, H5E_CANTINIT, FAIL, "unable to set file eoa")
             addr = 0;
         }
@@ -970,9 +1125,9 @@ done:
  *-------------------------------------------------------------------------
  */
 static haddr_t
-H5FD_family_get_eof(H5FD_t *_file)
+H5FD_family_get_eof(const H5FD_t *_file)
 {
-    H5FD_family_t	*file = (H5FD_family_t*)_file;
+    const H5FD_family_t	*file = (const H5FD_family_t*)_file;
     haddr_t		eof=0;
     int			i;      /* Local index variable */
     haddr_t ret_value;   /* Return value */
