@@ -26,12 +26,13 @@
  */
 #include <assert.h>
 #include <stdlib.h>
+#include <string.h>
 
 /* Disable certain warnings in PC-Lint: */
 /*lint --emacro( {534, 830}, H5P_DEFAULT, H5P_FILE_ACCESS, H5P_DATASET_XFER) */
 /*lint --emacro( {534, 830}, H5F_ACC_DEBUG, H5F_ACC_RDWR) */
 /*lint --emacro( {534, 830}, H5FD_MULTI) */
-/*lint -esym( 534, H5Eclear, H5Epush) */
+/*lint -esym( 534, H5Eclear2, H5Epush2) */
 
 #include "hdf5.h"
 
@@ -106,7 +107,9 @@ typedef struct H5FD_multi_t {
     H5FD_multi_fapl_t fa;	/*driver-specific file access properties*/
     haddr_t	memb_next[H5FD_MEM_NTYPES];/*addr of next member	*/
     H5FD_t	*memb[H5FD_MEM_NTYPES];	/*member pointers		*/
-    haddr_t	eoa;		/*end of allocated addresses		*/
+    /*haddr_t	eoa;*/		/*end of allocated addresses.  Took it out
+                                 *because individual files have their own
+                                 *eoa.                                  */
     unsigned	flags;		/*file open flags saved for debugging	*/
     char	*name;		/*name passed to H5Fopen or H5Fcreate	*/
 } H5FD_multi_t;
@@ -137,9 +140,9 @@ static H5FD_t *H5FD_multi_open(const char *name, unsigned flags,
 static herr_t H5FD_multi_close(H5FD_t *_file);
 static int H5FD_multi_cmp(const H5FD_t *_f1, const H5FD_t *_f2);
 static herr_t H5FD_multi_query(const H5FD_t *_f1, unsigned long *flags);
-static haddr_t H5FD_multi_get_eoa(H5FD_t *_file);
-static herr_t H5FD_multi_set_eoa(H5FD_t *_file, haddr_t eoa);
-static haddr_t H5FD_multi_get_eof(H5FD_t *_file);
+static haddr_t H5FD_multi_get_eoa(const H5FD_t *_file, H5FD_mem_t type);
+static herr_t H5FD_multi_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t eoa);
+static haddr_t H5FD_multi_get_eof(const H5FD_t *_file);
 static herr_t  H5FD_multi_get_handle(H5FD_t *_file, hid_t fapl, void** file_handle);
 static haddr_t H5FD_multi_alloc(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, hsize_t size);
 static herr_t H5FD_multi_free(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr,
@@ -232,7 +235,7 @@ hid_t
 H5FD_multi_init(void)
 {
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     if (H5I_VFL!=H5Iget_type(H5FD_MULTI_g)) {
 	H5FD_MULTI_g = H5FDregister(&H5FD_multi_g);
@@ -292,7 +295,7 @@ herr_t
 H5Pset_fapl_split(hid_t fapl, const char *meta_ext, hid_t meta_plist_id,
 		  const char *raw_ext, hid_t raw_plist_id)
 {
-    H5FD_mem_t		mt, memb_map[H5FD_MEM_NTYPES];
+    H5FD_mem_t		memb_map[H5FD_MEM_NTYPES];
     hid_t		memb_fapl[H5FD_MEM_NTYPES];
     const char		*memb_name[H5FD_MEM_NTYPES];
     char		meta_name[1024], raw_name[1024];
@@ -301,15 +304,15 @@ H5Pset_fapl_split(hid_t fapl, const char *meta_ext, hid_t meta_plist_id,
     /*NO TRACE*/
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Initialize */
-    for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
+    ALL_MEMBERS(mt) {
 	memb_map[mt] = (H5FD_MEM_DRAW==mt?mt:H5FD_MEM_SUPER);
 	memb_fapl[mt] = -1;
 	memb_name[mt] = NULL;
 	memb_addr[mt] = HADDR_UNDEF;
-    }
+    } END_MEMBERS;
 
     /* The file access properties */
     memb_fapl[H5FD_MEM_SUPER] = meta_plist_id;
@@ -445,12 +448,12 @@ H5Pset_fapl_multi(hid_t fapl_id, const H5FD_mem_t *memb_map,
     /*NO TRACE*/
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Check arguments and supply default values */
     if(H5I_GENPROP_LST != H5Iget_type(fapl_id) ||
             TRUE != H5Pisa_class(fapl_id, H5P_FILE_ACCESS))
-        H5Epush_ret(func, H5E_PLIST, H5E_BADVALUE, "not an access list", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADVALUE, "not an access list", -1)
     if (!memb_map) {
 	for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1))
 	    _memb_map[mt] = H5FD_MEM_DEFAULT;
@@ -479,7 +482,7 @@ H5Pset_fapl_multi(hid_t fapl_id, const H5FD_mem_t *memb_map,
 	/* Map usage type */
 	mmt = memb_map[mt];
 	if (mmt<0 || mmt>=H5FD_MEM_NTYPES)
-            H5Epush_ret(func, H5E_INTERNAL, H5E_BADRANGE, "file resource type out of range", -1)
+            H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADRANGE, "file resource type out of range", -1)
 	if (H5FD_MEM_DEFAULT==mmt) mmt = mt;
 
 	/*
@@ -487,11 +490,11 @@ H5Pset_fapl_multi(hid_t fapl_id, const H5FD_mem_t *memb_map,
 	 * access property lists.
 	 */
 	if (H5P_DEFAULT!=memb_fapl[mmt] && TRUE!=H5Pisa_class(memb_fapl[mmt], H5P_FILE_ACCESS))
-            H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "file resource type incorrect", -1)
+            H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "file resource type incorrect", -1)
 
 	/* All names must be defined */
 	if (!memb_name[mmt] || !memb_name[mmt][0])
-            H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "file resource type not set", -1)
+            H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "file resource type not set", -1)
     }
 
     /*
@@ -546,15 +549,15 @@ H5Pget_fapl_multi(hid_t fapl_id, H5FD_mem_t *memb_map/*out*/,
     /*NO TRACE*/
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     if(H5I_GENPROP_LST != H5Iget_type(fapl_id) ||
             TRUE != H5Pisa_class(fapl_id, H5P_FILE_ACCESS))
-        H5Epush_ret(func, H5E_PLIST, H5E_BADTYPE, "not an access list", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADTYPE, "not an access list", -1)
     if (H5FD_MULTI!=H5Pget_driver(fapl_id))
-        H5Epush_ret(func, H5E_PLIST, H5E_BADVALUE, "incorrect VFL driver", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADVALUE, "incorrect VFL driver", -1)
     if (NULL==(fa=H5Pget_driver_info(fapl_id)))
-        H5Epush_ret(func, H5E_PLIST, H5E_BADVALUE, "bad VFL driver info", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADVALUE, "bad VFL driver info", -1)
 
     if (memb_map)
         memcpy(memb_map, fa->memb_map, H5FD_MEM_NTYPES*sizeof(H5FD_mem_t));
@@ -613,16 +616,16 @@ H5Pset_dxpl_multi(hid_t dxpl_id, const hid_t *memb_dxpl)
     /*NO TRACE*/
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Check arguments */
     if (TRUE!=H5Pisa_class(dxpl_id,  H5P_DATASET_XFER))
-        H5Epush_ret(func, H5E_PLIST, H5E_BADTYPE, "not a data transfer property list", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADTYPE, "not a data transfer property list", -1)
     if (!memb_dxpl)
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "invalid pointer", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "invalid pointer", -1)
     for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
         if (memb_dxpl[mt]!=H5P_DEFAULT && TRUE!=H5Pisa_class(memb_dxpl[mt], H5P_DATASET_XFER))
-            H5Epush_ret(func, H5E_PLIST, H5E_BADTYPE, "not a data transfer property list", -1)
+            H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADTYPE, "not a data transfer property list", -1)
     }
 
     /* Initialize the data transfer property list */
@@ -665,14 +668,14 @@ H5Pget_dxpl_multi(hid_t dxpl_id, hid_t *memb_dxpl/*out*/)
     /*NO TRACE*/
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     if (TRUE!=H5Pisa_class(dxpl_id, H5P_DATASET_XFER))
-        H5Epush_ret(func, H5E_PLIST, H5E_BADTYPE, "not a file access property list", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADTYPE, "not a file access property list", -1)
     if (H5FD_MULTI!=H5Pget_driver(dxpl_id))
-        H5Epush_ret(func, H5E_PLIST, H5E_BADVALUE, "incorrect VFL driver", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADVALUE, "incorrect VFL driver", -1)
     if (NULL==(dx=H5Pget_driver_info(dxpl_id)))
-        H5Epush_ret(func, H5E_PLIST, H5E_BADVALUE, "bad VFL driver info", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_PLIST, H5E_BADVALUE, "bad VFL driver info", -1)
 
     if (memb_dxpl) {
 	for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
@@ -712,7 +715,7 @@ H5FD_multi_sb_size(H5FD_t *_file)
     hsize_t		nbytes = 8; /*size of header*/
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* How many unique files? */
     UNIQUE_MEMBERS(file->fa.memb_map, mt) {
@@ -770,15 +773,17 @@ H5FD_multi_sb_encode(H5FD_t *_file, char *name/*out*/,
     static const char *func="H5FD_multi_sb_encode";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Name and version number */
-    strncpy(name, "NCSAmulti",8);
+    strncpy(name, "NCSAmulti", (size_t)8);
     name[8] = '\0';
 
     assert(7==H5FD_MEM_NTYPES);
-    for (m=H5FD_MEM_SUPER; m<H5FD_MEM_NTYPES; m=(H5FD_mem_t)(m+1))
+
+    for (m=H5FD_MEM_SUPER; m<H5FD_MEM_NTYPES; m=(H5FD_mem_t)(m+1)) {
         buf[m-1] = (unsigned char)file->fa.memb_map[m];
+    }
     buf[6] = 0;
     buf[7] = 0;
 
@@ -792,7 +797,7 @@ H5FD_multi_sb_encode(H5FD_t *_file, char *name/*out*/,
     p = buf+8;
     assert(sizeof(haddr_t)<=8);
     UNIQUE_MEMBERS(file->fa.memb_map, mt) {
-        memb_eoa = H5FDget_eoa(file->memb[mt]);
+        memb_eoa = H5FDget_eoa(file->memb[mt], mt);
         memcpy(p, &(file->fa.memb_addr[mt]), sizeof(haddr_t));
         p += sizeof(haddr_t);
         memcpy(p, &memb_eoa, sizeof(haddr_t));
@@ -801,7 +806,7 @@ H5FD_multi_sb_encode(H5FD_t *_file, char *name/*out*/,
     } END_MEMBERS;
     if (H5Tconvert(H5T_NATIVE_HADDR, H5T_STD_U64LE, nseen*2, buf+8, NULL,
 		   H5P_DEFAULT)<0)
-        H5Epush_ret(func, H5E_DATATYPE, H5E_CANTCONVERT, "can't convert superblock info", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_DATATYPE, H5E_CANTCONVERT, "can't convert superblock info", -1)
 
     /* Encode all name templates */
     p = buf + 8 + nseen*2*8;
@@ -855,11 +860,11 @@ H5FD_multi_sb_decode(H5FD_t *_file, const char *name, const unsigned char *buf)
     static const char *func="H5FD_multi_sb_decode";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Make sure the name/version number is correct */
     if (strcmp(name, "NCSAmult"))
-        H5Epush_ret(func, H5E_FILE, H5E_BADVALUE, "invalid multi superblock", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_FILE, H5E_BADVALUE, "invalid multi superblock", -1)
 
     /* Set default values */
     ALL_MEMBERS(mt) {
@@ -872,11 +877,13 @@ H5FD_multi_sb_decode(H5FD_t *_file, const char *name, const unsigned char *buf)
      * Read the map and count the unique members.
      */
     memset(map, 0, sizeof map);
+
     for (i=0; i<6; i++) {
         map[i+1] = (H5FD_mem_t)buf[i];
         if (file->fa.memb_map[i+1]!=map[i+1])
             map_changed=TRUE;
     }
+
     UNIQUE_MEMBERS(map, mt) {
         nseen++;
     } END_MEMBERS;
@@ -887,7 +894,7 @@ H5FD_multi_sb_decode(H5FD_t *_file, const char *name, const unsigned char *buf)
     memcpy(x, buf, (nseen*2*8));
     buf += nseen*2*8;
     if (H5Tconvert(H5T_STD_U64LE, H5T_NATIVE_HADDR, nseen*2, x, NULL, H5P_DEFAULT)<0)
-        H5Epush_ret(func, H5E_DATATYPE, H5E_CANTCONVERT, "can't convert superblock info", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_DATATYPE, H5E_CANTCONVERT, "can't convert superblock info", -1)
     ap = (haddr_t*)x;
     UNIQUE_MEMBERS(map, mt) {
         memb_addr[_unmapped] = *ap++;
@@ -955,17 +962,17 @@ H5FD_multi_sb_decode(H5FD_t *_file, const char *name, const unsigned char *buf)
         }
     } END_MEMBERS;
     if (compute_next(file)<0)
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "compute_next() failed", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "compute_next() failed", -1)
 
     /* Open all necessary files */
     if (open_members(file)<0)
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "open_members() failed", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "open_members() failed", -1)
 
     /* Set the EOA marker for all open files */
     UNIQUE_MEMBERS(file->fa.memb_map, mt) {
         if (file->memb[mt])
-            if(H5FDset_eoa(file->memb[mt], memb_eoa[mt])<0)
-                H5Epush_ret(func, H5E_INTERNAL, H5E_CANTSET, "set_eoa() failed", -1)
+            if(H5FDset_eoa(file->memb[mt], mt, memb_eoa[mt])<0)
+                H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_CANTSET, "set_eoa() failed", -1)
     } END_MEMBERS;
 
     return 0;
@@ -997,7 +1004,7 @@ H5FD_multi_fapl_get(H5FD_t *_file)
     H5FD_multi_t	*file = (H5FD_multi_t*)_file;
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     return H5FD_multi_fapl_copy(&(file->fa));
 }
@@ -1024,17 +1031,16 @@ H5FD_multi_fapl_copy(const void *_old_fa)
 {
     const H5FD_multi_fapl_t *old_fa = (const H5FD_multi_fapl_t*)_old_fa;
     H5FD_multi_fapl_t *new_fa = malloc(sizeof(H5FD_multi_fapl_t));
-    H5FD_mem_t mt;
     int nerrors = 0;
     static const char *func="H5FD_multi_fapl_copy";  /* Function Name for error reporting */
 
     assert(new_fa);
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     memcpy(new_fa, old_fa, sizeof(H5FD_multi_fapl_t));
-    for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
+    ALL_MEMBERS(mt) {
 	if (old_fa->memb_fapl[mt]>=0) {
 	    new_fa->memb_fapl[mt] = H5Pcopy(old_fa->memb_fapl[mt]);
 	    if (new_fa->memb_fapl[mt]<0) nerrors++;
@@ -1044,15 +1050,15 @@ H5FD_multi_fapl_copy(const void *_old_fa)
 	    assert(new_fa->memb_name[mt]);
 	    strcpy(new_fa->memb_name[mt], old_fa->memb_name[mt]);
 	}
-    }
+    } END_MEMBERS;
 
     if (nerrors) {
-        for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
+        ALL_MEMBERS(mt) {
             if (new_fa->memb_fapl[mt]>=0) (void)H5Pclose(new_fa->memb_fapl[mt]);
             if (new_fa->memb_name[mt]) free(new_fa->memb_name[mt]);
-        }
+        } END_MEMBERS;
         free(new_fa);
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "invalid freespace objects", NULL)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "invalid freespace objects", NULL)
     }
     return new_fa;
 }
@@ -1078,19 +1084,18 @@ static herr_t
 H5FD_multi_fapl_free(void *_fa)
 {
     H5FD_multi_fapl_t	*fa = (H5FD_multi_fapl_t*)_fa;
-    H5FD_mem_t		mt;
     static const char *func="H5FD_multi_fapl_free";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
-    for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
+    ALL_MEMBERS(mt) {
 	if (fa->memb_fapl[mt]>=0)
             if(H5Pclose(fa->memb_fapl[mt])<0)
-                H5Epush_ret(func, H5E_FILE, H5E_CANTCLOSEOBJ, "can't close property list", -1)
+                H5Epush_ret(func, H5E_ERR_CLS, H5E_FILE, H5E_CANTCLOSEOBJ, "can't close property list", -1)
 	if (fa->memb_name[mt])
             free(fa->memb_name[mt]);
-    }
+    } END_MEMBERS;
     free(fa);
 
     return 0;
@@ -1118,28 +1123,28 @@ H5FD_multi_dxpl_copy(const void *_old_dx)
 {
     const H5FD_multi_dxpl_t *old_dx = (const H5FD_multi_dxpl_t*)_old_dx;
     H5FD_multi_dxpl_t *new_dx = malloc(sizeof(H5FD_multi_dxpl_t));
-    H5FD_mem_t mt;
     int nerrors = 0;
     static const char *func="H5FD_multi_dxpl_copy";  /* Function Name for error reporting */
 
     assert(new_dx);
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     memcpy(new_dx, old_dx, sizeof(H5FD_multi_dxpl_t));
-    for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
+    ALL_MEMBERS(mt) {
 	if (old_dx->memb_dxpl[mt]>=0) {
 	    new_dx->memb_dxpl[mt] = H5Pcopy(old_dx->memb_dxpl[mt]);
 	    if (new_dx->memb_dxpl[mt]<0) nerrors++;
 	}
-    }
+    } END_MEMBERS;
 
     if (nerrors) {
-        for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1))
+        ALL_MEMBERS(mt) {
             (void)H5Pclose(new_dx->memb_dxpl[mt]);
+        } END_MEMBERS;
         free(new_dx);
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "invalid freespace objects", NULL)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "invalid freespace objects", NULL)
     }
     return new_dx;
 }
@@ -1165,16 +1170,17 @@ static herr_t
 H5FD_multi_dxpl_free(void *_dx)
 {
     H5FD_multi_dxpl_t	*dx = (H5FD_multi_dxpl_t*)_dx;
-    H5FD_mem_t		mt;
     static const char *func="H5FD_multi_dxpl_free";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
-    for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1))
+    ALL_MEMBERS(mt) {
         if (dx->memb_dxpl[mt]>=0)
             if(H5Pclose(dx->memb_dxpl[mt])<0)
-                H5Epush_ret(func, H5E_FILE, H5E_CANTCLOSEOBJ, "can't close property list", -1)
+                H5Epush_ret(func, H5E_ERR_CLS, H5E_FILE, H5E_CANTCLOSEOBJ, "can't close property list", -1)
+    } END_MEMBERS;
+
     free(dx);
     return 0;
 }
@@ -1209,24 +1215,24 @@ H5FD_multi_open(const char *name, unsigned flags, hid_t fapl_id,
     static const char *func="H5FD_multi_open";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Check arguments */
     if (!name || !*name)
-        H5Epush_ret(func, H5E_ARGS, H5E_BADVALUE, "invalid file name", NULL)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_ARGS, H5E_BADVALUE, "invalid file name", NULL)
     if (0==maxaddr || HADDR_UNDEF==maxaddr)
-        H5Epush_ret(func, H5E_ARGS, H5E_BADRANGE, "bogus maxaddr", NULL)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_ARGS, H5E_BADRANGE, "bogus maxaddr", NULL)
 
     /*
      * Initialize the file from the file access properties, using default
      * values if necessary.
      */
-    if (NULL==(file=calloc(1, sizeof(H5FD_multi_t))))
-        H5Epush_ret(func, H5E_RESOURCE, H5E_NOSPACE, "memory allocation failed", NULL)
+    if (NULL==(file=calloc((size_t)1, sizeof(H5FD_multi_t))))
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_RESOURCE, H5E_NOSPACE, "memory allocation failed", NULL)
     if (H5P_FILE_ACCESS_DEFAULT==fapl_id || H5FD_MULTI!=H5Pget_driver(fapl_id)) {
         close_fapl = fapl_id = H5Pcreate(H5P_FILE_ACCESS);
         if(H5Pset_fapl_multi(fapl_id, NULL, NULL, NULL, NULL, TRUE)<0)
-            H5Epush_goto(func, H5E_FILE, H5E_CANTSET, "can't set property value", error)
+            H5Epush_goto(func, H5E_ERR_CLS, H5E_FILE, H5E_CANTSET, "can't set property value", error)
     }
     fa = H5Pget_driver_info(fapl_id);
     assert(fa);
@@ -1247,13 +1253,13 @@ H5FD_multi_open(const char *name, unsigned flags, hid_t fapl_id,
     file->name = my_strdup(name);
     if (close_fapl>=0)
         if(H5Pclose(close_fapl)<0)
-            H5Epush_goto(func, H5E_FILE, H5E_CANTCLOSEOBJ, "can't close property list", error)
+            H5Epush_goto(func, H5E_ERR_CLS, H5E_FILE, H5E_CANTCLOSEOBJ, "can't close property list", error)
 
     /* Compute derived properties and open member files */
     if (compute_next(file)<0)
-        H5Epush_goto(func, H5E_INTERNAL, H5E_BADVALUE, "compute_next() failed", error)
+        H5Epush_goto(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "compute_next() failed", error);
     if (open_members(file)<0)
-        H5Epush_goto(func, H5E_INTERNAL, H5E_BADVALUE, "open_members() failed", error)
+        H5Epush_goto(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "open_members() failed", error);
 
     /* We must have opened at least the superblock file */
     if (H5FD_MEM_DEFAULT==(m=file->fa.memb_map[H5FD_MEM_SUPER]))
@@ -1304,7 +1310,7 @@ H5FD_multi_close(H5FD_t *_file)
     static const char *func="H5FD_multi_close";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Close as many members as possible */
     ALL_MEMBERS(mt) {
@@ -1327,7 +1333,7 @@ H5FD_multi_close(H5FD_t *_file)
 	}
     } END_MEMBERS;
     if (nerrors)
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "error closing member files", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "error closing member files", -1)
 
     /* Clean up other stuff */
     ALL_MEMBERS(mt) {
@@ -1366,23 +1372,24 @@ H5FD_multi_cmp(const H5FD_t *_f1, const H5FD_t *_f2)
 {
     const H5FD_multi_t	*f1 = (const H5FD_multi_t*)_f1;
     const H5FD_multi_t	*f2 = (const H5FD_multi_t*)_f2;
-    H5FD_mem_t		mt;
+    H5FD_mem_t		out_mt = H5FD_MEM_DEFAULT;
     int			cmp=0;
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
-    for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
+    ALL_MEMBERS(mt) {
+        out_mt = mt;
 	if (f1->memb[mt] && f2->memb[mt]) break;
 	if (!cmp) {
 	    if (f1->memb[mt]) cmp = -1;
 	    else if (f2->memb[mt]) cmp = 1;
 	}
-    }
-    assert(cmp || mt<H5FD_MEM_NTYPES);
-    if (mt>=H5FD_MEM_NTYPES) return cmp;
+    } END_MEMBERS;
+    assert(cmp || out_mt<H5FD_MEM_NTYPES);
+    if (out_mt>=H5FD_MEM_NTYPES) return cmp;
 
-    return H5FDcmp(f1->memb[mt], f2->memb[mt]);
+    return H5FDcmp(f1->memb[out_mt], f2->memb[out_mt]);
 }
 
 
@@ -1435,18 +1442,80 @@ H5FD_multi_query(const H5FD_t *_f, unsigned long *flags /* out */)
  *              Wednesday, August  4, 1999
  *
  * Modifications:
+ *              Raymond Lu
+ *              21 Dec. 2006
+ *              Added the parameter TYPE.  It's only used for MULTI driver.
+ *              If the TYPE is H5FD_MEM_DEFAULT, simply find the biggest
+ *              EOA of individual file because the EOA for the whole file
+ *              is meaningless.
  *
  *-------------------------------------------------------------------------
  */
 static haddr_t
-H5FD_multi_get_eoa(H5FD_t *_file)
+H5FD_multi_get_eoa(const H5FD_t *_file, H5FD_mem_t type)
 {
-    H5FD_multi_t	*file = (H5FD_multi_t*)_file;
+    const H5FD_multi_t	*file = (const H5FD_multi_t*)_file;
+    haddr_t eoa = 0;
+    haddr_t memb_eoa = 0;
+    static const char *func="H5FD_multi_eof";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
-    return file->eoa;
+    /* The library used to have EOA for the whole file.  But it's
+     * taken out because it makes little sense for MULTI files.
+     * However, the library sometimes queries it through H5F_get_eoa.
+     * Here the code finds the biggest EOA for individual file if
+     * the query is from H5F_get_eoa (TYPE is H5FD_MEM_DEFAULT).
+     */ 
+    if(H5FD_MEM_DEFAULT == type) {
+        UNIQUE_MEMBERS(file->fa.memb_map, mt) {
+	    if (file->memb[mt]) {
+                /* Retrieve EOA */
+	        H5E_BEGIN_TRY {
+                    memb_eoa = H5FDget_eoa(file->memb[mt], mt);
+	        } H5E_END_TRY;
+
+	        if (HADDR_UNDEF==memb_eoa)
+                    H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "member file has unknown eoa", HADDR_UNDEF)
+	    } else if (file->fa.relax) {
+	        /*
+	         * The member is not open yet (maybe it doesn't exist). Make the
+	         * best guess about the end-of-file.
+	         */
+	        memb_eoa = file->memb_next[mt];
+	        assert(HADDR_UNDEF!=memb_eoa);
+	    } else {
+                H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "bad eoa", HADDR_UNDEF)
+	    }
+
+            if(memb_eoa > eoa)
+                eoa = memb_eoa;
+        } END_MEMBERS;
+    } else {
+        H5FD_mem_t mmt = file->fa.memb_map[type];
+        if (H5FD_MEM_DEFAULT==mmt) mmt = type;        
+
+	if (file->memb[mmt]) {
+            H5E_BEGIN_TRY {
+	        eoa = H5FDget_eoa(file->memb[mmt], mmt);
+            } H5E_END_TRY;
+
+	    if (HADDR_UNDEF==eoa)
+                H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "member file has unknown eoa", HADDR_UNDEF)
+	} else if (file->fa.relax) {
+	    /*
+	     * The member is not open yet (maybe it doesn't exist). Make the
+	     * best guess about the end-of-file.
+	     */
+	    eoa = file->memb_next[mmt];
+	    assert(HADDR_UNDEF!=eoa);
+	 } else {
+            H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "bad eoa", HADDR_UNDEF)
+	 }
+    }
+
+    return eoa;
 }
 
 
@@ -1466,43 +1535,30 @@ H5FD_multi_get_eoa(H5FD_t *_file)
  *              Wednesday, August  4, 1999
  *
  * Modifications:
+ *              Raymond Lu
+ *              10 January 2007
+ *              EOA for the whole file is discarded because it's meaningless
+ *              for MULTI file.  This function only sets eoa for individual
+ *              file.
  *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5FD_multi_set_eoa(H5FD_t *_file, haddr_t eoa)
+H5FD_multi_set_eoa(H5FD_t *_file, H5FD_mem_t type, haddr_t eoa)
 {
     H5FD_multi_t	*file = (H5FD_multi_t*)_file;
-    H5FD_mem_t		mt, mmt;
     herr_t		status;
     static const char *func="H5FD_multi_set_eoa";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
-    /* Find the subfile in which the new EOA value falls */
-    for (mt=H5FD_MEM_SUPER; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
-	mmt = file->fa.memb_map[mt];
-	if (H5FD_MEM_DEFAULT==mmt) mmt = mt;
-	assert(mmt>0 && mmt<H5FD_MEM_NTYPES);
+    H5E_BEGIN_TRY {
+	status = H5FDset_eoa(file->memb[type], type, eoa);
+    } H5E_END_TRY;
+    if (status<0)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_FILE, H5E_BADVALUE, "member H5FDset_eoa failed", -1)
 
-	if (eoa>=file->fa.memb_addr[mmt] && eoa<file->memb_next[mmt]) {
-	    break;
-	}
-    }
-    assert(mt<H5FD_MEM_NTYPES);
-
-    /* Set subfile eoa */
-    if (file->memb[mmt]) {
-	H5E_BEGIN_TRY {
-	    status = H5FDset_eoa(file->memb[mmt], eoa-file->fa.memb_addr[mmt]);
-	} H5E_END_TRY;
-	if (status<0)
-            H5Epush_ret(func, H5E_FILE, H5E_BADVALUE, "member H5FDset_eoa failed", -1)
-    }
-
-    /* Save new eoa for return later */
-    file->eoa = eoa;
     return 0;
 }
 
@@ -1523,44 +1579,61 @@ H5FD_multi_set_eoa(H5FD_t *_file, haddr_t eoa)
  *              Wednesday, August  4, 1999
  *
  * Modifications:
+ *              Raymond Lu
+ *              5 January 2007
+ *              Multi driver no longer has EOA for the whole file.  Calculate
+ *              it in the same way as EOF instead.
  *
  *-------------------------------------------------------------------------
  */
 static haddr_t
-H5FD_multi_get_eof(H5FD_t *_file)
+H5FD_multi_get_eof(const H5FD_t *_file)
 {
-    H5FD_multi_t	*file = (H5FD_multi_t*)_file;
-    haddr_t		eof=0, tmp;
+    const H5FD_multi_t	*file = (const H5FD_multi_t*)_file;
+    haddr_t		eof=0, tmp_eof;
+    haddr_t		eoa=0, tmp_eoa;
     static const char *func="H5FD_multi_eof";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     UNIQUE_MEMBERS(file->fa.memb_map, mt) {
 	if (file->memb[mt]) {
+            /* Retrieve EOF */
 	    H5E_BEGIN_TRY {
-		tmp = H5FDget_eof(file->memb[mt]);
+		tmp_eof = H5FDget_eof(file->memb[mt]);
 	    } H5E_END_TRY;
-	    if (HADDR_UNDEF==tmp)
-                H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "member file has unknown eof", HADDR_UNDEF)
-	    if (tmp>0) tmp += file->fa.memb_addr[mt];
 
+	    if (HADDR_UNDEF==tmp_eof)
+                H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "member file has unknown eof", HADDR_UNDEF)
+	    if (tmp_eof>0) tmp_eof += file->fa.memb_addr[mt];
+
+            /* Retrieve EOA */
+	    H5E_BEGIN_TRY {
+		tmp_eoa = H5FDget_eoa(file->memb[mt], mt);
+	    } H5E_END_TRY;
+
+	    if (HADDR_UNDEF==tmp_eoa)
+                H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "member file has unknown eoa", HADDR_UNDEF)
 	} else if (file->fa.relax) {
 	    /*
 	     * The member is not open yet (maybe it doesn't exist). Make the
 	     * best guess about the end-of-file.
 	     */
-	    tmp = file->memb_next[mt];
-	    assert(HADDR_UNDEF!=tmp);
+	    tmp_eof = file->memb_next[mt];
+	    assert(HADDR_UNDEF!=tmp_eof);
 
+	    tmp_eoa = file->memb_next[mt];
+	    assert(HADDR_UNDEF!=tmp_eoa);
 	} else {
-            H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "bad eof", HADDR_UNDEF)
+            H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "bad eof", HADDR_UNDEF)
 	}
 
-	if (tmp>eof) eof = tmp;
+	if (tmp_eof>eof) eof = tmp_eof;
+	if (tmp_eoa>eoa) eoa = tmp_eoa;
     } END_MEMBERS;
 
-    return MAX(file->eoa, eof);
+    return MAX(eoa, eof);
 }
 
 
@@ -1587,9 +1660,9 @@ H5FD_multi_get_handle(H5FD_t *_file, hid_t fapl, void** file_handle)
 
     /* Get data type for multi driver */
     if(H5Pget_multi_type(fapl, &type) < 0)
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "can't get data type for multi driver", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "can't get data type for multi driver", -1)
     if(type<H5FD_MEM_DEFAULT || type>=H5FD_MEM_NTYPES)
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "data type is out of range", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "data type is out of range", -1)
     mmt = file->fa.memb_map[type];
     if(H5FD_MEM_DEFAULT==mmt) mmt = type;
 
@@ -1625,9 +1698,23 @@ H5FD_multi_alloc(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, hsize_t size)
     if (H5FD_MEM_DEFAULT==mmt) mmt = type;
 
     if (HADDR_UNDEF==(addr=H5FDalloc(file->memb[mmt], type, dxpl_id, size)))
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "member file can't alloc", HADDR_UNDEF)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "member file can't alloc", HADDR_UNDEF)
     addr += file->fa.memb_addr[mmt];
-    if (addr+size>file->eoa) file->eoa = addr+size;
+
+/*#ifdef TMP
+    if ( addr + size > file->eoa ) {
+
+	if ( H5FD_multi_set_eoa(_file, addr + size) < 0 ) {
+
+            H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, \
+			"can't set eoa", HADDR_UNDEF)
+	}
+    }
+#else
+    if ( addr + size > file->eoa )
+	file->eoa = addr + size;
+#endif */
+
     return addr;
 }
 
@@ -1655,7 +1742,7 @@ H5FD_multi_free(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr, hsi
     H5FD_mem_t		mmt;
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     mmt = file->fa.memb_map[type];
     if (H5FD_MEM_DEFAULT==mmt) mmt = type;
@@ -1695,7 +1782,7 @@ H5FD_multi_read(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr, siz
     haddr_t		start_addr=0;
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Get the data transfer properties */
     if (H5P_FILE_ACCESS_DEFAULT!=dxpl_id && H5FD_MULTI==H5Pget_driver(dxpl_id)) {
@@ -1750,7 +1837,7 @@ H5FD_multi_write(H5FD_t *_file, H5FD_mem_t type, hid_t dxpl_id, haddr_t addr, si
     haddr_t		start_addr=0;
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Get the data transfer properties */
     if (H5P_FILE_ACCESS_DEFAULT!=dxpl_id && H5FD_MULTI==H5Pget_driver(dxpl_id)) {
@@ -1823,7 +1910,7 @@ H5FD_multi_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing)
 
     for (mt=1; mt<H5FD_MEM_NTYPES; mt++) {
 	if (HADDR_UNDEF!=file->memb_addr[mt]) {
-	    haddr_t eoa = H5FDget_eoa(file->memb[mt]);
+	    haddr_t eoa = H5FDget_eoa(file->memb[mt], mt);
 	    fprintf(stderr, "    %6d %20llu %20llu %20llu %s\n",
 		    (int)mt, (unsigned long_long)(file->memb_addr[mt]),
 		    (unsigned long_long)eoa,
@@ -1834,7 +1921,7 @@ H5FD_multi_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing)
 #endif
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     /* Flush each file */
     for (mt=H5FD_MEM_SUPER; mt<H5FD_MEM_NTYPES; mt=(H5FD_mem_t)(mt+1)) {
@@ -1845,7 +1932,7 @@ H5FD_multi_flush(H5FD_t *_file, hid_t dxpl_id, unsigned closing)
 	}
     }
     if (nerrors)
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "error flushing member files", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "error flushing member files", -1)
 
     return 0;
 }
@@ -1872,7 +1959,7 @@ static int
 compute_next(H5FD_multi_t *file)
 {
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     ALL_MEMBERS(mt) {
 	file->memb_next[mt] = HADDR_UNDEF;
@@ -1919,7 +2006,7 @@ open_members(H5FD_multi_t *file)
     static const char *func="(H5FD_multi)open_members";  /* Function Name for error reporting */
 
     /* Clear the error stack */
-    H5Eclear();
+    H5Eclear2(H5E_DEFAULT);
 
     UNIQUE_MEMBERS(file->fa.memb_map, mt) {
 	if (file->memb[mt]) continue; /*already open*/
@@ -1949,7 +2036,7 @@ open_members(H5FD_multi_t *file)
 	}
     } END_MEMBERS;
     if (nerrors)
-        H5Epush_ret(func, H5E_INTERNAL, H5E_BADVALUE, "error opening member files", -1)
+        H5Epush_ret(func, H5E_ERR_CLS, H5E_INTERNAL, H5E_BADVALUE, "error opening member files", -1)
 
     return 0;
 }
