@@ -29,9 +29,10 @@
 #include "H5MMprivate.h"	/* Memory management			*/
 
 /* PRIVATE PROTOTYPES */
-static herr_t H5F_mount(H5G_entry_t *loc, const char *name, H5F_t *child,
+static herr_t H5F_mount(H5G_loc_t *loc, const char *name, H5F_t *child,
     hid_t plist_id, hid_t dxpl_id);
-static herr_t H5F_unmount(H5G_entry_t *loc, const char *name, hid_t dxpl_id);
+static herr_t H5F_unmount(H5G_loc_t *loc, const char *name, hid_t dxpl_id);
+static void H5F_mount_count_ids_recurse(H5F_t *f, unsigned *nopen_files, unsigned *nopen_objs);
 
 
 /*--------------------------------------------------------------------------
@@ -44,7 +45,7 @@ RETURNS
     Non-negative on success/Negative on failure
 DESCRIPTION
     Initializes any interface-specific data or routines.  (Just calls
-    H5F_init_iterface currently).
+    H5F_init() currently).
 
 --------------------------------------------------------------------------*/
 static herr_t
@@ -113,7 +114,7 @@ done:
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5F_mount(H5G_entry_t *loc, const char *name, H5F_t *child,
+H5F_mount(H5G_loc_t *loc, const char *name, H5F_t *child,
 	  hid_t UNUSED plist_id, hid_t dxpl_id)
 {
     H5G_t	*mount_point = NULL;	/*mount point group		*/
@@ -121,9 +122,10 @@ H5F_mount(H5G_entry_t *loc, const char *name, H5F_t *child,
     H5F_t	*parent = NULL;		/*file containing mount point	*/
     unsigned	lt, rt, md;		/*binary search indices		*/
     int		cmp;			/*binary search comparison value*/
-    H5G_entry_t	*mp_ent = NULL;		/*mount point symbol table entry*/
-    H5G_entry_t mp_open_ent;     	/*entry of moint point to be opened */
-    H5G_entry_t *root_ent;              /* Symbol table for root of file to mount */
+    H5G_loc_t   mp_loc;                 /* entry of moint point to be opened */
+    H5G_name_t  mp_path;            	/* Mount point group hier. path */
+    H5O_loc_t   mp_oloc;            	/* Mount point object location */
+    H5G_loc_t   root_loc;               /* Group location of root of file to mount */
     herr_t	ret_value = SUCCEED;	/*return value			*/
 
     FUNC_ENTER_NOAPI_NOINIT(H5F_mount)
@@ -131,18 +133,32 @@ H5F_mount(H5G_entry_t *loc, const char *name, H5F_t *child,
     HDassert(loc);
     HDassert(name && *name);
     HDassert(child);
-    HDassert(TRUE == H5P_isa_class(plist_id,H5P_MOUNT));
+    HDassert(TRUE == H5P_isa_class(plist_id, H5P_FILE_MOUNT));
+
+    /* Set up dataset location to fill in */
+    mp_loc.oloc = &mp_oloc;
+    mp_loc.path = &mp_path;
+    H5G_loc_reset(&mp_loc);
 
     /*
      * Check that the child isn't mounted, that the mount point exists, that
+     * the mount point wasn't reached via external link, that
      * the parent & child files have the same file close degree, and
      * that the mount wouldn't introduce a cycle in the mount tree.
      */
     if(child->mtab.parent)
         HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "file is already mounted")
-    if(H5G_find(loc, name, &mp_open_ent/*out*/, H5AC_dxpl_id) < 0)
+    if(H5G_loc_find(loc, name, &mp_loc/*out*/, H5P_DEFAULT, dxpl_id) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "group not found")
-    if(NULL == (mount_point = H5G_open(&mp_open_ent, dxpl_id)))
+    /* If the mount location is holding its file open, that file will close
+     * and remove the mount as soon as we exit this function.  Prevent the
+     * user from doing this.
+     */
+    if(mp_loc.oloc->holding_file != FALSE)
+        HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "mount path cannot contain links to external files")    
+
+    /* Open the mount point group */
+    if(NULL == (mount_point = H5G_open(&mp_loc, dxpl_id)))
         HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "mount point not found")
 
     /* Retrieve information from the mount point group */
@@ -151,8 +167,10 @@ H5F_mount(H5G_entry_t *loc, const char *name, H5F_t *child,
      */
     parent = H5G_fileof(mount_point);
     HDassert(parent);
-    mp_ent = H5G_entof(mount_point);
-    HDassert(mp_ent);
+    mp_loc.oloc = H5G_oloc(mount_point);
+    HDassert(mp_loc.oloc);
+    mp_loc.path = H5G_nameof(mount_point);
+    HDassert(mp_loc.path);
     for(ancestor = parent; ancestor; ancestor = ancestor->mtab.parent) {
 	if(ancestor == child)
 	    HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "mount would introduce a cycle")
@@ -171,11 +189,11 @@ H5F_mount(H5G_entry_t *loc, const char *name, H5F_t *child,
     rt = parent->mtab.nmounts;
     cmp = -1;
     while(lt < rt && cmp) {
-        H5G_entry_t	*ent;		/*temporary symbol table entry	*/
+        H5O_loc_t	*oloc;		/*temporary symbol table entry	*/
 
 	md = (lt + rt) / 2;
-	ent = H5G_entof(parent->mtab.child[md].group);
-	cmp = H5F_addr_cmp(mp_ent->header, ent->header);
+	oloc = H5G_oloc(parent->mtab.child[md].group);
+	cmp = H5F_addr_cmp(mp_loc.oloc->addr, oloc->addr);
 	if(cmp < 0)
 	    rt = md;
 	else if(cmp > 0)
@@ -187,11 +205,11 @@ H5F_mount(H5G_entry_t *loc, const char *name, H5F_t *child,
 	HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "mount point is already in use")
 
     /* Make room in the table */
-    if(parent->mtab.nmounts>=parent->mtab.nalloc) {
-	unsigned n = MAX(16, 2*parent->mtab.nalloc);
+    if(parent->mtab.nmounts >= parent->mtab.nalloc) {
+	unsigned n = MAX(16, 2 * parent->mtab.nalloc);
 	H5F_mount_t *x = H5MM_realloc(parent->mtab.child,
 				      n * sizeof(parent->mtab.child[0]));
-	if (!x)
+	if(!x)
 	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed for mount table")
 	parent->mtab.child = x;
 	parent->mtab.nalloc = n;
@@ -210,18 +228,29 @@ H5F_mount(H5G_entry_t *loc, const char *name, H5F_t *child,
         HGOTO_ERROR(H5E_FILE, H5E_CANTCLOSEOBJ, FAIL, "unable to set group mounted flag")
 
     /* Get the group location for the root group in the file to unmount */
-    if(NULL == (root_ent = H5G_entof(child->shared->root_grp)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "unable to get entry for root group")
+    if(NULL == (root_loc.oloc = H5G_oloc(child->shared->root_grp)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "unable to get object location for root group")
+    if(NULL == (root_loc.path = H5G_nameof(child->shared->root_grp)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "unable to get path for root group")
 
     /* Search the open IDs and replace names for mount operation */
     /* We pass H5G_UNKNOWN as object type; search all IDs */
-    if(H5G_name_replace(H5G_UNKNOWN, mp_ent, NULL, root_ent, H5G_NAME_MOUNT) < 0)
+    if(H5G_name_replace(NULL, H5G_NAME_MOUNT, mp_loc.oloc->file,
+            mp_loc.path->full_path_r, root_loc.oloc->file, root_loc.path->full_path_r,
+            dxpl_id) < 0)
 	HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "unable to replace name")
 
 done:
-    if(ret_value < 0 && mount_point)
-	if(H5G_close(mount_point) < 0)
-            HDONE_ERROR(H5E_FILE, H5E_CANTCLOSEOBJ, FAIL, "unable to close mounted group")
+    if(ret_value < 0) {
+        if(mount_point) {
+            if(H5G_close(mount_point) < 0)
+                HDONE_ERROR(H5E_FILE, H5E_CANTCLOSEOBJ, FAIL, "unable to close mounted group")
+        } /* end if */
+        else {
+            if(H5G_loc_free(&mp_loc) < 0)
+                HDONE_ERROR(H5E_SYM, H5E_CANTRELEASE, FAIL, "unable to free mount location")
+        } /* end else */
+    } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5F_mount() */
@@ -246,16 +275,17 @@ done:
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5F_unmount(H5G_entry_t *loc, const char *name, hid_t dxpl_id)
+H5F_unmount(H5G_loc_t *loc, const char *name, hid_t dxpl_id)
 {
-    H5G_t	*mounted = NULL;	/*mount point group		*/
     H5G_t	*child_group = NULL;	/* Child's group in parent mtab	*/
     H5F_t	*child = NULL;		/*mounted file			*/
     H5F_t	*parent = NULL;		/*file where mounted		*/
-    H5G_entry_t	*mnt_ent = NULL;	/*mounted symbol table entry	*/
-    H5G_entry_t	*ent = NULL;		/*temporary symbol table entry	*/
-    H5G_entry_t mnt_open_ent;           /* entry used to open mount point*/
-    H5G_entry_t *root_ent;              /* Symbol table for root of file to mount */
+    H5O_loc_t   *mnt_oloc;            	/* symbol table entry for root of mounted file */
+    H5G_name_t  mp_path;            	/* Mount point group hier. path */
+    H5O_loc_t   mp_oloc;            	/* Mount point object location  */
+    H5G_loc_t   mp_loc;                 /* entry used to open mount point*/
+    hbool_t     mp_loc_setup = FALSE;   /* Whether mount point location is set up */
+    H5G_loc_t   root_loc;               /* Group location of root of file to unmount */
     int         child_idx;              /* Index of child in parent's mtab */
     herr_t	ret_value = SUCCEED;	/*return value			*/
 
@@ -264,21 +294,24 @@ H5F_unmount(H5G_entry_t *loc, const char *name, hid_t dxpl_id)
     HDassert(loc);
     HDassert(name && *name);
 
+    /* Set up mount point location to fill in */
+    mp_loc.oloc = &mp_oloc;
+    mp_loc.path = &mp_path;
+    H5G_loc_reset(&mp_loc);
+
     /*
      * Get the mount point, or more precisely the root of the mounted file.
      * If we get the root group and the file has a parent in the mount tree,
      * then we must have found the mount point.
      */
-    if(H5G_find(loc, name, &mnt_open_ent/*out*/, H5AC_dxpl_id) < 0)
+    if(H5G_loc_find(loc, name, &mp_loc/*out*/, H5P_DEFAULT, dxpl_id) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "group not found")
-    if(NULL == (mounted = H5G_open(&mnt_open_ent, dxpl_id)))
-        HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "mount point not found")
-    child = H5G_fileof(mounted);
-    mnt_ent = H5G_entof(mounted);
-    ent = H5G_entof(child->shared->root_grp);
+    mp_loc_setup = TRUE;
+    child = mp_loc.oloc->file;
+    mnt_oloc = H5G_oloc(child->shared->root_grp);
     child_idx = -1;
 
-    if(child->mtab.parent && H5F_addr_eq(mnt_ent->header, ent->header)) {
+    if(child->mtab.parent && H5F_addr_eq(mp_oloc.addr, mnt_oloc->addr)) {
         unsigned	u;		/*counters			*/
 
 	/*
@@ -307,8 +340,8 @@ H5F_unmount(H5G_entry_t *loc, const char *name, hid_t dxpl_id)
 	cmp = -1;
 	while(lt < rt && cmp) {
 	    md = (lt + rt) / 2;
-	    ent = H5G_entof(parent->mtab.child[md].group);
-	    cmp = H5F_addr_cmp(mnt_ent->header, ent->header);
+	    mnt_oloc = H5G_oloc(parent->mtab.child[md].group);
+	    cmp = H5F_addr_cmp(mp_oloc.addr, mnt_oloc->addr);
 	    if (cmp<0)
 		rt = md;
 	    else
@@ -319,7 +352,10 @@ H5F_unmount(H5G_entry_t *loc, const char *name, hid_t dxpl_id)
 
         /* Found the correct index, set the info about the child */
         child_idx = md;
-        mnt_ent = ent;
+        H5G_loc_free(&mp_loc);
+        mp_loc_setup = FALSE;
+        mp_loc.oloc = mnt_oloc;
+        mp_loc.path = H5G_nameof(parent->mtab.child[md].group);
         child = parent->mtab.child[child_idx].file;
     } /* end else */
     HDassert(child_idx >= 0);
@@ -328,11 +364,15 @@ H5F_unmount(H5G_entry_t *loc, const char *name, hid_t dxpl_id)
     child_group = parent->mtab.child[child_idx].group;
 
     /* Get the group location for the root group in the file to unmount */
-    if(NULL == (root_ent = H5G_entof(child->shared->root_grp)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "unable to get entry for root group")
+    if(NULL == (root_loc.oloc = H5G_oloc(child->shared->root_grp)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "unable to get object location for root group")
+    if(NULL == (root_loc.path = H5G_nameof(child->shared->root_grp)))
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "unable to get path for root group")
 
     /* Search the open IDs replace names to reflect unmount operation */
-    if(H5G_name_replace(H5G_UNKNOWN, mnt_ent, NULL, root_ent, H5G_NAME_UNMOUNT )<0)
+    if(H5G_name_replace(NULL, H5G_NAME_UNMOUNT, mp_loc.oloc->file,
+            mp_loc.path->full_path_r, root_loc.oloc->file, root_loc.path->full_path_r,
+            dxpl_id) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "unable to replace name")
 
     /* Eliminate the mount point from the table */
@@ -352,86 +392,13 @@ H5F_unmount(H5G_entry_t *loc, const char *name, hid_t dxpl_id)
         HGOTO_ERROR(H5E_FILE, H5E_CANTCLOSEFILE, FAIL, "unable to close unmounted file")
 
 done:
-    if(mounted)
-        if(H5G_close(mounted) < 0 && ret_value >= 0)
-	    HDONE_ERROR(H5E_FILE, H5E_CANTCLOSEOBJ, FAIL, "can't close group")
+    /* Free the mount point location's information, if it's been set up */
+    if(mp_loc_setup)
+        H5G_loc_free(&mp_loc);
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5F_unmount() */
 
-
-/*-------------------------------------------------------------------------
- * Function:	H5F_mountpoint
- *
- * Purpose:	If ENT is a mount point then copy the entry for the root
- *		group of the mounted file into ENT.
- *
- * Return:	Non-negative on success/Negative on failure
- *
- * Programmer:	Robb Matzke
- *              Tuesday, October  6, 1998
- *
- * Modifications:
- *
- *      Pedro Vicente, <pvn@ncsa.uiuc.edu> 22 Aug 2002
- *      Added `id to name' support.
- *
- *-------------------------------------------------------------------------
- */
-herr_t
-H5F_mountpoint(H5G_entry_t *find/*in,out*/)
-{
-    H5F_t	*parent = find->file;
-    unsigned	lt, rt, md=0;
-    int cmp;
-    H5G_entry_t	*ent = NULL;
-    herr_t ret_value=SUCCEED;   /* Return value */
-
-    FUNC_ENTER_NOAPI(H5F_mountpoint, FAIL)
-
-    assert(find);
-
-    /*
-     * The loop is necessary because we might have file1 mounted at the root
-     * of file2, which is mounted somewhere in file3.
-     */
-    do {
-	/*
-	 * Use a binary search to find the potential mount point in the mount
-	 * table for the parent
-	 */
-	lt = 0;
-	rt = parent->mtab.nmounts;
-	cmp = -1;
-	while (lt<rt && cmp) {
-	    md = (lt+rt)/2;
-	    ent = H5G_entof(parent->mtab.child[md].group);
-	    cmp = H5F_addr_cmp(find->header, ent->header);
-	    if (cmp<0) {
-		rt = md;
-	    } else {
-		lt = md+1;
-	    }
-	}
-
-	/* Copy root info over to ENT */
-	if(0 == cmp) {
-            /* Get the entry for the root group in the child's file */
-	    ent = H5G_entof(parent->mtab.child[md].file->shared->root_grp);
-
-            /* Don't lose the paths of the group when we copy the root group's entry */
-            if(H5G_ent_copy(find, ent, H5_COPY_LIMITED) < 0)
-                HGOTO_ERROR(H5E_FILE, H5E_CANTCOPY, FAIL, "unable to copy group entry")
-
-            /* Switch to child's file */
-	    parent = ent->file;
-	} /* end if */
-    } while (!cmp);
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
-}
-
 
 /*-------------------------------------------------------------------------
  * Function:	H5F_has_mount
@@ -511,28 +478,28 @@ H5F_is_mount(const H5F_t *file)
 herr_t
 H5Fmount(hid_t loc_id, const char *name, hid_t child_id, hid_t plist_id)
 {
-    H5G_entry_t		*loc = NULL;
+    H5G_loc_t	loc;
     H5F_t	*child = NULL;
     herr_t      ret_value=SUCCEED;       /* Return value */
 
     FUNC_ENTER_API(H5Fmount, FAIL)
-    H5TRACE4("e","isii",loc_id,name,child_id,plist_id);
+    H5TRACE4("e", "i*sii", loc_id, name, child_id, plist_id);
 
     /* Check arguments */
-    if(NULL == (loc = H5G_loc(loc_id)))
+    if(H5G_loc(loc_id, &loc) < 0)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a location")
     if(!name || !*name)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no name")
     if(NULL == (child = H5I_object_verify(child_id,H5I_FILE)))
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a file")
     if(H5P_DEFAULT == plist_id)
-        plist_id = H5P_MOUNT_DEFAULT;
+        plist_id = H5P_FILE_MOUNT_DEFAULT;
     else
-        if(TRUE != H5P_isa_class(plist_id, H5P_MOUNT))
+        if(TRUE != H5P_isa_class(plist_id, H5P_FILE_MOUNT))
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not property list")
 
     /* Do the mount */
-    if(H5F_mount(loc, name, child, plist_id, H5AC_dxpl_id) < 0)
+    if(H5F_mount(&loc, name, child, plist_id, H5AC_dxpl_id) < 0)
 	HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "unable to mount file")
 
 done:
@@ -562,20 +529,20 @@ done:
 herr_t
 H5Funmount(hid_t loc_id, const char *name)
 {
-    H5G_entry_t		*loc = NULL;
+    H5G_loc_t	loc;
     herr_t      ret_value=SUCCEED;       /* Return value */
 
     FUNC_ENTER_API(H5Funmount, FAIL)
-    H5TRACE2("e","is",loc_id,name);
+    H5TRACE2("e", "i*s", loc_id, name);
 
     /* Check args */
-    if (NULL==(loc=H5G_loc(loc_id)))
+    if(H5G_loc(loc_id, &loc) < 0)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a location")
     if(!name || !*name)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no name")
 
     /* Unmount */
-    if (H5F_unmount(loc, name, H5AC_dxpl_id) < 0)
+    if (H5F_unmount(&loc, name, H5AC_dxpl_id) < 0)
 	HGOTO_ERROR(H5E_FILE, H5E_MOUNT, FAIL, "unable to unmount file")
 
 done:
