@@ -29,11 +29,14 @@
 #include "H5Opkg.h"             /* Object headers			*/
 
 /* PRIVATE PROTOTYPES */
-static void *H5O_efl_decode(H5F_t *f, hid_t dxpl_id, const uint8_t *p);
-static herr_t H5O_efl_encode(H5F_t *f, uint8_t *p, const void *_mesg);
-static void *H5O_efl_copy(const void *_mesg, void *_dest, unsigned update_flags);
-static size_t H5O_efl_size(const H5F_t *f, const void *_mesg);
+static void *H5O_efl_decode(H5F_t *f, hid_t dxpl_id, unsigned mesg_flags, const uint8_t *p);
+static herr_t H5O_efl_encode(H5F_t *f, hbool_t disable_shared, uint8_t *p, const void *_mesg);
+static void *H5O_efl_copy(const void *_mesg, void *_dest);
+static size_t H5O_efl_size(const H5F_t *f, hbool_t disable_shared, const void *_mesg);
 static herr_t H5O_efl_reset(void *_mesg);
+static void *H5O_efl_copy_file(H5F_t *file_src, void *mesg_src,
+    H5F_t *file_dst, hbool_t *recompute_size, H5O_copy_t *cpy_info,
+    void *udata, hid_t dxpl_id);
 static herr_t H5O_efl_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg, FILE * stream,
 			    int indent, int fwidth);
 
@@ -42,6 +45,7 @@ const H5O_msg_class_t H5O_MSG_EFL[1] = {{
     H5O_EFL_ID,		    	/*message id number		*/
     "external file list",   	/*message name for debugging    */
     sizeof(H5O_efl_t),	    	/*native message size	    	*/
+    0,				/* messages are sharable?       */
     H5O_efl_decode,	    	/*decode message		*/
     H5O_efl_encode,	    	/*encode message		*/
     H5O_efl_copy,	    	/*copy native value		*/
@@ -50,8 +54,13 @@ const H5O_msg_class_t H5O_MSG_EFL[1] = {{
     NULL,		            /* free method			*/
     NULL,		        /* file delete method		*/
     NULL,			/* link method			*/
-    NULL,	  	    	/*get share method		*/
     NULL,			/*set share method		*/
+    NULL,		    	/*can share method		*/
+    NULL,			/* pre copy native value to file */
+    H5O_efl_copy_file,		/* copy native value to file    */
+    NULL,			/* post copy native value to file    */
+    NULL,			/* get creation index		*/
+    NULL,			/* set creation index		*/
     H5O_efl_debug	    	/*debug the message		*/
 }};
 
@@ -71,35 +80,32 @@ const H5O_msg_class_t H5O_MSG_EFL[1] = {{
  * Programmer:	Robb Matzke
  *		Tuesday, November 25, 1997
  *
- * Modifications:
- *	Robb Matzke, 1998-07-20
- *	Rearranged the message to add a version number near the beginning.
- *
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_efl_decode(H5F_t *f, hid_t dxpl_id, const uint8_t *p)
+H5O_efl_decode(H5F_t *f, hid_t dxpl_id, unsigned UNUSED mesg_flags,
+    const uint8_t *p)
 {
     H5O_efl_t		*mesg = NULL;
     int			version;
     const char		*s = NULL;
-    const H5HL_t        *heap;
+    H5HL_t              *heap;
     size_t		u;      /* Local index variable */
     void *ret_value;            /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_decode);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_decode)
 
     /* Check args */
-    assert(f);
-    assert(p);
+    HDassert(f);
+    HDassert(p);
 
-    if (NULL==(mesg = H5MM_calloc(sizeof(H5O_efl_t))))
-	HGOTO_ERROR (H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed");
+    if(NULL == (mesg = H5MM_calloc(sizeof(H5O_efl_t))))
+	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
     /* Version */
     version = *p++;
-    if (version!=H5O_EFL_VERSION)
-	HGOTO_ERROR(H5E_OHDR, H5E_CANTLOAD, NULL, "bad version number for external file list message");
+    if(version != H5O_EFL_VERSION)
+	HGOTO_ERROR(H5E_OHDR, H5E_CANTLOAD, NULL, "bad version number for external file list message")
 
     /* Reserved */
     p += 3;
@@ -114,56 +120,58 @@ H5O_efl_decode(H5F_t *f, hid_t dxpl_id, const uint8_t *p)
     H5F_addr_decode(f, &p, &(mesg->heap_addr));
 
 #ifndef NDEBUG
-    assert (H5F_addr_defined(mesg->heap_addr));
+    HDassert(H5F_addr_defined(mesg->heap_addr));
 
-    if (NULL == (heap = H5HL_protect(f, dxpl_id, mesg->heap_addr)))
+    if(NULL == (heap = H5HL_protect(f, dxpl_id, mesg->heap_addr, H5AC_READ)))
         HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read protect link value")
 
     s = H5HL_offset_into(f, heap, 0);
 
-    assert (s && !*s);
+    HDassert(s && !*s);
 
-    if (H5HL_unprotect(f, dxpl_id, heap, mesg->heap_addr) < 0)
+    if(H5HL_unprotect(f, dxpl_id, heap, mesg->heap_addr) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read unprotect link value")
+    heap = NULL;
 #endif
 
     /* Decode the file list */
-    mesg->slot = H5MM_calloc(mesg->nalloc*sizeof(H5O_efl_entry_t));
-    if (NULL==mesg->slot)
-	HGOTO_ERROR (H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed");
-    for (u=0; u<mesg->nused; u++) {
+    mesg->slot = H5MM_calloc(mesg->nalloc * sizeof(H5O_efl_entry_t));
+    if(NULL == mesg->slot)
+	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+
+    if(NULL == (heap = H5HL_protect(f, dxpl_id, mesg->heap_addr, H5AC_READ)))
+        HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read protect link value")
+    for(u = 0; u < mesg->nused; u++) {
 	/* Name */
 	H5F_DECODE_LENGTH (f, p, mesg->slot[u].name_offset);
 
-        if (NULL == (heap = H5HL_protect(f, dxpl_id, mesg->heap_addr)))
-            HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read protect link value")
-
         s = H5HL_offset_into(f, heap, mesg->slot[u].name_offset);
-	assert (s && *s);
+	HDassert(s && *s);
 	mesg->slot[u].name = H5MM_xstrdup (s);
-        assert(mesg->slot[u].name);
-
-        if (H5HL_unprotect(f, dxpl_id, heap, mesg->heap_addr) < 0)
-            HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read unprotect link value")
+        HDassert(mesg->slot[u].name);
 
 	/* File offset */
 	H5F_DECODE_LENGTH (f, p, mesg->slot[u].offset);
 
 	/* Size */
 	H5F_DECODE_LENGTH (f, p, mesg->slot[u].size);
-	assert (mesg->slot[u].size>0);
-    }
+	HDassert(mesg->slot[u].size > 0);
+    } /* end for */
+
+    if(H5HL_unprotect(f, dxpl_id, heap, mesg->heap_addr) < 0)
+        HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, NULL, "unable to read unprotect link value")
+    heap = NULL;
 
     /* Set return value */
-    ret_value=mesg;
+    ret_value = mesg;
 
 done:
-    if(ret_value==NULL) {
-        if(mesg!=NULL)
-            H5MM_xfree (mesg);
-    } /* end if */
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    if(ret_value == NULL)
+        if(mesg != NULL)
+            H5MM_xfree(mesg);
+
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_efl_decode() */
 
 
 /*-------------------------------------------------------------------------
@@ -176,29 +184,20 @@ done:
  * Programmer:	Robb Matzke
  *		Tuesday, November 25, 1997
  *
- * Modifications:
- *	Robb Matzke, 1998-07-20
- *	Rearranged the message to add a version number near the beginning.
- *
- * 	Robb Matzke, 1999-10-14
- *	Entering the name into the local heap happens when the dataset is
- *	created. Otherwise we could end up in infinite recursion if the heap
- *	happens to hash to the same cache slot as the object header.
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_efl_encode(H5F_t *f, uint8_t *p, const void *_mesg)
+H5O_efl_encode(H5F_t *f, hbool_t UNUSED disable_shared, uint8_t *p, const void *_mesg)
 {
     const H5O_efl_t	*mesg = (const H5O_efl_t *)_mesg;
     size_t		u;      /* Local index variable */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_encode);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_encode)
 
     /* check args */
-    assert(f);
-    assert(mesg);
-    assert(p);
+    HDassert(f);
+    HDassert(mesg);
+    HDassert(p);
 
     /* Version */
     *p++ = H5O_EFL_VERSION;
@@ -209,29 +208,29 @@ H5O_efl_encode(H5F_t *f, uint8_t *p, const void *_mesg)
     *p++ = 0;
 
     /* Number of slots */
-    assert (mesg->nalloc>0);
+    HDassert(mesg->nalloc > 0);
     UINT16ENCODE(p, mesg->nused); /*yes, twice*/
-    assert (mesg->nused>0 && mesg->nused<=mesg->nalloc);
+    HDassert(mesg->nused > 0 && mesg->nused <= mesg->nalloc);
     UINT16ENCODE(p, mesg->nused);
 
     /* Heap address */
-    assert (H5F_addr_defined(mesg->heap_addr));
+    HDassert(H5F_addr_defined(mesg->heap_addr));
     H5F_addr_encode(f, &p, mesg->heap_addr);
 
     /* Encode file list */
-    for (u=0; u<mesg->nused; u++) {
+    for(u = 0; u < mesg->nused; u++) {
 	/*
 	 * The name should have been added to the heap when the dataset was
 	 * created.
 	 */
-	assert(mesg->slot[u].name_offset);
-	H5F_ENCODE_LENGTH (f, p, mesg->slot[u].name_offset);
-	H5F_ENCODE_LENGTH (f, p, mesg->slot[u].offset);
-	H5F_ENCODE_LENGTH (f, p, mesg->slot[u].size);
-    }
+	HDassert(mesg->slot[u].name_offset);
+	H5F_ENCODE_LENGTH(f, p, mesg->slot[u].name_offset);
+	H5F_ENCODE_LENGTH(f, p, mesg->slot[u].offset);
+	H5F_ENCODE_LENGTH(f, p, mesg->slot[u].size);
+    } /* end for */
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
-}
+    FUNC_LEAVE_NOAPI(SUCCEED)
+} /* end H5O_efl_encode() */
 
 
 /*-------------------------------------------------------------------------
@@ -247,48 +246,44 @@ H5O_efl_encode(H5F_t *f, uint8_t *p, const void *_mesg)
  * Programmer:	Robb Matzke
  *		Tuesday, November 25, 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_efl_copy(const void *_mesg, void *_dest, unsigned UNUSED update_flags)
+H5O_efl_copy(const void *_mesg, void *_dest)
 {
     const H5O_efl_t	*mesg = (const H5O_efl_t *) _mesg;
     H5O_efl_t		*dest = (H5O_efl_t *) _dest;
     size_t		u;              /* Local index variable */
     void                *ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_copy);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_copy)
 
     /* check args */
-    assert(mesg);
-    if (!dest) {
-	if (NULL==(dest = H5MM_calloc(sizeof(H5O_efl_t))) ||
-                NULL==(dest->slot=H5MM_malloc(mesg->nalloc* sizeof(H5O_efl_entry_t))))
-	    HGOTO_ERROR (H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed");
-
-    } else if (dest->nalloc<mesg->nalloc) {
+    HDassert(mesg);
+    if(!dest) {
+	if(NULL == (dest = H5MM_calloc(sizeof(H5O_efl_t))) ||
+                NULL == (dest->slot = H5MM_malloc(mesg->nalloc * sizeof(H5O_efl_entry_t))))
+	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+    } else if(dest->nalloc < mesg->nalloc) {
 	H5MM_xfree(dest->slot);
-	if (NULL==(dest->slot = H5MM_malloc(mesg->nalloc*
-					    sizeof(H5O_efl_entry_t))))
-	    HGOTO_ERROR (H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed");
+	if(NULL == (dest->slot = H5MM_malloc(mesg->nalloc * sizeof(H5O_efl_entry_t))))
+	    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
     }
     dest->heap_addr = mesg->heap_addr;
     dest->nalloc = mesg->nalloc;
     dest->nused = mesg->nused;
 
-    for (u = 0; u < mesg->nused; u++) {
+    for(u = 0; u < mesg->nused; u++) {
 	dest->slot[u] = mesg->slot[u];
-	dest->slot[u].name = H5MM_xstrdup (mesg->slot[u].name);
-    }
+	dest->slot[u].name = H5MM_xstrdup(mesg->slot[u].name);
+    } /* end for */
 
     /* Set return value */
-    ret_value=dest;
+    ret_value = dest;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_efl_copy() */
 
 
 /*-------------------------------------------------------------------------
@@ -306,21 +301,19 @@ done:
  * Programmer:	Robb Matzke
  *		Tuesday, November 25, 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static size_t
-H5O_efl_size(const H5F_t *f, const void *_mesg)
+H5O_efl_size(const H5F_t *f, hbool_t UNUSED disable_shared, const void *_mesg)
 {
     const H5O_efl_t	*mesg = (const H5O_efl_t *) _mesg;
     size_t		ret_value = 0;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_size);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_size)
 
     /* check args */
-    assert(f);
-    assert(mesg);
+    HDassert(f);
+    HDassert(mesg);
 
     ret_value = H5F_SIZEOF_ADDR(f) +			/*heap address	*/
 		2 +					/*slots allocated*/
@@ -330,8 +323,8 @@ H5O_efl_size(const H5F_t *f, const void *_mesg)
 			       H5F_SIZEOF_SIZE(f) +	/*file offset	*/
 			       H5F_SIZEOF_SIZE(f));	/*file size	*/
 
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_efl_size() */
 
 
 /*-------------------------------------------------------------------------
@@ -345,8 +338,6 @@ H5O_efl_size(const H5F_t *f, const void *_mesg)
  * Programmer:	Robb Matzke
  *		Tuesday, November 25, 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
@@ -355,21 +346,21 @@ H5O_efl_reset(void *_mesg)
     H5O_efl_t	*mesg = (H5O_efl_t *) _mesg;
     size_t	u;              /* Local index variable */
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_reset);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_reset)
 
     /* check args */
-    assert(mesg);
+    HDassert(mesg);
 
     /* reset */
-    for (u=0; u<mesg->nused; u++)
-	mesg->slot[u].name = H5MM_xfree (mesg->slot[u].name);
+    for(u = 0; u < mesg->nused; u++)
+	mesg->slot[u].name = H5MM_xfree(mesg->slot[u].name);
     mesg->heap_addr = HADDR_UNDEF;
     mesg->nused = mesg->nalloc = 0;
     if(mesg->slot)
         mesg->slot = H5MM_xfree(mesg->slot);
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
-}
+    FUNC_LEAVE_NOAPI(SUCCEED)
+} /* end H5O_efl_reset() */
 
 
 /*-------------------------------------------------------------------------
@@ -385,8 +376,6 @@ H5O_efl_reset(void *_mesg)
  * Programmer:	Robb Matzke
  *              Tuesday, March  3, 1998
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 hsize_t
@@ -394,24 +383,112 @@ H5O_efl_total_size (H5O_efl_t *efl)
 {
     hsize_t	ret_value = 0, tmp;
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_total_size);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_total_size)
 
-    if (efl->nused>0 &&
-	H5O_EFL_UNLIMITED==efl->slot[efl->nused-1].size) {
+    if(efl->nused > 0 && H5O_EFL_UNLIMITED == efl->slot[efl->nused - 1].size)
 	ret_value = H5O_EFL_UNLIMITED;
-    } else {
+    else {
         size_t		u;      /* Local index variable */
 
-	for (u=0; u<efl->nused; u++, ret_value=tmp) {
+	for(u = 0; u < efl->nused; u++, ret_value = tmp) {
 	    tmp = ret_value + efl->slot[u].size;
-	    if (tmp<=ret_value)
-		HGOTO_ERROR (H5E_EFL, H5E_OVERFLOW, 0, "total external storage size overflowed");
-	}
-    }
+	    if(tmp <= ret_value)
+		HGOTO_ERROR(H5E_EFL, H5E_OVERFLOW, 0, "total external storage size overflowed");
+	} /* end for */
+    } /* end else */
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_efl_total_size() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5O_efl_copy_file
+ *
+ * Purpose:     Copies an efl message from _MESG to _DEST in file
+ *
+ * Return:      Success:        Ptr to _DEST
+ *
+ *              Failure:        NULL
+ *
+ * Programmer:  Peter Cao
+ *              September 29, 2005
+ *
+ *-------------------------------------------------------------------------
+ */
+static void *
+H5O_efl_copy_file(H5F_t UNUSED *file_src, void *mesg_src, H5F_t *file_dst,
+    hbool_t UNUSED *recompute_size, H5O_copy_t UNUSED *cpy_info,
+    void UNUSED *_udata, hid_t dxpl_id)
+{
+    H5O_efl_t   *efl_src = (H5O_efl_t *) mesg_src;
+    H5O_efl_t   *efl_dst = NULL;
+    H5HL_t      *heap = NULL;                           /* Pointer to local heap for EFL file names */
+    size_t      idx, size, name_offset, heap_size;
+    void        *ret_value;          /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT(H5O_efl_copy_file)
+
+    /* check args */
+    HDassert(efl_src);
+    HDassert(file_dst);
+
+    /* Allocate space for the destination efl */
+    if(NULL == (efl_dst = H5MM_calloc(sizeof(H5O_efl_t))))
+        HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+
+    /* Copy the "top level" information */
+    HDmemcpy(efl_dst, efl_src, sizeof(H5O_efl_t));
+
+    /* Determine size needed for destination heap */
+    heap_size = H5HL_ALIGN(1);  /* "empty" name */
+    for(idx = 0; idx < efl_src->nused; idx++)
+        heap_size += H5HL_ALIGN(HDstrlen(efl_src->slot[idx].name) + 1);
+
+    /* Create name heap */
+    if(H5HL_create(file_dst, dxpl_id, heap_size, &efl_dst->heap_addr/*out*/) < 0)
+        HGOTO_ERROR(H5E_EFL, H5E_CANTINIT, NULL, "can't create heap")
+
+    /* Pin the heap down in memory */
+    if(NULL == (heap = H5HL_protect(file_dst, dxpl_id, efl_dst->heap_addr, H5AC_WRITE)))
+        HGOTO_ERROR(H5E_EFL, H5E_PROTECT, NULL, "unable to protect EFL file name heap")
+
+    /* Insert "empty" name first */
+    if((size_t)(-1) == (name_offset = H5HL_insert(file_dst, dxpl_id, heap, (size_t)1, "")))
+        HGOTO_ERROR(H5E_EFL, H5E_CANTINSERT, NULL, "can't insert file name into heap")
+    HDassert(0 == name_offset);
+
+    /* allocate array of external file entries */
+    if(efl_src->nalloc > 0) {
+        size = efl_src->nalloc * sizeof(H5O_efl_entry_t);
+        if((efl_dst->slot = H5MM_calloc(size)) == NULL)
+            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+
+        /* copy content from the source. Need to update later */
+        HDmemcpy(efl_dst->slot, efl_src->slot, size);
+    } /* end if */
+
+    /* copy the name from the source */
+    for(idx = 0; idx < efl_src->nused; idx++) {
+        efl_dst->slot[idx].name = H5MM_xstrdup(efl_src->slot[idx].name);
+        if((size_t)(-1) == (efl_dst->slot[idx].name_offset = H5HL_insert(file_dst, dxpl_id, heap,
+                HDstrlen(efl_dst->slot[idx].name) + 1, efl_dst->slot[idx].name)))
+            HGOTO_ERROR(H5E_EFL, H5E_CANTINSERT, NULL, "can't insert file name into heap")
+    } /* end for */
+
+    /* Set return value */
+    ret_value = efl_dst;
+
+done:
+    /* Release resources */
+    if(heap && H5HL_unprotect(file_dst, dxpl_id, heap, efl_dst->heap_addr) < 0)
+        HDONE_ERROR(H5E_EFL, H5E_PROTECT, NULL, "unable to unprotect EFL file name heap")
+    if(!ret_value)
+        if(efl_dst)
+            H5MM_xfree(efl_dst);
+
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_efl_copy_file() */
 
 
 /*-------------------------------------------------------------------------
@@ -424,8 +501,6 @@ done:
  * Programmer:	Robb Matzke
  *		Tuesday, November 25, 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
@@ -436,14 +511,14 @@ H5O_efl_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FILE * s
     char		    buf[64];
     size_t		    u;
 
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_debug);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5O_efl_debug)
 
     /* check args */
-    assert(f);
-    assert(mesg);
-    assert(stream);
-    assert(indent >= 0);
-    assert(fwidth >= 0);
+    HDassert(f);
+    HDassert(mesg);
+    HDassert(stream);
+    HDassert(indent >= 0);
+    HDassert(fwidth >= 0);
 
     HDfprintf(stream, "%*s%-*s %a\n", indent, "", fwidth,
 	      "Heap address:", mesg->heap_addr);
@@ -452,7 +527,7 @@ H5O_efl_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FILE * s
 	      "Slots used/allocated:",
 	      mesg->nused, mesg->nalloc);
 
-    for (u = 0; u < mesg->nused; u++) {
+    for(u = 0; u < mesg->nused; u++) {
 	sprintf (buf, "File %u", (unsigned)u);
 	HDfprintf (stream, "%*s%s:\n", indent, "", buf);
 
@@ -471,8 +546,8 @@ H5O_efl_debug(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, const void *_mesg, FILE * s
 	HDfprintf (stream, "%*s%-*s %lu\n", indent+3, "", MAX (fwidth-3, 0),
 		   "Bytes reserved for data:",
 		   (unsigned long)(mesg->slot[u].size));
-    }
+    } /* end for */
 
-    FUNC_LEAVE_NOAPI(SUCCEED);
-}
+    FUNC_LEAVE_NOAPI(SUCCEED)
+} /* end H5O_efl_debug() */
 
