@@ -21,28 +21,32 @@
  *
  * Purpose:             Symbol table messages.
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 
-#define H5O_PACKAGE	/*suppress error about including H5Opkg	  */
-#define H5G_PACKAGE	/*suppress error about including H5Gpkg	  */
+#define H5G_PACKAGE		/*suppress error about including H5Gpkg	  */
+#define H5O_PACKAGE		/*suppress error about including H5Opkg	  */
 
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5FLprivate.h"	/* Free lists                           */
 #include "H5Gpkg.h"		/* Groups				*/
+#include "H5HLprivate.h"	/* Local Heaps				*/
 #include "H5Opkg.h"             /* Object headers			*/
 
 
 /* PRIVATE PROTOTYPES */
-static void *H5O_stab_decode(H5F_t *f, hid_t dxpl_id, const uint8_t *p);
-static herr_t H5O_stab_encode(H5F_t *f, uint8_t *p, const void *_mesg);
-static void *H5O_stab_copy(const void *_mesg, void *_dest, unsigned update_flags);
-static size_t H5O_stab_size(const H5F_t *f, const void *_mesg);
+static void *H5O_stab_decode(H5F_t *f, hid_t dxpl_id, unsigned mesg_flags, const uint8_t *p);
+static herr_t H5O_stab_encode(H5F_t *f, hbool_t disable_shared, uint8_t *p, const void *_mesg);
+static void *H5O_stab_copy(const void *_mesg, void *_dest);
+static size_t H5O_stab_size(const H5F_t *f, hbool_t disable_shared, const void *_mesg);
 static herr_t H5O_stab_free(void *_mesg);
-static herr_t H5O_stab_delete(H5F_t *f, hid_t dxpl_id, const void *_mesg, hbool_t adj_link);
+static herr_t H5O_stab_delete(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh, void *_mesg);
+static void *H5O_stab_copy_file(H5F_t *file_src, void *native_src,
+    H5F_t *file_dst, hbool_t *recompute_size, H5O_copy_t *cpy_info, void *udata,
+    hid_t dxpl_id);
+static herr_t H5O_stab_post_copy_file(const H5O_loc_t *src_oloc, const void *mesg_src, H5O_loc_t *dst_oloc,
+    void *mesg_dst, hid_t dxpl_id, H5O_copy_t *cpy_info);
 static herr_t H5O_stab_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg,
     FILE * stream, int indent, int fwidth);
 
@@ -51,6 +55,7 @@ const H5O_msg_class_t H5O_MSG_STAB[1] = {{
     H5O_STAB_ID,            	/*message id number             */
     "stab",                 	/*message name for debugging    */
     sizeof(H5O_stab_t),     	/*native message size           */
+    0,				/* messages are sharable?       */
     H5O_stab_decode,        	/*decode message                */
     H5O_stab_encode,        	/*encode message                */
     H5O_stab_copy,          	/*copy the native value         */
@@ -59,8 +64,13 @@ const H5O_msg_class_t H5O_MSG_STAB[1] = {{
     H5O_stab_free,	        /* free method			*/
     H5O_stab_delete,	        /* file delete method		*/
     NULL,			/* link method			*/
-    NULL,		    	/*get share method		*/
     NULL, 			/*set share method		*/
+    NULL,		    	/*can share method		*/
+    NULL,			/* pre copy native value to file */
+    H5O_stab_copy_file,		/* copy native value to file    */
+    H5O_stab_post_copy_file,	/* post copy native value to file    */
+    NULL,			/* get creation index		*/
+    NULL,			/* set creation index		*/
     H5O_stab_debug         	/*debug the message             */
 }};
 
@@ -82,12 +92,11 @@ H5FL_DEFINE_STATIC(H5O_stab_t);
  *              matzke@llnl.gov
  *              Aug  6 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_stab_decode(H5F_t *f, hid_t UNUSED dxpl_id, const uint8_t *p)
+H5O_stab_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
+    const uint8_t *p)
 {
     H5O_stab_t          *stab=NULL;
     void                *ret_value;     /* Return value */
@@ -128,12 +137,10 @@ done:
  *              matzke@llnl.gov
  *              Aug  6 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_stab_encode(H5F_t *f, uint8_t *p, const void *_mesg)
+H5O_stab_encode(H5F_t *f, hbool_t UNUSED disable_shared, uint8_t *p, const void *_mesg)
 {
     const H5O_stab_t       *stab = (const H5O_stab_t *) _mesg;
 
@@ -151,53 +158,6 @@ H5O_stab_encode(H5F_t *f, uint8_t *p, const void *_mesg)
     FUNC_LEAVE_NOAPI(SUCCEED);
 }
 
-
-/*-------------------------------------------------------------------------
- * Function:    H5O_stab_fast
- *
- * Purpose:     Initializes a new message struct with info from the cache of
- *              a symbol table entry.
- *
- * Return:      Success:        Ptr to message struct, allocated if none
- *                              supplied.
- *
- *              Failure:        NULL
- *
- * Programmer:  Robb Matzke
- *              matzke@llnl.gov
- *              Aug  6 1997
- *
- * Modifications:
- *
- *-------------------------------------------------------------------------
- */
-void *
-H5O_stab_fast(const H5G_cache_t *cache, const H5O_msg_class_t *type, void *_mesg)
-{
-    H5O_stab_t          *ret_value;     /* Return value */
-
-    FUNC_ENTER_NOAPI_NOINIT(H5O_stab_fast)
-
-    /* check args */
-    assert(cache);
-    assert(type);
-
-    if (H5O_MSG_STAB == type) {
-        if (_mesg) {
-	    ret_value = (H5O_stab_t *) _mesg;
-        } else if (NULL==(ret_value = H5FL_MALLOC(H5O_stab_t))) {
-	    HGOTO_ERROR (H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
-	}
-        ret_value->btree_addr = cache->stab.btree_addr;
-        ret_value->heap_addr = cache->stab.heap_addr;
-    }
-    else
-        ret_value=NULL;
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
-}
-
 
 /*-------------------------------------------------------------------------
  * Function:    H5O_stab_copy
@@ -213,33 +173,31 @@ done:
  *              matzke@llnl.gov
  *              Aug  6 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_stab_copy(const void *_mesg, void *_dest, unsigned UNUSED update_flags)
+H5O_stab_copy(const void *_mesg, void *_dest)
 {
     const H5O_stab_t       *stab = (const H5O_stab_t *) _mesg;
     H5O_stab_t             *dest = (H5O_stab_t *) _dest;
     void                *ret_value;     /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT(H5O_stab_copy);
+    FUNC_ENTER_NOAPI_NOINIT(H5O_stab_copy)
 
     /* check args */
-    assert(stab);
-    if (!dest && NULL==(dest = H5FL_MALLOC(H5O_stab_t)))
+    HDassert(stab);
+    if(!dest && NULL == (dest = H5FL_MALLOC(H5O_stab_t)))
 	HGOTO_ERROR (H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed");
 
     /* copy */
     *dest = *stab;
 
     /* Set return value */
-    ret_value=dest;
+    ret_value = dest;
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5O_stab_copy() */
 
 
 /*-------------------------------------------------------------------------
@@ -257,12 +215,10 @@ done:
  *              matzke@llnl.gov
  *              Aug  6 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static size_t
-H5O_stab_size(const H5F_t *f, const void UNUSED *_mesg)
+H5O_stab_size(const H5F_t *f, hbool_t UNUSED disable_shared, const void UNUSED *_mesg)
 {
     size_t ret_value;   /* Return value */
 
@@ -312,29 +268,131 @@ H5O_stab_free (void *mesg)
  * Programmer:  Quincey Koziol
  *              Thursday, March 20, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5O_stab_delete(H5F_t *f, hid_t dxpl_id, const void *mesg, hbool_t adj_link)
+H5O_stab_delete(H5F_t *f, hid_t dxpl_id, H5O_t UNUSED *open_oh, void *mesg)
 {
-    herr_t ret_value=SUCCEED;   /* Return value */
+    herr_t ret_value = SUCCEED;   /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5O_stab_delete)
 
     /* check args */
-    assert(f);
-    assert(mesg);
+    HDassert(f);
+    HDassert(mesg);
 
     /* Free the file space for the symbol table */
-    if (H5G_stab_delete(f, dxpl_id, mesg, adj_link)<0)
+    if(H5G_stab_delete(f, dxpl_id, mesg) < 0)
         HGOTO_ERROR(H5E_OHDR, H5E_CANTFREE, FAIL, "unable to free symbol table")
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_stab_delete() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:    H5O_stab_copy_file
+ *
+ * Purpose:     Copies a message from _MESG to _DEST in file
+ *
+ * Return:      Success:        Ptr to _DEST
+ *
+ *              Failure:        NULL
+ *
+ * Programmer:  Peter Cao
+ *              September 10, 2005
+ *
+ *-------------------------------------------------------------------------
+ */
+static void *
+H5O_stab_copy_file(H5F_t *file_src, void *native_src, H5F_t *file_dst,
+    hbool_t UNUSED *recompute_size, H5O_copy_t UNUSED *cpy_info, void UNUSED *udata,
+    hid_t dxpl_id)
+{
+    H5O_stab_t          *stab_src = (H5O_stab_t *) native_src;
+    H5O_stab_t          *stab_dst = NULL;
+    size_t              size_hint;              /* Local heap initial size */
+    void                *ret_value;             /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT(H5O_stab_copy_file)
+
+    /* check args */
+    HDassert(stab_src);
+    HDassert(file_dst);
+
+    /* Allocate space for the destination stab */
+    if(NULL == (stab_dst = H5FL_MALLOC(H5O_stab_t)))
+        HGOTO_ERROR (H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+
+    /* Get the old local heap's size and use that as the hint for the new heap */
+    if(H5HL_get_size(file_src, dxpl_id, stab_src->heap_addr, &size_hint) < 0)
+	HGOTO_ERROR(H5E_SYM, H5E_CANTGETSIZE, NULL, "can't query local heap size")
+
+    /* Create components of symbol table message */
+    if(H5G_stab_create_components(file_dst, stab_dst, size_hint, dxpl_id) < 0)
+	HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, NULL, "can't create symbol table components")
+
+    /* Set return value */
+    ret_value = stab_dst;
+
+done:
+    if(!ret_value)
+        if(stab_dst)
+            H5FL_FREE(H5O_stab_t, stab_dst);
+
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* H5O_stab_copy_file() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5O_stab_post_copy_file
+ *
+ * Purpose:     Finish copying a message from between files
+ *
+ * Return:      Non-negative on success/Negative on failure
+ *
+ * Programmer:  Peter Cao
+ *              September 28, 2005
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5O_stab_post_copy_file(const H5O_loc_t *src_oloc, const void *mesg_src, H5O_loc_t *dst_oloc,
+    void *mesg_dst, hid_t dxpl_id, H5O_copy_t *cpy_info)
+{
+    const H5O_stab_t    *stab_src = (const H5O_stab_t *)mesg_src;
+    H5O_stab_t          *stab_dst = (H5O_stab_t *)mesg_dst;
+    H5G_bt_it_cpy_t     udata;      /* B-tree user data */
+    herr_t ret_value = SUCCEED;   /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT(H5O_stab_post_copy_file)
+
+    /* check args */
+    HDassert(stab_src);
+    HDassert(H5F_addr_defined(dst_oloc->addr));
+    HDassert(dst_oloc->file);
+    HDassert(stab_dst);
+    HDassert(cpy_info);
+
+    /* If we are performing a 'shallow hierarchy' copy, get out now */
+    if(cpy_info->max_depth >= 0 && cpy_info->curr_depth >= cpy_info->max_depth)
+        HGOTO_DONE(SUCCEED)
+
+    /* Set up B-tree iteration user data */
+    udata.src_oloc = src_oloc;
+    udata.src_heap_addr = stab_src->heap_addr;
+    udata.dst_file = dst_oloc->file;
+    udata.dst_stab = stab_dst;
+    udata.cpy_info = cpy_info;
+
+    /* Iterate over objects in group, copying them */
+    if((H5B_iterate(src_oloc->file, dxpl_id, H5B_SNODE, H5G_node_copy, stab_src->btree_addr, &udata)) < 0)
+        HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "iteration operator failed")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* H5O_stab_post_copy_file() */
+
 
 /*-------------------------------------------------------------------------
  * Function:    H5O_stab_debug
