@@ -26,11 +26,33 @@
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Oprivate.h"		/* Object headers		  	*/
 
+/**************************/
+/* Library Private Macros */
+/**************************/
+
+/* ========  String creation property names ======== */
+#define H5P_STRCRT_CHAR_ENCODING_NAME  "character_encoding"     /* Character set encoding for string */
+
+
+/****************************/
+/* Library Private Typedefs */
+/****************************/
+
 /* Forward declarations for anonymous H5P objects */
 typedef struct H5P_genplist_t H5P_genplist_t;
 typedef struct H5P_genclass_t H5P_genclass_t;
 
-/* Private functions, not part of the publicly documented API */
+
+/*****************************/
+/* Library Private Variables */
+/*****************************/
+
+
+/******************************/
+/* Library Private Prototypes */
+/******************************/
+
+/* Package initialization routine */
 H5_DLL herr_t H5P_init(void);
 
 /* Internal versions of API routines */
@@ -46,12 +68,13 @@ H5_DLL herr_t H5P_insert(H5P_genplist_t *plist, const char *name, size_t size,
 H5_DLL herr_t H5P_remove(hid_t plist_id, H5P_genplist_t *plist, const char *name);
 H5_DLL htri_t H5P_exist_plist(H5P_genplist_t *plist, const char *name);
 H5_DLL char *H5P_get_class_name(H5P_genclass_t *pclass);
-H5_DLL herr_t H5P_get_nprops_pclass(H5P_genclass_t *pclass, size_t *nprops);
+H5_DLL herr_t H5P_get_nprops_pclass(const H5P_genclass_t *pclass, size_t *nprops,
+    hbool_t recurse);
 H5_DLL herr_t H5P_register(H5P_genclass_t *pclass, const char *name, size_t size,
-            void *def_value, H5P_prp_create_func_t prp_create, H5P_prp_set_func_t prp_set,
-            H5P_prp_get_func_t prp_get, H5P_prp_delete_func_t prp_delete,
-            H5P_prp_copy_func_t prp_copy, H5P_prp_compare_func_t prp_cmp,
-            H5P_prp_close_func_t prp_close);
+    const void *def_value, H5P_prp_create_func_t prp_create, H5P_prp_set_func_t prp_set,
+    H5P_prp_get_func_t prp_get, H5P_prp_delete_func_t prp_delete,
+    H5P_prp_copy_func_t prp_copy, H5P_prp_compare_func_t prp_cmp,
+    H5P_prp_close_func_t prp_close);
 H5_DLL hid_t H5P_get_driver(H5P_genplist_t *plist);
 H5_DLL void * H5P_get_driver_info(H5P_genplist_t *plist);
 H5_DLL herr_t H5P_set_driver(H5P_genplist_t *plist, hid_t new_driver_id,
@@ -59,8 +82,10 @@ H5_DLL herr_t H5P_set_driver(H5P_genplist_t *plist, hid_t new_driver_id,
 H5_DLL herr_t H5P_set_vlen_mem_manager(H5P_genplist_t *plist,
         H5MM_allocate_t alloc_func, void *alloc_info, H5MM_free_t free_func,
         void *free_info);
-H5_DLL herr_t H5P_is_fill_value_defined(const struct H5O_fill_t *fill,
+H5_DLL herr_t H5P_is_fill_value_defined(const H5O_fill_t *fill,
         H5D_fill_value_t *status);
+H5_DLL int H5P_fill_value_cmp(const void *value1, const void *value2,
+    size_t size);
 
 /* *SPECIAL* Don't make more of these! -QAK */
 H5_DLL htri_t H5P_isa_class(hid_t plist_id, hid_t pclass_id);
@@ -72,4 +97,17 @@ H5_DLL hid_t H5P_peek_hid_t(H5P_genplist_t *plist, const char *name);
 H5_DLL void *H5P_peek_voidp(H5P_genplist_t *plist, const char *name);
 H5_DLL size_t H5P_peek_size_t(H5P_genplist_t *plist, const char *name);
 
-#endif
+/* Private DCPL routines */
+H5_DLL herr_t H5P_modify_filter(H5P_genplist_t *plist, H5Z_filter_t filter,
+    unsigned flags, size_t cd_nelmts, const unsigned cd_values[/*cd_nelmts*/]);
+H5_DLL herr_t H5P_get_filter_by_id(H5P_genplist_t *plist, H5Z_filter_t id,
+    unsigned int *flags/*out*/, size_t *cd_nelmts/*in_out*/,
+    unsigned cd_values[]/*out*/, size_t namelen, char name[]/*out*/,
+    unsigned *filter_config);
+H5_DLL herr_t H5P_fill_value_defined(H5P_genplist_t *plist,
+    H5D_fill_value_t *status);
+H5_DLL herr_t H5P_get_fill_value(H5P_genplist_t *plist, const H5T_t *type,
+    void *value, hid_t dxpl_id);
+
+#endif /* _H5Pprivate_H */
+
