@@ -18,9 +18,10 @@
 
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
+#include "H5Iprivate.h"		/* IDs			  		*/
 #include "H5MMprivate.h"	/* Memory management			*/
-#include "H5Ppublic.h"		/* Property lists			*/
-#include "H5Tpublic.h"		/* Datatype functions			*/
+#include "H5Pprivate.h"         /* Property lists                       */
+#include "H5Tprivate.h"		/* Datatypes         			*/
 #include "H5Zpkg.h"		/* Data filters				*/
 
 #ifdef H5_HAVE_FILTER_SHUFFLE
@@ -32,7 +33,10 @@ static size_t H5Z_filter_shuffle(unsigned flags, size_t cd_nelmts,
 
 /* This message derives from H5Z */
 const H5Z_class_t H5Z_SHUFFLE[1] = {{
+    H5Z_CLASS_T_VERS,       /* H5Z_class_t version */
     H5Z_FILTER_SHUFFLE,		/* Filter id number		*/
+    1,              /* encoder_present flag (set to true) */
+    1,              /* decoder_present flag (set to true) */
     "shuffle",			/* Filter name for debugging	*/
     NULL,                       /* The "can apply" callback     */
     H5Z_set_local_shuffle,      /* The "set local" callback     */
@@ -40,8 +44,6 @@ const H5Z_class_t H5Z_SHUFFLE[1] = {{
 }};
 
 /* Local macros */
-#define H5Z_SHUFFLE_USER_NPARMS    0       /* Number of parameters that users can set */
-#define H5Z_SHUFFLE_TOTAL_NPARMS   1       /* Total number of parameters for filter */
 #define H5Z_SHUFFLE_PARM_SIZE      0       /* "Local" parameter for shuffling size */
 
 
@@ -65,23 +67,33 @@ const H5Z_class_t H5Z_SHUFFLE[1] = {{
 static herr_t
 H5Z_set_local_shuffle(hid_t dcpl_id, hid_t type_id, hid_t UNUSED space_id)
 {
-    unsigned flags;         /* Filter flags */
-    size_t cd_nelmts=H5Z_SHUFFLE_USER_NPARMS;     /* Number of filter parameters */
+    H5P_genplist_t *dcpl_plist;     /* Property list pointer */
+    const H5T_t	*type;                  /* Datatype */
+    unsigned flags;                     /* Filter flags */
+    size_t cd_nelmts = H5Z_SHUFFLE_USER_NPARMS;     /* Number of filter parameters */
     unsigned cd_values[H5Z_SHUFFLE_TOTAL_NPARMS];  /* Filter parameters */
-    herr_t ret_value=SUCCEED;   /* Return value */
+    herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI(H5Z_set_local_shuffle, FAIL)
 
+    /* Get the plist structure */
+    if(NULL == (dcpl_plist = H5P_object_verify(dcpl_id, H5P_DATASET_CREATE)))
+        HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't find object for ID")
+
+    /* Get datatype */
+    if(NULL == (type = H5I_object_verify(type_id, H5I_DATATYPE)))
+	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not a datatype")
+
     /* Get the filter's current parameters */
-    if(H5Pget_filter_by_id(dcpl_id,H5Z_FILTER_SHUFFLE,&flags,&cd_nelmts, cd_values,0,NULL)<0)
+    if(H5P_get_filter_by_id(dcpl_plist, H5Z_FILTER_SHUFFLE, &flags, &cd_nelmts, cd_values, (size_t)0, NULL, NULL) < 0)
 	HGOTO_ERROR(H5E_PLINE, H5E_CANTGET, FAIL, "can't get shuffle parameters")
 
     /* Set "local" parameter for this dataset */
-    if((cd_values[H5Z_SHUFFLE_PARM_SIZE]=(unsigned)H5Tget_size(type_id))==0)
+    if((cd_values[H5Z_SHUFFLE_PARM_SIZE] = (unsigned)H5T_get_size(type)) == 0)
 	HGOTO_ERROR(H5E_PLINE, H5E_BADTYPE, FAIL, "bad datatype size")
 
     /* Modify the filter's parameters for this dataset */
-    if(H5Pmodify_filter(dcpl_id, H5Z_FILTER_SHUFFLE, flags, H5Z_SHUFFLE_TOTAL_NPARMS, cd_values)<0)
+    if(H5P_modify_filter(dcpl_plist, H5Z_FILTER_SHUFFLE, flags, (size_t)H5Z_SHUFFLE_TOTAL_NPARMS, cd_values) < 0)
 	HGOTO_ERROR(H5E_PLINE, H5E_CANTSET, FAIL, "can't set local shuffle parameters")
 
 done:
