@@ -30,6 +30,9 @@
  */
 #include "H5pubconf.h"		/*from configure                             */
 
+/* API Version macro wrapper definitions */
+#include "H5version.h"
+
 #ifdef H5_HAVE_FEATURES_H
 #include <features.h>           /*for setting POSIX, BSD, etc. compatibility */
 #endif
@@ -57,15 +60,8 @@
 #endif
 #endif
 
-#ifdef H5_HAVE_GASS             /*for Globus GASS I/O                        */
-#include "globus_common.h"
-#include "globus_gass_file.h"
-#endif
-
-#ifdef H5_HAVE_SRB              /*for SRB I/O                                */
-#include <srbClient.h>
-#endif
 
+/* Include the Windows API adapter header early */
 #include "H5api_adpt.h"
 
 #ifdef __cplusplus
@@ -74,11 +70,11 @@ extern "C" {
 
 /* Version numbers */
 #define H5_VERS_MAJOR	1	/* For major interface/format changes  	     */
-#define H5_VERS_MINOR	6	/* For minor interface/format changes  	     */
-#define H5_VERS_RELEASE	6	/* For tweaks, bug-fixes, or development     */
+#define H5_VERS_MINOR	8	/* For minor interface/format changes  	     */
+#define H5_VERS_RELEASE	0	/* For tweaks, bug-fixes, or development     */
 #define H5_VERS_SUBRELEASE ""	/* For pre-releases like snap0       */
 				/* Empty string for real releases.           */
-#define H5_VERS_INFO    "HDF5 library version: 1.6.6"      /* Full version string */
+#define H5_VERS_INFO    "HDF5 library version: 1.8.0"      /* Full version string */
 
 #define H5check()	H5check_version(H5_VERS_MAJOR,H5_VERS_MINOR,	      \
 				        H5_VERS_RELEASE)
@@ -90,9 +86,8 @@ extern "C" {
  * The negative failure value is most commonly -1, but don't bet on it.  The
  * proper way to detect failure is something like:
  *
- * 	if ((dset = H5Dopen (file, name))<0) {
- *	    fprintf (stderr, "unable to open the requested dataset\n");
- *	}
+ * 	if((dset = H5Dopen2(file, name)) < 0)
+ *	    fprintf(stderr, "unable to open the requested dataset\n");
  */
 typedef int herr_t;
 
@@ -115,6 +110,24 @@ typedef int herr_t;
 typedef unsigned int hbool_t;
 typedef int htri_t;
 
+/*
+ * Although `long long' is part of the revised ANSI-C some compilers don't
+ * support it yet.  We define `long_long' as the longest integral integer type
+ * supported by the compiler, usually 64 bits.  It must be legal to qualify
+ * `long_long' with `unsigned'.
+ */
+#if H5_SIZEOF_LONG_LONG>0
+#   define long_long    long long
+#elif H5_SIZEOF___INT64>0
+#   define long_long    __int64 /*Win32*/
+#   undef H5_SIZEOF_LONG_LONG
+#   define H5_SIZEOF_LONG_LONG H5_SIZEOF___INT64
+#else
+#   define long_long    long int
+#   undef H5_SIZEOF_LONG_LONG
+#   define H5_SIZEOF_LONG_LONG H5_SIZEOF_LONG
+#endif
+
 /* Define the ssize_t type if it not is defined */
 #if H5_SIZEOF_SSIZE_T==0
 /* Undefine this size, we will re-define it in one of the sections below */
@@ -126,39 +139,28 @@ typedef int ssize_t;
 typedef long ssize_t;
 #       define H5_SIZEOF_SSIZE_T H5_SIZEOF_LONG
 #elif H5_SIZEOF_SIZE_T==H5_SIZEOF_LONG_LONG
-typedef long long ssize_t;
+typedef long_long ssize_t;
 #       define H5_SIZEOF_SSIZE_T H5_SIZEOF_LONG_LONG
-#elif H5_SIZEOF_SIZE_T==H5_SIZEOF___INT64
-typedef __int64 ssize_t;
-#       define H5_SIZEOF_SSIZE_T H5_SIZEOF___INT64
 #else /* Can't find matching type for ssize_t */
 #   error "nothing appropriate for ssize_t"
 #endif
 #endif
 
 /*
- * The sizes of file objects have their own types defined here.  If large
- * sizes are enabled then use a 64-bit data type, otherwise use the size of
- * memory objects.
+ * The sizes of file objects have their own types defined here, use a 64-bit
+ * type.
  */
-#ifdef H5_HAVE_LARGE_HSIZET
-#   if H5_SIZEOF_LONG_LONG>=8
-typedef unsigned long long 	hsize_t;
-typedef signed long long	hssize_t;
+#if H5_SIZEOF_LONG_LONG >= 8
+typedef unsigned long_long 	hsize_t;
+typedef signed long_long	hssize_t;
 #       define H5_SIZEOF_HSIZE_T H5_SIZEOF_LONG_LONG
-#   elif H5_SIZEOF___INT64>=8
-typedef unsigned __int64	hsize_t;
-typedef signed __int64		hssize_t;
-#       define H5_SIZEOF_HSIZE_T H5_SIZEOF___INT64
-#   endif
-#else /* H5_HAVE_LARGE_HSIZET */
-typedef size_t			hsize_t;
-typedef ssize_t			hssize_t;
-#       define H5_SIZEOF_HSIZE_T H5_SIZEOF_SIZE_T
-#endif /* H5_HAVE_LARGE_HSIZET */
+#       define H5_SIZEOF_HSSIZE_T H5_SIZEOF_LONG_LONG
+#else
+#   error "nothing appropriate for hsize_t"
+#endif
 
 /*
- * File addresses have there own types.
+ * File addresses have their own types.
  */
 #if H5_SIZEOF_INT64_T>=8
     typedef uint64_t                haddr_t;
@@ -182,19 +184,12 @@ typedef ssize_t			hssize_t;
 #       define HADDR_AS_MPI_TYPE    MPI_UNSIGNED_LONG
 #   endif  /* H5_HAVE_PARALLEL */
 #elif H5_SIZEOF_LONG_LONG>=8
-    typedef unsigned long long      haddr_t;
-#   define HADDR_UNDEF              ((haddr_t)(long long)(-1))
+    typedef unsigned long_long      haddr_t;
+#   define HADDR_UNDEF              ((haddr_t)(long_long)(-1))
 #   define H5_SIZEOF_HADDR_T        H5_SIZEOF_LONG_LONG
 #   ifdef H5_HAVE_PARALLEL
 #       define HADDR_AS_MPI_TYPE    MPI_LONG_LONG_INT
 #   endif  /* H5_HAVE_PARALLEL */
-#elif H5_SIZEOF___INT64>=8
-    typedef unsigned __int64        haddr_t;
-#   define HADDR_UNDEF              ((haddr_t)(__int64)(-1))
-#   define H5_SIZEOF_HADDR_T        H5_SIZEOF___INT64
-#   ifdef H5_HAVE_PARALLEL
-#       define HADDR_AS_MPI_TYPE    MPI_LONG_LONG_INT
-#   endif  /* H5_HAVE_PARALLEL */
 #else
 #   error "nothing appropriate for haddr_t"
 #endif
@@ -204,13 +199,108 @@ typedef ssize_t			hssize_t;
 #   define H5_PRINTF_HADDR_FMT  "%lu"
 #elif H5_SIZEOF_HADDR_T ==H5_SIZEOF_LONG_LONG
 #   define H5_PRINTF_HADDR_FMT  "%"H5_PRINTF_LL_WIDTH"u"
-#elif H5_SIZEOF_HADDR_T ==H5_SIZEOF___INT64
-#   define H5_PRINTF_HADDR_FMT  "%"H5_PRINTF_LL_WIDTH"u"
 #else
 #   error "nothing appropriate for H5_PRINTF_HADDR_FMT"
 #endif
 #define HADDR_MAX		(HADDR_UNDEF-1)
 
+/* uint32_t type is used for creation order field for messages.  It may be
+ * defined in Posix.1g, otherwise it is defined here.
+ */
+#if H5_SIZEOF_UINT32_T>=4
+#elif H5_SIZEOF_SHORT>=4
+    typedef short uint32_t;
+#   undef H5_SIZEOF_UINT32_T
+#   define H5_SIZEOF_UINT32_T H5_SIZEOF_SHORT
+#elif H5_SIZEOF_INT>=4
+    typedef unsigned int uint32_t;
+#   undef H5_SIZEOF_UINT32_T
+#   define H5_SIZEOF_UINT32_T H5_SIZEOF_INT
+#elif H5_SIZEOF_LONG>=4
+    typedef unsigned long uint32_t;
+#   undef H5_SIZEOF_UINT32_T
+#   define H5_SIZEOF_UINT32_T H5_SIZEOF_LONG
+#else
+#   error "nothing appropriate for uint32_t"
+#endif
+
+/* int64_t type is used for creation order field for links.  It may be
+ * defined in Posix.1g, otherwise it is defined here.
+ */
+#if H5_SIZEOF_INT64_T>=8
+#elif H5_SIZEOF_INT>=8
+    typedef int int64_t;
+#   undef H5_SIZEOF_INT64_T
+#   define H5_SIZEOF_INT64_T H5_SIZEOF_INT
+#elif H5_SIZEOF_LONG>=8
+    typedef long int64_t;
+#   undef H5_SIZEOF_INT64_T
+#   define H5_SIZEOF_INT64_T H5_SIZEOF_LONG
+#elif H5_SIZEOF_LONG_LONG>=8
+    typedef long_long int64_t;
+#   undef H5_SIZEOF_INT64_T
+#   define H5_SIZEOF_INT64_T H5_SIZEOF_LONG_LONG
+#else
+#   error "nothing appropriate for int64_t"
+#endif
+
+/* uint64_t type is used for fields for H5O_info_t.  It may be
+ * defined in Posix.1g, otherwise it is defined here.
+ */
+#if H5_SIZEOF_UINT64_T>=8
+#elif H5_SIZEOF_INT>=8
+    typedef unsigned uint64_t;
+#   undef H5_SIZEOF_UINT64_T
+#   define H5_SIZEOF_UINT64_T H5_SIZEOF_INT
+#elif H5_SIZEOF_LONG>=8
+    typedef unsigned long uint64_t;
+#   undef H5_SIZEOF_UINT64_T
+#   define H5_SIZEOF_UINT64_T H5_SIZEOF_LONG
+#elif H5_SIZEOF_LONG_LONG>=8
+    typedef unsigned long_long uint64_t;
+#   undef H5_SIZEOF_UINT64_T
+#   define H5_SIZEOF_UINT64_T H5_SIZEOF_LONG_LONG
+#else
+#   error "nothing appropriate for uint64_t"
+#endif
+
+/* Common iteration orders */
+typedef enum {
+    H5_ITER_UNKNOWN = -1,       /* Unknown order */
+    H5_ITER_INC,                /* Increasing order */
+    H5_ITER_DEC,                /* Decreasing order */
+    H5_ITER_NATIVE,             /* No particular order, whatever is fastest */
+    H5_ITER_N		        /* Number of iteration orders */
+} H5_iter_order_t;
+
+/* Iteration callback values */
+/* (Actually, any postive value will cause the iterator to stop and pass back
+ *      that positive value to the function that called the iterator)
+ */
+#define H5_ITER_ERROR   (-1)
+#define H5_ITER_CONT    (0)
+#define H5_ITER_STOP    (1)
+
+/*
+ * The types of indices on links in groups/attributes on objects.
+ * Primarily used for "<do> <foo> by index" routines and for iterating over
+ * links in groups/attributes on objects.
+ */
+typedef enum H5_index_t {
+    H5_INDEX_UNKNOWN = -1,	/* Unknown index type			*/
+    H5_INDEX_NAME,		/* Index on names 			*/
+    H5_INDEX_CRT_ORDER,		/* Index on creation order 		*/
+    H5_INDEX_N			/* Number of indices defined 		*/
+} H5_index_t;
+
+/*
+ * Storage info struct used by H5O_info_t and H5F_info_t
+ */
+typedef struct H5_ih_info_t {
+    hsize_t     index_size;     /* btree and/or list */
+    hsize_t     heap_size;
+} H5_ih_info_t;
+
 /* Functions in H5.c */
 H5_DLL herr_t H5open(void);
 H5_DLL herr_t H5close(void);
@@ -228,3 +318,4 @@ H5_DLL herr_t H5check_version(unsigned majnum, unsigned minnum,
 }
 #endif
 #endif
+
