@@ -21,10 +21,14 @@
  */
 #include "h5test.h"
 
+/* File for external link test.  Created with gen_udlinks.c */
+#define LINKED_FILE  "be_extlink2.h5"
+
 const char *FILENAME[] = {
     "extern_1",
     "extern_2",
     "extern_3",
+    "extern_4",
     NULL
 };
 
@@ -108,52 +112,51 @@ test_1a(hid_t file)
     TESTING("fixed-size data space, exact storage");
 
     /* Create the dataset */
-    if((dcpl=H5Pcreate(H5P_DATASET_CREATE))<0) goto error;
+    if((dcpl = H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
     cur_size[0] = max_size[0] = 100;
-    if (H5Pset_external(dcpl, "ext1.data", (off_t)0,
-	(hsize_t)(max_size[0]*sizeof(int)))<0) goto error;
-    if ((space = H5Screate_simple (1, cur_size, max_size))<0) goto error;
-    if ((dset = H5Dcreate (file, "dset1", H5T_NATIVE_INT, space, dcpl))<0)
+    if(H5Pset_external(dcpl, "ext1.data", (off_t)0,
+	(hsize_t)(max_size[0] * sizeof(int))) < 0) goto error;
+    if((space = H5Screate_simple(1, cur_size, max_size)) < 0) goto error;
+    if((dset = H5Dcreate2(file, "dset1", H5T_NATIVE_INT, space, H5P_DEFAULT, dcpl, H5P_DEFAULT)) < 0)
 	goto error;
-    if (H5Dclose (dset)<0) goto error;
-    if (H5Sclose (space)<0) goto error;
-    if (H5Pclose (dcpl)<0) goto error;
+    if(H5Dclose(dset) < 0) goto error;
+    if(H5Sclose(space) < 0) goto error;
+    if(H5Pclose(dcpl) < 0) goto error;
 
     /* Read dataset creation information */
-    if ((dset = H5Dopen (file, "dset1"))<0) goto error;
+    if((dset = H5Dopen2(file, "dset1", H5P_DEFAULT)) < 0) goto error;
 
     /* Test dataset address.  Should be undefined. */
-    if (H5Dget_offset(dset)!=HADDR_UNDEF) goto error;
+    if(H5Dget_offset(dset) != HADDR_UNDEF) goto error;
 
-    if ((dcpl = H5Dget_create_plist (dset))<0) goto error;
-    if ((n=H5Pget_external_count (dcpl))<0) goto error;
-    if (1!=n) {
+    if((dcpl = H5Dget_create_plist(dset)) < 0) goto error;
+    if((n = H5Pget_external_count(dcpl)) < 0) goto error;
+    if(1 != n) {
 	H5_FAILED();
 	puts("    Returned external count is wrong.");
 	printf("   got: %d\n    ans: 1\n", n);
 	goto error;
     }
-    strcpy (name+sizeof(name)-4, "...");
-    if (H5Pget_external (dcpl, 0, sizeof(name)-4, name, &file_offset,
-			 &file_size)<0) goto error;
-    if (file_offset!=0) {
+    HDstrcpy(name + sizeof(name) - 4, "...");
+    if(H5Pget_external(dcpl, 0, sizeof(name) - 4, name, &file_offset,
+			 &file_size) < 0) goto error;
+    if(file_offset != 0) {
 	H5_FAILED();
 	puts("    Wrong file offset.");
 	printf("    got: %lu\n    ans: 0\n", (unsigned long)file_offset);
 	goto error;
     }
-    if (file_size!=(max_size[0]*sizeof(int))) {
+    if(file_size != (max_size[0] * sizeof(int))) {
 	H5_FAILED();
 	puts("    Wrong file size.");
 	printf("    got: %lu\n    ans: %lu\n", (unsigned long)file_size,
 		(unsigned long)max_size[0]*sizeof(int));
 	goto error;
     }
-    if (H5Pclose (dcpl)<0) goto error;
-    if (H5Dclose (dset)<0) goto error;
+    if (H5Pclose (dcpl) < 0) goto error;
+    if (H5Dclose (dset) < 0) goto error;
     PASSED();
     return 0;
-
  error:
     H5E_BEGIN_TRY {
 	H5Pclose(dcpl);
@@ -184,28 +187,29 @@ test_1a(hid_t file)
 static int
 test_1b(hid_t file)
 {
-    hid_t	dcpl=-1;		/*dataset creation properties	*/
-    hid_t	space=-1;		/*data space			*/
-    hid_t	dset=-1;		/*dataset			*/
+    hid_t	dcpl = -1;		/*dataset creation properties	*/
+    hid_t	space = -1;		/*data space			*/
+    hid_t	dset = -1;		/*dataset			*/
     hsize_t	cur_size[1];		/*current data space size	*/
     hsize_t	max_size[1];		/*maximum data space size	*/
 
     TESTING("external storage is too small");
-    if((dcpl = H5Pcreate(H5P_DATASET_CREATE))<0) goto error;
+    if((dcpl = H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
     cur_size[0] = max_size[0] = 100;
-    if (H5Pset_external(dcpl, "ext1.data", (off_t)0,
-	(hsize_t)(max_size[0]*sizeof(int)-1))<0) goto error;
-    if ((space = H5Screate_simple (1, cur_size, max_size))<0) goto error;
+    if(H5Pset_external(dcpl, "ext1.data", (off_t)0,
+	(hsize_t)(max_size[0] * sizeof(int) - 1)) < 0) goto error;
+    if((space = H5Screate_simple(1, cur_size, max_size)) < 0) goto error;
     H5E_BEGIN_TRY {
-	dset = H5Dcreate (file, "dset2", H5T_NATIVE_INT, space, dcpl);
+	dset = H5Dcreate2(file, "dset2", H5T_NATIVE_INT, space, H5P_DEFAULT, dcpl, H5P_DEFAULT);
     } H5E_END_TRY;
-    if (dset>=0) {
+    if(dset >= 0) {
 	H5_FAILED();
 	puts("    Small external file succeeded instead of failing.");
 	goto error;
     }
-    if (H5Sclose (space)<0) goto error;
-    if (H5Pclose (dcpl)<0) goto error;
+    if(H5Sclose(space) < 0) goto error;
+    if(H5Pclose(dcpl) < 0) goto error;
+
     PASSED();
     return 0;
 
@@ -247,21 +251,23 @@ test_1c(hid_t file)
     hsize_t	max_size[1];		/*maximum data space size	*/
 
     TESTING("extendible dataspace, exact external size");
-    if((dcpl=H5Pcreate (H5P_DATASET_CREATE))<0) goto error;
+
+    if((dcpl = H5Pcreate (H5P_DATASET_CREATE)) < 0) goto error;
     cur_size[0] = 100;
     max_size[0] = 200;
-    if (H5Pset_external(dcpl, "ext1.data", (off_t)0,
-	(hsize_t)(max_size[0]*sizeof(int)))<0) goto error;
-    if ((space = H5Screate_simple (1, cur_size, max_size))<0) goto error;
-    if ((dset = H5Dcreate (file, "dset3", H5T_NATIVE_INT, space, dcpl))<0)
+    if(H5Pset_external(dcpl, "ext1.data", (off_t)0,
+	(hsize_t)(max_size[0] * sizeof(int))) < 0) goto error;
+    if((space = H5Screate_simple(1, cur_size, max_size)) < 0) goto error;
+    if((dset = H5Dcreate2(file, "dset3", H5T_NATIVE_INT, space, H5P_DEFAULT, dcpl, H5P_DEFAULT)) < 0)
 	goto error;
-    if (H5Dclose (dset)<0) goto error;
-    if (H5Sclose (space)<0) goto error;
-    if (H5Pclose (dcpl)<0) goto error;
+    if(H5Dclose(dset) < 0) goto error;
+    if(H5Sclose(space) < 0) goto error;
+    if(H5Pclose(dcpl) < 0) goto error;
+
     PASSED();
     return 0;
 
- error:
+error:
     H5E_BEGIN_TRY {
 	H5Dclose(dset);
 	H5Pclose(dcpl);
@@ -298,22 +304,24 @@ test_1d(hid_t file)
     hsize_t	max_size[1];		/*maximum data space size	*/
 
     TESTING("extendible dataspace, external storage is too small");
-    if((dcpl=H5Pcreate(H5P_DATASET_CREATE))<0) goto error;
+
+    if((dcpl = H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
     cur_size[0] = 100;
     max_size[0] = 200;
-    if (H5Pset_external(dcpl, "ext1.data", (off_t)0,
-	(hsize_t)(max_size[0]*sizeof(int)-1))<0) goto error;
-    if ((space=H5Screate_simple(1, cur_size, max_size))<0) goto error;
+    if(H5Pset_external(dcpl, "ext1.data", (off_t)0,
+	(hsize_t)(max_size[0] * sizeof(int) - 1)) < 0) goto error;
+    if((space = H5Screate_simple(1, cur_size, max_size)) < 0) goto error;
     H5E_BEGIN_TRY {
-	dset = H5Dcreate (file, "dset4", H5T_NATIVE_INT, space, dcpl);
+	dset = H5Dcreate2(file, "dset4", H5T_NATIVE_INT, space, H5P_DEFAULT, dcpl, H5P_DEFAULT);
     } H5E_END_TRY;
-    if (dset>=0) {
+    if(dset >= 0) {
 	H5_FAILED();
 	puts("    Small external file succeeded instead of failing.");
 	goto error;
     }
-    if (H5Sclose (space)<0) goto error;
-    if (H5Pclose (dcpl)<0) goto error;
+    if(H5Sclose(space) < 0) goto error;
+    if(H5Pclose(dcpl) < 0) goto error;
+
     PASSED();
     return 0;
 
@@ -360,44 +368,45 @@ test_1e(hid_t file)
     TESTING("unlimited dataspace, unlimited external storage");
 
     /* Create dataset */
-    if((dcpl=H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
-    if (H5Pset_external(dcpl, "ext1.data", (off_t)0, H5F_UNLIMITED)<0) goto error;
+    if((dcpl = H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
+    if(H5Pset_external(dcpl, "ext1.data", (off_t)0, H5F_UNLIMITED) < 0) goto error;
     cur_size[0] = 100;
     max_size[0] = H5S_UNLIMITED;
-    if ((space=H5Screate_simple(1, cur_size, max_size))<0) goto error;
-    if ((dset=H5Dcreate(file, "dset5", H5T_NATIVE_INT, space, dcpl))<0)
+    if((space = H5Screate_simple(1, cur_size, max_size)) < 0) goto error;
+    if((dset = H5Dcreate2(file, "dset5", H5T_NATIVE_INT, space, H5P_DEFAULT, dcpl, H5P_DEFAULT)) < 0)
 	goto error;
-    if (H5Dclose (dset)<0) goto error;
-    if (H5Sclose (space)<0) goto error;
-    if (H5Pclose (dcpl)<0) goto error;
+    if(H5Dclose(dset) < 0) goto error;
+    if(H5Sclose(space) < 0) goto error;
+    if(H5Pclose(dcpl) < 0) goto error;
 
     /* Read dataset creation information */
-    if ((dset = H5Dopen (file, "dset5"))<0) goto error;
-    if ((dcpl = H5Dget_create_plist (dset))<0) goto error;
-    if ((n = H5Pget_external_count (dcpl))<0) goto error;
-    if (1!=n) {
+    if((dset = H5Dopen2(file, "dset5", H5P_DEFAULT)) < 0) goto error;
+    if((dcpl = H5Dget_create_plist(dset)) < 0) goto error;
+    if((n = H5Pget_external_count(dcpl)) < 0) goto error;
+    if(1 != n) {
 	H5_FAILED();
 	puts("    Returned external count is wrong.");
 	printf("    got: %d\n    ans: 1\n", n);
 	goto error;
     }
-    strcpy (name+sizeof(name)-4, "...");
-    if (H5Pget_external (dcpl, 0, sizeof(name)-4, name, &file_offset,
-			 &file_size)<0) goto error;
-    if (file_offset!=0) {
+    HDstrcpy(name + sizeof(name) - 4, "...");
+    if(H5Pget_external(dcpl, 0, sizeof(name) - 4, name, &file_offset,
+			 &file_size) < 0) goto error;
+    if(file_offset != 0) {
 	H5_FAILED();
 	puts("    Wrong file offset.");
 	printf("    got: %lu\n    ans: 0\n", (unsigned long)file_offset);
 	goto error;
     }
-    if (H5F_UNLIMITED!=file_size) {
+    if(H5F_UNLIMITED != file_size) {
 	H5_FAILED();
 	puts("    Wrong file size.");
 	printf("    got: %lu\n    ans: INF\n", (unsigned long)file_size);
 	goto error;
     }
-    if (H5Pclose (dcpl)<0) goto error;
-    if (H5Dclose (dset)<0) goto error;
+    if(H5Pclose(dcpl) < 0) goto error;
+    if(H5Dclose(dset) < 0) goto error;
+
     PASSED();
     return 0;
 
@@ -437,26 +446,28 @@ test_1f(hid_t file)
     hsize_t	max_size[1];		/*data space maximum size	*/
 
     TESTING("multiple external files");
-    if((dcpl=H5Pcreate(H5P_DATASET_CREATE))<0) goto error;
+
+    if((dcpl = H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
     cur_size[0] = max_size[0] = 100;
-    if (H5Pset_external(dcpl, "ext1.data", (off_t)0,
-	(hsize_t)(max_size[0]*sizeof(int)/4))<0) goto error;
-    if (H5Pset_external(dcpl, "ext2.data", (off_t)0,
-	(hsize_t)(max_size[0]*sizeof(int)/4))<0) goto error;
-    if (H5Pset_external(dcpl, "ext3.data", (off_t)0,
-	(hsize_t)(max_size[0]*sizeof(int)/4))<0) goto error;
-    if (H5Pset_external(dcpl, "ext4.data", (off_t)0,
-	(hsize_t)(max_size[0]*sizeof(int)/4))<0) goto error;
-    if ((space=H5Screate_simple(1, cur_size, max_size))<0) goto error;
-    if ((dset=H5Dcreate(file, "dset6", H5T_NATIVE_INT, space, dcpl))<0)
+    if(H5Pset_external(dcpl, "ext1.data", (off_t)0,
+	(hsize_t)(max_size[0]*sizeof(int)/4)) < 0) goto error;
+    if(H5Pset_external(dcpl, "ext2.data", (off_t)0,
+	(hsize_t)(max_size[0]*sizeof(int)/4)) < 0) goto error;
+    if(H5Pset_external(dcpl, "ext3.data", (off_t)0,
+	(hsize_t)(max_size[0]*sizeof(int)/4)) < 0) goto error;
+    if(H5Pset_external(dcpl, "ext4.data", (off_t)0,
+	(hsize_t)(max_size[0]*sizeof(int)/4)) < 0) goto error;
+    if((space = H5Screate_simple(1, cur_size, max_size)) < 0) goto error;
+    if((dset = H5Dcreate2(file, "dset6", H5T_NATIVE_INT, space, H5P_DEFAULT, dcpl, H5P_DEFAULT)) < 0)
 	goto error;
-    if (H5Dclose(dset)<0) goto error;
-    if (H5Sclose(space)<0) goto error;
-    if (H5Pclose(dcpl)<0) goto error;
+    if(H5Dclose(dset) < 0) goto error;
+    if(H5Sclose(space) < 0) goto error;
+    if(H5Pclose(dcpl) < 0) goto error;
+
     PASSED();
     return 0;
 
- error:
+error:
     H5E_BEGIN_TRY {
 	H5Dclose(dset);
 	H5Pclose(dcpl);
@@ -491,8 +502,8 @@ test_1g(void)
     int		n;			/*number of external files	*/
 
     TESTING("external file following unlimited file");
-    if ((dcpl=H5Pcreate (H5P_DATASET_CREATE))<0) goto error;
-    if (H5Pset_external(dcpl, "ext1.data", (off_t)0, H5F_UNLIMITED)<0) goto error;
+    if ((dcpl=H5Pcreate (H5P_DATASET_CREATE)) < 0) goto error;
+    if (H5Pset_external(dcpl, "ext1.data", (off_t)0, H5F_UNLIMITED) < 0) goto error;
     H5E_BEGIN_TRY {
 	status = H5Pset_external(dcpl, "ext2.data", (off_t)0, (hsize_t)100);
     } H5E_END_TRY;
@@ -501,13 +512,13 @@ test_1g(void)
 	puts ("    H5Pset_external() succeeded when it should have failed.");
 	goto error;
     }
-    if ((n = H5Pget_external_count(dcpl))<0) goto error;
+    if ((n = H5Pget_external_count(dcpl)) < 0) goto error;
     if (1!=n) {
 	H5_FAILED();
 	puts("    Wrong external file count returned.");
 	goto error;
     }
-    if (H5Pclose(dcpl)<0) goto error;
+    if (H5Pclose(dcpl) < 0) goto error;
     PASSED();
     return 0;
 
@@ -543,8 +554,8 @@ test_1h(void)
     herr_t	status;			/*return status			*/
 
     TESTING("address overflow in external files");
-    if((dcpl=H5Pcreate(H5P_DATASET_CREATE))<0) goto error;
-    if (H5Pset_external(dcpl, "ext1.data", (off_t)0, H5F_UNLIMITED-1)<0) goto error;
+    if((dcpl=H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
+    if (H5Pset_external(dcpl, "ext1.data", (off_t)0, H5F_UNLIMITED-1) < 0) goto error;
     H5E_BEGIN_TRY {
 	status = H5Pset_external(dcpl, "ext2.data", (off_t)0, (hsize_t)100);
     } H5E_END_TRY;
@@ -553,7 +564,7 @@ test_1h(void)
 	puts("    H5Pset_external() succeeded when it should have failed.");
 	goto error;
     }
-    if (H5Pclose(dcpl)<0) goto error;
+    if (H5Pclose(dcpl) < 0) goto error;
     PASSED();
     return 0;
 
@@ -601,7 +612,7 @@ test_2 (hid_t fapl)
 	int temparray[10] = {0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f,0x0f0f0f0f};
 
     TESTING("read external dataset");
-
+    
     /* Write the data to external files directly */
     for (i=0; i<4; i++) {
 	for (j=0; j<25; j++) {
@@ -625,60 +636,55 @@ test_2 (hid_t fapl)
      * output looks like.
      */
     h5_fixname(FILENAME[1], fapl, filename, sizeof filename);
-    if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl))<0) {
-	goto error;
-    }
-    if ((grp=H5Gcreate(file, "emit-diagnostics", 8))<0) goto error;
-    if (H5Gclose(grp)<0) goto error;
+    if((file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl)) < 0) FAIL_STACK_ERROR
+    if((grp = H5Gcreate2(file, "emit-diagnostics", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
+    if(H5Gclose(grp) < 0) FAIL_STACK_ERROR
 
     /* Create the dataset */
-    if((dcpl=H5Pcreate(H5P_DATASET_CREATE))<0) goto error;
-    if (H5Pset_external (dcpl, "extern_1a.raw",  (off_t)0, (hsize_t)sizeof part)<0 ||
-	H5Pset_external (dcpl, "extern_2a.raw", (off_t)10, (hsize_t)sizeof part)<0 ||
-	H5Pset_external (dcpl, "extern_3a.raw", (off_t)20, (hsize_t)sizeof part)<0 ||
-	H5Pset_external (dcpl, "extern_4a.raw", (off_t)30, (hsize_t)sizeof part)<0)
+    if((dcpl = H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
+    if(H5Pset_external(dcpl, "extern_1a.raw",  (off_t)0, (hsize_t)sizeof part) < 0 ||
+            H5Pset_external(dcpl, "extern_2a.raw", (off_t)10, (hsize_t)sizeof part) < 0 ||
+            H5Pset_external(dcpl, "extern_3a.raw", (off_t)20, (hsize_t)sizeof part) < 0 ||
+            H5Pset_external(dcpl, "extern_4a.raw", (off_t)30, (hsize_t)sizeof part) < 0)
 	goto error;
     cur_size = 100;
-    if ((space=H5Screate_simple (1, &cur_size, NULL))<0) goto error;
-    if ((dset=H5Dcreate(file, "dset1", H5T_NATIVE_INT, space, dcpl))<0)
-	goto error;
+    if((space = H5Screate_simple(1, &cur_size, NULL)) < 0) goto error;
+    if((dset = H5Dcreate2(file, "dset1", H5T_NATIVE_INT, space, H5P_DEFAULT, dcpl, H5P_DEFAULT)) < 0) goto error;
 
     /*
      * Read the entire dataset and compare with the original
      */
-    memset (whole, 0, sizeof(whole));
-    if (H5Dread(dset, H5T_NATIVE_INT, space, space, H5P_DEFAULT, whole)<0)
-	goto error;
-    for (i=0; i<100; i++) {
-	if (whole[i]!=(signed)i) {
+    memset(whole, 0, sizeof(whole));
+    if(H5Dread(dset, H5T_NATIVE_INT, space, space, H5P_DEFAULT, whole) < 0) goto error;
+    for(i = 0; i < 100; i++)
+	if(whole[i] != (signed)i) {
 	    H5_FAILED();
 	    puts("    Incorrect value(s) read.");
 	    goto error;
-	}
-    }
+	} /* end if */
 
     /*
      * Read the middle of the dataset
      */
-    if ((hs_space=H5Scopy(space))<0) goto error;
-    if (H5Sselect_hyperslab(hs_space, H5S_SELECT_SET, &hs_start, NULL,
-			    &hs_count, NULL)<0) goto error;
-    memset(whole, 0, sizeof(whole));
-    if (H5Dread (dset, H5T_NATIVE_INT, hs_space, hs_space, H5P_DEFAULT,
-		 whole)<0) goto error;
-    if (H5Sclose (hs_space)<0) goto error;
-    for (i=hs_start; i<hs_start+hs_count; i++) {
-	if (whole[i]!=(signed)i) {
+    if((hs_space = H5Scopy(space)) < 0) goto error;
+    if(H5Sselect_hyperslab(hs_space, H5S_SELECT_SET, &hs_start, NULL,
+			    &hs_count, NULL) < 0) goto error;
+    HDmemset(whole, 0, sizeof(whole));
+    if(H5Dread(dset, H5T_NATIVE_INT, hs_space, hs_space, H5P_DEFAULT,
+		 whole) < 0) goto error;
+    if(H5Sclose(hs_space) < 0) goto error;
+    for(i = hs_start; i<hs_start+hs_count; i++) {
+	if(whole[i] != (signed)i) {
 	    H5_FAILED();
 	    puts("    Incorrect value(s) read.");
 	    goto error;
 	}
     }
 
-    if (H5Dclose(dset)<0) goto error;
-    if (H5Pclose(dcpl)<0) goto error;
-    if (H5Sclose(space)<0) goto error;
-    if (H5Fclose(file)<0) goto error;
+    if (H5Dclose(dset) < 0) goto error;
+    if (H5Pclose(dcpl) < 0) goto error;
+    if (H5Sclose(space) < 0) goto error;
+    if (H5Fclose(file) < 0) goto error;
     PASSED();
     return 0;
 
@@ -731,22 +737,22 @@ test_3 (hid_t fapl)
 
     /* Create another file */
     h5_fixname(FILENAME[2], fapl, filename, sizeof filename);
-    if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl))<0) {
+    if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl)) < 0) {
 	goto error;
     }
 
     /* Create the external file list */
-    if((dcpl=H5Pcreate(H5P_DATASET_CREATE))<0) goto error;
-    if (H5Pset_external(dcpl, "extern_1b.raw", (off_t)0, (hsize_t)sizeof part)<0 ||
-	H5Pset_external(dcpl, "extern_2b.raw", (off_t)10, (hsize_t)sizeof part)<0 ||
-	H5Pset_external(dcpl, "extern_3b.raw", (off_t)20, (hsize_t)sizeof part)<0 ||
-	H5Pset_external(dcpl, "extern_4b.raw", (off_t)30, H5F_UNLIMITED)<0)
+    if((dcpl=H5Pcreate(H5P_DATASET_CREATE)) < 0) goto error;
+    if (H5Pset_external(dcpl, "extern_1b.raw", (off_t)0, (hsize_t)sizeof part) < 0 ||
+	H5Pset_external(dcpl, "extern_2b.raw", (off_t)10, (hsize_t)sizeof part) < 0 ||
+	H5Pset_external(dcpl, "extern_3b.raw", (off_t)20, (hsize_t)sizeof part) < 0 ||
+	H5Pset_external(dcpl, "extern_4b.raw", (off_t)30, H5F_UNLIMITED) < 0)
 	goto error;
 
     /* Make sure the output files are fresh*/
     for (i=1; i<=4; i++) {
 	sprintf(filename, "extern_%db.raw", i);
-	if ((fd= open(filename, O_RDWR|O_CREAT|O_TRUNC, 0666))<0) {
+	if ((fd= open(filename, O_RDWR|O_CREAT|O_TRUNC, 0666)) < 0) {
 	    H5_FAILED();
 	    printf("    cannot open %s: %s\n", filename, strerror(errno));
 	    goto error;
@@ -757,44 +763,45 @@ test_3 (hid_t fapl)
     }
 
     /* Create the dataset */
-    if ((mem_space=H5Screate_simple(1, &cur_size, &max_size))<0) goto error;
-    if ((file_space=H5Scopy(mem_space))<0) goto error;
-    if ((dset=H5Dcreate(file, "dset1", H5T_NATIVE_INT, file_space, dcpl))<0)
+    if((mem_space = H5Screate_simple(1, &cur_size, &max_size)) < 0) goto error;
+    if((file_space = H5Scopy(mem_space)) < 0) goto error;
+    if((dset = H5Dcreate2(file, "dset1", H5T_NATIVE_INT, file_space, H5P_DEFAULT, dcpl, H5P_DEFAULT)) < 0)
 	goto error;
 
     /* Write the entire dataset and compare with the original */
-    for (i=0; i<cur_size; i++) whole[i] = i;
-    if (H5Dwrite(dset, H5T_NATIVE_INT, mem_space, file_space, H5P_DEFAULT,
-		 whole)<0) goto error;
-    for (i=0; i<4; i++) {
+    for(i = 0; i < cur_size; i++)
+        whole[i] = i;
+    if(H5Dwrite(dset, H5T_NATIVE_INT, mem_space, file_space, H5P_DEFAULT, whole) < 0) goto error;
+    for(i = 0; i < 4; i++) {
 	char name1[64], name2[64];
-	sprintf (name1, "extern_%da.raw", i+1);
-	sprintf (name2, "extern_%db.raw", i+1);
-	if (!same_contents (name1, name2)) {
+
+	sprintf(name1, "extern_%da.raw", i + 1);
+	sprintf(name2, "extern_%db.raw", i + 1);
+	if(!same_contents(name1, name2)) {
 	    H5_FAILED();
 	    puts ("   Output differs from expected value.");
 	    goto error;
-	}
-    }
+	} /* end if */
+    } /* end for */
 
     /* Extend the dataset by another 100 elements */
-    if (H5Dextend(dset, &max_size)<0) goto error;
-    if (H5Sclose(file_space)<0) goto error;
-    if ((file_space=H5Dget_space(dset))<0) goto error;
+    if(H5Dset_extent(dset, &max_size) < 0) goto error;
+    if(H5Sclose(file_space) < 0) goto error;
+    if((file_space = H5Dget_space(dset)) < 0) goto error;
 
     /* Write second half of dataset */
-    for (i=0; i<hs_count; i++) whole[i] = 100+i;
-    if (H5Sselect_hyperslab(file_space, H5S_SELECT_SET, &hs_start, NULL,
-			    &hs_count, NULL)<0) goto error;
-    if (H5Dwrite(dset, H5T_NATIVE_INT, mem_space, file_space, H5P_DEFAULT,
-		 whole)<0) goto error;
+    for(i = 0; i < hs_count; i++)
+        whole[i] = 100 + i;
+    if(H5Sselect_hyperslab(file_space, H5S_SELECT_SET, &hs_start, NULL, &hs_count, NULL) < 0) goto error;
+    if(H5Dwrite(dset, H5T_NATIVE_INT, mem_space, file_space, H5P_DEFAULT, whole) < 0) goto error;
 
 
-    if (H5Dclose (dset)<0) goto error;
-    if (H5Pclose (dcpl)<0) goto error;
-    if (H5Sclose (mem_space)<0) goto error;
-    if (H5Sclose (file_space)<0) goto error;
-    if (H5Fclose (file)<0) goto error;
+    if(H5Dclose(dset) < 0) goto error;
+    if(H5Pclose(dcpl) < 0) goto error;
+    if(H5Sclose(mem_space) < 0) goto error;
+    if(H5Sclose(file_space) < 0) goto error;
+    if(H5Fclose(file) < 0) goto error;
+
     PASSED();
     return 0;
 
@@ -809,6 +816,104 @@ test_3 (hid_t fapl)
     return 1;
 }
 
+
+/*-------------------------------------------------------------------------
+ * Function:	test_4
+ *
+ * Purpose:	Tests opening an external link twice.  It exposed a bug
+ *              in the library.  This function tests the fix.  This test
+ *              doesn't work with MULTI driver.
+ *
+ * Return:	Success:	0
+ *
+ * 		Failure:	number of errors
+ *
+ * Programmer:	Raymond Lu
+ *              5 November 2007
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+static int
+test_4 (hid_t fapl)
+{
+    hid_t       fid, gid, xid, xid2;
+    char	filename[1024];		/*file name			*/
+    char        pathname[1024];
+    char       *srcdir = getenv("srcdir"); /*where the src code is located*/
+    const char *envval = NULL;
+
+    TESTING("opening external link twice");
+
+    /* Don't run this test using the multi file driver */
+    envval = HDgetenv("HDF5_DRIVER");
+    if (envval == NULL)
+        envval = "nomatch";
+    if (HDstrcmp(envval, "multi") && HDstrcmp(envval, "family")) {
+        h5_fixname(FILENAME[3], fapl, filename, sizeof filename);
+
+        if((fid = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl)) < 0)
+	    goto error;
+
+        if((gid = H5Gopen2(fid, "/", H5P_DEFAULT)) < 0)
+	    goto error;
+
+        pathname[0] = '\0';
+        /* Generate correct name for test file by prepending the source path */
+        if(srcdir && ((HDstrlen(srcdir) + HDstrlen(LINKED_FILE) + 1) < sizeof(pathname))) {
+            HDstrcpy(pathname, srcdir);
+            HDstrcat(pathname, "/");
+        }
+        HDstrcat(pathname, LINKED_FILE);
+
+        /* Create an external link to an existing file*/
+        if(H5Lcreate_external(pathname, "/group", gid, " link", H5P_DEFAULT, H5P_DEFAULT) < 0)
+	    goto error;
+
+        if(H5Gclose(gid) < 0)
+	    goto error;
+
+        if(H5Fclose(fid) < 0)
+	    goto error;
+
+        /* Reopen the file */
+        if((fid = H5Fopen(filename, H5F_ACC_RDONLY, fapl)) < 0)
+	    goto error;
+
+        /* Open the external link */
+        if((xid = H5Gopen2(fid, "/ link", H5P_DEFAULT)) < 0)
+	    goto error;
+
+        /* Open the external link twice */
+        if((xid2 = H5Gopen2(xid, ".", H5P_DEFAULT)) < 0)
+	    goto error;
+
+        if(H5Gclose(xid2) < 0)
+	    goto error;
+
+        if(H5Gclose(xid) < 0)
+	    goto error;
+
+        if(H5Fclose(fid) < 0)
+	    goto error;
+
+        PASSED();
+    } else 
+        SKIPPED();
+
+    return 0;
+
+ error:
+    H5E_BEGIN_TRY {
+        H5Gclose(gid);
+        H5Gclose(xid);
+        H5Gclose(xid2);
+	H5Fclose(fid);
+    } H5E_END_TRY;
+    return 1;
+}
+
 
 /*-------------------------------------------------------------------------
  * Function:	main
@@ -834,48 +939,58 @@ main (void)
     char	filename[1024];		/*file name for test_1* funcs	*/
     hid_t	grp=-1;			/*group to emit diagnostics	*/
     int		nerrors=0;		/*number of errors		*/
+    const char *envval = NULL;
 
-    h5_reset();
-    fapl = h5_fileaccess();
-    h5_fixname(FILENAME[0], fapl, filename, sizeof filename);
-    if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl))<0) {
-	goto error;
+    /* Don't run this test using the split file driver */
+    envval = HDgetenv("HDF5_DRIVER");
+    if (envval == NULL)
+        envval = "nomatch";
+    if (HDstrcmp(envval, "split")) {
+	h5_reset();
+	fapl = h5_fileaccess();
+	h5_fixname(FILENAME[0], fapl, filename, sizeof filename);
+	if((file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl)) < 0) FAIL_STACK_ERROR
+	if((grp = H5Gcreate2(file, "emit-diagnostics", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
+	if(H5Gclose(grp) < 0) goto error;
+
+	nerrors += test_1a(file);
+	nerrors += test_1b(file);
+	nerrors += test_1c(file);
+	nerrors += test_1d(file);
+	nerrors += test_1e(file);
+	nerrors += test_1f(file);
+	nerrors += test_1g();
+	nerrors += test_1h();
+	nerrors += test_2(fapl);
+	nerrors += test_3(fapl);
+	nerrors += test_4(fapl);
+	if (nerrors>0) goto error;
+
+	if (H5Fclose(file) < 0) goto error;
+	puts("All external storage tests passed.");
+	if (h5_cleanup(FILENAME, fapl)) {
+	    remove("extern_1a.raw");
+	    remove("extern_1b.raw");
+	    remove("extern_2a.raw");
+	    remove("extern_2b.raw");
+	    remove("extern_3a.raw");
+	    remove("extern_3b.raw");
+	    remove("extern_4a.raw");
+	    remove("extern_4b.raw");
+	}
     }
-    if ((grp=H5Gcreate(file, "emit-diagnostics", 8))<0) goto error;
-    if (H5Gclose (grp)<0) goto error;
-
-    nerrors += test_1a(file);
-    nerrors += test_1b(file);
-    nerrors += test_1c(file);
-    nerrors += test_1d(file);
-    nerrors += test_1e(file);
-    nerrors += test_1f(file);
-    nerrors += test_1g();
-    nerrors += test_1h();
-    nerrors += test_2(fapl);
-    nerrors += test_3(fapl);
-    if (nerrors>0) goto error;
-
-    if (H5Fclose(file)<0) goto error;
-    puts("All external storage tests passed.");
-    if (h5_cleanup(FILENAME, fapl)) {
-	remove("extern_1a.raw");
-	remove("extern_1b.raw");
-	remove("extern_2a.raw");
-	remove("extern_2b.raw");
-	remove("extern_3a.raw");
-	remove("extern_3b.raw");
-	remove("extern_4a.raw");
-	remove("extern_4b.raw");
+    else
+    {
+	puts("All external storage tests skipped - Incompatible with current Virtual File Driver");
     }
     return 0;
 
- error:
-    H5E_BEGIN_TRY {
-	H5Fclose(file);
-	H5Pclose(fapl);
-    } H5E_END_TRY;
-    nerrors = MAX(1, nerrors);
-    printf ("%d TEST%s FAILED.\n", nerrors, 1==nerrors?"":"s");
-    return 1;
+    error:
+	H5E_BEGIN_TRY {
+	    H5Fclose(file);
+	    H5Pclose(fapl);
+	} H5E_END_TRY;
+	nerrors = MAX(1, nerrors);
+	printf ("%d TEST%s FAILED.\n", nerrors, 1==nerrors?"":"s");
+	return 1;
 }
