@@ -26,11 +26,11 @@
 #include <sys/stat.h>
 #include "h5test.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <process.h>
 #include <direct.h>
 #include <winsock.h>
-#endif  /* WIN32 */
+#endif  /* _WIN32 */
 
 /*
  * Define these environment variables or constants to influence functions in
@@ -90,7 +90,7 @@ MPI_Info    h5_io_info_g=MPI_INFO_NULL;/* MPI INFO object for IO */
  */
 static const char *multi_letters = "msbrglo";
 
-static herr_t h5_errors(void *client_data);
+static herr_t h5_errors(hid_t estack, void *client_data);
 
 
 /*-------------------------------------------------------------------------
@@ -110,10 +110,10 @@ static herr_t h5_errors(void *client_data);
  *-------------------------------------------------------------------------
  */
 static herr_t
-h5_errors(void UNUSED *client_data)
+h5_errors(hid_t estack, void UNUSED *client_data)
 {
     H5_FAILED();
-    H5Eprint(stdout);
+    H5Eprint2(estack, stdout);
     return 0;
 }
 
@@ -210,12 +210,23 @@ h5_cleanup(const char *base_name[], hid_t fapl)
 void
 h5_reset(void)
 {
-    char	filename[1024];
-
     HDfflush(stdout);
     HDfflush(stderr);
     H5close();
-    H5Eset_auto(h5_errors, NULL);
+    H5Eset_auto2(H5E_DEFAULT, h5_errors, NULL);
+
+/*
+ * I commented this chunk of code out because it's not clear what diagnostics
+ *      were being output and under what circumstances, and creating this file
+ *      is throwing off debugging some of the tests.  I can't see any _direct_
+ *      harm in keeping this section of code, but I can't see any _direct_
+ *      benefit right now either.  If we figure out under which circumstances
+ *      diagnostics are being output, we should enable this behavior based on
+ *      appropriate configure flags/macros.  QAK - 2007/12/20
+ */
+#ifdef OLD_WAY
+{
+    char	filename[1024];
 
     /*
      * Cause the library to emit some diagnostics early so they don't
@@ -225,12 +236,14 @@ h5_reset(void)
     H5E_BEGIN_TRY {
 	hid_t file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT,
 			       H5P_DEFAULT);
-	hid_t grp = H5Gcreate(file, "emit", 0);
+	hid_t grp = H5Gcreate2(file, "emit", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
 	H5Gclose(grp);
 	H5Fclose(file);
 	HDunlink(filename);
     } H5E_END_TRY;
 }
+#endif /* OLD_WAY */
+}
 
 
 /*-------------------------------------------------------------------------
@@ -287,14 +300,14 @@ h5_fixname(const char *base_name, hid_t fapl, char *fullname, size_t size)
 
 	if (H5FD_FAMILY == driver)
 	    suffix = "%05d.h5";
-	else if (H5FD_CORE == driver || H5FD_MULTI == driver)
+	else if (H5FD_MULTI == driver)
 	    suffix = NULL;
     }
 
     /* Must first check fapl is not H5P_DEFAULT (-1) because H5FD_XXX
      * could be of value -1 if it is not defined.
      */
-    isppdriver = H5P_DEFAULT != fapl && 
+    isppdriver = H5P_DEFAULT != fapl &&
 	(H5FD_MPIO==driver || H5FD_MPIPOSIX==driver);
 
     /* Check HDF5_NOCLEANUP environment setting.
@@ -467,16 +480,16 @@ h5_fixname(const char *base_name, hid_t fapl, char *fullname, size_t size)
  *
  *-------------------------------------------------------------------------
  */
-char *
+const char *
 h5_rmprefix(const char *filename)
 {
-    char *ret_ptr;
+    const char *ret_ptr;
 
     if ((ret_ptr = HDstrstr(filename, ":")) == NULL)
 	ret_ptr = filename;
     else
 	ret_ptr++;
-    
+
     return(ret_ptr);
 }
 
@@ -528,7 +541,7 @@ h5_fileaccess(void)
 	if (H5Pset_fapl_stdio(fapl)<0) return -1;
     } else if (!HDstrcmp(name, "core")) {
 	/* In-core temporary file with 1MB increment */
-	if (H5Pset_fapl_core(fapl, 1024*1024, FALSE)<0) return -1;
+	if (H5Pset_fapl_core(fapl, (size_t)1024*1024, TRUE)<0) return -1;
     } else if (!HDstrcmp(name, "split")) {
 	/* Split meta data and raw data each using default driver */
 	if (H5Pset_fapl_split(fapl,
@@ -570,25 +583,24 @@ h5_fileaccess(void)
 	if (H5Pset_fapl_family(fapl, fam_size, H5P_DEFAULT)<0)
             return -1;
     } else if (!HDstrcmp(name, "log")) {
-#ifdef H5_WANT_H5_V1_4_COMPAT
-        long verbosity = 1;
-
-        /* Log file access */
-        if ((val = strtok(NULL, " \t\n\r")))
-            verbosity = strtol(val, NULL, 0);
-
-        if (H5Pset_fapl_log(fapl, NULL, (int)verbosity) < 0)
-	    return -1;
-#else /* H5_WANT_H5_V1_4_COMPAT */
-        unsigned log_flags = H5FD_LOG_LOC_IO;
+        unsigned log_flags = H5FD_LOG_LOC_IO | H5FD_LOG_ALLOC;
 
         /* Log file access */
         if ((val = HDstrtok(NULL, " \t\n\r")))
             log_flags = (unsigned)HDstrtol(val, NULL, 0);
 
-        if (H5Pset_fapl_log(fapl, NULL, log_flags, 0) < 0)
+        if (H5Pset_fapl_log(fapl, NULL, log_flags, (size_t)0) < 0)
 	    return -1;
-#endif /* H5_WANT_H5_V1_4_COMPAT */
+    } else if (!HDstrcmp(name, "direct")) {
+#ifdef H5_HAVE_DIRECT
+	/* Linux direct read() and write() system calls.  Set memory boundary, file block size,
+	 * and copy buffer size to the default values. */
+	if (H5Pset_fapl_direct(fapl, 1024, 4096, 8*4096)<0) return -1;
+#endif
+    } else if(!HDstrcmp(name, "latest")) {
+	/* use the latest format */
+	if(H5Pset_libver_bounds(fapl, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0)
+            return -1;
     } else {
 	/* Unknown driver */
 	return -1;
@@ -637,7 +649,7 @@ void
 h5_show_hostname(void)
 {
     char	hostname[80];
-#ifdef WIN32
+#ifdef _WIN32
      WSADATA wsaData;
      int err;
 #endif
@@ -655,7 +667,7 @@ h5_show_hostname(void)
 	    printf("thread 0.");
     }
 #elif defined(H5_HAVE_THREADSAFE)
-#ifdef WIN32
+#ifdef _WIN32
     printf("some thread: no way to know the thread number from pthread on windows.");
 #else
     printf("thread %d.", (int)pthread_self());
@@ -664,7 +676,7 @@ h5_show_hostname(void)
 #else
     printf("thread 0.");
 #endif
-#ifdef WIN32
+#ifdef _WIN32
 
    err = WSAStartup( MAKEWORD(2,2), &wsaData );
    if ( err != 0 ) {
@@ -687,15 +699,14 @@ h5_show_hostname(void)
 
 #endif
 #ifdef H5_HAVE_GETHOSTNAME
-    if (gethostname(hostname, 80) < 0){
+    if (gethostname(hostname, (size_t)80) < 0)
 	printf(" gethostname failed\n");
-    }
     else
 	printf(" hostname=%s\n", hostname);
 #else
     printf(" gethostname not supported\n");
 #endif
-#ifdef WIN32
+#ifdef _WIN32
     WSACleanup();
 #endif
 }
@@ -831,19 +842,19 @@ h5_dump_info_object(MPI_Info info)
 
 
 /*-------------------------------------------------------------------------
- * Function:    h5_get_file_size
+ * Function:	h5_get_file_size
  *
- * Purpose:     Get the current size of a file (in bytes)
+ * Purpose:	Get the current size of a file (in bytes)
  *
- * Return:      Success:        Size of file in bytes
- *              Failure:        -1
+ * Return:	Success:	Size of file in bytes
+ *		Failure:	-1
  *
- * Programmer:  Quincey Koziol
+ * Programmer:	Quincey Koziol
  *              Saturday, March 22, 2003
  *
  * Modifications:
- *      Albert Cheng, Nov 22, 2006
- *      Changed Failure return value to -1.
+ * 	Albert Cheng, Oct 11, 2006
+ *	Changed Failure return value to -1.
  *
  *-------------------------------------------------------------------------
  */
@@ -946,58 +957,75 @@ int h5_szip_can_encode(void )
  *              4/4/05
  *
  * Modifications:
+ *		Use original getenv if MPI is not initialized. This happens
+ *		one uses the PHDF5 library to build a serial nature code.
+ *		Albert 2006/04/07
  *
  *-------------------------------------------------------------------------
  */
 
 char* getenv_all(MPI_Comm comm, int root, const char* name)
 {
-    int nID;
-    int len = -1;
+    int mpi_size, mpi_rank, mpi_initialized;
+    int len;
     static char* env = NULL;
     MPI_Status Status;
 
     assert(name);
 
-    MPI_Comm_rank(comm, &nID);
-
-    /* The root task does the getenv call
-     * and sends the result to the other tasks */
-    if(nID == root)
-    {
-	env = HDgetenv(name);
+    MPI_Initialized(&mpi_initialized);
+    if (!mpi_initialized){
+	/* use original getenv */
 	if(env)
-	{
-	    len = HDstrlen(env);
-	    MPI_Bcast(&len, 1, MPI_INT, root, comm);
-	    MPI_Bcast(env, len, MPI_CHAR, root, comm);
-	}
-	/* len -1 indicates that the variable was not in the environment */
-	else
-	    MPI_Bcast(&len, 1, MPI_INT, root, comm);
-    }
-    else
-    {
-	MPI_Bcast(&len, 1, MPI_INT, root, comm);
-	if(len >= 0)
-	{
-	    if(env == NULL)
-		env = (char*) HDmalloc(len+1);
-	    else if(strlen(env) < len)
-		env = (char*) HDrealloc(env, len+1);
+	    HDfree(env);
+	env = HDgetenv(name);
+    }else{
+	MPI_Comm_rank(comm, &mpi_rank);
+	MPI_Comm_size(comm, &mpi_size);
+	assert(root < mpi_size);
 
-	    MPI_Bcast(env, len, MPI_CHAR, root, comm);
-	    env[len] = '\0';
-	}
-	else
+	/* The root task does the getenv call
+	 * and sends the result to the other tasks */
+	if(mpi_rank == root)
 	{
+	    env = HDgetenv(name);
 	    if(env)
-		HDfree(env);
-	    env = NULL;
+	    {
+		len = HDstrlen(env);
+		MPI_Bcast(&len, 1, MPI_INT, root, comm);
+		MPI_Bcast(env, len, MPI_CHAR, root, comm);
+	    }
+	    else{
+		/* len -1 indicates that the variable was not in the environment */
+		len = -1;
+		MPI_Bcast(&len, 1, MPI_INT, root, comm);
+	    }
+	}
+	else
+	{
+	    MPI_Bcast(&len, 1, MPI_INT, root, comm);
+	    if(len >= 0)
+	    {
+		if(env == NULL)
+		    env = (char*) HDmalloc(len+1);
+		else if(strlen(env) < len)
+		    env = (char*) HDrealloc(env, len+1);
+
+		MPI_Bcast(env, len, MPI_CHAR, root, comm);
+		env[len] = '\0';
+	    }
+	    else
+	    {
+		if(env)
+		    HDfree(env);
+		env = NULL;
+	    }
 	}
     }
 
+#ifndef NDEBUG
     MPI_Barrier(comm);
+#endif
 
     return env;
 }
