@@ -68,17 +68,17 @@ int main( void )
 
 
     /* Create a new file using default properties. */
-    if ((file_id = H5Fcreate( "set_extent_create.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT ))<0) TEST_ERROR;
+    if((file_id = H5Fcreate("set_extent_create.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT)) < 0) TEST_ERROR;
 
     TESTING("extend dataset create with fill value");
 
     /* Create the data space with unlimited dimensions. */
-    if ((space_id = H5Screate_simple( RANK, dims, maxdims ))<0) TEST_ERROR;
+    if((space_id = H5Screate_simple(RANK, dims, maxdims)) < 0) TEST_ERROR;
 
     /* Modify dataset creation properties, i.e. enable chunking. */
-    if ((plist_id = H5Pcreate (H5P_DATASET_CREATE ))<0) TEST_ERROR;
-    if (H5Pset_chunk( plist_id, RANK, dims_chunk )<0) TEST_ERROR;
-    if (H5Pset_fill_value( plist_id, H5T_NATIVE_INT, &fillvalue )<0) TEST_ERROR;
+    if((plist_id = H5Pcreate (H5P_DATASET_CREATE)) < 0) TEST_ERROR;
+    if(H5Pset_chunk(plist_id, RANK, dims_chunk) < 0) TEST_ERROR;
+    if(H5Pset_fill_value(plist_id, H5T_NATIVE_INT, &fillvalue) < 0) TEST_ERROR;
 
 
     /*-------------------------------------------------------------------------
@@ -87,10 +87,10 @@ int main( void )
     */
 
     /* Create a new dataset */
-    if ((dataset_id = H5Dcreate( file_id , "Dataset1", H5T_NATIVE_INT, space_id, plist_id ))<0) TEST_ERROR;
+    if((dataset_id = H5Dcreate2(file_id , "Dataset1", H5T_NATIVE_INT, space_id, H5P_DEFAULT, plist_id, H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Write the data. */
-    if (H5Dwrite( dataset_id , H5T_NATIVE_INT, space_id, H5S_ALL, H5P_DEFAULT, data )<0) TEST_ERROR;
+    if(H5Dwrite(dataset_id , H5T_NATIVE_INT, space_id, H5S_ALL, H5P_DEFAULT, data) < 0) TEST_ERROR;
 
     /*-------------------------------------------------------------------------
     * Set new dimensions for the array; shrink it
@@ -98,16 +98,16 @@ int main( void )
     */
 
     /* Set new dimensions for the array. */
-    if (H5Dset_extent( dataset_id , dims_new )<0) TEST_ERROR;
+    if(H5Dset_extent(dataset_id , dims_new) < 0) TEST_ERROR;
 
     /* Get the space. */
-    if ((space_id = H5Dget_space( dataset_id ))<0) TEST_ERROR;
+    if((space_id = H5Dget_space(dataset_id)) < 0) TEST_ERROR;
 
     /* Get dimensions. */
-    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL )<0) TEST_ERROR;
+    if(H5Sget_simple_extent_dims(space_id, dims_out, NULL) < 0) TEST_ERROR;
 
-    if ( dims_out[0] != dims_new[0] ) TEST_ERROR;
-    if ( dims_out[1] != dims_new[1] ) TEST_ERROR;
+    if(dims_out[0] != dims_new[0]) TEST_ERROR;
+    if(dims_out[1] != dims_new[1]) TEST_ERROR;
 
 
     /*-------------------------------------------------------------------------
@@ -116,34 +116,35 @@ int main( void )
     */
 
     /* Read the new dataset. */
-    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf1 )<0) TEST_ERROR;
+    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf1 ) < 0) TEST_ERROR;
 
 
     /* Compare the read array with the original array */
     for( i = 0; i < (int)dims_out[0]; i++ )
         for( j = 0; j < (int)dims_out[1]; j++ )
             if (  buf1[i][j] != data[i][j] ) {
-                printf("buf1[%d][%d]=%d\n",i,j,buf1[i][j]);
-                printf("data[%d][%d]=%d\n",i,j,data[i][j]);
+                printf("buf1[%d][%d] = %d\n", i, j, buf1[i][j]);
+                printf("data[%d][%d] = %d\n", i, j, data[i][j]);
                 TEST_ERROR;
             } /* end if */
 
 
     /*-------------------------------------------------------------------------
-    * Set new dimensions for the array; expand it again
+    * Set new dimensions for the array; expand it back to original size
     *-------------------------------------------------------------------------
     */
 
     /* Set new dimensions for the array. */
-    if (H5Dset_extent( dataset_id , dims )<0) TEST_ERROR;
+    if(H5Dset_extent(dataset_id, dims) < 0) TEST_ERROR;
 
     /* Get the space. */
-    if ((space_id = H5Dget_space( dataset_id ))<0) TEST_ERROR;
+    if((space_id = H5Dget_space(dataset_id)) < 0) TEST_ERROR;
 
     /* Get dimensions. */
-    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL )<0) TEST_ERROR;
+    if(H5Sget_simple_extent_dims(space_id, dims_out, NULL) < 0) TEST_ERROR;
 
-    if ( dims_out[0] != dims[0] ) TEST_ERROR;
+    if(dims_out[0] != dims[0]) TEST_ERROR;
+    if(dims_out[1] != dims[1]) TEST_ERROR;
 
 
     /*-------------------------------------------------------------------------
@@ -152,19 +153,21 @@ int main( void )
     */
 
     /* Read the new dataset. */
-    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf2 )<0) TEST_ERROR;
+    if(H5Dread(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf2) < 0) TEST_ERROR;
 
     /* Compare the read array with the original array */
-    for( i = 0; i < (int)dims_out[0]; i++ ) {
-        for( j = 0; j < (int)dims_out[1]; j++ ) {
-            if ( i >= 70 || j >= 70 ) {
-                if (  buf2[i][j] != fillvalue ) TEST_ERROR;
-            }
+    for(i = 0; i < (int)dims_out[0]; i++ )
+        for(j = 0; j < (int)dims_out[1]; j++ )
+            if(i >= 70 || j >= 70) {
+                if(buf2[i][j] != fillvalue) {
+                    printf("buf1[%d][%d] = %d\n", i, j, buf1[i][j]);
+                    printf("fillvalue = %d\n", fillvalue);
+                    TEST_ERROR;
+                } /* end if */
+            } /* end if */
             else {
-                if (  buf2[i][j] != data[i][j] ) TEST_ERROR;
+                if(buf2[i][j] != data[i][j]) TEST_ERROR;
             }
-        }
-    }
 
 
     /*-------------------------------------------------------------------------
@@ -172,20 +175,20 @@ int main( void )
     *-------------------------------------------------------------------------
     */
 
-    H5Dclose( dataset_id );
-    H5Sclose( space_id );
-    H5Pclose( plist_id  );
+    if(H5Dclose(dataset_id) < 0) TEST_ERROR
+    if(H5Sclose(space_id) < 0) TEST_ERROR
+    if(H5Pclose(plist_id) < 0) TEST_ERROR
 
     PASSED();
     TESTING("extend dataset create without fill value");
 
     /* Create the data space with unlimited dimensions. */
-    if ((space_id = H5Screate_simple( RANK, dims, maxdims ))<0) TEST_ERROR;
+    if((space_id = H5Screate_simple(RANK, dims, maxdims)) < 0) TEST_ERROR;
 
     /* Modify dataset creation properties, i.e. enable chunking. */
-    if ((plist_id = H5Pcreate (H5P_DATASET_CREATE ))<0) TEST_ERROR;
-    if (H5Pset_chunk( plist_id, RANK, dims_chunk )<0) TEST_ERROR;
-    if (H5Pset_fill_time( plist_id, H5D_FILL_TIME_ALLOC)<0) TEST_ERROR;
+    if((plist_id = H5Pcreate(H5P_DATASET_CREATE)) < 0) TEST_ERROR;
+    if(H5Pset_chunk(plist_id, RANK, dims_chunk) < 0) TEST_ERROR;
+    if(H5Pset_fill_time(plist_id, H5D_FILL_TIME_ALLOC) < 0) TEST_ERROR;
 
     /*-------------------------------------------------------------------------
     * Create and write one dataset
@@ -193,10 +196,10 @@ int main( void )
     */
 
     /* Create a new dataset */
-    if ((dataset_id = H5Dcreate( file_id , "Dataset2", H5T_NATIVE_INT, space_id, plist_id ))<0) TEST_ERROR;
+    if((dataset_id = H5Dcreate2(file_id , "Dataset2", H5T_NATIVE_INT, space_id, H5P_DEFAULT, plist_id, H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Write the data. */
-    if (H5Dwrite( dataset_id , H5T_NATIVE_INT, space_id, H5S_ALL, H5P_DEFAULT, data )<0) TEST_ERROR;
+    if(H5Dwrite(dataset_id , H5T_NATIVE_INT, space_id, H5S_ALL, H5P_DEFAULT, data) < 0) TEST_ERROR;
 
     /*-------------------------------------------------------------------------
     * Set new dimensions for the array; shrink it
@@ -204,15 +207,15 @@ int main( void )
     */
 
     /* Set new dimensions for the array. */
-    if (H5Dset_extent( dataset_id , dims_new )<0) TEST_ERROR;
+    if(H5Dset_extent(dataset_id , dims_new) < 0) TEST_ERROR;
 
     /* Get the space. */
-    if ((space_id = H5Dget_space( dataset_id ))<0) TEST_ERROR;
+    if((space_id = H5Dget_space(dataset_id)) < 0) TEST_ERROR;
 
     /* Get dimensions. */
-    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL )<0) TEST_ERROR;
+    if(H5Sget_simple_extent_dims(space_id, dims_out, NULL) < 0) TEST_ERROR;
 
-    if ( dims_out[0] != dims_new[0] ) TEST_ERROR;
+    if(dims_out[0] != dims_new[0]) TEST_ERROR;
 
 
     /*-------------------------------------------------------------------------
@@ -221,7 +224,7 @@ int main( void )
     */
 
     /* Read the new dataset. */
-    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf1 )<0) TEST_ERROR;
+    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf1 ) < 0) TEST_ERROR;
 
 
     /* Compare the read array with the original array */
@@ -236,13 +239,13 @@ int main( void )
     */
 
     /* Set new dimensions for the array. */
-    if (H5Dset_extent( dataset_id , dims )<0) TEST_ERROR;
+    if (H5Dset_extent( dataset_id , dims ) < 0) TEST_ERROR;
 
     /* Get the space. */
-    if ((space_id = H5Dget_space( dataset_id ))<0) TEST_ERROR;
+    if ((space_id = H5Dget_space( dataset_id )) < 0) TEST_ERROR;
 
     /* Get dimensions. */
-    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL )<0) TEST_ERROR;
+    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL ) < 0) TEST_ERROR;
 
     if ( dims_out[0] != dims[0] ) TEST_ERROR;
 
@@ -253,7 +256,7 @@ int main( void )
     */
 
     /* Read the new dataset. */
-    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf2 )<0) TEST_ERROR;
+    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf2 ) < 0) TEST_ERROR;
 
     /* Compare the read array with the original array */
     for( i = 0; i < (int)dims_out[0]; i++ ) {
@@ -289,64 +292,65 @@ int main( void )
     */
 
     /* Create a file creation property list */
-    if((fcpl = H5Pcreate(H5P_FILE_CREATE))<0) TEST_ERROR;
+    if((fcpl = H5Pcreate(H5P_FILE_CREATE)) < 0) TEST_ERROR;
 
     /* Set non-default indexed storage B-tree internal 'K' value */
-    if(H5Pset_istore_k(fcpl,ISTORE_IK)<0) TEST_ERROR;
+    if(H5Pset_istore_k(fcpl,ISTORE_IK) < 0) TEST_ERROR;
 
     /* Create a new file using properties. */
-    if ((file_id = H5Fcreate( "set_extent_read.h5", H5F_ACC_TRUNC, fcpl, H5P_DEFAULT ))<0) TEST_ERROR;
+    if((file_id = H5Fcreate("set_extent_read.h5", H5F_ACC_TRUNC, fcpl, H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Close property list */
-    if(H5Pclose(fcpl)<0) TEST_ERROR;
+    if(H5Pclose(fcpl) < 0) TEST_ERROR;
 
     TESTING("extend dataset read with fill value");
 
     /* Create the data space with unlimited dimensions. */
-    if ((space_id = H5Screate_simple( RANK, dims, maxdims ))<0) TEST_ERROR;
+    if((space_id = H5Screate_simple(RANK, dims, maxdims)) < 0) TEST_ERROR;
 
     /* Modify dataset creation properties, i.e. enable chunking. */
-    if ((plist_id = H5Pcreate (H5P_DATASET_CREATE ))<0) TEST_ERROR;
-    if (H5Pset_chunk( plist_id, RANK, dims_chunk )<0) TEST_ERROR;
-    if (H5Pset_fill_value( plist_id, H5T_NATIVE_INT, &fillvalue )<0) TEST_ERROR;
+    if((plist_id = H5Pcreate(H5P_DATASET_CREATE)) < 0) TEST_ERROR;
+    if(H5Pset_chunk(plist_id, RANK, dims_chunk) < 0) TEST_ERROR;
+    if(H5Pset_fill_value(plist_id, H5T_NATIVE_INT, &fillvalue) < 0) TEST_ERROR;
 
     /* Create a new dataset within the file using cparms creation properties. */
-    if ((dataset_id = H5Dcreate( file_id , "Dataset1", H5T_NATIVE_INT, space_id, plist_id ))<0) TEST_ERROR;
+    if((dataset_id = H5Dcreate2(file_id , "Dataset1", H5T_NATIVE_INT, space_id, H5P_DEFAULT, plist_id, H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Write the data. */
-    if (H5Dwrite( dataset_id , H5T_NATIVE_INT, space_id, H5S_ALL, H5P_DEFAULT, data )<0) TEST_ERROR;
+    if(H5Dwrite(dataset_id , H5T_NATIVE_INT, space_id, H5S_ALL, H5P_DEFAULT, data) < 0) TEST_ERROR;
 
     /* Close/release resources. */
-    H5Dclose( dataset_id );
-    H5Sclose( space_id );
-    H5Pclose( plist_id  );
-    H5Fclose( file_id );
+    if(H5Dclose(dataset_id) < 0) FAIL_STACK_ERROR
+    if(H5Sclose(space_id) < 0) FAIL_STACK_ERROR
+    if(H5Pclose(plist_id) < 0) FAIL_STACK_ERROR
+    if(H5Fclose(file_id) < 0) FAIL_STACK_ERROR
 
 
     /* Open the file */
-    if ((file_id = H5Fopen( "set_extent_read.h5", H5F_ACC_RDWR, H5P_DEFAULT ))<0) TEST_ERROR;
+    if((file_id = H5Fopen("set_extent_read.h5", H5F_ACC_RDWR, H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Open the dataset */
-    if ((dataset_id = H5Dopen( file_id , "Dataset1" ))<0) TEST_ERROR;
+    if((dataset_id = H5Dopen2(file_id , "Dataset1", H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Set new dimensions for the array. */
-    if (H5Dset_extent( dataset_id, dims_new )<0) TEST_ERROR;
+    if(H5Dset_extent(dataset_id, dims_new) < 0) TEST_ERROR;
 
     /* Get the space. */
-    if ((space_id = H5Dget_space( dataset_id ))<0) TEST_ERROR;
+    if((space_id = H5Dget_space(dataset_id)) < 0) TEST_ERROR;
 
     /* Get dimensions. */
-    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL )<0) TEST_ERROR;
+    if(H5Sget_simple_extent_dims(space_id, dims_out, NULL) < 0) TEST_ERROR;
 
-    if ( dims_out[0] != dims_new[0] ) TEST_ERROR;
+    if(dims_out[0] != dims_new[0]) TEST_ERROR;
 
     /* Read the new dataset. */
-    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf1 )<0) TEST_ERROR;
+    if(H5Dread(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf1) < 0) TEST_ERROR;
 
     /* Compare the read array with the original array */
-    for( i = 0; i < (int)dims_out[0]; i++ )
-        for( j = 0; j < (int)dims_out[1]; j++ )
-            if (  buf1[i][j] != data[i][j] )    TEST_ERROR;
+    for(i = 0; i < (int)dims_out[0]; i++)
+        for(j = 0; j < (int)dims_out[1]; j++)
+            if(buf1[i][j] != data[i][j])
+                TEST_ERROR;
 
     /*-------------------------------------------------------------------------
     * Set new dimensions for the array; expand it again
@@ -354,18 +358,18 @@ int main( void )
     */
 
     /* Set new dimensions for the array. */
-    if (H5Dset_extent( dataset_id , dims )<0) TEST_ERROR;
+    if (H5Dset_extent( dataset_id , dims ) < 0) TEST_ERROR;
 
     /* Get the space. */
-    if ((space_id = H5Dget_space( dataset_id ))<0) TEST_ERROR;
+    if ((space_id = H5Dget_space( dataset_id )) < 0) TEST_ERROR;
 
     /* Get dimensions. */
-    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL )<0) TEST_ERROR;
+    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL ) < 0) TEST_ERROR;
 
     if ( dims_out[0] != dims[0] ) TEST_ERROR;
 
     /* Read the new dataset. */
-    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf2 )<0) TEST_ERROR;
+    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf2 ) < 0) TEST_ERROR;
 
     /* Compare the read array with the original array */
     for( i = 0; i < (int)dims_out[0]; i++ ) {
@@ -394,50 +398,51 @@ int main( void )
     TESTING("extend dataset read without fill value");
 
     /* Create the data space with unlimited dimensions. */
-    if ((space_id = H5Screate_simple( RANK, dims, maxdims ))<0) TEST_ERROR;
+    if((space_id = H5Screate_simple(RANK, dims, maxdims)) < 0) TEST_ERROR;
 
     /* Modify dataset creation properties, i.e. enable chunking. */
-    if ((plist_id = H5Pcreate (H5P_DATASET_CREATE ))<0) TEST_ERROR;
-    if (H5Pset_chunk( plist_id, RANK, dims_chunk )<0) TEST_ERROR;
-    if (H5Pset_fill_time( plist_id, H5D_FILL_TIME_ALLOC)<0) TEST_ERROR;
+    if((plist_id = H5Pcreate(H5P_DATASET_CREATE)) < 0) TEST_ERROR;
+    if(H5Pset_chunk(plist_id, RANK, dims_chunk) < 0) TEST_ERROR;
+    if(H5Pset_fill_time(plist_id, H5D_FILL_TIME_ALLOC) < 0) TEST_ERROR;
 
     /* Create a new dataset within the file using cparms creation properties. */
-    if ((dataset_id = H5Dcreate( file_id , "Dataset2", H5T_NATIVE_INT, space_id, plist_id ))<0) TEST_ERROR;
+    if((dataset_id = H5Dcreate2(file_id , "Dataset2", H5T_NATIVE_INT, space_id, H5P_DEFAULT, plist_id, H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Write the data. */
-    if (H5Dwrite( dataset_id , H5T_NATIVE_INT, space_id, H5S_ALL, H5P_DEFAULT, data )<0) TEST_ERROR;
+    if(H5Dwrite(dataset_id , H5T_NATIVE_INT, space_id, H5S_ALL, H5P_DEFAULT, data) < 0) TEST_ERROR;
 
     /* Close/release resources. */
-    H5Dclose( dataset_id );
-    H5Sclose( space_id );
-    H5Pclose( plist_id  );
-    H5Fclose( file_id );
+    if(H5Dclose(dataset_id) < 0) FAIL_STACK_ERROR
+    if(H5Sclose(space_id) < 0) FAIL_STACK_ERROR
+    if(H5Pclose(plist_id) < 0) FAIL_STACK_ERROR
+    if(H5Fclose(file_id) < 0) FAIL_STACK_ERROR
 
 
     /* Open the file */
-    if ((file_id = H5Fopen( "set_extent_read.h5", H5F_ACC_RDWR, H5P_DEFAULT ))<0) TEST_ERROR;
+    if((file_id = H5Fopen("set_extent_read.h5", H5F_ACC_RDWR, H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Open the dataset */
-    if ((dataset_id = H5Dopen( file_id , "Dataset2" ))<0) TEST_ERROR;
+    if((dataset_id = H5Dopen2(file_id , "Dataset2", H5P_DEFAULT)) < 0) TEST_ERROR;
 
     /* Set new dimensions for the array. */
-    if (H5Dset_extent( dataset_id, dims_new )<0) TEST_ERROR;
+    if(H5Dset_extent(dataset_id, dims_new) < 0) TEST_ERROR;
 
     /* Get the space. */
-    if ((space_id = H5Dget_space( dataset_id ))<0) TEST_ERROR;
+    if((space_id = H5Dget_space(dataset_id)) < 0) TEST_ERROR;
 
     /* Get dimensions. */
-    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL )<0) TEST_ERROR;
+    if(H5Sget_simple_extent_dims(space_id, dims_out, NULL) < 0) TEST_ERROR;
 
-    if ( dims_out[0] != dims_new[0] ) TEST_ERROR;
+    if(dims_out[0] != dims_new[0]) TEST_ERROR;
 
     /* Read the new dataset. */
-    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf1 )<0) TEST_ERROR;
+    if(H5Dread(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf1) < 0) TEST_ERROR;
 
     /* Compare the read array with the original array */
-    for( i = 0; i < (int)dims_out[0]; i++ )
-        for( j = 0; j < (int)dims_out[1]; j++ )
-            if (  buf1[i][j] != data[i][j] ) TEST_ERROR;
+    for(i = 0; i < (int)dims_out[0]; i++)
+        for(j = 0; j < (int)dims_out[1]; j++)
+            if(buf1[i][j] != data[i][j])
+                TEST_ERROR;
 
     /*-------------------------------------------------------------------------
     * Set new dimensions for the array; expand it again
@@ -445,18 +450,18 @@ int main( void )
     */
 
     /* Set new dimensions for the array. */
-    if (H5Dset_extent( dataset_id , dims )<0) TEST_ERROR;
+    if (H5Dset_extent( dataset_id , dims ) < 0) TEST_ERROR;
 
     /* Get the space. */
-    if ((space_id = H5Dget_space( dataset_id ))<0) TEST_ERROR;
+    if ((space_id = H5Dget_space( dataset_id )) < 0) TEST_ERROR;
 
     /* Get dimensions. */
-    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL )<0) TEST_ERROR;
+    if (H5Sget_simple_extent_dims( space_id, dims_out, NULL ) < 0) TEST_ERROR;
 
     if ( dims_out[0] != dims[0] ) TEST_ERROR;
 
     /* Read the new dataset. */
-    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf2 )<0) TEST_ERROR;
+    if (H5Dread( dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf2 ) < 0) TEST_ERROR;
 
     /* Compare the read array with the original array */
     for( i = 0; i < (int)dims_out[0]; i++ ) {
