@@ -80,9 +80,9 @@ test_array_atomic_1d(void)
     MESSAGE(5, ("Testing 1-D Array of Atomic Datatypes Functionality\n"));
 
     /* Allocate and initialize array data to write */
-    for(i=0; i<SPACE1_DIM1; i++)
-        for(j=0; j<ARRAY1_DIM1; j++)
-            wdata[i][j]=i*10+j;
+    for(i = 0; i < SPACE1_DIM1; i++)
+        for(j = 0; j < ARRAY1_DIM1; j++)
+            wdata[i][j] = i * 10 + j;
 
     /* Create file */
     fid1 = H5Fcreate(FILENAME, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
@@ -93,15 +93,15 @@ test_array_atomic_1d(void)
     CHECK(sid1, FAIL, "H5Screate_simple");
 
     /* Create a datatype to refer to */
-    tid1 = H5Tarray_create (H5T_NATIVE_INT,ARRAY1_RANK,tdims1,NULL);
-    CHECK(tid1, FAIL, "H5Tarray_create");
+    tid1 = H5Tarray_create2(H5T_NATIVE_INT, ARRAY1_RANK, tdims1);
+    CHECK(tid1, FAIL, "H5Tarray_create2");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid1,"Dataset1",tid1,sid1,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset1", tid1, sid1, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Close Dataset */
@@ -126,41 +126,39 @@ test_array_atomic_1d(void)
     CHECK(fid1, FAIL, "H5Fopen");
 
     /* Open the dataset */
-    dataset=H5Dopen(fid1,"Dataset1");
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid1, "Dataset1", H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     /* Get the datatype */
     tid1 = H5Dget_type (dataset);
     CHECK(tid1, FAIL, "H5Dget_type");
 
     /* Check the array rank */
-    ndims=H5Tget_array_ndims(tid1);
-    VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
+    ndims = H5Tget_array_ndims(tid1);
+    VERIFY(ndims, ARRAY1_RANK, "H5Tget_array_ndims");
 
     /* Get the array dimensions */
-    ret=H5Tget_array_dims(tid1,rdims1,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid1, rdims1);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
-    for(i=0; i<ndims; i++)
-        if(rdims1[i]!=tdims1[i]) {
-            TestErrPrintf("Array dimension information doesn't match!, rdims1[%d]=%d, tdims1[%d]=%d\n",(int)i,(int)rdims1[i],(int)i,(int)tdims1[i]);
+    for(i = 0; i < ndims; i++)
+        if(rdims1[i] != tdims1[i]) {
+            TestErrPrintf("Array dimension information doesn't match!, rdims1[%d]=%d, tdims1[%d]=%d\n", (int)i, (int)rdims1[i], (int)i, (int)tdims1[i]);
             continue;
         } /* end if */
 
     /* Read dataset from disk */
-    ret=H5Dread(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Compare data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        for(j=0; j<ARRAY1_DIM1; j++) {
-            if(wdata[i][j]!=rdata[i][j]) {
-                TestErrPrintf("Array data information doesn't match!, wdata[%d][%d]=%d, rdata[%d][%d]=%d\n",(int)i,(int)j,(int)wdata[i][j],(int)i,(int)j,(int)rdata[i][j]);
+    for(i = 0; i < SPACE1_DIM1; i++)
+        for(j = 0; j < ARRAY1_DIM1; j++)
+            if(wdata[i][j] != rdata[i][j]) {
+                TestErrPrintf("Array data information doesn't match!, wdata[%d][%d]=%d, rdata[%d][%d]=%d\n", (int)i, (int)j, (int)wdata[i][j], (int)i, (int)j, (int)rdata[i][j]);
                 continue;
             } /* end if */
-        } /* end for */
-    } /* end for */
 
     /* Close Datatype */
     ret = H5Tclose(tid1);
@@ -173,9 +171,66 @@ test_array_atomic_1d(void)
     /* Close file */
     ret = H5Fclose(fid1);
     CHECK(ret, FAIL, "H5Fclose");
-
 } /* end test_array_atomic_1d() */
 
+/****************************************************************
+**
+**  test_array_funcs(): Test some type functions that are and
+**      aren't supposed to work with array type.
+**
+****************************************************************/
+static void
+test_array_funcs(void)
+{
+    hid_t		type;       /* Datatype ID */
+    hsize_t		tdims1[] = {ARRAY1_DIM1};
+    int                 size;
+    H5T_pad_t           inpad;
+    H5T_norm_t          norm;
+    H5T_cset_t          cset;
+    H5T_str_t           strpad;
+    herr_t		ret;	    /* Generic return value */
+
+    /* Create a datatype to refer to */
+    type = H5Tarray_create2(H5T_IEEE_F32BE, ARRAY1_RANK, tdims1);
+    CHECK(type, FAIL, "H5Tarray_create2");
+
+    size=H5Tget_precision(type);
+    CHECK(size, FAIL, "H5Tget_precision");
+
+    size=H5Tget_size(type);
+    CHECK(size, FAIL, "H5Tget_size");
+
+    size=H5Tget_ebias(type);
+    CHECK(size, FAIL, "H5Tget_ebias");
+
+    ret=H5Tset_pad(type, H5T_PAD_ZERO, H5T_PAD_ONE);
+    CHECK(ret, FAIL, "H5Tset_pad");
+
+    inpad=H5Tget_inpad(type);
+    CHECK(inpad, FAIL, "H5Tget_inpad");
+
+    norm=H5Tget_norm(type);
+    CHECK(norm, FAIL, "H5Tget_norm");
+
+    ret=H5Tset_offset(type, (size_t)16);
+    CHECK(ret, FAIL, "H5Tset_offset");
+
+    H5E_BEGIN_TRY {
+        cset=H5Tget_cset(type);
+    } H5E_END_TRY;
+    VERIFY(cset, FAIL, "H5Tget_cset");
+
+    H5E_BEGIN_TRY {
+        strpad=H5Tget_strpad(type);
+    } H5E_END_TRY;
+    VERIFY(strpad, FAIL, "H5Tget_strpad");
+
+    /* Close datatype */
+    ret = H5Tclose(type);
+    CHECK(ret, FAIL, "H5Tclose");
+} /* end test_array_funcs */
+
 /****************************************************************
 **
 **  test_array_atomic_3d(): Test basic array datatype code.
@@ -202,11 +257,11 @@ test_array_atomic_3d(void)
     MESSAGE(5, ("Testing 3-D Array of Atomic Datatypes Functionality\n"));
 
     /* Allocate and initialize array data to write */
-    for(i=0; i<SPACE1_DIM1; i++)
-        for(j=0; j<ARRAY2_DIM1; j++)
-            for(k=0; k<ARRAY2_DIM2; k++)
-                for(l=0; l<ARRAY2_DIM3; l++)
-                    wdata[i][j][k][l]=i*1000+j*100+k*10+l;
+    for(i = 0; i < SPACE1_DIM1; i++)
+        for(j = 0; j < ARRAY2_DIM1; j++)
+            for(k = 0; k < ARRAY2_DIM2; k++)
+                for(l = 0; l < ARRAY2_DIM3; l++)
+                    wdata[i][j][k][l] = i * 1000 + j * 100 + k * 10 + l;
 
     /* Create file */
     fid = H5Fcreate(FILENAME, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
@@ -217,15 +272,15 @@ test_array_atomic_3d(void)
     CHECK(sid, FAIL, "H5Screate_simple");
 
     /* Create a datatype to refer to */
-    tid = H5Tarray_create (H5T_NATIVE_INT,ARRAY2_RANK,tdims2,NULL);
-    CHECK(tid, FAIL, "H5Tarray_create");
+    tid = H5Tarray_create2(H5T_NATIVE_INT, ARRAY2_RANK, tdims2);
+    CHECK(tid, FAIL, "H5Tarray_create2");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid,"Dataset1",tid,sid,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid, "Dataset1", tid, sid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Close Dataset */
@@ -250,45 +305,41 @@ test_array_atomic_3d(void)
     CHECK(fid, FAIL, "H5Fopen");
 
     /* Open the dataset */
-    dataset=H5Dopen(fid,"Dataset1");
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid, "Dataset1", H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     /* Get the datatype */
     tid = H5Dget_type (dataset);
     CHECK(tid, FAIL, "H5Dget_type");
 
     /* Check the array rank */
-    ndims=H5Tget_array_ndims(tid);
-    VERIFY(ndims,ARRAY2_RANK,"H5Tget_array_ndims");
+    ndims = H5Tget_array_ndims(tid);
+    VERIFY(ndims, ARRAY2_RANK, "H5Tget_array_ndims");
 
     /* Get the array dimensions */
-    ret=H5Tget_array_dims(tid,rdims2,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid, rdims2);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
-    for(i=0; i<ndims; i++)
-        if(rdims2[i]!=tdims2[i]) {
-            TestErrPrintf("Array dimension information doesn't match!, rdims2[%d]=%d, tdims2[%d]=%d\n",(int)i,(int)rdims2[i],(int)i,(int)tdims2[i]);
+    for(i = 0; i < ndims; i++)
+        if(rdims2[i] != tdims2[i]) {
+            TestErrPrintf("Array dimension information doesn't match!, rdims2[%d]=%d, tdims2[%d]=%d\n", (int)i, (int)rdims2[i], (int)i, (int)tdims2[i]);
             continue;
         } /* end if */
 
     /* Read dataset from disk */
-    ret=H5Dread(dataset,tid,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Compare data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        for(j=0; j<ARRAY2_DIM1; j++) {
-            for(k=0; k<ARRAY2_DIM2; k++) {
-                for(l=0; l<ARRAY2_DIM3; l++) {
-                    if(wdata[i][j][k][l]!=rdata[i][j][k][l]) {
-                        TestErrPrintf("Array data information doesn't match!, wdata[%d][%d][%d][%d]=%d, rdata[%d][%d][%d][%d]=%d\n",(int)i,(int)j,(int)k,(int)l,(int)wdata[i][j][k][l],(int)i,(int)j,(int)k,(int)l,(int)rdata[i][j][k][l]);
+    for(i = 0; i < SPACE1_DIM1; i++)
+        for(j = 0; j < ARRAY2_DIM1; j++)
+            for(k = 0; k < ARRAY2_DIM2; k++)
+                for(l = 0; l < ARRAY2_DIM3; l++)
+                    if(wdata[i][j][k][l] != rdata[i][j][k][l]) {
+                        TestErrPrintf("Array data information doesn't match!, wdata[%d][%d][%d][%d]=%d, rdata[%d][%d][%d][%d]=%d\n", (int)i, (int)j, (int)k, (int)l, (int)wdata[i][j][k][l], (int)i, (int)j, (int)k, (int)l, (int)rdata[i][j][k][l]);
                         continue;
                     } /* end if */
-                } /* end for */
-            } /* end for */
-        } /* end for */
-    } /* end for */
 
     /* Close Datatype */
     ret = H5Tclose(tid);
@@ -334,11 +385,11 @@ test_array_array_atomic(void)
     MESSAGE(5, ("Testing 1-D Array 2-D Arrays of Atomic Datatypes Functionality\n"));
 
     /* Allocate and initialize array data to write */
-    for(i=0; i<SPACE1_DIM1; i++)
-        for(j=0; j<ARRAY1_DIM1; j++)
-            for(k=0; k<ARRAY3_DIM1; k++)
-                for(l=0; l<ARRAY3_DIM2; l++)
-                    wdata[i][j][k][l]=i*1000+j*100+k*10+l;
+    for(i = 0; i < SPACE1_DIM1; i++)
+        for(j = 0; j < ARRAY1_DIM1; j++)
+            for(k = 0; k < ARRAY3_DIM1; k++)
+                for(l = 0; l < ARRAY3_DIM2; l++)
+                    wdata[i][j][k][l] = i * 1000 + j * 100 + k * 10 + l;
 
     /* Create file */
     fid = H5Fcreate(FILENAME, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
@@ -349,19 +400,19 @@ test_array_array_atomic(void)
     CHECK(sid, FAIL, "H5Screate_simple");
 
     /* Create a 2-D datatype to refer to */
-    tid2 = H5Tarray_create (H5T_NATIVE_INT,ARRAY3_RANK,tdims2,NULL);
-    CHECK(tid2, FAIL, "H5Tarray_create");
+    tid2 = H5Tarray_create2(H5T_NATIVE_INT, ARRAY3_RANK, tdims2);
+    CHECK(tid2, FAIL, "H5Tarray_create2");
 
     /* Create a 1-D datatype to refer to */
-    tid1 = H5Tarray_create (tid2,ARRAY1_RANK,tdims1,NULL);
-    CHECK(tid1, FAIL, "H5Tarray_create");
+    tid1 = H5Tarray_create2(tid2, ARRAY1_RANK, tdims1);
+    CHECK(tid1, FAIL, "H5Tarray_create2");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid,"Dataset1",tid1,sid,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid, "Dataset1", tid1, sid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Close Dataset */
@@ -388,64 +439,60 @@ test_array_array_atomic(void)
     CHECK(fid, FAIL, "H5Fopen");
 
     /* Open the dataset */
-    dataset=H5Dopen(fid,"Dataset1");
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid, "Dataset1", H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     /* Get the 1-D datatype */
-    tid1 = H5Dget_type (dataset);
+    tid1 = H5Dget_type(dataset);
     CHECK(tid1, FAIL, "H5Dget_type");
 
     /* Check the 1-D array rank */
-    ndims1=H5Tget_array_ndims(tid1);
-    VERIFY(ndims1,ARRAY1_RANK,"H5Tget_array_ndims");
+    ndims1 = H5Tget_array_ndims(tid1);
+    VERIFY(ndims1, ARRAY1_RANK, "H5Tget_array_ndims");
 
     /* Get the 1-D array dimensions */
-    ret=H5Tget_array_dims(tid1,rdims1,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid1, rdims1);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
-    for(i=0; i<ndims1; i++)
-        if(rdims1[i]!=tdims1[i]) {
-            TestErrPrintf("Array dimension information doesn't match!, rdims1[%d]=%d, tdims1[%d]=%d\n",(int)i,(int)rdims1[i],(int)i,(int)tdims1[i]);
+    for(i = 0; i < ndims1; i++)
+        if(rdims1[i] != tdims1[i]) {
+            TestErrPrintf("Array dimension information doesn't match!, rdims1[%d]=%d, tdims1[%d]=%d\n", (int)i, (int)rdims1[i], (int)i, (int)tdims1[i]);
             continue;
         } /* end if */
 
     /* Get the 2-D datatype */
-    tid2 = H5Tget_super (tid1);
+    tid2 = H5Tget_super(tid1);
     CHECK(tid2, FAIL, "H5Tget_super");
 
     /* Check the 2-D array rank */
-    ndims2=H5Tget_array_ndims(tid2);
-    VERIFY(ndims2,ARRAY3_RANK,"H5Tget_array_ndims");
+    ndims2 = H5Tget_array_ndims(tid2);
+    VERIFY(ndims2, ARRAY3_RANK, "H5Tget_array_ndims");
 
     /* Get the 2-D array dimensions */
-    ret=H5Tget_array_dims(tid2,rdims2,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid2, rdims2);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
-    for(i=0; i<ndims2; i++)
-        if(rdims2[i]!=tdims2[i]) {
-            TestErrPrintf("Array dimension information doesn't match!, rdims2[%d]=%d, tdims2[%d]=%d\n",(int)i,(int)rdims2[i],(int)i,(int)tdims2[i]);
+    for(i = 0; i < ndims2; i++)
+        if(rdims2[i] != tdims2[i]) {
+            TestErrPrintf("Array dimension information doesn't match!, rdims2[%d]=%d, tdims2[%d]=%d\n", (int)i, (int)rdims2[i], (int)i, (int)tdims2[i]);
             continue;
         } /* end if */
 
     /* Read dataset from disk */
-    ret=H5Dread(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Compare data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        for(j=0; j<ARRAY1_DIM1; j++) {
-            for(k=0; k<ARRAY3_DIM1; k++) {
-                for(l=0; l<ARRAY3_DIM2; l++) {
-                    if(wdata[i][j][k][l]!=rdata[i][j][k][l]) {
-                        TestErrPrintf("Array data information doesn't match!, wdata[%d][%d][%d][%d]=%d, rdata[%d][%d][%d][%d]=%d\n",(int)i,(int)j,(int)k,(int)l,(int)wdata[i][j][k][l],(int)i,(int)j,(int)k,(int)l,(int)rdata[i][j][k][l]);
+    for(i = 0; i < SPACE1_DIM1; i++)
+        for(j = 0; j < ARRAY1_DIM1; j++)
+            for(k = 0; k < ARRAY3_DIM1; k++)
+                for(l = 0; l < ARRAY3_DIM2; l++)
+                    if(wdata[i][j][k][l] != rdata[i][j][k][l]) {
+                        TestErrPrintf("Array data information doesn't match!, wdata[%d][%d][%d][%d]=%d, rdata[%d][%d][%d][%d]=%d\n", (int)i, (int)j, (int)k, (int)l, (int)wdata[i][j][k][l], (int)i, (int)j, (int)k, (int)l, (int)rdata[i][j][k][l]);
                         continue;
                     } /* end if */
-                } /* end for */
-            } /* end for */
-        } /* end for */
-    } /* end for */
 
     /* Close Datatypes */
     ret = H5Tclose(tid1);
@@ -460,7 +507,6 @@ test_array_array_atomic(void)
     /* Close file */
     ret = H5Fclose(fid);
     CHECK(ret, FAIL, "H5Fclose");
-
 } /* end test_array_array_atomic() */
 
 /****************************************************************
@@ -498,10 +544,10 @@ test_array_compound_atomic(void)
     MESSAGE(5, ("Testing 1-D Array of Compound Atomic Datatypes Functionality\n"));
 
     /* Initialize array data to write */
-    for(i=0; i<SPACE1_DIM1; i++)
-        for(j=0; j<ARRAY1_DIM1; j++) {
-            wdata[i][j].i=i*10+j;
-            wdata[i][j].f=(float)(i*2.5+j);
+    for(i = 0; i < SPACE1_DIM1; i++)
+        for(j = 0; j < ARRAY1_DIM1; j++) {
+            wdata[i][j].i = i * 10 + j;
+            wdata[i][j].f = (float)(i * 2.5 + j);
         } /* end for */
 
     /* Create file */
@@ -517,27 +563,27 @@ test_array_compound_atomic(void)
     CHECK(tid2, FAIL, "H5Tcreate");
 
     /* Insert integer field */
-    ret = H5Tinsert (tid2, "i", HOFFSET(s1_t,i), H5T_NATIVE_INT);
+    ret = H5Tinsert(tid2, "i", HOFFSET(s1_t, i), H5T_NATIVE_INT);
     CHECK(ret, FAIL, "H5Tinsert");
 
     /* Insert float field */
-    ret = H5Tinsert (tid2, "f", HOFFSET(s1_t,f), H5T_NATIVE_FLOAT);
+    ret = H5Tinsert(tid2, "f", HOFFSET(s1_t, f), H5T_NATIVE_FLOAT);
     CHECK(ret, FAIL, "H5Tinsert");
 
     /* Create an array datatype to refer to */
-    tid1 = H5Tarray_create (tid2,ARRAY1_RANK,tdims1,NULL);
-    CHECK(tid1, FAIL, "H5Tarray_create");
+    tid1 = H5Tarray_create2(tid2, ARRAY1_RANK, tdims1);
+    CHECK(tid1, FAIL, "H5Tarray_create2");
 
     /* Close compound datatype */
-    ret=H5Tclose(tid2);
+    ret = H5Tclose(tid2);
     CHECK(ret, FAIL, "H5Tclose");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid1,"Dataset1",tid1,sid1,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset1", tid1, sid1, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Close Dataset */
@@ -562,25 +608,25 @@ test_array_compound_atomic(void)
     CHECK(fid1, FAIL, "H5Fopen");
 
     /* Open the dataset */
-    dataset=H5Dopen(fid1,"Dataset1");
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid1, "Dataset1", H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     /* Get the datatype */
-    tid1 = H5Dget_type (dataset);
+    tid1 = H5Dget_type(dataset);
     CHECK(tid1, FAIL, "H5Dget_type");
 
     /* Check the array rank */
-    ndims=H5Tget_array_ndims(tid1);
+    ndims = H5Tget_array_ndims(tid1);
     VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
 
     /* Get the array dimensions */
-    ret=H5Tget_array_dims(tid1,rdims1,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid1, rdims1);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
-    for(i=0; i<ndims; i++)
-        if(rdims1[i]!=tdims1[i]) {
-            TestErrPrintf("Array dimension information doesn't match!, rdims1[%d]=%d, tdims1[%d]=%d\n",(int)i,(int)rdims1[i],(int)i,(int)tdims1[i]);
+    for(i = 0; i < ndims; i++)
+        if(rdims1[i] != tdims1[i]) {
+            TestErrPrintf("Array dimension information doesn't match!, rdims1[%d]=%d, tdims1[%d]=%d\n", (int)i, (int)rdims1[i], (int)i, (int)tdims1[i]);
             continue;
         } /* end if */
 
@@ -589,45 +635,45 @@ test_array_compound_atomic(void)
     CHECK(tid2, FAIL, "H5Tget_super");
 
     /* Check the number of members */
-    nmemb=H5Tget_nmembers(tid2);
+    nmemb = H5Tget_nmembers(tid2);
     VERIFY(nmemb,2,"H5Tget_nmembers");
 
     /* Check the 1st field's name */
-    mname=H5Tget_member_name(tid2,0);
+    mname = H5Tget_member_name(tid2, 0);
     CHECK(mname, NULL, "H5Tget_member_name");
-    if(HDstrcmp(mname,"i")!=0)
-        TestErrPrintf("Compound field name doesn't match!, mname=%s\n",mname);
+    if(HDstrcmp(mname, "i") != 0)
+        TestErrPrintf("Compound field name doesn't match!, mname=%s\n", mname);
     free(mname);
 
     /* Check the 1st field's offset */
-    off=H5Tget_member_offset(tid2,0);
-    VERIFY(off, HOFFSET(s1_t,i), "H5Tget_member_offset");
+    off = H5Tget_member_offset(tid2, 0);
+    VERIFY(off, HOFFSET(s1_t, i), "H5Tget_member_offset");
 
     /* Check the 1st field's datatype */
-    mtid=H5Tget_member_type(tid2,0);
+    mtid = H5Tget_member_type(tid2, 0);
     CHECK(mtid, FAIL, "H5Tget_member_type");
-    if((ret=H5Tequal(mtid,H5T_NATIVE_INT))<=0)
-        TestErrPrintf("Compound data type is incorrect!, ret=%d\n",(int)ret);
-    ret=H5Tclose(mtid);
+    if((ret = H5Tequal(mtid,H5T_NATIVE_INT)) <= 0)
+        TestErrPrintf("Compound data type is incorrect!, ret=%d\n", (int)ret);
+    ret = H5Tclose(mtid);
     CHECK(mtid, FAIL, "H5Tclose");
 
     /* Check the 2nd field's name */
-    mname=H5Tget_member_name(tid2,1);
+    mname = H5Tget_member_name(tid2, 1);
     CHECK(mname, NULL, "H5Tget_member_name");
-    if(HDstrcmp(mname,"f")!=0)
-        TestErrPrintf("Compound field name doesn't match!, mname=%s\n",mname);
+    if(HDstrcmp(mname, "f") != 0)
+        TestErrPrintf("Compound field name doesn't match!, mname=%s\n", mname);
     free(mname);
 
     /* Check the 2nd field's offset */
-    off=H5Tget_member_offset(tid2,1);
-    VERIFY(off, HOFFSET(s1_t,f), "H5Tget_member_offset");
+    off = H5Tget_member_offset(tid2, 1);
+    VERIFY(off, HOFFSET(s1_t, f), "H5Tget_member_offset");
 
     /* Check the 2nd field's datatype */
-    mtid=H5Tget_member_type(tid2,1);
+    mtid = H5Tget_member_type(tid2, 1);
     CHECK(mtid, FAIL, "H5Tget_member_type");
-    if((ret=H5Tequal(mtid,H5T_NATIVE_FLOAT))<=0)
-        TestErrPrintf("Compound data type is incorrect!, ret=%d\n",(int)ret);
-    ret=H5Tclose(mtid);
+    if((ret = H5Tequal(mtid, H5T_NATIVE_FLOAT)) <= 0)
+        TestErrPrintf("Compound data type is incorrect!, ret=%d\n", (int)ret);
+    ret = H5Tclose(mtid);
     CHECK(mtid, FAIL, "H5Tclose");
 
     /* Close Compound Datatype */
@@ -635,22 +681,21 @@ test_array_compound_atomic(void)
     CHECK(ret, FAIL, "H5Tclose");
 
     /* Read dataset from disk */
-    ret=H5Dread(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Compare data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        for(j=0; j<ARRAY1_DIM1; j++) {
-            if(wdata[i][j].i!=rdata[i][j].i) {
-                TestErrPrintf("Array data information doesn't match!, wdata[%d][%d].i=%d, rdata[%d][%d].i=%d\n",(int)i,(int)j,(int)wdata[i][j].i,(int)i,(int)j,(int)rdata[i][j].i);
+    for(i = 0; i < SPACE1_DIM1; i++)
+        for(j = 0; j < ARRAY1_DIM1; j++) {
+            if(wdata[i][j].i != rdata[i][j].i) {
+                TestErrPrintf("Array data information doesn't match!, wdata[%d][%d].i=%d, rdata[%d][%d].i=%d\n", (int)i, (int)j, (int)wdata[i][j].i, (int)i, (int)j, (int)rdata[i][j].i);
                 continue;
             } /* end if */
-            if(wdata[i][j].f!=rdata[i][j].f) {
-                TestErrPrintf("Array data information doesn't match!, wdata[%d][%d].f=%f, rdata[%d][%d].f=%f\n",(int)i,(int)j,wdata[i][j].f,(int)i,(int)j,rdata[i][j].f);
+            if(!FLT_ABS_EQUAL(wdata[i][j].f, rdata[i][j].f)) {
+                TestErrPrintf("Array data information doesn't match!, wdata[%d][%d].f=%f, rdata[%d][%d].f=%f\n", (int)i, (int)j, wdata[i][j].f, (int)i, (int)j, rdata[i][j].f);
                 continue;
             } /* end if */
         } /* end for */
-    } /* end for */
 
     /* Close Datatype */
     ret = H5Tclose(tid1);
@@ -663,7 +708,6 @@ test_array_compound_atomic(void)
     /* Close file */
     ret = H5Fclose(fid1);
     CHECK(ret, FAIL, "H5Fclose");
-
 } /* end test_array_compound_atomic() */
 
 /****************************************************************
@@ -727,31 +771,31 @@ test_array_compound_array(void)
     CHECK(ret, FAIL, "H5Tinsert");
 
     /* Create an array of floats datatype */
-    tid3 = H5Tarray_create (H5T_NATIVE_FLOAT,ARRAY1_RANK,tdims1,NULL);
-    CHECK(tid3, FAIL, "H5Tarray_create");
+    tid3 = H5Tarray_create2(H5T_NATIVE_FLOAT, ARRAY1_RANK, tdims1);
+    CHECK(tid3, FAIL, "H5Tarray_create2");
 
     /* Insert float array field */
-    ret = H5Tinsert (tid2, "f", HOFFSET(s1_t,f), tid3);
+    ret = H5Tinsert(tid2, "f", HOFFSET(s1_t, f), tid3);
     CHECK(ret, FAIL, "H5Tinsert");
 
     /* Close array of floats field datatype */
-    ret=H5Tclose(tid3);
+    ret = H5Tclose(tid3);
     CHECK(ret, FAIL, "H5Tclose");
 
     /* Create an array datatype to refer to */
-    tid1 = H5Tarray_create (tid2,ARRAY1_RANK,tdims1,NULL);
-    CHECK(tid1, FAIL, "H5Tarray_create");
+    tid1 = H5Tarray_create2(tid2, ARRAY1_RANK, tdims1);
+    CHECK(tid1, FAIL, "H5Tarray_create2");
 
     /* Close compound datatype */
-    ret=H5Tclose(tid2);
+    ret = H5Tclose(tid2);
     CHECK(ret, FAIL, "H5Tclose");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid1,"Dataset1",tid1,sid1,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset1", tid1, sid1, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Close Dataset */
@@ -776,8 +820,8 @@ test_array_compound_array(void)
     CHECK(fid1, FAIL, "H5Fopen");
 
     /* Open the dataset */
-    dataset=H5Dopen(fid1,"Dataset1");
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid1, "Dataset1", H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     /* Get the datatype */
     tid1 = H5Dget_type (dataset);
@@ -788,8 +832,8 @@ test_array_compound_array(void)
     VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
 
     /* Get the array dimensions */
-    ret=H5Tget_array_dims(tid1,rdims1,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid1, rdims1);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
     for(i=0; i<ndims; i++)
@@ -849,8 +893,8 @@ test_array_compound_array(void)
     VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
 
     /* Get the array dimensions */
-    ret=H5Tget_array_dims(mtid,rdims1,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(mtid, rdims1);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
     for(i=0; i<ndims; i++)
@@ -890,7 +934,7 @@ test_array_compound_array(void)
                 continue;
             } /* end if */
             for(k=0; k<ARRAY1_DIM1; k++)
-                if(wdata[i][j].f[k]!=rdata[i][j].f[k]) {
+                if(!FLT_ABS_EQUAL(wdata[i][j].f[k],rdata[i][j].f[k])) {
                     TestErrPrintf("Array data information doesn't match!, wdata[%d][%d].f[%d]=%f, rdata[%d][%d].f[%d]=%f\n",(int)i,(int)j,(int)k,wdata[i][j].f[k],(int)i,(int)j,(int)k,rdata[i][j].f[k]);
                     continue;
                 } /* end if */
@@ -1022,19 +1066,19 @@ test_array_vlen_atomic(void)
     CHECK(tid2, FAIL, "H5Tcreate");
 
     /* Create an array datatype to refer to */
-    tid1 = H5Tarray_create (tid2,ARRAY1_RANK,tdims1,NULL);
-    CHECK(tid1, FAIL, "H5Tarray_create");
+    tid1 = H5Tarray_create2(tid2, ARRAY1_RANK, tdims1);
+    CHECK(tid1, FAIL, "H5Tarray_create2");
 
     /* Close VL datatype */
-    ret=H5Tclose(tid2);
+    ret = H5Tclose(tid2);
     CHECK(ret, FAIL, "H5Tclose");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid1,"Dataset1",tid1,sid1,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset1", tid1, sid1, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Close Dataset */
@@ -1059,8 +1103,8 @@ test_array_vlen_atomic(void)
     CHECK(fid1, FAIL, "H5Fopen");
 
     /* Open the dataset */
-    dataset=H5Dopen(fid1,"Dataset1");
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid1, "Dataset1", H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     /* Get the dataspace */
     sid1 = H5Dget_space (dataset);
@@ -1075,8 +1119,8 @@ test_array_vlen_atomic(void)
     VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
 
     /* Get the array dimensions */
-    ret=H5Tget_array_dims(tid1,rdims1,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid1, rdims1);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
     for(i=0; i<ndims; i++)
@@ -1230,31 +1274,31 @@ test_array_vlen_array(void)
     CHECK(sid1, FAIL, "H5Screate_simple");
 
     /* Create the nested array datatype to refer to */
-    tid3 = H5Tarray_create(H5T_NATIVE_UINT,ARRAY1_RANK,tdims1,NULL);
-    CHECK(tid3, FAIL, "H5Tcreate");
+    tid3 = H5Tarray_create2(H5T_NATIVE_UINT, ARRAY1_RANK, tdims1);
+    CHECK(tid3, FAIL, "H5Tarray_create2");
 
     /* Create a VL datatype of 1-D arrays to refer to */
     tid2 = H5Tvlen_create(tid3);
     CHECK(tid2, FAIL, "H5Tcreate");
 
     /* Close nested array datatype */
-    ret=H5Tclose(tid3);
+    ret = H5Tclose(tid3);
     CHECK(ret, FAIL, "H5Tclose");
 
     /* Create an array datatype to refer to */
-    tid1 = H5Tarray_create (tid2,ARRAY1_RANK,tdims1,NULL);
-    CHECK(tid1, FAIL, "H5Tarray_create");
+    tid1 = H5Tarray_create2(tid2, ARRAY1_RANK, tdims1);
+    CHECK(tid1, FAIL, "H5Tarray_create2");
 
     /* Close VL datatype */
-    ret=H5Tclose(tid2);
+    ret = H5Tclose(tid2);
     CHECK(ret, FAIL, "H5Tclose");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid1,"Dataset1",tid1,sid1,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset1", tid1, sid1, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Close Dataset */
@@ -1279,8 +1323,8 @@ test_array_vlen_array(void)
     CHECK(fid1, FAIL, "H5Fopen");
 
     /* Open the dataset */
-    dataset=H5Dopen(fid1,"Dataset1");
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid1, "Dataset1", H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     /* Get the dataspace */
     sid1 = H5Dget_space (dataset);
@@ -1295,8 +1339,8 @@ test_array_vlen_array(void)
     VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
 
     /* Get the array dimensions */
-    ret=H5Tget_array_dims(tid1,rdims1,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid1, rdims1);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
     for(i=0; i<ndims; i++)
@@ -1326,8 +1370,8 @@ test_array_vlen_array(void)
     VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
 
     /* Get the array dimensions */
-    ret=H5Tget_array_dims(tid3,rdims1,NULL);
-    CHECK(ret, FAIL, "H5Tget_array_dims");
+    ret = H5Tget_array_dims2(tid3, rdims1);
+    CHECK(ret, FAIL, "H5Tget_array_dims2");
 
     /* Check the array dimensions */
     for(i=0; i<ndims; i++)
@@ -1465,7 +1509,7 @@ test_array_bkg(void)
     hsize_t      dima[] = {ALEN};
 
     int          i, j;
-    int          ndims[3] = {1,1,1};
+    unsigned     ndims[3] = {1,1,1};
 
     typedef struct
     {
@@ -1522,7 +1566,7 @@ test_array_bkg(void)
     /* Initialize the names of data members */
     /* ------------------------------------ */
     for (i = 0; i < dtsinfo.nsubfields; i++)
-      dtsinfo.name[i] = (char *)calloc(20, sizeof(char));
+      dtsinfo.name[i] = (char *)HDcalloc((size_t)20, sizeof(char));
 
 	strcpy(dtsinfo.name[0], "One");
 	strcpy(dtsinfo.name[1], "Two");
@@ -1549,8 +1593,8 @@ test_array_bkg(void)
     /* -------------------------------------- */
     for ( i = 0; i < dtsinfo.nsubfields; i++)
     {
-        array_dt = H5Tarray_create (dtsinfo.datatype[i], ndims[i], dima, NULL);
-        CHECK(array_dt, FAIL, "H5Tarray_create");
+        array_dt = H5Tarray_create2(dtsinfo.datatype[i], ndims[i], dima);
+        CHECK(array_dt, FAIL, "H5Tarray_create2");
 
         status = H5Tinsert (type, dtsinfo.name[i], dtsinfo.offset[i], array_dt);
         CHECK(status, FAIL, "H5Tinsert");
@@ -1561,8 +1605,8 @@ test_array_bkg(void)
 
     /* Create the dataset */
     /* ------------------ */
-    dataset = H5Dcreate(fid, FIELDNAME, type, space, H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid, FIELDNAME, type, space, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write data to the dataset */
     /* ------------------------- */
@@ -1580,11 +1624,11 @@ test_array_bkg(void)
                 TestErrPrintf("Field a data doesn't match, cf[%d].a[%d]=%d, cfr[%d].a[%d]=%d\n",(int)i,(int)j,(int)cf[i].a[j],(int)i,(int)j,(int)cfr[i].a[j]);
                 continue;
             }
-            if(cf[i].b[j]!=cfr[i].b[j]) {
+            if(!FLT_ABS_EQUAL(cf[i].b[j],cfr[i].b[j])) {
                 TestErrPrintf("Field b data doesn't match, cf[%d].b[%d]=%f, cfr[%d].b[%d]=%f\n",(int)i,(int)j,(float)cf[i].b[j],(int)i,(int)j,(float)cfr[i].b[j]);
                 continue;
             }
-            if(cf[i].c[j]!=cfr[i].c[j]) {
+            if(!DBL_ABS_EQUAL(cf[i].c[j],cfr[i].c[j])) {
                 TestErrPrintf("Field c data doesn't match, cf[%d].b[%d]=%f, cfr[%d].b[%d]=%f\n",(int)i,(int)j,(float)cf[i].c[j],(int)i,(int)j,(float)cfr[i].c[j]);
                 continue;
             }
@@ -1620,14 +1664,14 @@ test_array_bkg(void)
     fid = H5Fopen(FILENAME, H5F_ACC_RDWR, H5P_DEFAULT);
     CHECK(fid, FAIL, "H5Fopen");
 
-    dataset = H5Dopen(fid, FIELDNAME );
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid, FIELDNAME, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     type = H5Tcreate(H5T_COMPOUND, sizeof(fld_t));
     CHECK(type, FAIL, "H5Tcreate");
 
-    array_dt = H5Tarray_create (H5T_NATIVE_FLOAT, 1, dima, NULL);
-    CHECK(array_dt, FAIL, "H5Tarray_create");
+    array_dt = H5Tarray_create2(H5T_NATIVE_FLOAT, 1, dima);
+    CHECK(array_dt, FAIL, "H5Tarray_create2");
 
     status = H5Tinsert (type, "Two", HOFFSET(fld_t, b), array_dt);
     CHECK(status, FAIL, "H5Tinsert");
@@ -1647,7 +1691,7 @@ test_array_bkg(void)
 
     for (i=0; i< LENGTH; i++)
         for (j = 0; j < ALEN; j++)
-            if(fld[i].b[j]!=fldr[i].b[j]) {
+            if(!FLT_ABS_EQUAL(fld[i].b[j],fldr[i].b[j])) {
                 TestErrPrintf("Field data doesn't match, fld[%d].b[%d]=%f, fldr[%d].b[%d]=%f\n",(int)i,(int)j,(float)fld[i].b[j],(int)i,(int)j,(float)fldr[i].b[j]);
                 continue;
             }
@@ -1673,11 +1717,11 @@ test_array_bkg(void)
                 TestErrPrintf("Field a data doesn't match, cf[%d].a[%d]=%d, cfr[%d].a[%d]=%d\n",(int)i,(int)j,(int)cf[i].a[j],(int)i,(int)j,(int)cfr[i].a[j]);
                 continue;
             }
-            if(cf[i].b[j]!=cfr[i].b[j]) {
+            if(!FLT_ABS_EQUAL(cf[i].b[j],cfr[i].b[j])) {
                 TestErrPrintf("Field b data doesn't match, cf[%d].b[%d]=%f, cfr[%d].b[%d]=%f\n",(int)i,(int)j,(float)cf[i].b[j],(int)i,(int)j,(float)cfr[i].b[j]);
                 continue;
             }
-            if(cf[i].c[j]!=cfr[i].c[j]) {
+            if(!DBL_ABS_EQUAL(cf[i].c[j],cfr[i].c[j])) {
                 TestErrPrintf("Field c data doesn't match, cf[%d].b[%d]=%f, cfr[%d].b[%d]=%f\n",(int)i,(int)j,(float)cf[i].c[j],(int)i,(int)j,(float)cfr[i].c[j]);
                 continue;
             }
@@ -1700,8 +1744,8 @@ test_array_bkg(void)
     fid = H5Fopen(FILENAME, H5F_ACC_RDWR, H5P_DEFAULT);
     CHECK(fid, FAIL, "H5Fopen");
 
-    dataset = H5Dopen(fid, FIELDNAME );
-    CHECK(dataset, FAIL, "H5Dopen");
+    dataset = H5Dopen2(fid, FIELDNAME, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dopen2");
 
     type = H5Dget_type(dataset);
     CHECK(type, FAIL, "H5Dget_type");
@@ -1722,11 +1766,11 @@ test_array_bkg(void)
                 TestErrPrintf("Field a data doesn't match, cf[%d].a[%d]=%d, cfr[%d].a[%d]=%d\n",(int)i,(int)j,(int)cf[i].a[j],(int)i,(int)j,(int)cfr[i].a[j]);
                 continue;
             }
-            if(cf[i].b[j]!=cfr[i].b[j]) {
+            if(!FLT_ABS_EQUAL(cf[i].b[j],cfr[i].b[j])) {
                 TestErrPrintf("Field b data doesn't match, cf[%d].b[%d]=%f, cfr[%d].b[%d]=%f\n",(int)i,(int)j,(float)cf[i].b[j],(int)i,(int)j,(float)cfr[i].b[j]);
                 continue;
             }
-            if(cf[i].c[j]!=cfr[i].c[j]) {
+            if(!DBL_ABS_EQUAL(cf[i].c[j],cfr[i].c[j])) {
                 TestErrPrintf("Field c data doesn't match, cf[%d].b[%d]=%f, cfr[%d].b[%d]=%f\n",(int)i,(int)j,(float)cf[i].c[j],(int)i,(int)j,(float)cfr[i].c[j]);
                 continue;
             }
@@ -1808,8 +1852,8 @@ test_compat(void)
     /* Only try to proceed if the file is around */
     if (fid1 >= 0){
         /* Open the first dataset (with no array fields) */
-        dataset = H5Dopen(fid1, "Dataset1");
-        CHECK_I(dataset, "H5Dopen");
+        dataset = H5Dopen2(fid1, "Dataset1", H5P_DEFAULT);
+        CHECK_I(dataset, "H5Dopen2");
 
         /* Get the datatype */
         tid1=H5Dget_type(dataset);
@@ -1890,8 +1934,8 @@ test_compat(void)
 
 
         /* Open the second dataset (with array fields) */
-        dataset = H5Dopen(fid1, "Dataset2");
-        CHECK_I(dataset, "H5Dopen");
+        dataset = H5Dopen2(fid1, "Dataset2", H5P_DEFAULT);
+        CHECK_I(dataset, "H5Dopen2");
 
         /* Get the datatype */
         tid1=H5Dget_type(dataset);
@@ -1948,8 +1992,8 @@ test_compat(void)
         VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
 
         /* Get the array dimensions */
-        ret=H5Tget_array_dims(mtid,rdims1,NULL);
-        CHECK(ret, FAIL, "H5Tget_array_dims");
+        ret = H5Tget_array_dims2(mtid, rdims1);
+        CHECK(ret, FAIL, "H5Tget_array_dims2");
 
         /* Check the array dimensions */
         for(i=0; i<ndims; i++)
@@ -1993,8 +2037,8 @@ test_compat(void)
         VERIFY(ndims,ARRAY1_RANK,"H5Tget_array_ndims");
 
         /* Get the array dimensions */
-        ret=H5Tget_array_dims(mtid,rdims1,NULL);
-        CHECK(ret, FAIL, "H5Tget_array_dims");
+        ret = H5Tget_array_dims2(mtid, rdims1);
+        CHECK(ret, FAIL, "H5Tget_array_dims2");
 
         /* Check the array dimensions */
         for(i=0; i<ndims; i++)
@@ -2069,6 +2113,7 @@ test_array(void)
     test_array_compound_array();    /* Test 1-D array of compound datatypes (with array fields) */
     test_array_vlen_atomic();   /* Test 1-D array of atomic VL datatypes */
     test_array_vlen_array();    /* Test 1-D array of 1-D array VL datatypes */
+    test_array_funcs();         /* Test type functions with array types */
 
     test_array_bkg();           /* Read compound datatype with array fields and background fields read */
 
