@@ -100,18 +100,14 @@ test_file_create(void)
     size_t		parm;		/*file-creation parameters	*/
     size_t		parm2;		/*file-creation parameters	*/
     unsigned		iparm;
-#ifdef H5_WANT_H5_V1_4_COMPAT
-    int		iparm2;
-#else /* H5_WANT_H5_V1_4_COMPAT */
     unsigned		iparm2;
-#endif /* H5_WANT_H5_V1_4_COMPAT */
     herr_t		ret;		/*generic return value		*/
 
     /* Output message about test being performed */
     MESSAGE(5, ("Testing Low-Level File Creation I/O\n"));
 
     /* First ensure the file does not exist */
-    remove(FILE1);
+    HDremove(FILE1);
 
     /* Try opening a non-existant file */
     fid1 = H5Fopen(FILE1, H5F_ACC_RDWR, H5P_DEFAULT);
@@ -128,14 +124,19 @@ test_file_create(void)
      * try to create the same file with H5F_ACC_TRUNC. This should fail
      * because fid1 is the same file and is currently open.
      */
+#ifndef H5_HAVE_FILE_VERSIONS
     fid2 = H5Fcreate(FILE1, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
     VERIFY(fid2, FAIL, "H5Fcreate");
+#endif /*H5_DONT_HAVE_FILE_VERSIONS*/
 
     /* Close all files */
     ret = H5Fclose(fid1);
     CHECK(ret, FAIL, "H5Fclose");
+
+#ifndef H5_HAVE_FILE_VERSIONS
     ret = H5Fclose(fid2);
     VERIFY(ret, FAIL, "H5Fclose"); /*file should not have been open */
+#endif /*H5_HAVE_FILE_VERSIONS*/
 
     /*
      * Try again with H5F_ACC_EXCL. This should fail because the file already
@@ -148,6 +149,7 @@ test_file_create(void)
     fid1 = H5Fcreate(FILE1, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
     CHECK(fid1, FAIL, "H5Fcreate");
 
+#ifndef H5_HAVE_FILE_VERSIONS
     /*
      * Try to truncate first file again. This should fail because fid1 is the
      * same file and is currently open.
@@ -161,6 +163,7 @@ test_file_create(void)
      */
     fid2 = H5Fcreate(FILE1, H5F_ACC_EXCL, H5P_DEFAULT, H5P_DEFAULT);
     VERIFY(fid2, FAIL, "H5Fcreate");
+#endif /*H5_HAVE_FILE_VERSIONS*/
 
     /* Get the file-creation template */
     tmpl1 = H5Fget_create_plist(fid1);
@@ -230,12 +233,12 @@ test_file_create(void)
        CHECK(dataspace_id, FAIL, "H5Screate_simple");
 
        /* Create the dataset. */
-       dataset_id = H5Dcreate(fid2, F2_DSET, H5T_NATIVE_INT, dataspace_id, H5P_DEFAULT);
-       CHECK(dataset_id, FAIL, "H5Dcreate");
+       dataset_id = H5Dcreate2(fid2, F2_DSET, H5T_NATIVE_INT, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+       CHECK(dataset_id, FAIL, "H5Dcreate2");
 
-       for(i=0; i<F2_DIM0; i++)
-           for(j=0; j<F2_DIM1; j++)
-               data[i][j]=i*10+j;
+       for(i = 0; i < F2_DIM0; i++)
+           for(j = 0; j < F2_DIM1; j++)
+               data[i][j] = i * 10 + j;
 
        /* Write data to the new dataset */
        ret = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data);
@@ -344,11 +347,8 @@ test_file_open(void)
     size_t		parm;		/*file-creation parameters	*/
     size_t		parm2;		/*file-creation parameters	*/
     unsigned		iparm;
-#ifdef H5_WANT_H5_V1_4_COMPAT
-    int		iparm2;
-#else /* H5_WANT_H5_V1_4_COMPAT */
     unsigned		iparm2;
-#endif /* H5_WANT_H5_V1_4_COMPAT */
+    unsigned		intent;
     herr_t		ret;		/*generic return value		*/
 
     /*
@@ -362,6 +362,11 @@ test_file_open(void)
     fid1 = H5Fopen(FILE2, H5F_ACC_RDWR, H5P_DEFAULT);
     CHECK(fid1, FAIL, "H5Fopen");
 
+    /* Get the intent */
+    ret = H5Fget_intent(fid1, &intent);
+    CHECK(ret, FAIL, "H5Fget_intent");
+    VERIFY(intent, H5F_ACC_RDWR, "H5Fget_intent");
+
     /* Get the file-creation template */
     tmpl1 = H5Fget_create_plist(fid1);
     CHECK(tmpl1, FAIL, "H5Fget_create_plist");
@@ -411,9 +416,18 @@ test_file_open(void)
     fid1 = H5Fopen(FILE2, H5F_ACC_RDONLY, fapl_id);
     CHECK(fid1, FAIL, "H5Fopen");
 
+    /* Check the intent */
+    ret = H5Fget_intent(fid1, &intent);
+    CHECK(ret, FAIL, "H5Fget_intent");
+    VERIFY(intent, H5F_ACC_RDONLY, "H5Fget_intent");
+
     /* Open dataset */
-    did = H5Dopen(fid1, F2_DSET);
-    CHECK(did, FAIL, "H5Dopen");
+    did = H5Dopen2(fid1, F2_DSET, H5P_DEFAULT);
+    CHECK(did, FAIL, "H5Dopen2");
+
+    /* Check that the intent works even if NULL is passed in */
+    ret = H5Fget_intent(fid1, NULL);
+    CHECK(ret, FAIL, "H5Fget_intent");
 
     /* Close first open */
     ret = H5Fclose(fid1);
@@ -423,6 +437,10 @@ test_file_open(void)
     fid2 = H5Fopen(FILE2, H5F_ACC_RDWR, fapl_id);
     VERIFY(fid2, FAIL, "H5Fopen");
 
+    /* Check that the intent fails for an invalid ID */
+    ret = H5Fget_intent(fid1, &intent);
+    VERIFY(ret, FAIL, "H5Fget_intent");
+
     /* Close dataset from first open */
     ret = H5Dclose(did);
     CHECK(ret, FAIL, "H5Dclose");
@@ -786,13 +804,12 @@ create_objects(hid_t fid1, hid_t fid2, hid_t *ret_did, hid_t *ret_gid1,
        CHECK(dataspace_id, FAIL, "H5Screate_simple");
 
        /* Create the dataset. */
-       dataset_id = H5Dcreate(fid1, "/dset", H5T_NATIVE_INT, dataspace_id,
-                        H5P_DEFAULT);
-       CHECK(dataset_id, FAIL, "H5Dcreate");
+       dataset_id = H5Dcreate2(fid1, "/dset", H5T_NATIVE_INT, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+       CHECK(dataset_id, FAIL, "H5Dcreate2");
 
-       for(i=0; i<F2_DIM0; i++)
-           for(j=0; j<F2_DIM1; j++)
-               data[i][j]=i*10+j;
+       for(i = 0; i < F2_DIM0; i++)
+           for(j = 0; j < F2_DIM1; j++)
+               data[i][j] = i * 10 + j;
 
        /* Write data to the new dataset */
        ret = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL,
@@ -810,18 +827,18 @@ create_objects(hid_t fid1, hid_t fid2, hid_t *ret_did, hid_t *ret_gid1,
     /* Create a group in the second file open */
     {
         hid_t   gid1, gid2, gid3;
-        gid1 = H5Gcreate(fid2, "/group", (size_t)0);
-        CHECK(gid1, FAIL, "H5Gcreate");
+        gid1 = H5Gcreate2(fid2, "/group", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+        CHECK(gid1, FAIL, "H5Gcreate2");
         if(ret_gid1 != NULL)
             *ret_gid1 = gid1;
 
-        gid2 = H5Gopen(fid2, "/group");
-        CHECK(gid2, FAIL, "H5Gopen");
+        gid2 = H5Gopen2(fid2, "/group", H5P_DEFAULT);
+        CHECK(gid2, FAIL, "H5Gopen2");
         if(ret_gid2 != NULL)
             *ret_gid2 = gid2;
 
-        gid3 = H5Gopen(fid2, "/group");
-        CHECK(gid3, FAIL, "H5Gopen");
+        gid3 = H5Gopen2(fid2, "/group", H5P_DEFAULT);
+        CHECK(gid3, FAIL, "H5Gopen2");
         if(ret_gid3 != NULL)
             *ret_gid3 = gid3;
     }
@@ -861,20 +878,26 @@ test_get_file_id(void)
     hid_t		datatype_id, dataset_id, dataspace_id, group_id, attr_id;
     hid_t               plist;
     hsize_t             dims[F2_RANK];
+    unsigned            intent;
     herr_t              ret;
 
     /* Create a file */
     fid = H5Fcreate(FILE4, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
     CHECK(fid, FAIL, "H5Fcreate");
 
+    /* Check the intent */
+    ret = H5Fget_intent(fid, &intent);
+    CHECK(ret, FAIL, "H5Fget_intent");
+    VERIFY(intent, H5F_ACC_RDWR, "H5Fget_intent");
+
     /* Test H5Iget_file_id() */
     check_file_id(fid, fid);
 
     /* Create a group in the file.  Make a duplicated file ID from the group.
      * And close this duplicated ID
      */
-    group_id = H5Gcreate(fid, GRP_NAME, (size_t)0);
-    CHECK(group_id, FAIL, "H5Gcreate");
+    group_id = H5Gcreate2(fid, GRP_NAME, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(group_id, FAIL, "H5Gcreate2");
 
     /* Test H5Iget_file_id() */
     check_file_id(fid, group_id);
@@ -893,8 +916,8 @@ test_get_file_id(void)
     fid = H5Fopen(FILE4, H5F_ACC_RDWR, H5P_DEFAULT);
     CHECK(fid, FAIL, "H5Fcreate");
 
-    group_id = H5Gopen(fid, GRP_NAME);
-    CHECK(group_id, FAIL, "H5Gcreate");
+    group_id = H5Gopen2(fid, GRP_NAME, H5P_DEFAULT);
+    CHECK(group_id, FAIL, "H5Gopen2");
 
     /* Test H5Iget_file_id() */
     check_file_id(fid, group_id);
@@ -917,9 +940,8 @@ test_get_file_id(void)
     dataspace_id = H5Screate_simple(F2_RANK, dims, NULL);
     CHECK(dataspace_id, FAIL, "H5Screate_simple");
 
-    dataset_id = H5Dcreate(group_id, DSET_NAME, H5T_NATIVE_INT, dataspace_id,
-                        H5P_DEFAULT);
-    CHECK(dataset_id, FAIL, "H5Dcreate");
+    dataset_id = H5Dcreate2(group_id, DSET_NAME, H5T_NATIVE_INT, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset_id, FAIL, "H5Dcreate2");
 
     /* Test H5Iget_file_id() */
     check_file_id(fid, dataset_id);
@@ -927,8 +949,8 @@ test_get_file_id(void)
     /* Create an attribute for the dataset.  Make a duplicated file ID from
      * this attribute.  And close it.
      */
-    attr_id=H5Acreate(dataset_id,ATTR_NAME,H5T_NATIVE_INT,dataspace_id,H5P_DEFAULT);
-    CHECK(ret, FAIL, "H5Acreate");
+    attr_id = H5Acreate2(dataset_id, ATTR_NAME, H5T_NATIVE_INT, dataspace_id, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Acreate2");
 
     /* Test H5Iget_file_id() */
     check_file_id(fid, attr_id);
@@ -936,11 +958,11 @@ test_get_file_id(void)
     /* Create a named datatype.  Make a duplicated file ID from
      * this attribute.  And close it.
      */
-    datatype_id=H5Tcopy(H5T_NATIVE_INT);
+    datatype_id = H5Tcopy(H5T_NATIVE_INT);
     CHECK(ret, FAIL, "H5Tcopy");
 
-    ret = H5Tcommit(fid, TYPE_NAME, datatype_id);
-    CHECK(ret, FAIL, "H5Tcommit");
+    ret = H5Tcommit2(fid, TYPE_NAME, datatype_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Tcommit2");
 
     /* Test H5Iget_file_id() */
     check_file_id(fid, datatype_id);
@@ -1125,21 +1147,22 @@ test_file_perm(void)
     CHECK(file, FAIL, "H5Fcreate");
 
     /* Create a dataset with the read-write file handle */
-    dset = H5Dcreate(file, F2_DSET, H5T_NATIVE_INT, dspace, H5P_DEFAULT);
-    CHECK(dset, FAIL, "H5Dcreate");
+    dset = H5Dcreate2(file, F2_DSET, H5T_NATIVE_INT, dspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dset, FAIL, "H5Dcreate2");
 
     ret = H5Dclose(dset);
     CHECK(ret, FAIL, "H5Dclose");
 
+#ifndef H5_CANNOT_OPEN_TWICE
     /* Open the file (with read-only permission) */
     filero = H5Fopen(FILE2, H5F_ACC_RDONLY, H5P_DEFAULT);
     CHECK(filero, FAIL, "H5Fopen");
 
     /* Create a dataset with the read-only file handle (should fail) */
     H5E_BEGIN_TRY {
-        dset = H5Dcreate(filero, F2_DSET, H5T_NATIVE_INT, dspace, H5P_DEFAULT);
+        dset = H5Dcreate2(filero, F2_DSET, H5T_NATIVE_INT, dspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(dset, FAIL, "H5Dcreate");
+    VERIFY(dset, FAIL, "H5Dcreate2");
     if(dset!=FAIL) {
         ret = H5Dclose(dset);
         CHECK(ret, FAIL, "H5Dclose");
@@ -1147,6 +1170,7 @@ test_file_perm(void)
 
     ret = H5Fclose(filero);
     CHECK(ret, FAIL, "H5Fclose");
+#endif /*H5_CANNOT_OPEN_TWICE*/
 
     ret = H5Fclose(file);
     CHECK(ret, FAIL, "H5Fclose");
@@ -1196,14 +1220,14 @@ test_file_freespace(void)
     CHECK(dcpl, FAIL, "H5Pcreate");
 
     /* Set the space allocation time to early */
-    ret = H5Pset_alloc_time(dcpl,H5D_ALLOC_TIME_EARLY);
+    ret = H5Pset_alloc_time(dcpl, H5D_ALLOC_TIME_EARLY);
     CHECK(ret, FAIL, "H5Pset_alloc_time");
 
     /* Create datasets in file */
-    for(u=0; u<10; u++) {
-        sprintf(name,"Dataset %u",u);
-        dset = H5Dcreate(file, name, H5T_STD_U32LE, dspace, dcpl);
-        CHECK(dset, FAIL, "H5Dcreate");
+    for(u = 0; u < 10; u++) {
+        sprintf(name, "Dataset %u", u);
+        dset = H5Dcreate2(file, name, H5T_STD_U32LE, dspace, H5P_DEFAULT, dcpl, H5P_DEFAULT);
+        CHECK(dset, FAIL, "H5Dcreate2");
 
         ret = H5Dclose(dset);
         CHECK(ret, FAIL, "H5Dclose");
@@ -1214,38 +1238,29 @@ test_file_freespace(void)
     CHECK(ret, FAIL, "H5Sclose");
 
     /* Close dataset creation property list */
-    ret=H5Pclose(dcpl);
+    ret = H5Pclose(dcpl);
     CHECK(ret, FAIL, "H5Pclose");
 
     /* Check that there is the right amount of free space in the file */
     free_space = H5Fget_freespace(file);
     CHECK(free_space, FAIL, "H5Fget_freespace");
-#ifdef H5_HAVE_LARGE_HSIZET
-    VERIFY(free_space, 168, "H5Fget_freespace");
-#else /* H5_HAVE_LARGE_HSIZET */
-    VERIFY(free_space, 76, "H5Fget_freespace");
-#endif /* H5_HAVE_LARGE_HSIZET */
+    VERIFY(free_space, 2376, "H5Fget_freespace");
 
     /* Delete datasets in file */
-    for(u=0; u<10; u++) {
-        sprintf(name,"Dataset %u",u);
-        ret = H5Gunlink(file, name);
-        CHECK(ret, FAIL, "H5Gunlink");
+    for(u = 0; u < 10; u++) {
+        sprintf(name, "Dataset %u", u);
+        ret = H5Ldelete(file, name, H5P_DEFAULT);
+        CHECK(ret, FAIL, "H5Ldelete");
     } /* end for */
 
     /* Check that there is the right amount of free space in the file */
     free_space = H5Fget_freespace(file);
     CHECK(free_space, FAIL, "H5Fget_freespace");
-#ifdef H5_HAVE_LARGE_HSIZET
-    VERIFY(free_space, 3584, "H5Fget_freespace");
-#else /* H5_HAVE_LARGE_HSIZET */
-    VERIFY(free_space, 3428, "H5Fget_freespace");
-#endif /* H5_HAVE_LARGE_HSIZET */
+    VERIFY(free_space, 0, "H5Fget_freespace");
 
     /* Close file */
     ret = H5Fclose(file);
     CHECK(ret, FAIL, "H5Fclose");
-
 } /* end test_file_freespace() */
 
 /****************************************************************
@@ -1354,8 +1369,8 @@ test_file_open_dot(void)
     CHECK(fid, FAIL, "H5Fcreate");
 
     /* Create a group in the HDF5 file */
-    gid = H5Gcreate(fid, GRP_NAME, (size_t)0);
-    CHECK(gid, FAIL, "H5Gcreate");
+    gid = H5Gcreate2(fid, GRP_NAME, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(gid, FAIL, "H5Gcreate2");
 
     /* Create a dataspace for creating datasets */
     sid = H5Screate(H5S_SCALAR);
@@ -1363,27 +1378,27 @@ test_file_open_dot(void)
 
     /* Create a dataset with no name using the file ID */
     H5E_BEGIN_TRY {
-        did = H5Dcreate(fid, ".", H5T_NATIVE_INT, sid, H5P_DEFAULT);
+        did = H5Dcreate2(fid, ".", H5T_NATIVE_INT, sid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(did, FAIL, "H5Dcreate");
+    VERIFY(did, FAIL, "H5Dcreate2");
 
     /* Create a dataset with no name using the group ID */
     H5E_BEGIN_TRY {
-        did = H5Dcreate(gid, ".", H5T_NATIVE_INT, sid, H5P_DEFAULT);
+        did = H5Dcreate2(gid, ".", H5T_NATIVE_INT, sid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(did, FAIL, "H5Dcreate");
+    VERIFY(did, FAIL, "H5Dcreate2");
 
     /* Open a dataset with no name using the file ID */
     H5E_BEGIN_TRY {
-        did = H5Dopen(fid, ".");
+        did = H5Dopen2(fid, ".", H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(did, FAIL, "H5Dopen");
+    VERIFY(did, FAIL, "H5Dopen2");
 
     /* Open a dataset with no name using the group ID */
     H5E_BEGIN_TRY {
-        did = H5Dopen(gid, ".");
+        did = H5Dopen2(gid, ".", H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(did, FAIL, "H5Dopen");
+    VERIFY(did, FAIL, "H5Dopen2");
 
     /* Make a copy of a datatype to use for creating a named datatype */
     tid = H5Tcopy(H5T_NATIVE_INT);
@@ -1391,50 +1406,50 @@ test_file_open_dot(void)
 
     /* Create a named datatype with no name using the file ID */
     H5E_BEGIN_TRY {
-        ret = H5Tcommit(fid, ".", tid);
+        ret = H5Tcommit2(fid, ".", tid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(ret, FAIL, "H5Tcommit");
+    VERIFY(ret, FAIL, "H5Tcommit2");
 
     /* Create a named datatype with no name using the group ID */
     H5E_BEGIN_TRY {
-        ret = H5Tcommit(gid, ".", tid);
+        ret = H5Tcommit2(gid, ".", tid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(ret, FAIL, "H5Tcommit");
+    VERIFY(ret, FAIL, "H5Tcommit2");
 
     /* Open a named datatype with no name using the file ID */
     H5E_BEGIN_TRY {
-        tid2 = H5Topen(fid, ".");
+        tid2 = H5Topen2(fid, ".", H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(tid2, FAIL, "H5Topen");
+    VERIFY(tid2, FAIL, "H5Topen2");
 
     /* Open a named datatype with no name using the group ID */
     H5E_BEGIN_TRY {
-        tid2 = H5Topen(gid, ".");
+        tid2 = H5Topen2(gid, ".", H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(tid2, FAIL, "H5Topen");
+    VERIFY(tid2, FAIL, "H5Topen2");
 
     /* Create a group with no name using the file ID */
     H5E_BEGIN_TRY {
-        gid2 = H5Gcreate(fid, ".", (size_t)0);
+        gid2 = H5Gcreate2(fid, ".", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(gid2, FAIL, "H5Gcreate");
+    VERIFY(gid2, FAIL, "H5Gcreate2");
 
     /* Create a group with no name using the group ID */
     H5E_BEGIN_TRY {
-        gid2 = H5Gcreate(gid, ".", (size_t)0);
+        gid2 = H5Gcreate2(gid, ".", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
     } H5E_END_TRY;
-    VERIFY(gid2, FAIL, "H5Gcreate");
+    VERIFY(gid2, FAIL, "H5Gcreate2");
 
     /* Open a group with no name using the file ID (should open the root group) */
-    gid2 = H5Gopen(fid, ".");
-    CHECK(gid2, FAIL, "H5Gopen");
+    gid2 = H5Gopen2(fid, ".", H5P_DEFAULT);
+    CHECK(gid2, FAIL, "H5Gopen2");
 
     ret = H5Gclose(gid2);
     CHECK(ret, FAIL, "H5Gclose");
 
     /* Open a group with no name using the group ID (should open the group again) */
-    gid2 = H5Gopen(gid, ".");
-    CHECK(gid2, FAIL, "H5Gopen");
+    gid2 = H5Gopen2(gid, ".", H5P_DEFAULT);
+    CHECK(gid2, FAIL, "H5Gopen2");
 
     ret = H5Gclose(gid2);
     CHECK(ret, FAIL, "H5Gclose");
@@ -1467,6 +1482,7 @@ test_file_open_overlap(void)
     hid_t gid;
     hid_t sid;
     int nobjs;          /* # of open objects */
+    unsigned intent;
     herr_t ret;         /* Generic return value */
 
     /* Output message about test being performed */
@@ -1480,17 +1496,22 @@ test_file_open_overlap(void)
     fid2 = H5Fopen(FILE1, H5F_ACC_RDWR, H5P_DEFAULT);
     CHECK(fid2, FAIL, "H5Fopen");
 
+    /* Check the intent */
+    ret = H5Fget_intent(fid1, &intent);
+    CHECK(ret, FAIL, "H5Fget_intent");
+    VERIFY(intent, H5F_ACC_RDWR, "H5Fget_intent");
+
     /* Create a group in file */
-    gid = H5Gcreate(fid1, GROUP1, (size_t)0);
-    CHECK(gid, FAIL, "H5Gcreate");
+    gid = H5Gcreate2(fid1, GROUP1, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(gid, FAIL, "H5Gcreate2");
 
     /* Create dataspace for dataset */
     sid = H5Screate(H5S_SCALAR);
     CHECK(sid, FAIL, "H5Screate");
 
     /* Create dataset in group w/first file ID */
-    did1 = H5Dcreate(gid, DSET1, H5T_NATIVE_INT, sid, H5P_DEFAULT);
-    CHECK(did1, FAIL, "H5Dcreate");
+    did1 = H5Dcreate2(gid, DSET1, H5T_NATIVE_INT, sid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(did1, FAIL, "H5Dcreate2");
 
     /* Check number of objects opened in first file */
     nobjs = H5Fget_obj_count(fid1, H5F_OBJ_LOCAL|H5F_OBJ_ALL);
@@ -1510,8 +1531,8 @@ test_file_open_overlap(void)
 
 
     /* Create dataset with second file ID */
-    did2 = H5Dcreate(fid2, DSET2, H5T_NATIVE_INT, sid, H5P_DEFAULT);
-    CHECK(did2, FAIL, "H5Dcreate");
+    did2 = H5Dcreate2(fid2, DSET2, H5T_NATIVE_INT, sid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(did2, FAIL, "H5Dcreate2");
 
     /* Check number of objects opened in first file */
     nobjs = H5Fget_obj_count(fid2, H5F_OBJ_ALL);
@@ -1564,16 +1585,16 @@ test_file_getname(void)
     CHECK(file_id, FAIL, "H5Fcreate");
 
     /* Get and verify file name */
-    name_len = H5Fget_name(file_id, name, TESTA_NAME_BUF_SIZE);
+    name_len = H5Fget_name(file_id, name, (size_t)TESTA_NAME_BUF_SIZE);
     CHECK(name_len, FAIL, "H5Fget_name");
     VERIFY_STR(name, FILE1, "H5Fget_name");
 
     /* Create a group in the root group */
-    group_id = H5Gcreate(file_id, TESTA_GROUPNAME, 0);
-    CHECK(group_id, FAIL, "H5Gcreate");
+    group_id = H5Gcreate2(file_id, TESTA_GROUPNAME, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(group_id, FAIL, "H5Gcreate2");
 
     /* Get and verify file name */
-    name_len = H5Fget_name(group_id, name, TESTA_NAME_BUF_SIZE);
+    name_len = H5Fget_name(group_id, name, (size_t)TESTA_NAME_BUF_SIZE);
     CHECK(name_len, FAIL, "H5Fget_name");
     VERIFY_STR(name, FILE1, "H5Fget_name");
 
@@ -1584,25 +1605,25 @@ test_file_getname(void)
     /* Try get file name from data space.  Supposed to fail because
      * it's illegal operation. */
     H5E_BEGIN_TRY {
-        name_len = H5Fget_name(space_id, name, TESTA_NAME_BUF_SIZE);
+        name_len = H5Fget_name(space_id, name, (size_t)TESTA_NAME_BUF_SIZE);
     } H5E_END_TRY;
     VERIFY(name_len, FAIL, "H5Fget_name");
 
     /* Create a new dataset */
-    dataset_id = H5Dcreate(file_id, TESTA_DSETNAME, H5T_NATIVE_INT, space_id, H5P_DEFAULT);
-    CHECK(dataset_id, FAIL, "H5Dcreate");
+    dataset_id = H5Dcreate2(file_id, TESTA_DSETNAME, H5T_NATIVE_INT, space_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset_id, FAIL, "H5Dcreate2");
 
     /* Get and verify file name */
-    name_len = H5Fget_name(dataset_id, name, TESTA_NAME_BUF_SIZE);
+    name_len = H5Fget_name(dataset_id, name, (size_t)TESTA_NAME_BUF_SIZE);
     CHECK(name_len, FAIL, "H5Fget_name");
     VERIFY_STR(name, FILE1, "H5Fget_name");
 
     /* Create an attribute for the dataset */
-    attr_id = H5Acreate(dataset_id,TESTA_ATTRNAME,H5T_NATIVE_INT,space_id,H5P_DEFAULT);
-    CHECK(attr_id, FAIL, "H5Acreate");
+    attr_id = H5Acreate2(dataset_id, TESTA_ATTRNAME, H5T_NATIVE_INT, space_id, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(attr_id, FAIL, "H5Acreate2");
 
     /* Get and verify file name */
-    name_len = H5Fget_name(attr_id, name, TESTA_NAME_BUF_SIZE);
+    name_len = H5Fget_name(attr_id, name, (size_t)TESTA_NAME_BUF_SIZE);
     CHECK(name_len, FAIL, "H5Fget_name");
     VERIFY_STR(name, FILE1, "H5Fget_name");
 
@@ -1618,11 +1639,11 @@ test_file_getname(void)
     CHECK(ret, FAIL, "H5Tinsert");
 
     /* Save it on file */
-    ret = H5Tcommit(file_id, TESTA_DTYPENAME, type_id);
-    CHECK(ret, FAIL, "H5Tcommit");
+    ret = H5Tcommit2(file_id, TESTA_DTYPENAME, type_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Tcommit2");
 
     /* Get and verify file name */
-    name_len = H5Fget_name(type_id, name, TESTA_NAME_BUF_SIZE);
+    name_len = H5Fget_name(type_id, name, (size_t)TESTA_NAME_BUF_SIZE);
     CHECK(name_len, FAIL, "H5Fget_name");
     VERIFY_STR(name, FILE1, "H5Fget_name");
 
@@ -1657,8 +1678,8 @@ test_file_getname(void)
 static void
 test_file_double_root_open(void)
 {
-    hid_t file1_id, file2_id; 
-    hid_t grp1_id, grp2_id; 
+    hid_t file1_id, file2_id;
+    hid_t grp1_id, grp2_id;
     herr_t ret;         /* Generic return value */
 
     /* Output message about test being performed */
@@ -1669,10 +1690,10 @@ test_file_double_root_open(void)
     file2_id = H5Fopen (FILE1, H5F_ACC_RDONLY, H5P_DEFAULT);
     CHECK(file2_id, FAIL, "H5Fopen");
 
-    grp1_id  = H5Gopen(file1_id, "/");
-    CHECK(grp1_id, FAIL, "H5Gopen");
-    grp2_id  = H5Gopen(file2_id, "/");
-    CHECK(grp2_id, FAIL, "H5Gopen");
+    grp1_id  = H5Gopen2(file1_id, "/", H5P_DEFAULT);
+    CHECK(grp1_id, FAIL, "H5Gopen2");
+    grp2_id  = H5Gopen2(file2_id, "/", H5P_DEFAULT);
+    CHECK(grp2_id, FAIL, "H5Gopen2");
 
     /* Note "assymetric" close order */
     ret = H5Gclose(grp1_id);
@@ -1696,8 +1717,8 @@ test_file_double_root_open(void)
 static void
 test_file_double_group_open(void)
 {
-    hid_t file1_id, file2_id; 
-    hid_t grp1_id, grp2_id; 
+    hid_t file1_id, file2_id;
+    hid_t grp1_id, grp2_id;
     herr_t ret;         /* Generic return value */
 
     /* Output message about test being performed */
@@ -1708,10 +1729,10 @@ test_file_double_group_open(void)
     file2_id = H5Fopen (FILE1, H5F_ACC_RDONLY, H5P_DEFAULT);
     CHECK(file2_id, FAIL, "H5Fopen");
 
-    grp1_id  = H5Gcreate(file1_id, GRP_NAME, (size_t)0);
-    CHECK(grp1_id, FAIL, "H5Gcreate");
-    grp2_id  = H5Gopen(file2_id, GRP_NAME);
-    CHECK(grp2_id, FAIL, "H5Gopen");
+    grp1_id  = H5Gcreate2(file1_id, GRP_NAME, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(grp1_id, FAIL, "H5Gcreate2");
+    grp2_id  = H5Gopen2(file2_id, GRP_NAME, H5P_DEFAULT);
+    CHECK(grp2_id, FAIL, "H5Gopen2");
 
     /* Note "assymetric" close order */
     ret = H5Gclose(grp1_id);
@@ -1735,9 +1756,9 @@ test_file_double_group_open(void)
 static void
 test_file_double_dataset_open(void)
 {
-    hid_t file1_id, file2_id; 
-    hid_t dset1_id, dset2_id; 
-    hid_t space_id; 
+    hid_t file1_id, file2_id;
+    hid_t dset1_id, dset2_id;
+    hid_t space_id;
     herr_t ret;         /* Generic return value */
 
     /* Output message about test being performed */
@@ -1752,10 +1773,10 @@ test_file_double_dataset_open(void)
     space_id = H5Screate(H5S_SCALAR);
     CHECK(space_id, FAIL, "H5Screate");
 
-    dset1_id  = H5Dcreate(file1_id, DSET_NAME, H5T_NATIVE_INT, space_id, H5P_DEFAULT);
-    CHECK(dset1_id, FAIL, "H5Dcreate");
-    dset2_id  = H5Dopen(file2_id, DSET_NAME);
-    CHECK(dset2_id, FAIL, "H5Dopen");
+    dset1_id  = H5Dcreate2(file1_id, DSET_NAME, H5T_NATIVE_INT, space_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dset1_id, FAIL, "H5Dcreate2");
+    dset2_id  = H5Dopen2(file2_id, DSET_NAME, H5P_DEFAULT);
+    CHECK(dset2_id, FAIL, "H5Dopen2");
 
     /* Close "supporting" dataspace */
     ret = H5Sclose(space_id);
@@ -1783,8 +1804,8 @@ test_file_double_dataset_open(void)
 static void
 test_file_double_datatype_open(void)
 {
-    hid_t file1_id, file2_id; 
-    hid_t type1_id, type2_id; 
+    hid_t file1_id, file2_id;
+    hid_t type1_id, type2_id;
     herr_t ret;         /* Generic return value */
 
     /* Output message about test being performed */
@@ -1797,10 +1818,10 @@ test_file_double_datatype_open(void)
 
     type1_id  = H5Tcopy(H5T_NATIVE_INT);
     CHECK(type1_id, FAIL, "H5Tcopy");
-    ret  = H5Tcommit(file1_id, TYPE_NAME, type1_id);
-    CHECK(ret, FAIL, "H5Tcommit");
-    type2_id  = H5Topen(file2_id, TYPE_NAME);
-    CHECK(type2_id, FAIL, "H5Topen");
+    ret  = H5Tcommit2(file1_id, TYPE_NAME, type1_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Tcommit2");
+    type2_id  = H5Topen2(file2_id, TYPE_NAME, H5P_DEFAULT);
+    CHECK(type2_id, FAIL, "H5Topen2");
 
     /* Note "assymetric" close order */
     ret = H5Tclose(type1_id);
@@ -1835,12 +1856,16 @@ test_file(void)
     test_file_freespace();      /* Test file free space information */
     test_file_ishdf5();         /* Test detecting HDF5 files correctly */
     test_file_open_dot();       /* Test opening objects with "." for a name */
+#ifndef H5_CANNOT_OPEN_TWICE
     test_file_open_overlap();   /* Test opening files in an overlapping manner */
+#endif /*H5_CANNOT_OPEN_TWICE*/
     test_file_getname();        /* Test basic H5Fget_name() functionality */
+#ifndef H5_CANNOT_OPEN_TWICE
     test_file_double_root_open();       /* Test opening root group from two files works properly */
     test_file_double_group_open();      /* Test opening same group from two files works properly */
     test_file_double_dataset_open();    /* Test opening same dataset from two files works properly */
     test_file_double_datatype_open();   /* Test opening same named datatype from two files works properly */
+#endif /*H5_CANNOT_OPEN_TWICE*/
 }				/* test_file() */
 
 
@@ -1861,8 +1886,8 @@ test_file(void)
 void
 cleanup_file(void)
 {
-    remove(FILE1);
-    remove(FILE2);
-    remove(FILE3);
-    remove(FILE4);
+    HDremove(FILE1);
+    HDremove(FILE2);
+    HDremove(FILE3);
+    HDremove(FILE4);
 }
