@@ -29,7 +29,7 @@
  *
  * HDF5 APIs exercised in thread:
  *
- *     H5Screate_simple, H5Tcopy, H5Tset_order, H5Dcreate, H5Dclose,
+ *     H5Screate_simple, H5Tcopy, H5Tset_order, H5Dcreate2, H5Dclose,
  *     H5Tclose, H5Sclose.
  *
  * Created: Apr 28 2000
@@ -55,28 +55,19 @@
 #define EXPECTED_ERROR_DEPTH	8
 #define WRITE_NUMBER		37
 
-static herr_t error_callback(void *);
-static herr_t walk_error_callback(int, H5E_error_t *, void *);
+static herr_t error_callback(hid_t , void *);
+static herr_t walk_error_callback(unsigned, const H5E_error2_t *, void *);
 static void *tts_error_thread(void *);
 
 /* Global variables */
 hid_t error_file;
 
 typedef struct err_num_struct {
-	int maj_num;
-	int min_num;
+    hid_t maj_num;
+    hid_t min_num;
 } err_num_t;
 
-err_num_t expected[] = {
-	{H5E_DATASET, H5E_CANTINIT},
-	{H5E_DATASET, H5E_CANTINIT},
-	{H5E_SYM, H5E_EXISTS},
-	{H5E_SYM, H5E_CANTINSERT},
-	{H5E_SYM, H5E_CANTINSERT},
-	{H5E_BTREE, H5E_CANTINIT},
-	{H5E_BTREE, H5E_CANTINSERT},
-	{H5E_SYM, H5E_CANTINSERT},
-};
+err_num_t expected[8];
 
 int error_flag = 0;
 int error_count = 0;
@@ -88,14 +79,41 @@ void tts_error(void)
     pthread_attr_t attribute;
     hid_t dataset;
     int value, i;
+    int ret;
+
+    /* Must initialize these at runtime */
+    expected[0].maj_num = H5E_DATASET;
+    expected[0].min_num = H5E_CANTINIT;
+
+    expected[1].maj_num = H5E_DATASET;
+    expected[1].min_num = H5E_CANTINIT;
+
+    expected[2].maj_num = H5E_LINK;
+    expected[2].min_num = H5E_CANTINIT;
+
+    expected[3].maj_num = H5E_SYM;
+    expected[3].min_num = H5E_CANTINSERT;
+
+    expected[4].maj_num = H5E_SYM;
+    expected[4].min_num = H5E_NOTFOUND;
+
+    expected[5].maj_num = H5E_SYM;
+    expected[5].min_num = H5E_CALLBACK;
+
+    expected[6].maj_num = H5E_SYM;
+    expected[6].min_num = H5E_EXISTS;
 
     /* set up mutex for global count of errors */
-    pthread_mutex_init(&error_mutex, NULL);
+    ret=pthread_mutex_init(&error_mutex, NULL);
+    assert(ret==0);
 
     /* make thread scheduling global */
-    pthread_attr_init(&attribute);
+    ret=pthread_attr_init(&attribute);
+    assert(ret==0);
+
 #ifdef H5_HAVE_SYSTEM_SCOPE_THREADS
-    pthread_attr_setscope(&attribute, PTHREAD_SCOPE_SYSTEM);
+    ret=pthread_attr_setscope(&attribute, PTHREAD_SCOPE_SYSTEM);
+    assert(ret==0);
 #endif /* H5_HAVE_SYSTEM_SCOPE_THREADS */
 
     /*
@@ -103,12 +121,17 @@ void tts_error(void)
      * creation plist and default file access plist
      */
     error_file = H5Fcreate(FILENAME, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+    assert(error_file>=0);
 
-    for (i = 0; i < NUM_THREAD; i++)
-        pthread_create(&threads[i], &attribute, tts_error_thread, NULL);
+    for (i = 0; i < NUM_THREAD; i++){
+        ret=pthread_create(&threads[i], &attribute, tts_error_thread, NULL);
+        assert(ret==0);
+    }
 
-    for (i = 0; i < NUM_THREAD; i++)
-        pthread_join(threads[i],NULL);
+    for (i = 0; i < NUM_THREAD; i++){
+        ret=pthread_join(threads[i],NULL);
+        assert(ret==0);
+    }
 
     if (error_flag)
         TestErrPrintf("Threads reporting different error values!\n");
@@ -116,17 +139,23 @@ void tts_error(void)
     if (error_count != NUM_THREAD - 1)
         TestErrPrintf("Error: %d threads failed instead of %d\n", error_count, NUM_THREAD-1);
 
-    dataset = H5Dopen(error_file, DATASETNAME);
-    H5Dread(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &value);
+    dataset = H5Dopen2(error_file, DATASETNAME, H5P_DEFAULT);
+    assert(dataset >= 0);
+
+    ret=H5Dread(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &value);
+    assert(ret>=0);
 
     if (value != WRITE_NUMBER)
         TestErrPrintf("Error: Successful thread wrote value %d instead of %d\n", value, WRITE_NUMBER);
 
-    H5Dclose(dataset);
-    H5Fclose(error_file);
+    ret=H5Dclose(dataset);
+    assert(ret>=0);
+    ret=H5Fclose(error_file);
+    assert(ret>=0);
 
     /* Destroy the thread attribute */
-    pthread_attr_destroy(&attribute);
+    ret=pthread_attr_destroy(&attribute);
+    assert(ret==0);
 }
 
 static
@@ -134,55 +163,63 @@ void *tts_error_thread(void UNUSED *arg)
 {
     hid_t dataspace, datatype, dataset;
     hsize_t dimsf[1]; /* dataset dimensions */
-    H5E_auto_t old_error_cb;
+    H5E_auto2_t old_error_cb;
     void *old_error_client_data;
     int value;
+    int ret;
 
     /* preserve previous error stack handler */
-    H5Eget_auto(&old_error_cb, &old_error_client_data);
+    H5Eget_auto2(H5E_DEFAULT, &old_error_cb, &old_error_client_data); 
 
     /* set each thread's error stack handler */
-    H5Eset_auto(error_callback, NULL);
+    H5Eset_auto2(H5E_DEFAULT, error_callback, NULL); 
 
     /* define dataspace for dataset */
     dimsf[0] = 1;
-    dataspace = H5Screate_simple(1,dimsf,NULL);
+    dataspace = H5Screate_simple(1, dimsf, NULL);
+    assert(dataspace >= 0);
 
     /* define datatype for the data using native little endian integers */
     datatype = H5Tcopy(H5T_NATIVE_INT);
+    assert(datatype >= 0);
     H5Tset_order(datatype, H5T_ORDER_LE);
 
     /* create a new dataset within the file */
-    dataset = H5Dcreate(error_file, DATASETNAME, datatype, dataspace, H5P_DEFAULT);
-    if (dataset >= 0) {   /* not an error */
+    dataset = H5Dcreate2(error_file, DATASETNAME, datatype, dataspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    if(dataset >= 0) {   /* not an error */
         value = WRITE_NUMBER;
         H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &value);
         H5Dclose(dataset);
-    }
+    } /* end if */
 
-    H5Tclose(datatype);
-    H5Sclose(dataspace);
+    ret = H5Tclose(datatype);
+    assert(ret >= 0);
+    ret = H5Sclose(dataspace);
+    assert(ret >= 0);
 
     /* turn our error stack handler off */
-    H5Eset_auto(old_error_cb, old_error_client_data);
+    H5Eset_auto2(H5E_DEFAULT, old_error_cb, old_error_client_data); 
 
     return NULL;
 }
 
 static
-herr_t error_callback(void *client_data)
+herr_t error_callback(hid_t estack_id, void *client_data)
 {
-    pthread_mutex_lock(&error_mutex);
-    error_count++;
-    pthread_mutex_unlock(&error_mutex);
+    int ret;
 
-    return H5Ewalk(H5E_WALK_DOWNWARD, walk_error_callback, client_data);
+    ret=pthread_mutex_lock(&error_mutex);
+    assert(ret==0);
+    error_count++;
+    ret=pthread_mutex_unlock(&error_mutex);
+    assert(ret==0);
+    return H5Ewalk2(H5E_DEFAULT, H5E_WALK_DOWNWARD, walk_error_callback, client_data);
 }
 
 static
-herr_t walk_error_callback(int n, H5E_error_t *err_desc, void UNUSED *client_data)
+herr_t walk_error_callback(unsigned n, const H5E_error2_t *err_desc, void UNUSED *client_data)
 {
-    int maj_num, min_num;
+    hid_t maj_num, min_num;
 
     if (err_desc) {
         maj_num = err_desc->maj_num;
