@@ -21,7 +21,7 @@
 *
 *************************************************************/
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <stdio.h>
 #endif
 
@@ -152,29 +152,29 @@ test_vlstrings_basic(void)
     CHECK(sid1, FAIL, "H5Screate_simple");
 
     /* Create a datatype to refer to */
-    tid1 = H5Tcopy (H5T_C_S1);
+    tid1 = H5Tcopy(H5T_C_S1);
     CHECK(tid1, FAIL, "H5Tcopy");
 
-    ret = H5Tset_size (tid1,H5T_VARIABLE);
+    ret = H5Tset_size(tid1,H5T_VARIABLE);
     CHECK(ret, FAIL, "H5Tset_size");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid1,"Dataset1",tid1,sid1,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset1", tid1, sid1, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     dataspace = H5Screate(H5S_SCALAR);
 
-    dataset2=H5Dcreate(fid1,"Dataset2",tid1,dataspace,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset2 = H5Dcreate2(fid1, "Dataset2", tid1, dataspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
-    wdata2 = (char*)HDcalloc(65534, sizeof(char));
-    HDmemset(wdata2, 'A', 65533);
+    wdata2 = (char*)HDcalloc((size_t)65534, sizeof(char));
+    HDmemset(wdata2, 'A', (size_t)65533);
 
-    ret=H5Dwrite(dataset2,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,&wdata2);
+    ret = H5Dwrite(dataset2, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, &wdata2);
     CHECK(ret, FAIL, "H5Dwrite");
 
     H5Sclose(dataspace);
@@ -182,7 +182,7 @@ test_vlstrings_basic(void)
     HDfree(wdata2);
 
     /* Change to the custom memory allocation routines for reading VL string */
-    xfer_pid=H5Pcreate(H5P_DATASET_XFER);
+    xfer_pid = H5Pcreate(H5P_DATASET_XFER);
     CHECK(xfer_pid, FAIL, "H5Pcreate");
 
     ret=H5Pset_vlen_mem_manager(xfer_pid,test_vlstr_alloc_custom,&mem_used,test_vlstr_free_custom,&mem_used);
@@ -281,52 +281,51 @@ test_vlstrings_special(void)
     CHECK(sid1, FAIL, "H5Screate_simple");
 
     /* Create a datatype to refer to */
-    tid1 = H5Tcopy (H5T_C_S1);
+    tid1 = H5Tcopy(H5T_C_S1);
     CHECK(tid1, FAIL, "H5Tcopy");
 
-    ret = H5Tset_size (tid1,H5T_VARIABLE);
+    ret = H5Tset_size(tid1,H5T_VARIABLE);
     CHECK(ret, FAIL, "H5Tset_size");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid1,"Dataset3",tid1,sid1,H5P_DEFAULT);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset3", tid1, sid1, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Read from dataset before writing data */
-    ret=H5Dread(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Check data read in */
-    for(i=0; i<SPACE1_DIM1; i++)
-        if(rdata[i]!=NULL)
+    for(i = 0; i < SPACE1_DIM1; i++)
+        if(rdata[i] != NULL)
             TestErrPrintf("VL doesn't match!, rdata[%d]=%p\n",(int)i,rdata[i]);
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Read dataset from disk */
-    ret=H5Dread(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Compare data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        if(HDstrlen(wdata[i])!=strlen(rdata[i])) {
+    for(i = 0; i < SPACE1_DIM1; i++) {
+        if(HDstrlen(wdata[i]) != HDstrlen(rdata[i])) {
             TestErrPrintf("VL data length don't match!, strlen(wdata[%d])=%d, strlen(rdata[%d])=%d\n",(int)i,(int)strlen(wdata[i]),(int)i,(int)strlen(rdata[i]));
             continue;
         } /* end if */
-        if((wdata[i]==NULL && rdata[i]!=NULL) || (rdata[i]==NULL && wdata[i]!=NULL)) {
+        if((wdata[i] == NULL && rdata[i] != NULL) || (rdata[i] == NULL && wdata[i] != NULL)) {
             TestErrPrintf("VL data values don't match!\n");
             continue;
         } /* end if */
-
-        if( HDstrcmp(wdata[i],rdata[i]) != 0 ) {
+        if(HDstrcmp(wdata[i], rdata[i]) != 0 ) {
             TestErrPrintf("VL data values don't match!, wdata[%d]=%s, rdata[%d]=%s\n",(int)i,wdata[i],(int)i,rdata[i]);
             continue;
         } /* end if */
     } /* end for */
 
     /* Reclaim the read VL data */
-    ret=H5Dvlen_reclaim(tid1,sid1,H5P_DEFAULT,rdata);
+    ret = H5Dvlen_reclaim(tid1, sid1, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dvlen_reclaim");
 
     /* Close Dataset */
@@ -334,41 +333,41 @@ test_vlstrings_special(void)
     CHECK(ret, FAIL, "H5Dclose");
 
     /* Create another dataset to test nil strings */
-    dcpl=H5Pcreate(H5P_DATASET_CREATE);
+    dcpl = H5Pcreate(H5P_DATASET_CREATE);
     CHECK(dcpl, FAIL, "H5Pcreate");
 
     /* Set the fill value for the second dataset */
-    fill=NULL;
-    ret=H5Pset_fill_value(dcpl, tid1, &fill);
+    fill = NULL;
+    ret = H5Pset_fill_value(dcpl, tid1, &fill);
     CHECK(ret, FAIL, "H5Pset_fill_value");
 
-    dataset=H5Dcreate(fid1,"Dataset4",tid1,sid1,dcpl);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset4", tid1, sid1, H5P_DEFAULT, dcpl, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Close dataset creation property list */
     ret = H5Pclose(dcpl);
     CHECK(ret, FAIL, "H5Pclose");
 
     /* Read from dataset before writing data */
-    ret=H5Dread(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Check data read in */
-    for(i=0; i<SPACE1_DIM1; i++)
-        if(rdata[i]!=NULL)
+    for(i = 0; i < SPACE1_DIM1; i++)
+        if(rdata[i] != NULL)
             TestErrPrintf("VL doesn't match!, rdata[%d]=%p\n",(int)i,rdata[i]);
 
     /* Try to write nil strings to disk. */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata2);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata2);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Read nil strings back from disk */
-    ret=H5Dread(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Check data read in */
-    for(i=0; i<SPACE1_DIM1; i++)
-        if(rdata[i]!=NULL)
+    for(i = 0; i < SPACE1_DIM1; i++)
+        if(rdata[i] != NULL)
             TestErrPrintf("VL doesn't match!, rdata[%d]=%p\n",(int)i,rdata[i]);
 
     /* Close Dataset */
@@ -436,15 +435,15 @@ static void test_vlstring_type(void)
     VERIFY(pad, H5T_STR_NULLPAD, "H5Tget_strpad");
 
     /* Commit variable-length string datatype to storage */
-    ret = H5Tcommit(fid, VLSTR_TYPE, tid_vlstr);
-    CHECK(ret, FAIL, "H5Tcommit");
+    ret = H5Tcommit2(fid, VLSTR_TYPE, tid_vlstr, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Tcommit2");
 
     /* Close datatype */
     ret = H5Tclose(tid_vlstr);
     CHECK(ret, FAIL, "H5Tclose");
 
-    tid_vlstr = H5Topen(fid, VLSTR_TYPE);
-    CHECK(tid_vlstr, FAIL, "H5Topen");
+    tid_vlstr = H5Topen2(fid, VLSTR_TYPE, H5P_DEFAULT);
+    CHECK(tid_vlstr, FAIL, "H5Topen2");
 
     ret = H5Tclose(tid_vlstr);
     CHECK(ret, FAIL, "H5Tclose");
@@ -457,8 +456,8 @@ static void test_vlstring_type(void)
     CHECK(fid, FAIL, "H5Fopen");
 
     /* Open the variable-length string datatype just created */
-    tid_vlstr = H5Topen(fid, VLSTR_TYPE);
-    CHECK(tid_vlstr, FAIL, "H5Topen");
+    tid_vlstr = H5Topen2(fid, VLSTR_TYPE, H5P_DEFAULT);
+    CHECK(tid_vlstr, FAIL, "H5Topen2");
 
     /* Verify character set and padding */
     cset = H5Tget_cset(tid_vlstr);
@@ -519,31 +518,31 @@ test_compact_vlstring(void)
     CHECK(ret, FAIL, "H5Pset_layout");
 
     /* Create a dataset */
-    dataset=H5Dcreate(fid1,"Dataset5",tid1,sid1,plist);
-    CHECK(dataset, FAIL, "H5Dcreate");
+    dataset = H5Dcreate2(fid1, "Dataset5", tid1, sid1, H5P_DEFAULT, plist, H5P_DEFAULT);
+    CHECK(dataset, FAIL, "H5Dcreate2");
 
     /* Write dataset to disk */
-    ret=H5Dwrite(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,wdata);
+    ret = H5Dwrite(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, wdata);
     CHECK(ret, FAIL, "H5Dwrite");
 
     /* Read dataset from disk */
-    ret=H5Dread(dataset,tid1,H5S_ALL,H5S_ALL,H5P_DEFAULT,rdata);
+    ret = H5Dread(dataset, tid1, H5S_ALL, H5S_ALL, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dread");
 
     /* Compare data read in */
-    for(i=0; i<SPACE1_DIM1; i++) {
-        if(HDstrlen(wdata[i])!=strlen(rdata[i])) {
+    for(i = 0; i < SPACE1_DIM1; i++) {
+        if(HDstrlen(wdata[i]) != HDstrlen(rdata[i])) {
             TestErrPrintf("VL data length don't match!, strlen(wdata[%d])=%d, strlen(rdata[%d])=%d\n",(int)i,(int)strlen(wdata[i]),(int)i,(int)strlen(rdata[i]));
             continue;
         } /* end if */
-        if( HDstrcmp(wdata[i],rdata[i]) != 0 ) {
+        if(HDstrcmp(wdata[i], rdata[i]) != 0) {
             TestErrPrintf("VL data values don't match!, wdata[%d]=%s, rdata[%d]=%s\n",(int)i,wdata[i],(int)i,rdata[i]);
             continue;
         } /* end if */
     } /* end for */
 
     /* Reclaim the read VL data */
-    ret=H5Dvlen_reclaim(tid1,sid1,H5P_DEFAULT,rdata);
+    ret = H5Dvlen_reclaim(tid1, sid1, H5P_DEFAULT, rdata);
     CHECK(ret, FAIL, "H5Dvlen_reclaim");
 
     /* Close Dataset */
@@ -591,15 +590,15 @@ static void test_write_vl_string_attribute(void)
     ret = H5Tset_size (type, H5T_VARIABLE);
     CHECK(ret, FAIL, "H5Tset_size");
 
-    root = H5Gopen(file, "/");
-    CHECK(root, FAIL, "H5Gopen");
+    root = H5Gopen2(file, "/", H5P_DEFAULT);
+    CHECK(root, FAIL, "H5Gopen2");
 
     dataspace = H5Screate(H5S_SCALAR);
     CHECK(dataspace, FAIL, "H5Screate");
 
     /* Test creating a "normal" sized string attribute */
-    att = H5Acreate(root, "test_scalar", type, dataspace, H5P_DEFAULT);
-    CHECK(att, FAIL, "H5Acreate");
+    att = H5Acreate2(root, "test_scalar", type, dataspace, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(att, FAIL, "H5Acreate2");
 
     ret = H5Awrite(att, type, &string_att);
     CHECK(ret, FAIL, "H5Awrite");
@@ -616,11 +615,11 @@ static void test_write_vl_string_attribute(void)
     CHECK(ret, FAIL, "HAclose");
 
     /* Test creating a "large" sized string attribute */
-    att = H5Acreate(root, "test_scalar_large", type, dataspace, H5P_DEFAULT);
-    CHECK(att, FAIL, "H5Acreate");
+    att = H5Acreate2(root, "test_scalar_large", type, dataspace, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(att, FAIL, "H5Acreate2");
 
-    string_att_write = (char*)HDcalloc(8192, sizeof(char));
-    HDmemset(string_att_write, 'A', 8191);
+    string_att_write = (char*)HDcalloc((size_t)8192, sizeof(char));
+    HDmemset(string_att_write, 'A', (size_t)8191);
 
     ret = H5Awrite(att, type, &string_att_write);
     CHECK(ret, FAIL, "H5Awrite");
@@ -677,12 +676,12 @@ static void test_read_vl_string_attribute(void)
     ret = H5Tset_size (type, H5T_VARIABLE);
     CHECK(ret, FAIL, "H5Tset_size");
 
-    root = H5Gopen(file, "/");
-    CHECK(root, FAIL, "H5Gopen");
+    root = H5Gopen2(file, "/", H5P_DEFAULT);
+    CHECK(root, FAIL, "H5Gopen2");
 
     /* Test reading "normal" sized string attribute */
-    att = H5Aopen_name(root, "test_scalar");
-    CHECK(att, FAIL, "H5Aopen_name");
+    att = H5Aopen(root, "test_scalar", H5P_DEFAULT);
+    CHECK(att, FAIL, "H5Aopen");
 
     ret = H5Aread(att, type, &string_att_check);
     CHECK(ret, FAIL, "H5Aread");
@@ -696,8 +695,8 @@ static void test_read_vl_string_attribute(void)
     CHECK(ret, FAIL, "HAclose");
 
     /* Test reading "large" sized string attribute */
-    att = H5Aopen_name(root, "test_scalar_large");
-    CHECK(att, FAIL, "H5Aopen_name");
+    att = H5Aopen(root, "test_scalar_large", H5P_DEFAULT);
+    CHECK(att, FAIL, "H5Aopen");
 
     ret = H5Aread(att, type, &string_att_check);
     CHECK(ret, FAIL, "H5Aread");
@@ -729,8 +728,8 @@ static void write_scalar_dset(hid_t file, hid_t type, hid_t space, char *name, c
       hid_t dset;
       herr_t ret;
 
-      dset = H5Dcreate (file, name, type, space, H5P_DEFAULT);
-      CHECK(dset, FAIL, "H5Dcreate");
+      dset = H5Dcreate2(file, name, type, space, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+      CHECK(dset, FAIL, "H5Dcreate2");
 
       ret = H5Dwrite(dset, type, space, space, H5P_DEFAULT, &data);
       CHECK(ret, FAIL, "H5Dwrite");
@@ -746,8 +745,8 @@ static void read_scalar_dset(hid_t file, hid_t type, hid_t space, char *name, ch
     herr_t ret;
     char *data_read;
 
-    dset = H5Dopen (file, name);
-    CHECK(dset, FAIL, "H5Dopen");
+    dset = H5Dopen2(file, name, H5P_DEFAULT);
+    CHECK(dset, FAIL, "H5Dopen2");
 
     ret = H5Dread(dset, type, space, space, H5P_DEFAULT, &data_read);
     CHECK(ret, FAIL, "H5Dread");
@@ -809,24 +808,24 @@ static void test_vl_rewrite(void)
     }
 
     /* Read back from file 2 */
-    for(i=0; i<REWRITE_NDATASETS; i++) {
+    for(i = 0; i < REWRITE_NDATASETS; i++) {
         sprintf(name, "/set_%d", i);
         read_scalar_dset(file2, type, space, name, name);
-    }
+    } /* end for */
 
     /* Remove from file 2. */
-    for(i=0; i<REWRITE_NDATASETS; i++) {
+    for(i = 0; i < REWRITE_NDATASETS; i++) {
         sprintf(name, "/set_%d", i);
-        ret = H5Gunlink(file2, name);
-        CHECK(ret, FAIL, "H5Gunlink");
-    }
+        ret = H5Ldelete(file2, name, H5P_DEFAULT);
+        CHECK(ret, FAIL, "H5Ldelete");
+    } /* end for */
 
     /* Effectively copy from file 1 to file 2 */
-    for(i=0; i<REWRITE_NDATASETS; i++) {
+    for(i = 0; i < REWRITE_NDATASETS; i++) {
         sprintf(name, "/set_%d", i);
         read_scalar_dset(file1, type, space, name, name);
         write_scalar_dset(file2, type, space, name, name);
-    }
+    } /* end for */
 
     /* Close everything */
     ret = H5Tclose(type);
