@@ -19,6 +19,7 @@
 
 #include "hdf5.h"
 #include "h5trav.h"
+#include "H5Zprivate.h"	/* H5Z_COMMON_CD_VALUES */	
 
 
 
@@ -48,14 +49,17 @@ typedef struct {
  H5Z_FILTER_SHUFFLE     2 , shuffle the data
  H5Z_FILTER_FLETCHER32  3 , letcher32 checksum of EDC
  H5Z_FILTER_SZIP        4 , szip compression
-
+ H5Z_FILTER_NBIT        5 , nbit compression
+ H5Z_FILTER_SCALEOFFSET 6 , scaleoffset compression
 */
 
-#define CDVALUES 2
+/* #define CD_VALUES H5Z_COMMON_CD_VALUES */
+#define CD_VALUES 20
 
 typedef struct {
- H5Z_filter_t filtn;               /* filter identification number */
- int          cd_values[CDVALUES]; /* filter client data values */
+ H5Z_filter_t filtn;                           /* filter identification number */
+ unsigned     cd_values[CD_VALUES];            /* filter client data values */
+ size_t       cd_nelmts;                       /* filter client number of values */
 } filter_info_t;
 
 /* chunk lengths along each dimension and rank */
@@ -96,12 +100,18 @@ typedef struct {
  pack_opttbl_t   *op_tbl;     /*table with all -c and -f options */
  int             all_layout;  /*apply the layout to all objects */
  int             all_filter;  /*apply the filter to all objects */
- filter_info_t   filter_g;    /*global filter INFO for the ALL case */
+ filter_info_t   filter_g[H5_REPACK_MAX_NFILTERS];    /*global filter array for the ALL case */
+ int             n_filter_g;  /*number of global filters */
  chunk_info_t    chunk_g;     /*global chunk INFO for the ALL case */
  H5D_layout_t    layout_g;    /*global layout information for the ALL case */
  int             verbose;     /*verbose mode */
  hsize_t         threshold;   /*minimum size to compress, in bytes */
  int             use_native;  /*use a native type in write */  
+ int             latest;      /*pack file with the latest file format */
+ int             grp_compact; /* Set the maximum number of links to store as header messages in the group */
+ int             grp_indexed; /* Set the minimum number of links to store in the indexed format */
+ int             msg_size[8]; /* Minumum size of shared messages: dataspace, 
+                                 datatype, fill value, filter pipleline, attribute */
 } pack_opt_t;
 
 
@@ -137,62 +147,38 @@ int h5repack_cmpdcpl   (const char *fname1,
  */
 
 
-int check_objects(const char* fname,
-                  pack_opt_t *options);
+/*-------------------------------------------------------------------------
+ * copy module
+ *-------------------------------------------------------------------------
+ */
 
-int copy_objects(const char* fnamein,
-                 const char* fnameout,
-                 pack_opt_t *options);
+int copy_objects   (const char* fnamein,
+                    const char* fnameout,
+                    pack_opt_t *options);
 
 int do_copy_refobjs(hid_t fidin,
                     hid_t fidout,
                     trav_table_t *travt,
-                    pack_opt_t *options); /* repack options */
-
+                    pack_opt_t *options); 
 
+/*-------------------------------------------------------------------------
+ * filters and verify module
+ *-------------------------------------------------------------------------
+ */
 void init_packobject(pack_info_t *obj);
-int print_filters(hid_t dcpl_id);
-int have_request(pack_opt_t *options);
-
-
 
 
 /*-------------------------------------------------------------------------
- * filters
+ * filters and copy module
  *-------------------------------------------------------------------------
  */
 
 int apply_filters(const char* name,    /* object name from traverse list */
                   int rank,            /* rank of dataset */
                   hsize_t *dims,       /* dimensions of dataset */
-                  hid_t dcpl_id,       /* (IN,OUT) dataset creation property list */
+                  hid_t dcpl_id,       /* dataset creation property list */
                   pack_opt_t *options, /* repack options */
-                  int *has_filter);     /* (OUT) object NAME has a filter */
-
-int has_filter(hid_t dcpl_id,
-               H5Z_filter_t filtnin);
-
-
-int can_read(const char* name,    /* object name from traverse list */
-             hid_t dcpl_id,       /* dataset creation property list */
-             pack_opt_t *options); /* repack options */
-
-
-/*-------------------------------------------------------------------------
- * layout functions
- *-------------------------------------------------------------------------
- */
-
-int has_layout(hid_t dcpl_id,
-               pack_info_t *obj);
-
-int layout_this(hid_t dcpl_id,             /* DCPL from input object */
-                const char* name,          /* object name from traverse list */
-                pack_opt_t *options,       /* repack options */
-                pack_info_t *pack /*OUT*/) /* object to apply layout */;
-
-int apply_layout(hid_t dcpl_id,
-                 pack_info_t *pack);  /* info about object  */
+                  int *has_filter);    /* (OUT) object NAME has a filter */
 
 
 /*-------------------------------------------------------------------------
@@ -220,15 +206,14 @@ pack_info_t* options_get_object( const char *path,
 obj_list_t* parse_filter(const char *str,
                          int *n_objs,
                          filter_info_t *filt,
-                         pack_opt_t *options);
+                         pack_opt_t *options,
+                         int *is_glb);
 
 obj_list_t* parse_layout(const char *str,
                          int *n_objs,
                          pack_info_t *pack,    /* info about object */
                          pack_opt_t *options);
 
-const char* get_sfilter (H5Z_filter_t filtn);
-int         parse_number(char *str);
 
 
 
